@@ -47,7 +47,7 @@ def _getBuiltinExceptionNames():
 
     # Hide Python3 changes for built-in exception names
     if python_version < 0x300:
-        import exceptions as builtin_exceptions
+        import exceptions as builtin_exceptions  # python2 code, pylint: disable=import-error
 
         for key in sorted(dir(builtin_exceptions)):
             name = str(key)
@@ -184,6 +184,7 @@ def _getAnonBuiltins():
         # pylint: disable=I0021,no-name-in-module
         from types import ClassType, InstanceType, MethodType
 
+        # We can to be sure of the type here, so use open without encoding, pylint: disable=unspecified-encoding
         with open(sys.executable) as any_file:
             anon_names["file"] = type(any_file)
         anon_codes["file"] = "&PyFile_Type"
@@ -197,6 +198,10 @@ def _getAnonBuiltins():
         anon_names["instancemethod"] = MethodType
         anon_codes["instancemethod"] = "&PyMethod_Type"
 
+    if python_version >= 0x270:
+        anon_names["version_info"] = type(sys.version_info)
+        anon_codes["version_info"] = 'Py_TYPE(Nuitka_SysGetObject("version_info"))'
+
     return anon_names, anon_codes
 
 
