@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,26 +21,23 @@ This contains tools to compare, classify and test constants.
 """
 
 import math
+import sys
 from types import BuiltinFunctionType
 
 from nuitka.Builtins import builtin_type_names
 from nuitka.PythonVersions import python_version
 
-from .__past__ import (  # pylint: disable=I0021,redefined-builtin
-    iterItems,
-    long,
-    unicode,
-    xrange
-)
+from .__past__ import GenericAlias, UnionType, iterItems, long, unicode, xrange
 from .Builtins import (
     builtin_anon_names,
     builtin_anon_value_list,
     builtin_exception_values_list,
-    builtin_named_values_list
+    builtin_named_values_list,
 )
 
 NoneType = type(None)
 
+
 def compareConstants(a, b):
     # Many many cases to deal with, pylint: disable=too-many-branches,too-many-return-statements
 
@@ -51,8 +48,7 @@ def compareConstants(a, b):
     # Now it's either not the same, or it is a container that contains NaN or it
     # is a complex or float that is NaN, the other cases can use == at the end.
     if type(a) is complex:
-        return compareConstants(a.imag, b.imag) and \
-               compareConstants(a.real, b.real)
+        return compareConstants(a.imag, b.imag) and compareConstants(a.real, b.real)
 
     if type(a) is float:
         # Check sign first, -0.0 is not 0.0, or -nan is not nan, it has a
@@ -80,8 +76,7 @@ def compareConstants(a, b):
 
         for ea1, ea2 in iterItems(a):
             for eb1, eb2 in iterItems(b):
-                if compareConstants(ea1, eb1) and \
-                   compareConstants(ea2, eb2):
+                if compareConstants(ea1, eb1) and compareConstants(ea2, eb2):
                     break
             else:
                 return False
@@ -109,6 +104,7 @@ def compareConstants(a, b):
     # constants are built in the same way, therefore above checks.
     return a == b
 
+
 # These built-in type references are kind of constant too. The list should be
 # complete.
 constant_builtin_types = (
@@ -125,16 +121,14 @@ constant_builtin_types = (
     NoneType,
 )
 
-if python_version >= 300:
-    constant_builtin_types += (
-        bytes,
-    )
+if python_version >= 0x300:
+    constant_builtin_types += (bytes,)
 else:
     constant_builtin_types += (
         unicode,
         long,
         # This has no name in Python, but the natural one in C-API.
-        builtin_anon_names["instance"]
+        builtin_anon_names["instance"],
     )
 
 
@@ -157,15 +151,30 @@ def isConstant(constant):
                 return False
         return True
     elif constant_type is slice:
-        if not isConstant(constant.start) or \
-           not isConstant(constant.stop) or \
-           not isConstant(constant.step):
+        if (
+            not isConstant(constant.start)
+            or not isConstant(constant.stop)
+            or not isConstant(constant.step)
+        ):
             return False
 
         return True
-    elif constant_type in (str, unicode, complex, int, long, bool, float,
-                           NoneType, range, bytes, set, frozenset, xrange,
-                           bytearray):
+    elif constant_type in (
+        str,
+        unicode,
+        complex,
+        int,
+        long,
+        bool,
+        float,
+        NoneType,
+        range,
+        bytes,
+        set,
+        frozenset,
+        xrange,
+        bytearray,
+    ):
         return True
     elif constant in (Ellipsis, NoneType, NotImplemented):
         return True
@@ -173,31 +182,52 @@ def isConstant(constant):
         return True
     elif constant_type is type:
         # Maybe pre-build this as a set for quicker testing.
-        return constant.__name__ in builtin_type_names or \
-               constant in builtin_exception_values_list
+        return (
+            constant.__name__ in builtin_type_names
+            or constant in builtin_exception_values_list
+        )
     elif constant_type is BuiltinFunctionType and constant in builtin_named_values_list:
         # TODO: Some others could also be usable and even interesting, but
         # then probably should go into other node types, e.g. str.join is
         # a candidate.
         return True
+    elif constant_type is GenericAlias:
+        return True
+    elif constant_type is UnionType:
+        return True
+    elif constant is sys.version_info:
+        return True
     else:
         return False
 
 
 def isMutable(constant):
-    """ Is a constant mutable
+    """Is a constant mutable
 
-        That means a user of a reference to it, can modify it. Strings are
-        a prime example of mutable, dictionaries are mutable.
+    That means a user of a reference to it, can modify it. Strings are
+    a prime example of immutable, dictionaries are mutable.
     """
     # Many cases and all return, that is how we do it here,
-    # pylint: disable=too-many-return-statements
+    # pylint: disable=too-many-branches,too-many-return-statements
 
     constant_type = type(constant)
 
-    if constant_type in (str, unicode, complex, int, long, bool, float,
-                         NoneType, range, bytes, slice, xrange, type,
-                         BuiltinFunctionType):
+    if constant_type in (
+        str,
+        unicode,
+        complex,
+        int,
+        long,
+        bool,
+        float,
+        NoneType,
+        range,
+        bytes,
+        slice,
+        xrange,
+        type,
+        BuiltinFunctionType,
+    ):
         return False
     elif constant_type in (dict, list, set, bytearray):
         return True
@@ -215,24 +245,42 @@ def isMutable(constant):
         return False
     elif constant is NotImplemented:
         return False
+    elif constant_type is GenericAlias:
+        return isMutable(constant.__origin__) or isMutable(constant.__args__)
+    elif constant_type is UnionType:
+        return False
+    elif constant is sys.version_info:
+        return False
     else:
         assert False, repr(constant)
 
 
 def isHashable(constant):
-    """ Is a constant hashable
+    """Is a constant hashable
 
-        That means a user of a reference to it, can use it for dicts and set
-        keys. This is distinct from mutable, there is one types that is not
-        mutable, and still not hashable: slices.
+    That means a user of a reference to it, can use it for dicts and set
+    keys. This is distinct from mutable, there is one types that is not
+    mutable, and still not hashable: slices.
     """
     # Many cases and all return, that is how we do it here,
     # pylint: disable=too-many-return-statements
 
     constant_type = type(constant)
 
-    if constant_type in (str, unicode, complex, int, long, bool, float,
-                         NoneType, xrange, bytes, type, BuiltinFunctionType):
+    if constant_type in (
+        str,
+        unicode,
+        complex,
+        int,
+        long,
+        bool,
+        float,
+        NoneType,
+        xrange,
+        bytes,
+        type,
+        BuiltinFunctionType,
+    ):
         return True
     elif constant_type in (dict, list, set, slice, bytearray):
         return False
@@ -258,8 +306,20 @@ def getUnhashableConstant(constant):
 
     constant_type = type(constant)
 
-    if constant_type in (str, unicode, complex, int, long, bool, float,
-                         NoneType, xrange, bytes, type, BuiltinFunctionType):
+    if constant_type in (
+        str,
+        unicode,
+        complex,
+        int,
+        long,
+        bool,
+        float,
+        NoneType,
+        xrange,
+        bytes,
+        type,
+        BuiltinFunctionType,
+    ):
         return None
     elif constant_type in (dict, list, set):
         return constant
@@ -279,32 +339,9 @@ def getUnhashableConstant(constant):
         assert False, constant_type
 
 
-def isIterableConstant(constant):
-    return type(constant) in (
-        str, unicode, list, tuple, set, frozenset, dict, xrange, bytes, bytearray
-    )
-
-
-def getConstantIterationLength(constant):
-    assert isIterableConstant(constant)
-
-    return len(constant)
-
-
-def isNumberConstant(constant):
-    return type(constant) in (int, long, float, bool)
-
-
-def isIndexConstant(constant):
-    return type(constant) in (int, long, bool)
-
-
 def createConstantDict(keys, values):
     # Create it proper size immediately.
-    constant_value = dict.fromkeys(
-        keys,
-        None
-    )
+    constant_value = dict.fromkeys(keys, None)
 
     for key, value in zip(keys, values):
         constant_value[key] = value
@@ -312,32 +349,8 @@ def createConstantDict(keys, values):
     return constant_value
 
 
-def getConstantWeight(constant):
-    constant_type = type(constant)
-
-    if constant_type is dict:
-        result = 0
-
-        for key, value in iterItems(constant):
-            result += getConstantWeight(key)
-            result += getConstantWeight(value)
-
-        return result
-    elif constant_type in (tuple, list, set, frozenset):
-        result = 0
-
-        for element_value in constant:
-            result += getConstantWeight(element_value)
-
-        return result
-    else:
-        return 1
-
-
 def isCompileTimeConstantValue(value):
-    """ Determine if a value will be usable at compile time.
-
-    """
+    """Determine if a value will be usable at compile time."""
     # This needs to match code in makeCompileTimeConstantReplacementNode
     if isConstant(value):
         return True
@@ -345,3 +358,14 @@ def isCompileTimeConstantValue(value):
         return True
     else:
         return False
+
+
+# Shared empty values, it would cost time to create them locally.
+the_empty_dict = {}
+the_empty_list = []
+the_empty_set = set()
+the_empty_bytearray = bytearray()
+
+the_empty_tuple = ()
+the_empty_frozenset = frozenset()
+the_empty_slice = slice(None)
