@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -25,6 +25,7 @@
     the existing set of modules.
 """
 
+import collections
 import os
 
 from nuitka.containers.oset import OrderedSet
@@ -37,6 +38,13 @@ root_modules = OrderedSet()
 # To be traversed modules
 active_modules = OrderedSet()
 
+# Information about why a module became active.
+active_modules_info = {}
+
+ActiveModuleInfo = collections.namedtuple(
+    "ActiveModuleInfo", ("using_module", "usage_tag", "reason", "source_ref")
+)
+
 # Already traversed modules
 done_modules = set()
 
@@ -52,6 +60,14 @@ def getRootModules():
     return root_modules
 
 
+def getRootTopModule():
+    top_module = next(iter(root_modules))
+
+    assert top_module.isTopModule(), top_module
+
+    return top_module
+
+
 def hasRootModule(module_name):
     for module in root_modules:
         if module.getFullName() == module_name:
@@ -77,48 +93,29 @@ def addUncompiledModule(module):
 
 
 def getUncompiledModules():
-    return sorted(
-        uncompiled_modules,
-        key = lambda module : module.getFullName()
-    )
+    return sorted(uncompiled_modules, key=lambda module: module.getFullName())
 
 
 def getUncompiledTechnicalModules():
-    result = [
-        module
-        for module in
-        uncompiled_modules
-        if module.isTechnical()
-    ]
+    result = [module for module in uncompiled_modules if module.isTechnical()]
 
-    return sorted(
-        result,
-        key = lambda module : module.getFullName()
-    )
+    return sorted(result, key=lambda module: module.getFullName())
 
 
 def getUncompiledNonTechnicalModules():
-    result = [
-        module
-        for module in
-        uncompiled_modules
-        if not module.isTechnical()
-    ]
+    result = [module for module in uncompiled_modules if not module.isTechnical()]
 
-    return sorted(
-        result,
-        key = lambda module : module.getFullName()
-    )
+    return sorted(result, key=lambda module: module.getFullName())
 
 
 def _normalizeModuleFilename(filename):
-    if python_version >= 300:
+    if python_version >= 0x300:
         filename = filename.replace("__pycache__", "")
 
         suffix = ".cpython-%d.pyc" % (python_version // 10)
 
         if filename.endswith(suffix):
-            filename = filename[:-len(suffix)] + ".py"
+            filename = filename[: -len(suffix)] + ".py"
     else:
         if filename.endswith(".pyc"):
             filename = filename[:-3] + ".py"
@@ -134,7 +131,7 @@ def getUncompiledModule(module_name, module_filename):
         if module_name == uncompiled_module.getFullName():
             if areSamePaths(
                 _normalizeModuleFilename(module_filename),
-                _normalizeModuleFilename(uncompiled_module.filename)
+                _normalizeModuleFilename(uncompiled_module.filename),
             ):
                 return uncompiled_module
 
@@ -148,19 +145,35 @@ def removeUncompiledModule(module):
 def startTraversal():
     # Using global here, as this is really a singleton, in the form of a module,
     # pylint: disable=global-statement
-    global active_modules, done_modules
+    global active_modules, done_modules, active_modules_info
 
     active_modules = OrderedSet(root_modules)
+
+    active_modules_info = {}
+    for root_module in root_modules:
+        active_modules_info[root_module] = ActiveModuleInfo(
+            using_module=None,
+            usage_tag="root_module",
+            reason="Root module",
+            source_ref=None,
+        )
     done_modules = set()
 
     for active_module in active_modules:
         active_module.startTraversal()
 
 
-def addUsedModule(module):
+def addUsedModule(module, using_module, usage_tag, reason, source_ref):
     if module not in done_modules and module not in active_modules:
         active_modules.add(module)
 
+        active_modules_info[module] = ActiveModuleInfo(
+            using_module=using_module,
+            usage_tag=usage_tag,
+            reason=reason,
+            source_ref=source_ref,
+        )
+
         module.startTraversal()
 
 
@@ -174,27 +187,20 @@ def nextModule():
         return None
 
 
-def remainingCount():
+def getRemainingModulesCount():
     return len(active_modules)
 
 
+def getDoneModulesCount():
+    return len(done_modules)
+
+
 def getDoneModules():
-    return sorted(
-        done_modules,
-        key = lambda module : module.getFullName()
-    )
+    return sorted(done_modules, key=lambda module: (module.getFullName(), module.kind))
 
 
-def getDoneUserModules():
-    return sorted(
-        [
-            module
-            for module in
-            done_modules
-            if not module.isInternalModule()
-        ],
-        key = lambda module : module.getFullName()
-    )
+def getModuleInclusionInfos():
+    return active_modules_info
 
 
 def removeDoneModule(module):
