@@ -31,8 +31,24 @@ import re
 import sys
 from optparse import SUPPRESS_HELP, OptionGroup, OptionParser
 
-from nuitka.utils import Utils
+from nuitka.PythonFlavors import (
+    isAnacondaPython,
+    isApplePython,
+    isDebianPackagePython,
+    isMSYS2MingwPython,
+    isNuitkaPython,
+    isPyenvPython,
+    isWinPython,
+)
 from nuitka.utils.FileOperations import getFileContentByLine
+from nuitka.utils.Utils import (
+    getArchitecture,
+    getLinuxDistribution,
+    getOS,
+    getWindowsRelease,
+    isLinux,
+    isPosixWindows,
+)
 from nuitka.Version import getCommercialVersion, getNuitkaVersion
 
 # Indicator if we were called as "nuitka-run" in which case we assume some
@@ -44,18 +60,51 @@ if not is_nuitka_run:
 else:
     usage = "usage: %prog [options] main_module.py"
 
+
+def _getPythonFlavor():
+    # return driven, pylint: disable=too-many-return-statements
+
+    if isNuitkaPython():
+        return "Nuitka Python"
+    elif isAnacondaPython():
+        return "Anaconda Python"
+    elif isWinPython():
+        return "WinPython"
+    elif isDebianPackagePython():
+        return "Debian Python"
+    elif isApplePython():
+        return "Apple Python"
+    elif isPyenvPython():
+        return "pyenv"
+    elif isPosixWindows():
+        return "MSYS2 Posix"
+    elif isMSYS2MingwPython():
+        return "MSYS2 MinGW"
+    else:
+        return "Unknown"
+
+
+def _getVersionInformationValues():
+    # TODO: Might be nice if we could delay version information computation
+    # until it's actually used.
+    yield getNuitkaVersion()
+    yield "Commercial: %s" % getCommercialVersion()
+    yield "Python: %s" % sys.version.split("\n", 1)[0]
+    yield "Flavor: %s" % _getPythonFlavor()
+    yield "Executable: %s" % sys.executable
+    yield "OS: %s" % getOS()
+    yield "Arch: %s" % getArchitecture()
+
+    if isLinux():
+        yield "Distribution: %s %s" % getLinuxDistribution()
+
+    if getOS() == "Windows":
+        yield "WindowsRelease: %s" % getWindowsRelease()
+
+
 parser = OptionParser(
     usage=usage,
-    version="\n".join(
-        (
-            getNuitkaVersion(),
-            "Commercial: %s" % getCommercialVersion(),
-            "Python: " + sys.version.split("\n")[0],
-            "Executable: " + sys.executable,
-            "OS: " + Utils.getOS(),
-            "Arch: " + Utils.getArchitecture(),
-        )
-    ),
+    version="\n".join(_getVersionInformationValues()),
 )
 
 parser.add_option(
@@ -118,7 +167,7 @@ if os.name == "nt":
         help="""\
 Architecture of Python to use. One of "x86" or "x86_64".
 Defaults to what you run Nuitka with (currently "%s")."""
-        % (Utils.getArchitecture()),
+        % (getArchitecture()),
     )
 
 parser.add_option(
@@ -143,7 +192,7 @@ enforces a specific mode. These are options that also exist to standard
 Python executable. Currently supported: "-S" (alias "no_site"),
 "static_hashes" (do not use hash randomization), "no_warnings" (do not
 give Python runtime warnings), "-O" (alias "no_asserts"), "no_docstrings"
-(do not use docstrings). Default empty.""",
+(do not use docstrings), and "-m".  Default empty.""",
 )
 
 parser.add_option(
@@ -183,7 +232,8 @@ parser.add_option(
     default=False,
     help="""\
 Allow Nuitka to download external code if necessary, e.g. dependency
-walker, ccache, and even gcc on Windows.""",
+walker, ccache, and even gcc on Windows. To disable, redirect input
+from nul device, e.g. "</dev/null" or "<NUL:". Default is to prompt.""",
 )
 
 
@@ -198,10 +248,11 @@ include_group.add_option(
     metavar="PACKAGE",
     default=[],
     help="""\
-Include a whole package. Give as a Python namespace, e.g. ``some_package.sub_package``
+Include a whole package. Give as a Python namespace, e.g. "some_package.sub_package"
 and Nuitka will then find it and include it and all the modules found below that
 disk location in the binary or extension module it creates, and make it available
-for import by the code. Default empty.""",
+for import by the code. To avoid unwanted sub packages, e.g. tests you can e.g. do
+this "--nofollow-import-to=*.tests". Default empty.""",
 )
 
 include_group.add_option(
@@ -211,7 +262,7 @@ include_group.add_option(
     metavar="MODULE",
     default=[],
     help="""\
-Include a single module. Give as a Python namespace, e.g. ``some_package.some_module``
+Include a single module. Give as a Python namespace, e.g. "some_package.some_module"
 and Nuitka will then find it and include it in the binary or extension module
 it creates, and make it available for import by the code. Default empty.""",
 )
@@ -219,19 +270,19 @@ it creates, and make it available for import by the code. Default empty.""",
 include_group.add_option(
     "--include-plugin-directory",
     action="append",
-    dest="recurse_extra",
+    dest="include_extra",
     metavar="MODULE/PACKAGE",
     default=[],
     help="""\
 Include the content of that directory, no matter if it's used by the given main
-program in a visible form. Overrides all other recursion options. Can be given
+program in a visible form. Overrides all other inclusion options. Can be given
 multiple times. Default empty.""",
 )
 
 include_group.add_option(
     "--include-plugin-files",
     action="append",
-    dest="recurse_extra_files",
+    dest="include_extra_files",
     metavar="PATTERN",
     default=[],
     help="""\
@@ -268,7 +319,7 @@ follow_group = OptionGroup(parser, "Control the following into imported modules"
 follow_group.add_option(
     "--follow-stdlib",
     action="store_true",
-    dest="recurse_stdlib",
+    dest="follow_stdlib",
     default=False,
     help="""\
 Also descend into imported modules from standard library. This will increase
@@ -282,7 +333,7 @@ follow_group.add_option(
     default=False,
     help="""\
 When --nofollow-imports is used, do not descend into any imported modules at all,
-overrides all other recursion options. Defaults to off.""",
+overrides all other inclusion options. Defaults to off.""",
 )
 
 follow_group.add_option(
@@ -491,7 +542,7 @@ include path information that needs to exist though. Defaults to '%s' on this
 platform.
 """
     % "<program_name>"
-    + (".exe" if Utils.getOS() == "Windows" else ".bin"),
+    + (".exe" if getOS() == "Windows" else ".bin"),
 )
 
 output_group.add_option(
@@ -563,12 +614,13 @@ Enable vmprof based profiling of time spent. Not working currently. Defaults to
 )
 
 debug_group.add_option(
-    "--graph",
+    "--internal-graph",
     action="store_true",
     dest="graph",
     default=False,
     help="""\
-Create graph of optimization process. Defaults to off.""",
+Create graph of optimization process internals, do not use for whole programs, but only
+for small test cases. Defaults to off.""",
 )
 
 debug_group.add_option(
@@ -625,6 +677,17 @@ debug_group.add_option(
     help=SUPPRESS_HELP,
 )
 
+debug_group.add_option(
+    "--low-memory",
+    action="store_true",
+    dest="low_memory",
+    default=False,
+    help="""\
+Attempt to use less memory, by forking less C compilation jobs and using
+options that use less memory. For use on embedded machines. Use this in
+case of out of memory problems. Defaults to off.""",
+)
+
 if os.name == "nt":
     debug_group.add_option(
         "--disable-dll-dependency-cache",
@@ -688,14 +751,15 @@ Enforce the use of MinGW64 on Windows. Defaults to off.""",
 c_compiler_group.add_option(
     "--msvc",
     action="store",
-    dest="msvc",
+    dest="msvc_version",
     default=None,
     help="""\
 Enforce the use of specific MSVC version on Windows. Allowed values
-are e.g. 14.0, specify an illegal value for a list of installed compilers,
-beware that only latest MSVC is really supported.
+are e.g. "14.2" (MSVC 2019), specify an illegal value for a list of
+installed compilers, or use "latest". Notice that only latest
+MSVC is really supported, and you can use "latest" to enforce that.
 
-Defaults to the most recent version.""",
+Defaults to MSVC on Windows being used if installed, otherwise MinGW64.""",
 )
 
 c_compiler_group.add_option(
@@ -704,7 +768,7 @@ c_compiler_group.add_option(
     action="store",
     dest="jobs",
     metavar="N",
-    default=Utils.getCoreCount(),
+    default=None,
     help="""\
 Specify the allowed number of parallel C compiler jobs. Defaults to the
 system CPU count.""",
@@ -735,6 +799,15 @@ Use static link library of Python. Allowed values are "yes", "no",
 and "auto" (when it's known to work). Defaults to "auto".""",
 )
 
+c_compiler_group.add_option(
+    "--disable-ccache",
+    action="store_true",
+    dest="disable_ccache",
+    default=False,
+    help="""\
+Do not attempt to use ccache (gcc, clang, etc.) or clcache (MSVC, clangcl).""",
+)
+
 parser.add_option_group(c_compiler_group)
 
 pgo_group = OptionGroup(parser, "PGO compilation choices")
@@ -742,12 +815,38 @@ pgo_group = OptionGroup(parser, "PGO compilation choices")
 pgo_group.add_option(
     "--pgo",
     action="store_true",
-    dest="is_pgo",
+    dest="is_c_pgo",
     default=False,
     help="""\
-Enables profile guided optimization (PGO), by executing a dedicated build first for a profiling
-run, and then using the result to feedback into the C compilation. Note: This is highly
-experimental and not working with many modes of Nuitka yet. Defaults to off.""",
+Enables C level profile guided optimization (PGO), by executing a dedicated build first
+for a profiling run, and then using the result to feedback into the C compilation.
+Note: This is experimental and not working with standalone modes of Nuitka yet.
+Defaults to off.""",
+)
+
+pgo_group.add_option(
+    "--pgo-python",
+    action="store_true",
+    dest="is_python_pgo",
+    default=False,
+    help=SUPPRESS_HELP,
+)
+
+pgo_group.add_option(
+    "--pgo-python-input",
+    action="store",
+    dest="python_pgo_input",
+    default=None,
+    help=SUPPRESS_HELP,
+)
+
+pgo_group.add_option(
+    "--pgo-python-policy-unused-module",
+    action="store",
+    dest="python_pgo_policy_unused_module",
+    choices=("include", "exclude", "bytecode"),
+    default="include",
+    help=SUPPRESS_HELP,
 )
 
 pgo_group.add_option(
@@ -844,6 +943,15 @@ tracing_group.add_option(
 Where to output --show-modules, should be a filename. Default is standard output.""",
 )
 
+tracing_group.add_option(
+    "--report",
+    action="store",
+    dest="compilation_report_filename",
+    default=None,
+    help="""\
+Report module inclusion in an XML output file. Default is off.""",
+)
+
 tracing_group.add_option(
     "--verbose",
     action="store_true",
@@ -969,7 +1077,7 @@ windows_group.add_option(
     default=None,
     help="""\
 File version to use in Windows Version information. Must be a sequence of
-up to 4 numbers, nothing else allowed.
+up to 4 numbers, e.g. 1.0.0.0, only this format is allowed.
 One of file or product version is required, when a version resource needs to be
 added, e.g. to specify product name, or company name. Defaults to unused.""",
 )
@@ -982,7 +1090,7 @@ windows_group.add_option(
     default=None,
     help="""\
 Product version to use in Windows Version information. Must be a sequence of
-up to 4 numbers, nothing else allowed.
+up to 4 numbers, e.g. 1.0.0.0, only this format is allowed.
 One of file or product version is required, when a version resource needs to be
 added, e.g. to specify product name, or company name. Defaults to unused.""",
 )
@@ -1102,6 +1210,18 @@ Name of the product to use in macOS bundle information. Defaults to base
 filename of the binary.""",
 )
 
+macos_group.add_option(
+    "--macos-app-version",
+    action="store",
+    dest="macos_app_version",
+    metavar="MACOS_APP_VERSION",
+    default=None,
+    help="""\
+Product version to use in macOS bundle information. Defaults to 1.0 if
+not given.""",
+)
+
+
 parser.add_option_group(macos_group)
 
 linux_group = OptionGroup(parser, "Linux specific controls")
@@ -1120,8 +1240,8 @@ parser.add_option_group(linux_group)
 plugin_group = OptionGroup(parser, "Plugin control")
 
 plugin_group.add_option(
-    "--plugin-enable",
     "--enable-plugin",
+    "--plugin-enable",
     action="append",
     dest="plugins_enabled",
     metavar="PLUGIN_NAME",
@@ -1132,8 +1252,8 @@ full list and exit. Default empty.""",
 )
 
 plugin_group.add_option(
-    "--plugin-disable",
     "--disable-plugin",
+    "--plugin-disable",
     action="append",
     dest="plugins_disabled",
     metavar="PLUGIN_NAME",
@@ -1150,9 +1270,9 @@ plugin_group.add_option(
     default=True,
     help="""\
 Plugins can detect if they might be used, and the you can disable the warning
-via --plugin-disable=plugin-that-warned, or you can use this option to disable
+via "--disable-plugin=plugin-that-warned", or you can use this option to disable
 the mechanism entirely, which also speeds up compilation slightly of course as
-this detection code is run in vain once you are certain of which plug-ins to
+this detection code is run in vain once you are certain of which plugins to
 use. Defaults to off.""",
 )
 
@@ -1191,7 +1311,7 @@ even with pure Python. Default False.""",
 
 
 def _considerPluginOptions(logger):
-    # Recursive dependency on plugins during parsing of command line.
+    # Cyclic dependency on plugins during parsing of command line.
     from nuitka.plugins.Plugins import (
         addPluginCommandLineOptions,
         addUserPluginCommandLineOptions,
@@ -1202,7 +1322,7 @@ def _considerPluginOptions(logger):
             plugin_names = arg[16:]
             if "=" in plugin_names:
                 logger.sysexit(
-                    "Error, plugin options format changed. Use '--plugin-enable=%s --help' to know new options."
+                    "Error, plugin options format changed. Use '--enable-plugin=%s --help' to know new options."
                     % plugin_names.split("=", 1)[0]
                 )
 
@@ -1233,12 +1353,25 @@ def _expandProjectArg(arg, filename_arg, for_eval):
             return value
 
     values = {
-        "OS": wrap(Utils.getOS()),
-        "Arch": wrap(Utils.getArchitecture()),
+        "OS": wrap(getOS()),
+        "Arch": wrap(getArchitecture()),
+        "Flavor": wrap(_getPythonFlavor()),
         "Version": getNuitkaVersion(),
-        "Commercial": getCommercialVersion(),
+        "Commercial": wrap(getCommercialVersion()),
         "MAIN_DIRECTORY": wrap(os.path.dirname(filename_arg) or "."),
     }
+
+    if isLinux():
+        dist_info = getLinuxDistribution()
+    else:
+        dist_info = "N/A", "0"
+
+    values["Linux_Distribution_Name"] = dist_info[0]
+    values["Linux_Distribution_Version"] = dist_info[1]
+
+    if getOS() == "Windows":
+        values["WindowsRelease"] = getWindowsRelease()
+
     arg = arg.format(**values)
 
     return arg
@@ -1246,9 +1379,6 @@ def _expandProjectArg(arg, filename_arg, for_eval):
 
 def _getProjectOptions(logger, filename_arg, module_mode):
     # Complex stuff, pylint: disable=too-many-branches,too-many-locals
-    # Do it only once.
-    if os.environ.get("NUITKA_REEXECUTION", "0") == "1":
-        return
 
     if os.path.isdir(filename_arg):
         if module_mode:
@@ -1324,6 +1454,9 @@ def _getProjectOptions(logger, filename_arg, module_mode):
             elif command == "":
                 arg = re.sub(r"""^([\w-]*=)(['"])(.*)\2$""", r"\1\3", arg.lstrip())
 
+                if not arg:
+                    continue
+
                 yield _expandProjectArg(arg, filename_arg, for_eval=False)
             else:
                 assert False, (command, line)
