@@ -23,15 +23,19 @@ should attempt to make run time detections.
 
 """
 
+import __future__
+
 import os
 import re
 import sys
 
+from nuitka.__past__ import WindowsError  # pylint: disable=I0021,redefined-builtin
+
 
 def getSupportedPythonVersions():
     """Officially supported Python versions for Nuitka."""
 
-    return ("2.6", "2.7", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9")
+    return ("2.6", "2.7", "3.3", "3.4", "3.5", "3.6", "3.7", "3.8", "3.9", "3.10")
 
 
 def getPartiallySupportedPythonVersions():
@@ -54,7 +58,6 @@ def getSupportedPythonVersionStr():
 def _getPythonVersion():
     big, major, minor = sys.version_info[0:3]
 
-    # TODO: Give up on decimal versions already.
     return big * 256 + major * 16 + min(15, minor)
 
 
@@ -132,29 +135,6 @@ def needsDuplicateArgumentColOffset():
         return True
 
 
-def isUninstalledPython():
-    if os.name == "nt":
-        import ctypes.wintypes
-
-        GetSystemDirectory = ctypes.windll.kernel32.GetSystemDirectoryW
-        GetSystemDirectory.argtypes = (ctypes.wintypes.LPWSTR, ctypes.wintypes.DWORD)
-        GetSystemDirectory.restype = ctypes.wintypes.DWORD
-
-        MAX_PATH = 4096
-        buf = ctypes.create_unicode_buffer(MAX_PATH)
-
-        res = GetSystemDirectory(buf, MAX_PATH)
-        assert res != 0
-
-        system_path = os.path.normcase(buf.value)
-        return not getRunningPythonDLLPath().startswith(system_path)
-
-    return (
-        os.path.exists(os.path.join(sys.prefix, "conda-meta"))
-        or "WinPython" in sys.version
-    )
-
-
 def getRunningPythonDLLPath():
     import ctypes.wintypes
 
@@ -206,6 +186,12 @@ def getTargetPythonDLLPath():
 
 
 def isStaticallyLinkedPython():
+    # On Windows, there is no way to detect this from syconfig.
+    if os.name == "nt":
+        import ctypes
+
+        return ctypes.pythonapi is None
+
     try:
         import sysconfig
     except ImportError:
@@ -215,26 +201,6 @@ def isStaticallyLinkedPython():
 
     result = sysconfig.get_config_var("Py_ENABLE_SHARED") == 0
 
-    if result:
-        from nuitka.utils.Execution import check_output
-
-        with open(os.devnull, "w") as devnull:
-            output = check_output(
-                (os.path.realpath(sys.executable) + "-config", "--ldflags"),
-                stderr=devnull,
-            )
-
-        if str is not bytes:
-            output = output.decode("utf8")
-
-        import shlex
-
-        output = shlex.split(output)
-
-        python_abi_version = python_version_str + getPythonABI()
-
-        result = ("-lpython" + python_abi_version) not in output
-
     return result
 
 
@@ -254,14 +220,98 @@ def getPythonABI():
     return abiflags
 
 
+_the_sys_prefix = None
+
+
 def getSystemPrefixPath():
-    """Return real sys.prefix as an absolute path.
+    """Return real sys.prefix as an absolute path breaking out of virtualenv.
 
-    Note: This breaks out of virtualenvs as necessary.
+    Note:
 
-    Returns: path to system prefix
+        For Nuitka, it often is OK to break out of the virtualenv, and use the
+        original install. Mind you, this is not about executing anything, this is
+        about building, and finding the headers to compile against that Python, we
+        do not care about any site packages, and so on.
+
+    Returns:
+        str - path to system prefix
     """
 
-    sys_prefix = getattr(sys, "real_prefix", getattr(sys, "base_prefix", sys.prefix))
-    sys_prefix = os.path.abspath(sys_prefix)
-    return sys_prefix
+    global _the_sys_prefix  # Cached result, pylint: disable=global-statement
+    if _the_sys_prefix is None:
+
+        sys_prefix = getattr(
+            sys, "real_prefix", getattr(sys, "base_prefix", sys.prefix)
+        )
+        sys_prefix = os.path.abspath(sys_prefix)
+
+        # Some virtualenv contain the "orig-prefix.txt" as a textual link to the
+        # target, this is often on Windows with virtualenv. There are two places to
+        # look for.
+        for candidate in (
+            "Lib/orig-prefix.txt",
+            "lib/python%s/orig-prefix.txt" % python_version_str,
+        ):
+            candidate = os.path.join(sys_prefix, candidate)
+            if os.path.exists(candidate):
+                # Cannot use FileOperations.getFileContents() here, because of circular dependency.
+                # pylint: disable=unspecified-encoding
+                with open(candidate) as f:
+                    sys_prefix = f.read()
+
+                # Trailing spaces in the python prefix, please not.
+                assert sys_prefix == sys_prefix.strip()
+
+        # This is another for of virtualenv references:
+        if os.name != "nt" and os.path.islink(os.path.join(sys_prefix, ".Python")):
+            sys_prefix = os.path.normpath(
+                os.path.join(os.readlink(os.path.join(sys_prefix, ".Python")), "..")
+            )
+
+        # Some virtualenv created by "venv" seem to have a different structure, where
+        # library and include files are outside of it.
+        if (
+            os.name != "nt"
+            and python_version >= 0x330
+            and os.path.exists(os.path.join(sys_prefix, "bin/activate"))
+        ):
+            python_binary = os.path.join(sys_prefix, "bin", "python")
+            python_binary = os.path.realpath(python_binary)
+
+            sys_prefix = os.path.normpath(os.path.join(python_binary, "..", ".."))
+
+        _the_sys_prefix = sys_prefix
+
+    return _the_sys_prefix
+
+
+def getFutureModuleKeys():
+    result = [
+        "unicode_literals",
+        "absolute_import",
+        "division",
+        "print_function",
+        "generator_stop",
+        "nested_scopes",
+        "generators",
+        "with_statement",
+    ]
+
+    if hasattr(__future__, "barry_as_FLUFL"):
+        result.append("barry_as_FLUFL")
+    if hasattr(__future__, "annotations"):
+        result.append("annotations")
+
+    return result
+
+
+def getImportlibSubPackages():
+    result = []
+    if python_version >= 0x270:
+        import importlib
+        import pkgutil
+
+        for module_info in pkgutil.walk_packages(importlib.__path__):
+            result.append(module_info[1])
+
+    return result
