@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,15 +22,14 @@ XML tree to ASCII or output it.
 """
 
 from nuitka.__past__ import StringIO
-from nuitka.PythonVersions import python_version
 
 from . import Tracing
 
 
-def indent(elem, level = 0, more_sibs = False):
-    i = '\n'
+def _indent(elem, level=0, more_sibs=False):
+    i = "\n"
     if level:
-        i += (level-1) * "  "
+        i += (level - 1) * "  "
     num_kids = len(elem)
     if num_kids:
         if not elem.text or not elem.text.strip():
@@ -39,7 +38,7 @@ def indent(elem, level = 0, more_sibs = False):
                 elem.text += "  "
         count = 0
         for kid in elem:
-            indent(kid, level+1, count < num_kids - 1)
+            _indent(kid, level + 1, count < num_kids - 1)
             count += 1
         if not elem.tail or not elem.tail.strip():
             elem.tail = i
@@ -53,39 +52,59 @@ def indent(elem, level = 0, more_sibs = False):
 
     return elem
 
+
+def _dedent(elem, level=0):
+    if not elem.text or not elem.text.strip():
+        elem.text = ""
+
+    for child in elem:
+        _dedent(child, level + 1)
+
+    if not elem.tail or not elem.tail.strip():
+        elem.tail = ""
+
+    return elem
+
+
 try:
-    import lxml.etree # pylint: disable=I0021,import-error
-    xml_module = lxml.etree
+    import xml.etree.ElementTree
+
+    xml_module = xml.etree.ElementTree
+
+    Element = xml.etree.ElementTree.Element
+
+    def xml_tostring(tree, indent=True):
+        if indent:
+            _indent(tree)
+        elif not indent:
+            _dedent(tree)
+
+        return xml_module.tostring(tree)
 
-    Element = xml_module.Element
-    xml_tostring = lambda tree: lxml.etree.tostring(tree, pretty_print = True)
 except ImportError:
-    try:
-        import xml.etree.ElementTree
-        xml_module = xml.etree.ElementTree
-
-        Element = xml.etree.ElementTree.Element
-        xml_tostring = lambda tree: xml_module.tostring(indent(tree))
-    except ImportError:
-        xml_module = None
-        Element = None
-        xml_tostring = None
+    xml_module = None
+    Element = None
+    xml_tostring = None
 
 # TODO: Use the writer to create the XML we output. That should be more
 # scalable and/or faster.
 try:
-    import lxml.xmlfile # pylint: disable=I0021,import-error
+    import lxml.xmlfile  # pylint: disable=I0021,import-error
 
     xml_writer = lxml.xmlfile
 except ImportError:
     xml_writer = None
 
 
-def toString(tree):
-    result = xml_tostring(tree)
+def toBytes(tree, indent=True):
+    return xml_tostring(tree, indent=indent)
 
-    if python_version >= 300:
-        result = result.decode("utf-8")
+
+def toString(tree):
+    result = toBytes(tree)
+
+    if str is not bytes:
+        result = result.decode("utf8")
 
     return result
 
