@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,6 +21,7 @@
 
 from .CodeHelpers import generateStatementSequenceCode
 from .ConditionalCodes import generateConditionCode
+from .Emission import withSubCollector
 from .LabelCodes import getGotoCode, getLabelCode
 
 
@@ -36,11 +37,12 @@ def generateBranchCode(statement, emit, context):
     context.setTrueBranchTarget(true_target)
     context.setFalseBranchTarget(false_target)
 
-    generateConditionCode(
-        condition = statement.getCondition(),
-        emit      = emit,
-        context   = context
-    )
+    # Have own declaration scope for condition, to limit visibility from branches
+    # which can be huge.
+    with withSubCollector(emit, context) as condition_emit:
+        generateConditionCode(
+            condition=statement.subnode_condition, emit=condition_emit, context=context
+        )
 
     context.setTrueBranchTarget(old_true_target)
     context.setFalseBranchTarget(old_false_target)
@@ -48,19 +50,15 @@ def generateBranchCode(statement, emit, context):
     getLabelCode(true_target, emit)
 
     generateStatementSequenceCode(
-        statement_sequence = statement.getBranchYes(),
-        emit               = emit,
-        context            = context
+        statement_sequence=statement.subnode_yes_branch, emit=emit, context=context
     )
 
-    if statement.getBranchNo() is not None:
+    if statement.subnode_no_branch is not None:
         getGotoCode(end_target, emit)
         getLabelCode(false_target, emit)
 
         generateStatementSequenceCode(
-            statement_sequence = statement.getBranchNo(),
-            emit               = emit,
-            context            = context
+            statement_sequence=statement.subnode_no_branch, emit=emit, context=context
         )
 
         getLabelCode(end_target, emit)
