@@ -26,7 +26,13 @@ branches and make a code block out of it. But it doesn't contain any target
 language syntax.
 """
 
-from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
+from nuitka.nodes.AttributeNodesGenerated import (
+    attribute_classes,
+    attribute_typed_classes,
+)
+from nuitka.nodes.StrNodes import getStrOperationClasses
+from nuitka.plugins.Plugins import Plugins
+from nuitka.utils.CStrings import encodePythonStringToC
 
 from . import Contexts
 from .AsyncgenCodes import (
@@ -72,23 +78,24 @@ from .BuiltinCodes import (
     generateBuiltinXrange2Code,
     generateBuiltinXrange3Code,
 )
-from .CallCodes import generateCallCode, getCallsCode, getCallsDecls
+from .CallCodes import generateCallCode, getCallsCode
 from .ClassCodes import generateBuiltinSuperCode, generateSelectMetaclassCode
-from .CodeHelpers import setExpressionDispatchDict, setStatementDispatchDict
+from .CodeHelpers import addExpressionDispatchDict, setStatementDispatchDict
 from .ComparisonCodes import (
     generateBuiltinIsinstanceCode,
+    generateBuiltinIssubclassCode,
     generateComparisonExpressionCode,
+    generateMatchTypeCheckMappingCode,
+    generateMatchTypeCheckSequenceCode,
+    generateTypeCheckCode,
 )
 from .ConditionalCodes import (
     generateConditionalAndOrCode,
     generateConditionalCode,
 )
 from .ConstantCodes import (
-    generateConstantEllipsisReferenceCode,
-    generateConstantFalseReferenceCode,
-    generateConstantNoneReferenceCode,
+    generateConstantGenericAliasCode,
     generateConstantReferenceCode,
-    generateConstantTrueReferenceCode,
     getConstantsDefinitionCode,
 )
 from .CoroutineCodes import (
@@ -102,12 +109,31 @@ from .CoroutineCodes import (
 from .DictCodes import (
     generateBuiltinDictCode,
     generateDictionaryCreationCode,
-    generateDictOperationGetCode,
+    generateDictOperationClearCode,
+    generateDictOperationCopyCode,
+    generateDictOperationGet2Code,
+    generateDictOperationGet3Code,
     generateDictOperationInCode,
+    generateDictOperationItemCode,
+    generateDictOperationItemsCode,
+    generateDictOperationIteritemsCode,
+    generateDictOperationIterkeysCode,
+    generateDictOperationItervaluesCode,
+    generateDictOperationKeysCode,
+    generateDictOperationPop2Code,
+    generateDictOperationPop3Code,
     generateDictOperationRemoveCode,
     generateDictOperationSetCode,
     generateDictOperationSetCodeKeyValue,
+    generateDictOperationSetdefault2Code,
+    generateDictOperationSetdefault3Code,
+    generateDictOperationUpdate2Code,
+    generateDictOperationUpdate3Code,
     generateDictOperationUpdateCode,
+    generateDictOperationValuesCode,
+    generateDictOperationViewitemsCode,
+    generateDictOperationViewkeysCode,
+    generateDictOperationViewvaluesCode,
 )
 from .EvalCodes import (
     generateBuiltinCompileCode,
@@ -156,6 +182,9 @@ from .GlobalsLocalsCodes import (
 from .IdCodes import generateBuiltinHashCode, generateBuiltinIdCode
 from .ImportCodes import (
     generateBuiltinImportCode,
+    generateConstantSysVersionInfoCode,
+    generateImportlibImportCallCode,
+    generateImportModuleFixedCode,
     generateImportModuleHardCode,
     generateImportModuleNameHardCode,
     generateImportNameCode,
@@ -250,10 +279,12 @@ from .StringCodes import (
     generateBuiltinStrCode,
     generateBuiltinUnicodeCode,
     generateStringContenationCode,
+    generateStrOperationCode,
 )
 from .SubscriptCodes import (
     generateAssignmentSubscriptCode,
     generateDelSubscriptCode,
+    generateSubscriptCheckCode,
     generateSubscriptLookupCode,
 )
 from .TryCodes import generateTryCode
@@ -411,7 +442,7 @@ def generateFunctionBodyCode(function_body, context):
     return function_code, function_decl
 
 
-def generateModuleCode(module, data_filename):
+def _generateModuleCode(module, data_filename):
     # As this not only creates all modules, but also functions, it deals
     # also with its functions.
 
@@ -467,15 +498,22 @@ def generateModuleCode(module, data_filename):
         module=module,
         function_decl_codes=function_decl_codes,
         function_body_codes=function_body_codes,
-        module_const_blob_name=deriveModuleConstantsBlobName(data_filename),
+        module_const_blob_name=encodePythonStringToC(
+            Plugins.deriveModuleConstantsBlobName(data_filename)
+        ),
         context=context,
     )
 
 
-def generateHelpersCode():
-    calls_decl_code = getCallsDecls()
+def generateModuleCode(module, data_filename):
+    try:
+        return _generateModuleCode(module=module, data_filename=data_filename)
+    except KeyboardInterrupt:
+        raise KeyboardInterrupt("Interrupted while working on", module)
 
-    calls_body_code = getCallsCode()
+
+def generateHelpersCode():
+    calls_decl_code, calls_body_code = getCallsCode()
 
     constants_header_code, constants_body_code = getConstantsDefinitionCode()
 
@@ -487,7 +525,7 @@ def generateHelpersCode():
     )
 
 
-setExpressionDispatchDict(
+addExpressionDispatchDict(
     {
         "EXPRESSION_ATTRIBUTE_CHECK": generateAttributeCheckCode,
         "EXPRESSION_ATTRIBUTE_LOOKUP": generateAttributeLookupCode,
@@ -543,8 +581,13 @@ setExpressionDispatchDict(
         "EXPRESSION_BUILTIN_LOCALS_UPDATED": generateBuiltinLocalsCode,
         "EXPRESSION_BUILTIN_LOCALS_REF": generateBuiltinLocalsRefCode,
         "EXPRESSION_BUILTIN_GLOBALS": generateBuiltinGlobalsCode,
-        "EXPRESSION_BUILTIN_SUPER": generateBuiltinSuperCode,
+        "EXPRESSION_BUILTIN_SUPER0": generateBuiltinSuperCode,
+        "EXPRESSION_BUILTIN_SUPER2": generateBuiltinSuperCode,
         "EXPRESSION_BUILTIN_ISINSTANCE": generateBuiltinIsinstanceCode,
+        "EXPRESSION_BUILTIN_ISSUBCLASS": generateBuiltinIssubclassCode,
+        "EXPRESSION_TYPE_CHECK": generateTypeCheckCode,
+        "EXPRESSION_MATCH_TYPE_CHECK_SEQUENCE": generateMatchTypeCheckSequenceCode,
+        "EXPRESSION_MATCH_TYPE_CHECK_MAPPING": generateMatchTypeCheckMappingCode,
         "EXPRESSION_BUILTIN_DIR1": generateBuiltinDir1Code,
         "EXPRESSION_BUILTIN_VARS": generateBuiltinVarsCode,
         "EXPRESSION_BUILTIN_HASATTR": generateBuiltinHasattrCode,
@@ -572,9 +615,9 @@ setExpressionDispatchDict(
         "EXPRESSION_CALL_KEYWORDS_ONLY": generateCallCode,
         "EXPRESSION_CALL_NO_KEYWORDS": generateCallCode,
         "EXPRESSION_CALL": generateCallCode,
-        "EXPRESSION_CONSTANT_NONE_REF": generateConstantNoneReferenceCode,
-        "EXPRESSION_CONSTANT_TRUE_REF": generateConstantTrueReferenceCode,
-        "EXPRESSION_CONSTANT_FALSE_REF": generateConstantFalseReferenceCode,
+        "EXPRESSION_CONSTANT_NONE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TRUE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_FALSE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_STR_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_STR_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_UNICODE_REF": generateConstantReferenceCode,
@@ -585,7 +628,7 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_LONG_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_FLOAT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_COMPLEX_REF": generateConstantReferenceCode,
-        "EXPRESSION_CONSTANT_ELLIPSIS_REF": generateConstantEllipsisReferenceCode,
+        "EXPRESSION_CONSTANT_ELLIPSIS_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_DICT_EMPTY_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TUPLE_REF": generateConstantReferenceCode,
@@ -600,7 +643,11 @@ setExpressionDispatchDict(
         "EXPRESSION_CONSTANT_SLICE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_XRANGE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_TYPE_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_TYPE_SUBSCRIPTABLE_REF": generateConstantReferenceCode,
         "EXPRESSION_CONSTANT_BYTEARRAY_REF": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_GENERIC_ALIAS": generateConstantGenericAliasCode,
+        "EXPRESSION_CONSTANT_UNION_TYPE": generateConstantReferenceCode,
+        "EXPRESSION_CONSTANT_SYS_VERSION_INFO_REF": generateConstantSysVersionInfoCode,
         "EXPRESSION_CONDITIONAL": generateConditionalCode,
         "EXPRESSION_CONDITIONAL_OR": generateConditionalAndOrCode,
         "EXPRESSION_CONDITIONAL_AND": generateConditionalAndOrCode,
@@ -617,14 +664,37 @@ setExpressionDispatchDict(
         "EXPRESSION_COMPARISON_GTE": generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_EQ": generateComparisonExpressionCode,
         "EXPRESSION_COMPARISON_NEQ": generateComparisonExpressionCode,
-        "EXPRESSION_DICT_OPERATION_GET": generateDictOperationGetCode,
+        "EXPRESSION_DICT_OPERATION_ITEM": generateDictOperationItemCode,
+        "EXPRESSION_DICT_OPERATION_GET2": generateDictOperationGet2Code,
+        "EXPRESSION_DICT_OPERATION_GET3": generateDictOperationGet3Code,
+        "EXPRESSION_DICT_OPERATION_HASKEY": generateDictOperationInCode,
         "EXPRESSION_DICT_OPERATION_IN": generateDictOperationInCode,
         "EXPRESSION_DICT_OPERATION_NOT_IN": generateDictOperationInCode,
+        "EXPRESSION_DICT_OPERATION_COPY": generateDictOperationCopyCode,
+        "EXPRESSION_DICT_OPERATION_CLEAR": generateDictOperationClearCode,
+        "EXPRESSION_DICT_OPERATION_ITEMS": generateDictOperationItemsCode,
+        "EXPRESSION_DICT_OPERATION_ITERITEMS": generateDictOperationIteritemsCode,
+        "EXPRESSION_DICT_OPERATION_VIEWITEMS": generateDictOperationViewitemsCode,
+        "EXPRESSION_DICT_OPERATION_KEYS": generateDictOperationKeysCode,
+        "EXPRESSION_DICT_OPERATION_ITERKEYS": generateDictOperationIterkeysCode,
+        "EXPRESSION_DICT_OPERATION_VIEWKEYS": generateDictOperationViewkeysCode,
+        "EXPRESSION_DICT_OPERATION_VALUES": generateDictOperationValuesCode,
+        "EXPRESSION_DICT_OPERATION_ITERVALUES": generateDictOperationItervaluesCode,
+        "EXPRESSION_DICT_OPERATION_VIEWVALUES": generateDictOperationViewvaluesCode,
+        "EXPRESSION_DICT_OPERATION_SETDEFAULT2": generateDictOperationSetdefault2Code,
+        "EXPRESSION_DICT_OPERATION_SETDEFAULT3": generateDictOperationSetdefault3Code,
+        "EXPRESSION_DICT_OPERATION_POP2": generateDictOperationPop2Code,
+        "EXPRESSION_DICT_OPERATION_POP3": generateDictOperationPop3Code,
+        "EXPRESSION_DICT_OPERATION_UPDATE2": generateDictOperationUpdate2Code,
+        "EXPRESSION_DICT_OPERATION_UPDATE3": generateDictOperationUpdate3Code,
         "EXPRESSION_FUNCTION_CREATION": generateFunctionCreationCode,
         "EXPRESSION_FUNCTION_CALL": generateFunctionCallCode,
         "EXPRESSION_FUNCTION_ERROR_STR": generateFunctionErrorStrCode,
+        "EXPRESSION_IMPORT_MODULE_FIXED": generateImportModuleFixedCode,
         "EXPRESSION_IMPORT_MODULE_HARD": generateImportModuleHardCode,
         "EXPRESSION_IMPORT_MODULE_NAME_HARD": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_IMPORT_MODULE_REF": generateImportModuleNameHardCode,
+        "EXPRESSION_IMPORTLIB_IMPORT_MODULE_CALL": generateImportlibImportCallCode,
         "EXPRESSION_IMPORT_NAME": generateImportNameCode,
         "EXPRESSION_LIST_OPERATION_EXTEND": generateListOperationExtendCode,
         "EXPRESSION_LIST_OPERATION_EXTEND_FOR_UNPACK": generateListOperationExtendCode,
@@ -671,14 +741,18 @@ setExpressionDispatchDict(
         "EXPRESSION_OPERATION_INPLACE_BIT_AND": generateOperationBinaryCode,
         "EXPRESSION_OPERATION_INPLACE_BIT_XOR": generateOperationBinaryCode,
         "EXPRESSION_OPERATION_INPLACE_MAT_MULT": generateOperationBinaryCode,
-        "EXPRESSION_OPERATION_UNARY": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_REPR": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_SUB": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_ADD": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_INVERT": generateOperationUnaryCode,
+        "EXPRESSION_OPERATION_UNARY_ABS": generateBuiltinAbsCode,
         "EXPRESSION_OPERATION_NOT": generateOperationNotCode,
-        "EXPRESSION_OPERATION_ABS": generateBuiltinAbsCode,
         "EXPRESSION_OUTLINE_BODY": generateFunctionOutlineCode,
         "EXPRESSION_OUTLINE_FUNCTION": generateFunctionOutlineCode,
         # TODO: Rename to make more clear it is an outline
         "EXPRESSION_CLASS_BODY": generateFunctionOutlineCode,
         "EXPRESSION_SUBSCRIPT_LOOKUP": generateSubscriptLookupCode,
+        "EXPRESSION_SUBSCRIPT_CHECK": generateSubscriptCheckCode,
         "EXPRESSION_SLICE_LOOKUP": generateSliceLookupCode,
         "EXPRESSION_SET_OPERATION_UPDATE": generateSetOperationUpdateCode,
         "EXPRESSION_SIDE_EFFECTS": generateSideEffectsCode,
@@ -706,6 +780,21 @@ setExpressionDispatchDict(
     }
 )
 
+# Add code generation for the EXPRESSION_ATTRIBUTE_LOOKUP_FIXED_* variety
+addExpressionDispatchDict(
+    dict((cls.kind, generateAttributeLookupCode) for cls in attribute_classes.values())
+)
+
+# Add code generation for the EXPRESSION_ATTRIBUTE_LOOKUP_DICT|LIST|STR_* variety
+addExpressionDispatchDict(
+    dict((cls.kind, generateAttributeLookupCode) for cls in attribute_typed_classes)
+)
+
+addExpressionDispatchDict(
+    dict((cls.kind, generateStrOperationCode) for cls in getStrOperationClasses())
+)
+
+
 setStatementDispatchDict(
     {
         "STATEMENT_ASSIGNMENT_VARIABLE": generateAssignmentVariableCode,
