@@ -24,10 +24,11 @@ typical support functions to building parts.
 
 from contextlib import contextmanager
 
+from nuitka import Options
 from nuitka.nodes.NodeMetaClasses import NuitkaNodeDesignError
-from nuitka.Options import isExperimental, shallTraceExecution
+from nuitka.Options import shallTraceExecution
 from nuitka.PythonVersions import python_version
-from nuitka.Tracing import my_print, printError
+from nuitka.Tracing import printError
 
 from .Emission import withSubCollector
 from .LabelCodes import getGotoCode, getLabelCode, getStatementTrace
@@ -36,15 +37,9 @@ from .Reports import onMissingHelper
 expression_dispatch_dict = {}
 
 
-def setExpressionDispatchDict(dispatch_dict):
-    # Using global here, as this is really a singleton, in the form of a module,
-    # and this is to break the cyclic dependency it has, pylint: disable=global-statement
-
-    # Please call us only once.
-    global expression_dispatch_dict
-
-    assert not expression_dispatch_dict
-    expression_dispatch_dict = dispatch_dict
+def addExpressionDispatchDict(dispatch_dict):
+    # We use this to break the cyclic dependency.
+    expression_dispatch_dict.update(dispatch_dict)
 
 
 def generateExpressionCode(to_name, expression, emit, context, allow_none=False):
@@ -80,10 +75,6 @@ def _generateExpressionCode(to_name, expression, emit, context, allow_none=False
     assert not hasattr(expression, "code_generated"), expression
     expression.code_generated = True
 
-    old_source_ref = context.setCurrentSourceCodeReference(
-        expression.getSourceReference()
-    )
-
     if not expression.isExpression():
         printError("No expression %r" % expression)
 
@@ -99,9 +90,10 @@ def _generateExpressionCode(to_name, expression, emit, context, allow_none=False
             expression.kind,
         )
 
-    code_generator(to_name=to_name, expression=expression, emit=emit, context=context)
-
-    context.setCurrentSourceCodeReference(old_source_ref)
+    with context.withCurrentSourceCodeReference(expression.getSourceReference()):
+        code_generator(
+            to_name=to_name, expression=expression, emit=emit, context=context
+        )
 
 
 def generateExpressionsCode(names, expressions, emit, context):
@@ -127,9 +119,32 @@ def generateChildExpressionsCode(expression, emit, context):
     value_names = []
 
     for child_name, child_value in expression.getVisitableNodesNamed():
-        if child_value is not None:
+        if type(child_value) is tuple:
+            child_names = []
+
+            for child_val in child_value:
+                if child_val.isExpressionKeyValuePair():
+                    child_names.append(
+                        tuple(
+                            generateChildExpressionsCode(
+                                expression=child_val, emit=emit, context=context
+                            )
+                        )
+                    )
+                else:
+                    generateExpressionCode(
+                        to_name=value_name,
+                        expression=child_val,
+                        emit=emit,
+                        context=context,
+                    )
+
+                    child_names.append(value_name)
+
+            value_names.append(tuple(child_names))
+        elif child_value is not None:
             # Allocate anyway, so names are aligned.
-            value_name = context.allocateTempName(child_name + "_name")
+            value_name = context.allocateTempName(child_name + "_value")
 
             generateExpressionCode(
                 to_name=value_name, expression=child_value, emit=emit, context=context
@@ -137,6 +152,7 @@ def generateChildExpressionsCode(expression, emit, context):
 
             value_names.append(value_name)
         else:
+            # Allocate anyway, so names are aligned.
             context.skipTempName(child_name + "_value")
 
             value_names.append(None)
@@ -151,7 +167,9 @@ def generateChildExpressionCode(expression, emit, context, child_name=None):
         child_name = expression.getChildName()
 
     # Allocate anyway, so names are aligned.
-    value_name = context.allocateTempName(child_name + "_name")
+    value_name = context.allocateTempName(
+        child_name + "_value",
+    )
 
     generateExpressionCode(
         to_name=value_name, expression=expression, emit=emit, context=context
@@ -249,7 +267,7 @@ def decideConversionCheckNeeded(to_name, expression):
 def withObjectCodeTemporaryAssignment2(
     to_name, value_name, needs_conversion_check, emit, context
 ):
-    """ Converting to the target type, provide temporary object value name only if necessary. """
+    """Converting to the target type, provide temporary object value name only if necessary."""
 
     if to_name.c_type == "PyObject *":
         value_name = to_name
@@ -274,7 +292,7 @@ def withObjectCodeTemporaryAssignment2(
 
 @contextmanager
 def withObjectCodeTemporaryAssignment(to_name, value_name, expression, emit, context):
-    """ Converting to the target type, provide temporary object value name only if necessary. """
+    """Converting to the target type, provide temporary object value name only if necessary."""
 
     if to_name.c_type == "PyObject *":
         value_name = to_name
@@ -297,6 +315,19 @@ def withObjectCodeTemporaryAssignment(to_name, value_name, expression, emit, con
         getReleaseCode(value_name, emit, context)
 
 
+def assignConstantNoneResult(to_name, emit, context):
+    # TODO: This is also in SetCode, and should be common for statement only
+    # operations that return None in Python, but only in case of non-error
+    to_name.getCType().emitAssignmentCodeFromConstant(
+        to_name=to_name, constant=None, may_escape=False, emit=emit, context=context
+    )
+
+    # This assignment will not necessarily use it, and since it is borrowed,
+    # debug mode would otherwise complain.
+    if to_name.c_type == "nuitka_void":
+        to_name.maybe_unused = True
+
+
 class HelperCallHandle(object):
     def __init__(
         self,
@@ -449,10 +480,7 @@ def pickCodeHelper(
             helper_right=right_shape,
         )
 
-    if isExperimental("nuitka_ilong"):
-        my_print(ideal_helper)
-
-    if source_ref is not None and (not nonhelpers or ideal_helper not in nonhelpers):
+    if Options.is_report_missing and (not nonhelpers or ideal_helper not in nonhelpers):
         onMissingHelper(ideal_helper, source_ref)
 
     fallback_helper = "%s_%s_%s_%s%s" % (prefix, "OBJECT", "OBJECT", "OBJECT", suffix)
