@@ -23,37 +23,48 @@ Rich comparisons, "in", and "not in", also "is", and "is not", and the
 
 from nuitka.containers.oset import OrderedSet
 from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_bool
+from nuitka.Options import isExperimental
 
 from . import OperatorCodes
 from .CodeHelpers import generateExpressionCode, pickCodeHelper
-from .ErrorCodes import getErrorExitBoolCode, getReleaseCodes
+from .ErrorCodes import getErrorExitBoolCode, getReleaseCode, getReleaseCodes
 
 specialized_cmp_helpers_set = OrderedSet(
     (
         "RICH_COMPARE_xx_OBJECT_OBJECT_OBJECT",
         "RICH_COMPARE_xx_CBOOL_OBJECT_OBJECT",
         "RICH_COMPARE_xx_NBOOL_OBJECT_OBJECT",
-        #        "RICH_COMPARE_xx_OBJECT_INT",
-        #        "RICH_COMPARE_xx_OBJECT_LONG",
-        #        "RICH_COMPARE_xx_OBJECT_STR",
+        "RICH_COMPARE_xx_OBJECT_OBJECT_STR",
+        "RICH_COMPARE_xx_CBOOL_OBJECT_STR",
+        "RICH_COMPARE_xx_NBOOL_OBJECT_STR",
+        "RICH_COMPARE_xx_OBJECT_STR_OBJECT",
+        "RICH_COMPARE_xx_CBOOL_STR_OBJECT",
+        "RICH_COMPARE_xx_NBOOL_STR_OBJECT",
+        "RICH_COMPARE_xx_OBJECT_STR_STR",
+        "RICH_COMPARE_xx_CBOOL_STR_STR",
+        "RICH_COMPARE_xx_NBOOL_STR_STR",
+        "RICH_COMPARE_xx_OBJECT_OBJECT_UNICODE",
+        "RICH_COMPARE_xx_CBOOL_OBJECT_UNICODE",
+        "RICH_COMPARE_xx_NBOOL_OBJECT_UNICODE",
+        "RICH_COMPARE_xx_OBJECT_UNICODE_OBJECT",
+        "RICH_COMPARE_xx_CBOOL_UNICODE_OBJECT",
+        "RICH_COMPARE_xx_NBOOL_UNICODE_OBJECT",
+        "RICH_COMPARE_xx_OBJECT_UNICODE_UNICODE",
+        "RICH_COMPARE_xx_CBOOL_UNICODE_UNICODE",
+        "RICH_COMPARE_xx_NBOOL_UNICODE_UNICODE",
+        "RICH_COMPARE_xx_CBOOL_OBJECT_BYTES",
+        "RICH_COMPARE_xx_NBOOL_OBJECT_BYTES",
+        "RICH_COMPARE_xx_OBJECT_BYTES_OBJECT",
+        "RICH_COMPARE_xx_CBOOL_BYTES_OBJECT",
+        "RICH_COMPARE_xx_NBOOL_BYTES_OBJECT",
+        "RICH_COMPARE_xx_OBJECT_BYTES_BYTES",
+        "RICH_COMPARE_xx_CBOOL_BYTES_BYTES",
+        "RICH_COMPARE_xx_NBOOL_BYTES_BYTES",
         #        "RICH_COMPARE_xx_OBJECT_UNICODE",
-        #        "RICH_COMPARE_xx_OBJECT_TUPLE",
-        #        "RICH_COMPARE_xx_OBJECT_LIST",
-        #        "RICH_COMPARE_xx_OBJECT_BYTES",
-        #        "RICH_COMPARE_xx_INT_OBJECT",
         #        "RICH_COMPARE_xx_LONG_OBJECT",
-        #        "RICH_COMPARE_xx_STR_OBJECT",
         #        "RICH_COMPARE_xx_UNICODE_OBJECT",
-        #        "RICH_COMPARE_xx_TUPLE_OBJECT",
-        #        "RICH_COMPARE_xx_LIST_OBJECT",
-        #        "RICH_COMPARE_xx_BYTES_OBJECT",
-        #        "RICH_COMPARE_xx_INT_INT",
         #        "RICH_COMPARE_xx_LONG_LONG",
-        #        "RICH_COMPARE_xx_STR_STR",
         #        "RICH_COMPARE_xx_UNICODE_UNICODE",
-        #        "RICH_COMPARE_xx_TUPLE_TUPLE",
-        #        "RICH_COMPARE_xx_LIST_LIST",
-        #        "RICH_COMPARE_xx_BYTES_BYTES",
         "RICH_COMPARE_xx_OBJECT_INT_INT",
         "RICH_COMPARE_xx_CBOOL_INT_INT",
         "RICH_COMPARE_xx_NBOOL_INT_INT",
@@ -63,6 +74,15 @@ specialized_cmp_helpers_set = OrderedSet(
         "RICH_COMPARE_xx_OBJECT_INT_OBJECT",
         "RICH_COMPARE_xx_CBOOL_INT_OBJECT",
         "RICH_COMPARE_xx_NBOOL_INT_OBJECT",
+        "RICH_COMPARE_xx_OBJECT_LONG_LONG",
+        "RICH_COMPARE_xx_CBOOL_LONG_LONG",
+        "RICH_COMPARE_xx_NBOOL_LONG_LONG",
+        "RICH_COMPARE_xx_OBJECT_OBJECT_LONG",
+        "RICH_COMPARE_xx_CBOOL_OBJECT_LONG",
+        "RICH_COMPARE_xx_NBOOL_OBJECT_LONG",
+        "RICH_COMPARE_xx_OBJECT_LONG_OBJECT",
+        "RICH_COMPARE_xx_CBOOL_LONG_OBJECT",
+        "RICH_COMPARE_xx_NBOOL_LONG_OBJECT",
         "RICH_COMPARE_xx_OBJECT_FLOAT_FLOAT",
         "RICH_COMPARE_xx_CBOOL_FLOAT_FLOAT",
         "RICH_COMPARE_xx_NBOOL_FLOAT_FLOAT",
@@ -81,24 +101,21 @@ specialized_cmp_helpers_set = OrderedSet(
         "RICH_COMPARE_xx_OBJECT_TUPLE_OBJECT",
         "RICH_COMPARE_xx_CBOOL_TUPLE_OBJECT",
         "RICH_COMPARE_xx_NBOOL_TUPLE_OBJECT",
+        "RICH_COMPARE_xx_OBJECT_LIST_LIST",
+        "RICH_COMPARE_xx_CBOOL_LIST_LIST",
+        "RICH_COMPARE_xx_NBOOL_LIST_LIST",
+        "RICH_COMPARE_xx_OBJECT_OBJECT_LIST",
+        "RICH_COMPARE_xx_CBOOL_OBJECT_LIST",
+        "RICH_COMPARE_xx_NBOOL_OBJECT_LIST",
+        "RICH_COMPARE_xx_OBJECT_LIST_OBJECT",
+        "RICH_COMPARE_xx_CBOOL_LIST_OBJECT",
+        "RICH_COMPARE_xx_NBOOL_LIST_OBJECT",
         #        "RICH_COMPARE_xx_CBOOL_OBJECT_LONG",
-        #        "RICH_COMPARE_xx_CBOOL_OBJECT_STR",
         #        "RICH_COMPARE_xx_CBOOL_OBJECT_UNICODE",
-        #        "RICH_COMPARE_xx_CBOOL_OBJECT_LIST",
-        #        "RICH_COMPARE_xx_CBOOL_OBJECT_BYTES",
         #        "RICH_COMPARE_xx_CBOOL_LONG_OBJECT",
-        #        "RICH_COMPARE_xx_CBOOL_STR_OBJECT",
-        #        "RICH_COMPARE_xx_CBOOL_OBJECT_FLOAT",
         #        "RICH_COMPARE_xx_CBOOL_UNICODE_OBJECT",
-        #        "RICH_COMPARE_xx_CBOOL_TUPLE_OBJECT",
-        #        "RICH_COMPARE_xx_CBOOL_LIST_OBJECT",
-        #        "RICH_COMPARE_xx_CBOOL_BYTES_OBJECT",
         #        "RICH_COMPARE_xx_CBOOL_LONG_LONG",
-        #        "RICH_COMPARE_xx_CBOOL_STR_STR",
         #        "RICH_COMPARE_xx_CBOOL_UNICODE_UNICODE",
-        #        "RICH_COMPARE_xx_CBOOL_TUPLE_TUPLE",
-        #        "RICH_COMPARE_xx_CBOOL_LIST_LIST",
-        #        "RICH_COMPARE_xx_CBOOL_BYTES_BYTES",
     )
 )
 
@@ -180,8 +197,7 @@ def generateComparisonExpressionCode(to_name, expression, emit, context):
             right_shape=expression.subnode_right.getTypeShape(),
             helpers=specialized_cmp_helpers_set,
             nonhelpers=(),
-            # TODO: Only temporary, we need to be more complete with these.
-            source_ref=None,  # expression.source_ref,
+            source_ref=expression.source_ref,
         )
 
         # Lets patch this up here, instead of having one set per comparison operation.
@@ -239,9 +255,14 @@ def generateBuiltinIsinstanceCode(to_name, expression, emit, context):
         context=context,
     )
 
+    context.setCurrentSourceCodeReference(expression.getCompatibleSourceReference())
+
     res_name = context.getIntResName()
 
-    emit("%s = Nuitka_IsInstance(%s, %s);" % (res_name, inst_name, cls_name))
+    if isExperimental("function-base"):
+        emit("%s = PyObject_IsInstance(%s, %s);" % (res_name, inst_name, cls_name))
+    else:
+        emit("%s = Nuitka_IsInstance(%s, %s);" % (res_name, inst_name, cls_name))
 
     getErrorExitBoolCode(
         condition="%s == -1" % res_name,
@@ -253,3 +274,114 @@ def generateBuiltinIsinstanceCode(to_name, expression, emit, context):
     to_name.getCType().emitAssignmentCodeFromBoolCondition(
         to_name=to_name, condition="%s != 0" % res_name, emit=emit
     )
+
+
+def generateBuiltinIssubclassCode(to_name, expression, emit, context):
+    cls_name = context.allocateTempName("issubclass_cls")
+    classes_name = context.allocateTempName("issubclass_classes")
+
+    generateExpressionCode(
+        to_name=cls_name,
+        expression=expression.subnode_cls,
+        emit=emit,
+        context=context,
+    )
+    generateExpressionCode(
+        to_name=classes_name,
+        expression=expression.subnode_classes,
+        emit=emit,
+        context=context,
+    )
+
+    context.setCurrentSourceCodeReference(expression.getCompatibleSourceReference())
+
+    res_name = context.getIntResName()
+
+    emit("%s = PyObject_IsSubclass(%s, %s);" % (res_name, cls_name, classes_name))
+
+    getErrorExitBoolCode(
+        condition="%s == -1" % res_name,
+        release_names=(cls_name, classes_name),
+        emit=emit,
+        context=context,
+    )
+
+    to_name.getCType().emitAssignmentCodeFromBoolCondition(
+        to_name=to_name, condition="%s != 0" % res_name, emit=emit
+    )
+
+
+def generateTypeCheckCode(to_name, expression, emit, context):
+    cls_name = context.allocateTempName("issubclass_cls")
+
+    generateExpressionCode(
+        to_name=cls_name,
+        expression=expression.subnode_cls,
+        emit=emit,
+        context=context,
+    )
+    context.setCurrentSourceCodeReference(expression.getCompatibleSourceReference())
+
+    res_name = context.getIntResName()
+
+    emit("%s = PyType_Check(%s);" % (res_name, cls_name))
+
+    getReleaseCode(
+        release_name=cls_name,
+        emit=emit,
+        context=context,
+    )
+
+    to_name.getCType().emitAssignmentCodeFromBoolCondition(
+        to_name=to_name, condition="%s != 0" % res_name, emit=emit
+    )
+
+
+def generateMatchTypeCheckMappingCode(to_name, expression, emit, context):
+    cls_name = context.allocateTempName("mapping_check_cls")
+
+    generateExpressionCode(
+        to_name=cls_name,
+        expression=expression.subnode_value,
+        emit=emit,
+        context=context,
+    )
+
+    res_name = context.getIntResName()
+
+    emit("%s = Py_TYPE(%s)->tp_flags & Py_TPFLAGS_MAPPING;" % (res_name, cls_name))
+
+    getReleaseCode(
+        release_name=cls_name,
+        emit=emit,
+        context=context,
+    )
+
+    to_name.getCType().emitAssignmentCodeFromBoolCondition(
+        to_name=to_name, condition="%s" % res_name, emit=emit
+    )
+
+
+def generateMatchTypeCheckSequenceCode(to_name, expression, emit, context):
+    cls_name = context.allocateTempName("sequence_check_cls")
+
+    generateExpressionCode(
+        to_name=cls_name,
+        expression=expression.subnode_value,
+        emit=emit,
+        context=context,
+    )
+
+    res_name = context.getIntResName()
+
+    emit("%s = Py_TYPE(%s)->tp_flags & Py_TPFLAGS_SEQUENCE;" % (res_name, cls_name))
+
+    getReleaseCode(
+        release_name=cls_name,
+        emit=emit,
+        context=context,
+    )
+
+    to_name.getCType().emitAssignmentCodeFromBoolCondition(
+        to_name=to_name, condition="%s" % res_name, emit=emit
+    )
