@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,19 +22,28 @@ Branches, conditions, truth checks.
 
 from .CodeHelpers import decideConversionCheckNeeded, generateExpressionCode
 from .Emission import SourceCodeCollector
-from .ErrorCodes import getReleaseCode, getTakeReferenceCode
+from .ErrorCodes import (
+    getErrorExitBoolCode,
+    getReleaseCode,
+    getTakeReferenceCode,
+)
 from .LabelCodes import getBranchingCode, getGotoCode, getLabelCode
 
 
 def generateConditionCode(condition, emit, context):
-    compare_name = context.allocateTempName("condition_result", "nuitka_bool")
+    if condition.mayRaiseExceptionBool(BaseException):
+        compare_name = context.allocateTempName("condition_result", "nuitka_bool")
+    else:
+        compare_name = context.allocateTempName("condition_result", "bool")
 
     generateExpressionCode(
         to_name=compare_name, expression=condition, emit=emit, context=context
     )
 
     getBranchingCode(
-        condition="%s == NUITKA_BOOL_TRUE" % compare_name, emit=emit, context=context
+        condition=compare_name.getCType().getTruthCheckCode(compare_name),
+        emit=emit,
+        context=context,
     )
 
     getReleaseCode(compare_name, emit, context)
@@ -61,7 +70,7 @@ def generateConditionalAndOrCode(to_name, expression, emit, context):
     left_name = context.allocateTempName(prefix + "left_value", to_name.c_type)
     right_name = context.allocateTempName(prefix + "right_value", to_name.c_type)
 
-    left_value = expression.getLeft()
+    left_value = expression.subnode_left
 
     generateExpressionCode(
         to_name=left_name, expression=left_value, emit=emit, context=context
@@ -81,11 +90,19 @@ def generateConditionalAndOrCode(to_name, expression, emit, context):
     left_name.getCType().emitTruthCheckCode(
         to_name=truth_name,
         value_name=left_name,
-        needs_check=left_value.mayRaiseExceptionBool(BaseException),
         emit=emit,
-        context=context,
     )
 
+    needs_check = left_value.mayRaiseExceptionBool(BaseException)
+
+    if needs_check:
+        getErrorExitBoolCode(
+            condition="%s == -1" % truth_name,
+            needs_check=True,
+            emit=emit,
+            context=context,
+        )
+
     getBranchingCode(condition="%s == 1" % truth_name, emit=emit, context=context)
 
     getLabelCode(false_target, emit)
@@ -94,7 +111,7 @@ def generateConditionalAndOrCode(to_name, expression, emit, context):
     # is not needed, but we remember if it should be added above.
     getReleaseCode(release_name=left_name, emit=emit, context=context)
 
-    right_value = expression.getRight()
+    right_value = expression.subnode_right
 
     # Evaluate the "right" value then.
     generateExpressionCode(
@@ -154,13 +171,13 @@ def generateConditionalCode(to_name, expression, emit, context):
     context.setFalseBranchTarget(false_target)
 
     generateConditionCode(
-        condition=expression.getCondition(), emit=emit, context=context
+        condition=expression.subnode_condition, emit=emit, context=context
     )
 
     getLabelCode(true_target, emit)
     generateExpressionCode(
         to_name=to_name,
-        expression=expression.getExpressionYes(),
+        expression=expression.subnode_expression_yes,
         emit=emit,
         context=context,
     )
@@ -175,7 +192,7 @@ def generateConditionalCode(to_name, expression, emit, context):
 
     generateExpressionCode(
         to_name=to_name,
-        expression=expression.getExpressionNo(),
+        expression=expression.subnode_expression_no,
         emit=emit,
         context=context,
     )
