@@ -26,7 +26,6 @@ by a try statement is accessible this way.
 from nuitka.PythonVersions import python_version
 
 from .CodeHelpers import generateExpressionCode
-from .ConstantCodes import getConstantAccess
 from .ExceptionCodes import getExceptionUnpublishedReleaseCode
 from .LabelCodes import getGotoCode
 
@@ -39,6 +38,7 @@ def generateReturnCode(statement, emit, context):
     return_value_name = context.getReturnValueName()
 
     if context.getReturnReleaseMode():
+        emit("CHECK_OBJECT(%s);" % return_value_name)
         emit("Py_DECREF(%s);" % return_value_name)
 
     generateExpressionCode(
@@ -70,11 +70,13 @@ def generateReturnConstantCode(statement, emit, context):
     return_value_name = context.getReturnValueName()
 
     if context.getReturnReleaseMode():
+        emit("CHECK_OBJECT(%s);" % return_value_name)
         emit("Py_DECREF(%s);" % return_value_name)
 
-    getConstantAccess(
+    return_value_name.getCType().emitAssignmentCodeFromConstant(
         to_name=return_value_name,
         constant=statement.getConstant(),
+        may_escape=True,
         emit=emit,
         context=context,
     )
@@ -96,6 +98,7 @@ def generateGeneratorReturnValueCode(statement, emit, context):
         expression = statement.subnode_expression
 
         if context.getReturnReleaseMode():
+            emit("CHECK_OBJECT(%s);" % return_value_name)
             emit("Py_DECREF(%s);" % return_value_name)
 
         generateExpressionCode(
@@ -125,10 +128,15 @@ def generateGeneratorReturnNoneCode(statement, emit, context):
         return_value_name = context.getGeneratorReturnValueName()
 
         if context.getReturnReleaseMode():
+            emit("CHECK_OBJECT(%s);" % return_value_name)
             emit("Py_DECREF(%s);" % return_value_name)
 
-        getConstantAccess(
-            to_name=return_value_name, constant=None, emit=emit, context=context
+        return_value_name.getCType().emitAssignmentCodeFromConstant(
+            to_name=return_value_name,
+            constant=None,
+            may_escape=False,
+            emit=emit,
+            context=context,
         )
 
         if context.needsCleanup(return_value_name):
