@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -23,6 +23,7 @@ from nuitka.nodes.shapes.BuiltinTypeShapes import (
     tshape_bool,
     tshape_int_or_long,
 )
+from nuitka.PythonVersions import python_version
 
 from .c_types.CTypeNuitkaBools import CTypeNuitkaBoolEnum
 from .c_types.CTypePyObjectPtrs import (
@@ -30,9 +31,14 @@ from .c_types.CTypePyObjectPtrs import (
     CTypePyObjectPtr,
     CTypePyObjectPtrPtr,
 )
-from .CodeHelpers import decideConversionCheckNeeded, generateExpressionCode
+from .CodeHelpers import (
+    decideConversionCheckNeeded,
+    generateExpressionCode,
+    withObjectCodeTemporaryAssignment2,
+)
 from .ErrorCodes import (
     getAssertionCode,
+    getErrorExitCode,
     getLocalVariableReferenceErrorCode,
     getNameReferenceErrorCode,
 )
@@ -85,66 +91,86 @@ def generateAssignmentVariableCode(statement, emit, context):
 
 
 def generateDelVariableCode(statement, emit, context):
-    old_source_ref = context.setCurrentSourceCodeReference(
-        statement.getSourceReference()
-    )
-
-    _getVariableDelCode(
-        variable=statement.getVariable(),
-        variable_trace=statement.variable_trace,
-        previous_trace=statement.previous_trace,
-        tolerant=statement.isTolerant(),
-        needs_check=statement.isTolerant()
-        or statement.mayRaiseException(BaseException),
-        emit=emit,
-        context=context,
-    )
-
-    context.setCurrentSourceCodeReference(old_source_ref)
+    with context.withCurrentSourceCodeReference(statement.getSourceReference()):
+        _getVariableDelCode(
+            variable=statement.getVariable(),
+            variable_trace=statement.variable_trace,
+            previous_trace=statement.previous_trace,
+            tolerant=statement.isTolerant(),
+            needs_check=statement.isTolerant()
+            or statement.mayRaiseException(BaseException),
+            emit=emit,
+            context=context,
+        )
 
 
 def getVariableReferenceCode(
     to_name, variable, variable_trace, needs_check, conversion_check, emit, context
 ):
     if variable.isModuleVariable():
-        variable_declaration = VariableDeclaration(
-            "module_var", variable.getName(), None, None
-        )
+        owner = context.getOwner()
+
+        with withObjectCodeTemporaryAssignment2(
+            to_name, "mvar_value", conversion_check, emit, context
+        ) as value_name:
+            # TODO: Rather have this passed from a distinct node type, so inlining
+            # doesn't change things.
+
+            emit(
+                """\
+%(value_name)s = GET_STRING_DICT_VALUE(moduledict_%(module_identifier)s, (Nuitka_StringObject *)%(var_name)s);
+
+if (unlikely(%(value_name)s == NULL)) {
+    %(value_name)s = %(helper_code)s(%(var_name)s);
+}
+"""
+                % {
+                    "helper_code": "GET_MODULE_VARIABLE_VALUE_FALLBACK_IN_FUNCTION"
+                    if python_version < 0x340
+                    and not owner.isCompiledPythonModule()
+                    and not owner.isExpressionClassBody()
+                    else "GET_MODULE_VARIABLE_VALUE_FALLBACK",
+                    "module_identifier": context.getModuleCodeName(),
+                    "value_name": value_name,
+                    "var_name": context.getConstantCode(constant=variable.getName()),
+                }
+            )
+
+            getErrorExitCode(
+                check_name=value_name,
+                emit=emit,
+                context=context,
+                needs_check=needs_check,
+            )
     else:
         variable_declaration = getLocalVariableDeclaration(
             context, variable, variable_trace
         )
 
-    value_name = variable_declaration.getCType().emitValueAccessCode(
-        value_name=variable_declaration, emit=emit, context=context
-    )
+        value_name = variable_declaration.getCType().emitValueAccessCode(
+            value_name=variable_declaration, emit=emit, context=context
+        )
 
-    if needs_check:
-        condition = value_name.getCType().getLocalVariableInitTestCode(value_name, True)
-
-        if variable.isModuleVariable():
-            getNameReferenceErrorCode(
-                variable_name=variable.getName(),
-                condition=condition,
-                emit=emit,
-                context=context,
+        if needs_check:
+            condition = value_name.getCType().getInitTestConditionCode(
+                value_name, inverted=True
             )
-        else:
+
             getLocalVariableReferenceErrorCode(
                 variable=variable, condition=condition, emit=emit, context=context
             )
-    else:
-        value_name.getCType().emitValueAssertionCode(
-            value_name=value_name, emit=emit, context=context
-        )
+        else:
+            value_name.getCType().emitValueAssertionCode(
+                value_name=value_name, emit=emit
+            )
 
-    to_name.getCType().emitAssignConversionCode(
-        to_name=to_name,
-        value_name=value_name,
-        needs_check=conversion_check,
-        emit=emit,
-        context=context,
-    )
+        to_name.getCType().emitAssignConversionCode(
+            to_name=to_name,
+            value_name=value_name,
+            needs_check=conversion_check,
+            emit=emit,
+            context=context,
+        )
 
 
 def generateVariableReferenceCode(to_name, expression, emit, context):
@@ -177,7 +203,7 @@ def _getVariableCodeName(in_context, variable):
 
 
 def getPickedCType(variable, context):
-    """ Return type to use for specific context. """
+    """Return type to use for specific context."""
 
     user = context.getEntryPoint()
     owner = variable.getEntryPoint()
@@ -411,29 +437,25 @@ def _getVariableDelCode(
 def generateVariableReleaseCode(statement, emit, context):
     variable = statement.getVariable()
 
+    # Only for normal variables we do this.
+    assert not variable.isModuleVariable()
+
+    variable_trace = statement.getVariableTrace()
+
     if variable.isSharedTechnically():
         # TODO: We might start to not allocate the cell object, then a check
         # would be due. But currently we always allocate it.
         needs_check = False
     else:
-        needs_check = not statement.variable_trace.mustHaveValue()
+        needs_check = not variable_trace.mustHaveValue()
 
-    _getVariableReleaseCode(
-        variable=statement.getVariable(),
-        variable_trace=statement.getVariableTrace(),
-        needs_check=needs_check,
-        emit=emit,
-        context=context,
-    )
+    value_name = getLocalVariableDeclaration(context, variable, variable_trace)
 
+    c_type = value_name.getCType()
 
-def _getVariableReleaseCode(variable, variable_trace, needs_check, emit, context):
-    assert not variable.isModuleVariable()
+    if not needs_check:
+        c_type.emitReleaseAssertionCode(value_name=value_name, emit=emit)
 
-    variable_declaration = getLocalVariableDeclaration(
-        context, variable, variable_trace
-    )
+    c_type.getReleaseCode(value_name=value_name, needs_check=needs_check, emit=emit)
 
-    variable_declaration.getCType().getReleaseCode(
-        variable_code_name=variable_declaration, needs_check=needs_check, emit=emit
-    )
+    c_type.emitReinitCode(value_name=value_name, emit=emit)
