@@ -61,6 +61,9 @@ PyDictObject *moduledict_%(module_identifier)s;
 
 /* The declarations of module constants used, if any. */
 static PyObject *mod_consts[%(constants_count)d];
+#ifndef __NUITKA_NO_ASSERT__
+static Py_hash_t mod_consts_hash[%(constants_count)d];
+#endif
 
 static PyObject *module_filename_obj = NULL;
 
@@ -69,13 +72,20 @@ static bool constants_created = false;
 
 /* Function to create module private constants. */
 static void createModuleConstants(void) {
-    loadConstantsBlob(&mod_consts[0], "%(module_const_blob_name)s", %(constants_count)d);
+    if (constants_created == false) {
+        loadConstantsBlob(&mod_consts[0], UNTRANSLATE(%(module_const_blob_name)s));
+        constants_created = true;
 
-    constants_created = true;
+#ifndef __NUITKA_NO_ASSERT__
+        for(int i = 0; i < %(constants_count)d; i++) {
+            mod_consts_hash[i] = DEEP_HASH(mod_consts[i]);
+        }
+#endif
+    }
 }
 
-/* For multiprocessing, we want to be able to initialize the __main__ constants. */
-#if (_NUITKA_PLUGIN_MULTIPROCESSING_ENABLED || _NUITKA_PLUGIN_TRACEBACK_ENCRYPTION_ENABLED) && %(is_main_module)s
+// We want to be able to initialize the "__main__" constants in any case.
+#if %(is_dunder_main)s
 void createMainModuleConstants(void) {
     createModuleConstants();
 }
@@ -87,7 +97,10 @@ void checkModuleConstants_%(module_identifier)s(void) {
     // The module may not have been used at all, then ignore this.
     if (constants_created == false) return;
 
-    checkConstantsBlob(&mod_consts[0], "%(module_name)s", %(constants_count)d);
+    for(int i = 0; i < %(constants_count)d; i++) {
+        assert(mod_consts_hash[i] == DEEP_HASH(mod_consts[i]));
+        CHECK_OBJECT_DEEP(mod_consts[i]);
+    }
 }
 #endif
 
@@ -115,6 +128,7 @@ extern PyTypeObject Nuitka_Loader_Type;
 #ifdef _NUITKA_PLUGIN_DILL_ENABLED
 // Provide a way to create find a function via its C code and create it back
 // in another process, useful for multiprocessing extensions like dill
+extern void registerDillPluginTables(char const *module_name, PyMethodDef *reduce_compiled_function, PyMethodDef *create_compiled_function);
 
 function_impl_code functable_%(module_identifier)s[] = {
 %(module_function_table_entries)s
@@ -233,12 +247,12 @@ static PyObject *_create_compiled_function(PyObject *self, PyObject *args, PyObj
         flags_int,
         function_name,
         argnames,
+        NULL, // freevars
         arg_count_int,
         0, // TODO: Missing kw_only_count
         0 // TODO: Missing pos_only_count
     );
 
-    // TODO: More stuff needed for Python3, best to re-order arguments of MAKE_CODEOBJECT.
     struct Nuitka_FunctionObject *result = Nuitka_Function_New(
         functable_%(module_identifier)s[offset],
         code_object->co_name,
@@ -269,101 +283,75 @@ static PyMethodDef _method_def_create_compiled_function = {
 
 #endif
 
-// Actual name might be different when loaded as a package.
-NUITKA_MAY_BE_UNUSED static char const *module_full_name = "%(module_name)s";
-
 // Internal entry point for module code.
-PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaPathBasedLoaderEntry const *module_entry) {
+PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaPathBasedLoaderEntry const *loader_entry) {
+    // Report entry to PGO.
+    PGO_onModuleEntered("%(module_name)s");
+
+    // Store the module for future use.
     module_%(module_identifier)s = module;
 
-#if defined(_NUITKA_EXE) || PYTHON_VERSION >= 0x300
-    static bool _init_done = false;
+    // Modules can be loaded again in case of errors, avoid the init being done again.
+    static bool init_done = false;
 
-    // Modules might be imported repeatedly, which is to be ignored.
-    if (_init_done) {
-#ifdef _NUITKA_TRACE
-        PRINT_STRING("%(module_name)s: Skipping module init, already done.\n");
-#endif
+    if (init_done == false) {
+#if defined(_NUITKA_MODULE) && %(is_top)d
+        // In case of an extension module loaded into a process, we need to call
+        // initialization here because that's the first and potentially only time
+        // we are going called.
 
-        return module_%(module_identifier)s;
-    } else {
-        _init_done = true;
-    }
-#endif
+        // Initialize the constant values used.
+        _initBuiltinModule();
+        createGlobalConstants();
 
-#ifdef _NUITKA_MODULE
-    // In case of a stand alone extension module, need to call initialization
-    // the init here because that's the first and only time we are going to get
-    // called here.
+        /* Initialize the compiled types of Nuitka. */
+        _initCompiledCellType();
+        _initCompiledGeneratorType();
+        _initCompiledFunctionType();
+        _initCompiledMethodType();
+        _initCompiledFrameType();
 
-    // Initialize the constant values used.
-    _initBuiltinModule();
-    createGlobalConstants();
-
-    /* Initialize the compiled types of Nuitka. */
-    _initCompiledCellType();
-    _initCompiledGeneratorType();
-    _initCompiledFunctionType();
-    _initCompiledMethodType();
-    _initCompiledFrameType();
-
-#if PYTHON_VERSION < 0x300
-    _initSlotCompare();
-#endif
+        _initSlotCompare();
 #if PYTHON_VERSION >= 0x270
-    _initSlotIternext();
+        _initSlotIternext();
 #endif
 
-    patchBuiltinModule();
-    patchTypeComparison();
+        patchBuiltinModule();
+        patchTypeComparison();
 
-    // Enable meta path based loader if not already done.
+        // Enable meta path based loader if not already done.
 #ifdef _NUITKA_TRACE
-    PRINT_STRING("%(module_name)s: Calling setupMetaPathBasedLoader().\n");
+        PRINT_STRING("%(module_name)s: Calling setupMetaPathBasedLoader().\n");
 #endif
-    setupMetaPathBasedLoader();
+        setupMetaPathBasedLoader();
 
 #if PYTHON_VERSION >= 0x300
-    patchInspectModule();
+        patchInspectModule();
 #endif
 
 #endif
 
-    /* The constants only used by this module are created now. */
+        /* The constants only used by this module are created now. */
 #ifdef _NUITKA_TRACE
-    PRINT_STRING("%(module_name)s: Calling createModuleConstants().\n");
+        PRINT_STRING("%(module_name)s: Calling createModuleConstants().\n");
 #endif
-    createModuleConstants();
+        createModuleConstants();
 
-    /* The code objects used by this module are created now. */
+        /* The code objects used by this module are created now. */
 #ifdef _NUITKA_TRACE
-    PRINT_STRING("%(module_name)s: Calling createModuleCodeObjects().\n");
+        PRINT_STRING("%(module_name)s: Calling createModuleCodeObjects().\n");
 #endif
-    createModuleCodeObjects();
+        createModuleCodeObjects();
+
+        init_done = true;
+    }
 
     // PRINT_STRING("in init%(module_identifier)s\n");
 
-    // Create the module object first. There are no methods initially, all are
-    // added dynamically in actual code only.  Also no "__doc__" is initially
-    // set at this time, as it could not contain NUL characters this way, they
-    // are instead set in early module code.  No "self" for modules, we have no
-    // use for it.
-
     moduledict_%(module_identifier)s = MODULE_DICT(module_%(module_identifier)s);
 
 #ifdef _NUITKA_PLUGIN_DILL_ENABLED
-    {
-        PyObject *function_tables = PyObject_GetAttrString((PyObject *)builtin_module, "compiled_function_tables");
-        if (function_tables == NULL) {
-            DROP_ERROR_OCCURRED();
-            function_tables = PyDict_New();
-        }
-        PyObject_SetAttrString((PyObject *)builtin_module, "compiled_function_tables", function_tables);
-        PyObject *funcs = PyTuple_New(2);
-        PyTuple_SET_ITEM(funcs, 0, PyCFunction_New(&_method_def_reduce_compiled_function, NULL));
-        PyTuple_SET_ITEM(funcs, 1, PyCFunction_New(&_method_def_create_compiled_function, NULL));
-        PyDict_SetItemString(function_tables, module_full_name, funcs);
-    }
+    registerDillPluginTables(loader_entry->name, &_method_def_reduce_compiled_function, &_method_def_create_compiled_function);
 #endif
 
     // Set "__compiled__" to what version information we have.
@@ -379,7 +367,7 @@ PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaP
         UPDATE_STRING_DICT0(
             moduledict_%(module_identifier)s,
             (Nuitka_StringObject *)const_str_plain___package__,
-            const_str_empty
+            %(dunder_main_package)s
         );
 #elif %(is_package)s
         PyObject *module_name = GET_STRING_DICT_VALUE(moduledict_%(module_identifier)s, (Nuitka_StringObject *)const_str_plain___name__);
@@ -429,7 +417,7 @@ PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaP
         PyObject *value = (PyObject *)builtin_module;
 
         // Check if main module, not a dict then but the module itself.
-#if !defined(_NUITKA_EXE) || !%(is_main_module)s
+#if defined(_NUITKA_MODULE) || !%(is_dunder_main)s
         value = PyModule_GetDict(value);
 #endif
 
@@ -443,7 +431,7 @@ PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaP
 #if PYTHON_VERSION >= 0x340
 // Set the "__spec__" value
 
-#if %(is_main_module)s
+#if %(is_dunder_main)s
     // Main modules just get "None" as spec.
     UPDATE_STRING_DICT0(moduledict_%(module_identifier)s, (Nuitka_StringObject *)const_str_plain___spec__, Py_None);
 #else
@@ -480,6 +468,9 @@ PyObject *modulecode_%(module_identifier)s(PyObject *module, struct Nuitka_MetaP
     // Module code.
 %(module_code)s
 
+    // Report to PGO about leaving the module without error.
+    PGO_onModuleExit("%(module_name)s", false);
+
     return module_%(module_identifier)s;
 %(module_exit)s
 """
@@ -544,6 +535,10 @@ static struct PyModuleDef mdef_%(module_identifier)s = {
  * gets called. It has to have an exact function name, in cases it's a shared
  * library export. This is hidden behind the MOD_INIT_DECL macro.
  */
+
+// Actual name might be different when loaded as a package.
+static char const *module_full_name = "%(module_name)s";
+
 MOD_INIT_DECL(%(module_identifier)s) {
     if (_Py_PackageContext != NULL) {
         module_full_name = _Py_PackageContext;
@@ -580,6 +575,18 @@ MOD_INIT_DECL(%(module_identifier)s) {
 
 template_module_exception_exit = """\
     module_exception_exit:
+
+#if defined(_NUITKA_MODULE) && %(is_top)d
+    {
+        PyObject *module_name = GET_STRING_DICT_VALUE(moduledict_%(module_identifier)s, (Nuitka_StringObject *)const_str_plain___name__);
+
+        if (module_name != NULL) {
+            Nuitka_DelModule(module_name);
+        }
+    }
+#endif
+    PGO_onModuleExit("%(module_identifier)s", false);
+
     RESTORE_ERROR_OCCURRED(exception_type, exception_value, exception_tb);
     return NULL;
 }"""
@@ -593,7 +600,7 @@ template_helper_impl_decl = """\
 
 #include "nuitka/prelude.h"
 
-extern PyObject *callPythonFunction( PyObject *func, PyObject **args, int count );
+extern PyObject *callPythonFunction(PyObject *func, PyObject *const *args, int count);
 
 """
 
