@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,19 +21,21 @@ This wraps strings with a class derived from "str" that does more checks.
 """
 
 
+from nuitka import Options
 from nuitka.__past__ import iterItems
-from nuitka.Options import isDebug
+from nuitka.Tracing import optimization_logger
 
 
 def enableDebug(globals_dict):
     templates = dict(globals_dict)
 
     class TemplateWrapper(object):
-        """ Wrapper around templates.
+        """Wrapper around templates.
 
-            To better trace and control template usage.
+        To better trace and control template usage.
 
         """
+
         def __init__(self, name, value):
             self.name = name
             self.value = value
@@ -41,17 +43,18 @@ def enableDebug(globals_dict):
         def __str__(self):
             return self.value
 
+        def __add__(self, other):
+            return self.__class__(
+                self.name + "+" + other.name, self.value + other.value
+            )
+
         def __mod__(self, other):
             assert type(other) is dict, self.name
 
             for key in other.keys():
                 if "%%(%s)" % key not in self.value:
-                    from logging import warning
-
-                    warning(
-                        "Extra value '%s' provided to template '%s'.",
-                        key,
-                        self.name
+                    optimization_logger.warning(
+                        "Extra value %r provided to template %r." % (key, self.name)
                     )
 
             try:
@@ -65,15 +68,13 @@ def enableDebug(globals_dict):
     for template_name, template_value in iterItems(templates):
         # Ignore internal attribute like "__name__" that the module will also
         # have of course.
-        if template_name.startswith('_'):
+        if template_name.startswith("_"):
             continue
 
         if type(template_value) is str:
-            globals_dict[template_name] = TemplateWrapper(
-                template_name,
-                template_value
-            )
+            globals_dict[template_name] = TemplateWrapper(template_name, template_value)
+
 
 def checkDebug(globals_dict):
-    if isDebug():
+    if Options.is_debug:
         enableDebug(globals_dict)
