@@ -1,4 +1,4 @@
-#     Copyright 2013, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,18 +22,11 @@ can be computed here, so the code generation can be quick and doesn't have to
 check it many times.
 
 """
-from .FinalizeMarkups import FinalizeMarkups
-from .FinalizeClosureTaking import FinalizeClosureTaking
-
-# Bug of pylint, it's there but it reports it wrongly, pylint: disable=E0611
 from nuitka.tree import Operations
 
-def prepareCodeGeneration( tree ):
-    visitor = FinalizeMarkups()
-    Operations.visitTree( tree, visitor )
-    for function in tree.getUsedFunctions():
-        Operations.visitTree( function, visitor )
+from .FinalizeMarkups import FinalizeMarkups
 
-    visitor = FinalizeClosureTaking()
-    for function in tree.getUsedFunctions():
-        Operations.visitFunction( function, visitor )
+
+def prepareCodeGeneration(tree):
+    visitor = FinalizeMarkups()
+    Operations.visitTree(tree, visitor)
