@@ -35,7 +35,6 @@ it's from the standard library, one can abuse the attribute ``__file__`` of the
 
 """
 
-from __future__ import print_function
 
 import collections
 import hashlib
@@ -44,19 +43,21 @@ import os
 import sys
 import zipfile
 
-from nuitka import Options
+from nuitka import Options, SourceCodeReferences
+from nuitka.__past__ import iter_modules
 from nuitka.containers.oset import OrderedSet
 from nuitka.importing import StandardLibrary
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
-from nuitka.Tracing import recursion_logger
+from nuitka.Tracing import my_print, recursion_logger
 from nuitka.utils.AppDirs import getCacheDir
-from nuitka.utils.FileOperations import listDir
+from nuitka.utils.FileOperations import listDir, removeDirectory
 from nuitka.utils.Importing import getSharedLibrarySuffixes
 from nuitka.utils.ModuleNames import ModuleName
+from nuitka.utils.Utils import getOS, isMacOS
 
+from .IgnoreListing import isIgnoreListedNotExistingModule
 from .PreloadedPackages import getPreloadedPackagePath, isPreloadedPackagePath
-from .Whitelisting import isWhiteListedNotExistingModule
 
 _debug_module_finding = Options.shallExplainImports()
 
@@ -77,6 +78,10 @@ def setMainScriptDirectory(main_dir):
     main_path = main_dir
 
 
+def getMainScriptDirectory():
+    return main_path
+
+
 def isPackageDir(dirname):
     """Decide if a directory is a package.
 
@@ -115,12 +120,14 @@ def getModuleNameAndKindFromFilename(module_filename):
         module_name = ModuleName(os.path.basename(module_filename))
         module_kind = "py"
     elif module_filename.endswith(".py"):
-        module_name = os.path.basename(module_filename)[:-3]
+        module_name = ModuleName(os.path.basename(module_filename)[:-3])
         module_kind = "py"
     else:
         for suffix in getSharedLibrarySuffixes():
             if module_filename.endswith(suffix):
-                module_name = os.path.basename(module_filename)[: -len(suffix)]
+                module_name = ModuleName(
+                    os.path.basename(module_filename)[: -len(suffix)]
+                )
                 module_kind = "shlib"
 
                 break
@@ -131,31 +138,26 @@ def getModuleNameAndKindFromFilename(module_filename):
     return module_name, module_kind
 
 
-def isWhiteListedImport(node):
-    module = node.getParentModule()
-
-    return StandardLibrary.isStandardLibraryPath(module.getFilename())
+def isIgnoreListedImportMaker(source_ref):
+    return StandardLibrary.isStandardLibraryPath(source_ref.getFilename())
 
 
-def warnAbout(importing, module_name, parent_package, level, tried_names):
-    # This probably should not be dealt with here, pylint: disable=too-many-branches
+def warnAbout(importing, module_name, level, source_ref):
+    # This probably should not be dealt with here
     if module_name == "":
         return
 
-    if not isWhiteListedImport(importing) and not isWhiteListedNotExistingModule(
+    if not isIgnoreListedNotExistingModule(
         module_name
-    ):
-        key = module_name, parent_package, level
+    ) and not isIgnoreListedImportMaker(source_ref):
+        key = module_name, level
 
         if key not in warned_about:
             warned_about.add(key)
 
-            if parent_package is None:
-                full_name = module_name
-            else:
-                full_name = module_name
-
-            if Plugins.suppressUnknownImportWarning(importing, full_name):
+            if Plugins.suppressUnknownImportWarning(
+                importing=importing, source_ref=source_ref, module_name=module_name
+            ):
                 return
 
             if level == 0:
@@ -168,15 +170,14 @@ def warnAbout(importing, module_name, parent_package, level, tried_names):
                 level_desc = "%d package levels up" % level
 
             if _debug_module_finding:
-                if parent_package is not None:
+                if importing.getPackageName() is not None:
                     recursion_logger.warning(
-                        "%s: Cannot find '%s' in package '%s' %s (tried %s)."
+                        "%s: Cannot find '%s' in package '%s' %s."
                         % (
                             importing.getSourceReference().getAsString(),
                             module_name,
-                            parent_package,
+                            importing.getPackageName().asString(),
                             level_desc,
-                            ",".join(tried_names),
                         )
                     )
                 else:
@@ -200,7 +201,7 @@ def normalizePackageName(module_name):
     return module_name
 
 
-def findModule(importing, module_name, parent_package, level, warn):
+def findModule(module_name, parent_package, level):
     """Find a module with given package name as parent.
 
     The package name can be None of course. Level is the same
@@ -212,12 +213,12 @@ def findModule(importing, module_name, parent_package, level, warn):
         method used.
     """
     # We have many branches here, because there are a lot of cases to try.
-    # pylint: disable=too-many-branches
+    # pylint: disable=too-many-branches,too-many-return-statements,too-many-statements
 
     assert type(module_name) is ModuleName, module_name
 
     if _debug_module_finding:
-        print(
+        my_print(
             "findModule: Enter to search %r in package %r level %s."
             % (module_name, parent_package, level)
         )
@@ -232,13 +233,7 @@ def findModule(importing, module_name, parent_package, level, warn):
         # TODO: Should give a warning and return not found if the levels
         # exceed the package name.
         if parent_package is not None:
-            # TODO: This should be done with the API instead.
-            parent_package = ModuleName(
-                ".".join(parent_package.asString().split(".")[: -level + 1])
-            )
-
-            if parent_package == "":
-                parent_package = None
+            parent_package = parent_package.getRelativePackageName(level)
         else:
             return None, None, "not-found"
 
@@ -252,6 +247,17 @@ def findModule(importing, module_name, parent_package, level, warn):
         full_name = normalizePackageName(full_name)
         tried_names.append(full_name)
 
+        preloaded_path = getPreloadedPackagePath(module_name)
+
+        if preloaded_path is not None:
+            for module_filename in preloaded_path:
+                if os.path.exists(module_filename):
+                    break
+            else:
+                module_filename = None
+
+            return full_name.getPackageName(), module_filename, "pth"
+
         try:
             module_filename = _findModule(module_name=full_name)
         except ImportError:
@@ -259,7 +265,7 @@ def findModule(importing, module_name, parent_package, level, warn):
             pass
         else:
             if _debug_module_finding:
-                print(
+                my_print(
                     "findModule: Relative imported module '%s' as '%s' in filename '%s':"
                     % (module_name, full_name, module_filename)
                 )
@@ -275,12 +281,40 @@ def findModule(importing, module_name, parent_package, level, warn):
         # Built-in module names must not be searched any further.
         if module_name in sys.builtin_module_names:
             if _debug_module_finding:
-                print(
+                my_print(
                     "findModule: Absolute imported module '%s' in as built-in':"
                     % (module_name,)
                 )
             return package_name, None, "built-in"
 
+        # Frozen module names are similar to built-in, but there is no list of
+        # them, therefore check loader name. Not useful at this time
+        # to make a difference with built-in.
+        if python_version >= 0x300 and module_name in sys.modules:
+            loader = getattr(sys.modules[module_name], "__loader__", None)
+
+            if (
+                loader is not None
+                and getattr(loader, "__name__", "") == "FrozenImporter"
+            ):
+                if _debug_module_finding:
+                    my_print(
+                        "findModule: Absolute imported module '%s' in as frozen':"
+                        % (module_name,)
+                    )
+                return package_name, None, "built-in"
+
+        preloaded_path = getPreloadedPackagePath(module_name)
+
+        if preloaded_path is not None:
+            for module_filename in preloaded_path:
+                if os.path.exists(module_filename):
+                    break
+            else:
+                module_filename = None
+
+            return package_name, module_filename, "pth"
+
         try:
             module_filename = _findModule(module_name=module_name)
         except ImportError:
@@ -288,34 +322,36 @@ def findModule(importing, module_name, parent_package, level, warn):
             pass
         else:
             if _debug_module_finding:
-                print(
+                my_print(
                     "findModule: Found absolute imported module '%s' in filename '%s':"
                     % (module_name, module_filename)
                 )
 
             return package_name, module_filename, "absolute"
 
-    if warn:
-        warnAbout(
-            importing=importing,
-            module_name=module_name,
-            parent_package=parent_package,
-            tried_names=tried_names,
-            level=level,
-        )
-
     return None, None, "not-found"
 
 
 # Some platforms are case insensitive.
-case_sensitive = not sys.platform.startswith(("win", "cygwin", "darwin"))
+case_sensitive = not isMacOS() and getOS() != "Windows"
 
 ImportScanFinding = collections.namedtuple(
     "ImportScanFinding", ("found_in", "priority", "full_path", "search_order")
 )
 
+# We put here things that are not worth it (Cython is not really used by
+# anything really, or where it's know to not have a big # impact, e.g. lxml.
+
+unworthy_namespaces = ("Cython", "lxml")
+
+
+def _reportCandidates(package_name, module_name, candidate, candidates):
+    module_name = (
+        package_name.getChildNamed(module_name)
+        if package_name is not None
+        else module_name
+    )
 
-def _reportCandidates(module_name, candidate, candidates):
     if (
         candidate.priority == 1
         and Options.shallPreferSourcecodeOverExtensionModules() is None
@@ -326,15 +362,17 @@ def _reportCandidates(module_name, candidate, candidates):
                 continue
 
             if c.search_order == candidate.search_order:
-                recursion_logger.info(
-                    """\
+                if not module_name.hasOneOfNamespaces(unworthy_namespaces):
+
+                    recursion_logger.info(
+                        """\
 Should decide '--prefer-source-code' vs. '--no-prefer-source-code', using \
 existing '%s' extension module by default. Candidates were: %s <-> %s."""
-                    % (module_name, candidate, c)
-                )
+                        % (module_name, candidate, c)
+                    )
 
 
-def _findModuleInPath2(module_name, search_path):
+def _findModuleInPath2(package_name, module_name, search_path):
     """This is out own module finding low level implementation.
 
     Just the full module name and search path are given. This is then
@@ -413,7 +451,7 @@ def _findModuleInPath2(module_name, search_path):
                 candidates.add(
                     ImportScanFinding(
                         found_in=entry,
-                        priority=priority_map[mtype],
+                        priority=4 + priority_map[mtype],
                         full_path=full_path,
                         search_order=count,
                     )
@@ -421,7 +459,7 @@ def _findModuleInPath2(module_name, search_path):
                 last_mtype = mtype
 
     if _debug_module_finding:
-        print("Candidates:", candidates)
+        my_print("Candidates:", candidates)
 
     if candidates:
         # Sort by priority, with entries from same path element coming first, then desired type.
@@ -429,13 +467,23 @@ def _findModuleInPath2(module_name, search_path):
 
         # On case sensitive systems, no resolution needed.
         if case_sensitive:
-            _reportCandidates(module_name, candidates[0], candidates)
+            _reportCandidates(
+                package_name=package_name,
+                module_name=module_name,
+                candidate=candidates[0],
+                candidates=candidates,
+            )
             return candidates[0].full_path
         else:
             for candidate in candidates:
                 for fullname, _filename in listDir(candidate[0]):
                     if fullname == candidate.full_path:
-                        _reportCandidates(module_name, candidate, candidates)
+                        _reportCandidates(
+                            package_name=package_name,
+                            module_name=module_name,
+                            candidate=candidate,
+                            candidates=candidates,
+                        )
                         return candidate.full_path
 
             # Only exact case matches matter, all candidates were ignored,
@@ -459,9 +507,15 @@ def _unpackPathElement(path_entry):
 
             target_dir = os.path.join(getCacheDir(), "egg-content", checksum)
 
-            zip_ref = zipfile.ZipFile(path_entry, "r")
-            zip_ref.extractall(target_dir)
-            zip_ref.close()
+            if not os.path.exists(target_dir):
+                try:
+                    # Not all Python versions allow using with here, pylint: disable=consider-using-with
+                    zip_ref = zipfile.ZipFile(path_entry, "r")
+                    zip_ref.extractall(target_dir)
+                    zip_ref.close()
+                except BaseException:
+                    removeDirectory(target_dir, ignore_errors=True)
+                    raise
 
             _egg_files[path_entry] = target_dir
 
@@ -520,7 +574,7 @@ def _findModuleInPath(module_name):
     package_name, module_name = module_name.splitModuleBasename()
 
     if _debug_module_finding:
-        print("_findModuleInPath: Enter", module_name, "in", package_name)
+        my_print("_findModuleInPath: Enter", module_name, "in", package_name)
 
     # The "site" module must be located based on PYTHONPATH before it was
     # executed, while we normally search in PYTHONPATH after it was executed,
@@ -538,11 +592,13 @@ def _findModuleInPath(module_name):
     search_path = getPackageSearchPath(package_name)
 
     if _debug_module_finding:
-        print("_findModuleInPath: Using search path", search_path, "for", package_name)
+        my_print(
+            "_findModuleInPath: Using search path", search_path, "for", package_name
+        )
 
     try:
         module_filename = _findModuleInPath2(
-            module_name=module_name, search_path=search_path
+            package_name=package_name, module_name=module_name, search_path=search_path
         )
     except SyntaxError:
         # Warn user, as this is kind of unusual.
@@ -554,7 +610,7 @@ def _findModuleInPath(module_name):
         return None
 
     if _debug_module_finding:
-        print("_findModuleInPath: _findModuleInPath2 gave", module_filename)
+        my_print("_findModuleInPath: _findModuleInPath2 gave", module_filename)
 
     return module_filename
 
@@ -563,8 +619,11 @@ module_search_cache = {}
 
 
 def _findModule(module_name):
+    # Not a good module name. TODO: Push this to ModuleName() creation maybe.
+    assert module_name != ""
+
     if _debug_module_finding:
-        print("_findModule: Enter to search '%s'." % (module_name,))
+        my_print("_findModule: Enter to search '%s'." % (module_name,))
 
     assert module_name.getBasename(), module_name
 
@@ -574,34 +633,130 @@ def _findModule(module_name):
         result = module_search_cache[key]
 
         if _debug_module_finding:
-            print("_findModule: Cached result (see previous call).")
+            my_print("_findModule: Cached result (see previous call).")
 
         if result is ImportError:
             raise ImportError
 
         return result
 
-    try:
-        module_search_cache[key] = _findModule2(module_name)
-    except ImportError:
-        new_module_name = Plugins.considerFailedImportReferrals(module_name)
-
-        if new_module_name is None:
-            module_search_cache[key] = ImportError
-            raise
-
-        module_search_cache[key] = _findModule(new_module_name)
+    module_search_cache[key] = _findModuleInPath(module_name)
 
     return module_search_cache[key]
 
 
-def _findModule2(module_name):
-    # Need a real module name.
-    assert module_name != ""
+def locateModule(module_name, parent_package, level):
+    """Locate a module with given package name as parent.
 
-    preloaded_path = getPreloadedPackagePath(module_name)
+    The package name can be None of course. Level is the same
+    as with "__import__" built-in.
 
-    if preloaded_path is not None:
-        return preloaded_path[0]
+    Returns:
+        Returns a triple of module name the module has considering
+        package containing it, and filename of it which can be a
+        directory for packages, and the location method used.
+    """
+    module_package, module_filename, finding = findModule(
+        module_name=module_name,
+        parent_package=parent_package,
+        level=level,
+    )
 
-    return _findModuleInPath(module_name=module_name)
+    assert module_package is None or (
+        type(module_package) is ModuleName and module_package != ""
+    ), repr(module_package)
+
+    if module_filename is not None:
+        module_filename = os.path.normpath(module_filename)
+
+        module_name, module_kind = getModuleNameAndKindFromFilename(module_filename)
+
+        assert module_kind is not None, module_filename
+
+        module_name = ModuleName.makeModuleNameInPackage(module_name, module_package)
+
+    return module_name, module_filename, finding
+
+
+def locateModules(package_name):
+    """Determine child module names.
+
+    Return:
+        generator of ModuleName objects
+    """
+    package_name = ModuleName(package_name)
+
+    module_filename = locateModule(
+        module_name=ModuleName(package_name), parent_package=None, level=0
+    )[1]
+
+    if module_filename is not None:
+        for sub_module in iter_modules([module_filename]):
+            yield package_name.getChildNamed(sub_module.name)
+
+
+def decideModuleSourceRef(filename, module_name, is_main, is_fake, logger):
+    # Many branches due to the many cases
+
+    assert type(module_name) is ModuleName
+    assert filename is not None
+
+    is_namespace = False
+    is_package = False
+
+    if is_main and os.path.isdir(filename):
+        source_filename = os.path.join(filename, "__main__.py")
+
+        if not os.path.isfile(source_filename):
+            sys.stderr.write(
+                "%s: can't find '__main__' module in '%s'\n"
+                % (os.path.basename(sys.argv[0]), filename)
+            )
+            sys.exit(2)
+
+        filename = source_filename
+
+        main_added = True
+    else:
+        main_added = False
+
+    if is_fake:
+        source_filename = filename
+
+        source_ref = SourceCodeReferences.fromFilename(filename=filename)
+
+        module_name = is_fake
+
+    elif os.path.isfile(filename):
+        source_filename = filename
+
+        source_ref = SourceCodeReferences.fromFilename(filename=filename)
+
+    elif isPackageDir(filename):
+        is_package = True
+
+        source_filename = os.path.join(filename, "__init__.py")
+
+        if not os.path.isfile(source_filename):
+            source_ref = SourceCodeReferences.fromFilename(
+                filename=filename
+            ).atInternal()
+            is_namespace = True
+        else:
+            source_ref = SourceCodeReferences.fromFilename(
+                filename=os.path.abspath(source_filename)
+            )
+
+    else:
+        logger.sysexit(
+            "%s: can't open file '%s'." % (os.path.basename(sys.argv[0]), filename),
+            exit_code=2,
+        )
+
+    return (
+        main_added,
+        is_package,
+        is_namespace,
+        source_ref,
+        source_filename,
+    )
