@@ -147,9 +147,8 @@ class StatementAssignmentVariable(StatementChildHavingBase):
     def __init__(self, source, variable, source_ref, version=None):
         assert source is not None, source_ref
 
-        if variable is not None:
-            if version is None:
-                version = variable.allocateTargetNumber()
+        if version is None:
+            version = variable.allocateTargetNumber()
 
         self.variable = variable
         self.variable_version = version
@@ -292,9 +291,6 @@ Assignment raises exception in assigned value, removed assignment.""",
 
         variable = self.variable
 
-        # Not allowed anymore at this point.
-        assert variable is not None
-
         # Assigning from and to the same variable, can be optimized away
         # immediately, there is no point in doing it. Exceptions are of course
         # module variables that collide with built-in names.
@@ -346,7 +342,10 @@ Removed assignment of %s from itself which is known to be defined."""
 
             if last_trace is not None and not last_trace.getMergeOrNameUsageCount():
                 if source.isCompileTimeConstant():
-                    if variable.isModuleVariable():
+                    if (
+                        variable.isModuleVariable()
+                        or variable.owner.locals_scope.isUnoptimizedFunctionScope()
+                    ):
                         # TODO: We do not trust these yet a lot, but more might be
                         pass
                     else:
@@ -572,14 +571,20 @@ class StatementDelVariable(StatementBase):
                     % (self.getVariableName(),),
                 )
             else:
-                assert self.variable.isLocalVariable(), self.variable
-
-                result = makeRaiseExceptionReplacementStatement(
-                    statement=self,
-                    exception_type="UnboundLocalError",
-                    exception_value="""local variable '%s' referenced before assignment"""
-                    % variable.getName(),
-                )
+                if self.variable.isLocalVariable():
+                    result = makeRaiseExceptionReplacementStatement(
+                        statement=self,
+                        exception_type="UnboundLocalError",
+                        exception_value="""local variable '%s' referenced before assignment"""
+                        % variable.getName(),
+                    )
+                else:
+                    result = makeRaiseExceptionReplacementStatement(
+                        statement=self,
+                        exception_type="NameError",
+                        exception_value="""name '%s' is not defined"""
+                        % variable.getName(),
+                    )
 
                 return trace_collection.computedStatementResult(
                     result,
@@ -622,16 +627,11 @@ class StatementDelVariable(StatementBase):
             variable=variable, version=self.variable_version, del_node=self
         )
 
-        trace_collection.onVariableContentEscapes(variable)
-
         # Any code could be run, note that.
         trace_collection.onControlFlowEscape(self)
 
         return self, None, None
 
-    def mayHaveSideEffects(self):
-        return True
-
     def mayRaiseException(self, exception_type):
         if self.tolerant:
             return False
@@ -715,8 +715,8 @@ class StatementReleaseVariable(StatementBase):
                 return (
                     None,
                     "new_statements",
-                    "Original parameter variable value %s is not released."
-                    % (self.variable.getDescription()),
+                    "Original parameter variable value of '%s' is not released."
+                    % self.variable.getName(),
                 )
 
         self.variable_trace = trace_collection.getVariableCurrentTrace(self.variable)
@@ -728,7 +728,7 @@ class StatementReleaseVariable(StatementBase):
                 "Uninitialized %s is not released." % (self.variable.getDescription()),
             )
 
-        trace_collection.onVariableContentEscapes(self.variable)
+        # TODO: Annotate value content as escaped, as destruction might run.
 
         # Any code could be run, note that.
         trace_collection.onControlFlowEscape(self)
@@ -738,9 +738,11 @@ class StatementReleaseVariable(StatementBase):
 
         return self, None, None
 
-    def mayHaveSideEffects(self):
-        # May execute __del__ code, it would be sweet to be able to predict
-        # that another reference will still be active for a value though.
+    @staticmethod
+    def mayHaveSideEffects():
+        # TODO: May execute "__del__" code, it would be sweet to be able to predict
+        # if another reference will still be active for a value though, or if there
+        # is such a code for the type shape.
         return True
 
     def mayRaiseException(self, exception_type):
