@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,113 +22,78 @@ whose implementation lives here. The creation itself also lives here.
 
 """
 
-from .Checkers import checkStatementsSequenceOrNone
-from .ExpressionBases import ExpressionChildrenHavingBase
+from .ExpressionBases import (
+    ExpressionChildHavingBase,
+    ExpressionNoSideEffectsMixin,
+)
 from .FunctionNodes import ExpressionFunctionEntryPointBase
-from .IndicatorMixins import MarkLocalsDictIndicatorMixin
 
 
-class ExpressionMakeAsyncgenObject(ExpressionChildrenHavingBase):
+class ExpressionMakeAsyncgenObject(
+    ExpressionNoSideEffectsMixin, ExpressionChildHavingBase
+):
     kind = "EXPRESSION_MAKE_ASYNCGEN_OBJECT"
 
-    named_children = (
-        "asyncgen_ref",
-    )
+    named_child = "asyncgen_ref"
 
-    getAsyncgenRef = ExpressionChildrenHavingBase.childGetter("asyncgen_ref")
+    __slots__ = ("variable_closure_traces",)
 
-    def __init__(self, asyncgen_ref, code_object, source_ref):
+    def __init__(self, asyncgen_ref, source_ref):
         assert asyncgen_ref.getFunctionBody().isExpressionAsyncgenObjectBody()
 
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "asyncgen_ref" : asyncgen_ref,
-            },
-            source_ref = source_ref
+        ExpressionChildHavingBase.__init__(
+            self, value=asyncgen_ref, source_ref=source_ref
         )
 
-        self.code_object = code_object
-
         self.variable_closure_traces = []
 
-    def getDetails(self):
-        return {
-            "code_object" : self.code_object
-        }
-
-    def getCodeObject(self):
-        return self.code_object
-
     def getDetailsForDisplay(self):
-        return {
-            "asyncgen" : self.getAsyncgenRef().getFunctionBody().getCodeName()
-        }
+        return {"asyncgen": self.subnode_asyncgen_ref.getFunctionBody().getCodeName()}
 
     def computeExpression(self, trace_collection):
         self.variable_closure_traces = []
 
-        for closure_variable in self.getAsyncgenRef().getFunctionBody().getClosureVariables():
+        for (
+            closure_variable
+        ) in self.subnode_asyncgen_ref.getFunctionBody().getClosureVariables():
             trace = trace_collection.getVariableCurrentTrace(closure_variable)
-            trace.addClosureUsage()
+            trace.addNameUsage()
 
-            self.variable_closure_traces.append(trace)
+            self.variable_closure_traces.append((closure_variable, trace))
 
         # TODO: Asyncgen body may know something too.
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
-        return False
-
-    def mayHaveSideEffects(self):
-        return False
-
     def getClosureVariableVersions(self):
-        return [
-            (trace.getVariable(), trace.getVersion())
-            for trace in self.variable_closure_traces
-        ]
+        return self.variable_closure_traces
 
 
-class ExpressionAsyncgenObjectBody(MarkLocalsDictIndicatorMixin,
-                                   ExpressionFunctionEntryPointBase):
-    # We really want these many ancestors, as per design, we add properties via
-    # base class mix-ins a lot, pylint: disable=R0901
+class ExpressionAsyncgenObjectBody(ExpressionFunctionEntryPointBase):
     kind = "EXPRESSION_ASYNCGEN_OBJECT_BODY"
 
-    named_children = (
-        "body",
-    )
+    __slots__ = ("qualname_setup", "needs_generator_return_exit")
 
-    checkers = {
-        # TODO: Is "None" really an allowed value.
-        "body" : checkStatementsSequenceOrNone
-    }
-
-    qualname_setup = None
-
-    def __init__(self, provider, name, flags, source_ref):
+    def __init__(self, provider, name, code_object, flags, auto_release, source_ref):
         ExpressionFunctionEntryPointBase.__init__(
             self,
-            provider    = provider,
-            name        = name,
-            code_prefix = "asyncgen",
-            flags       = flags,
-            source_ref  = source_ref
+            provider=provider,
+            name=name,
+            code_object=code_object,
+            code_prefix="asyncgen",
+            flags=flags,
+            auto_release=auto_release,
+            source_ref=source_ref,
         )
 
-        MarkLocalsDictIndicatorMixin.__init__(self)
-
         self.needs_generator_return_exit = False
 
+        self.qualname_setup = None
+
     def getFunctionName(self):
         return self.name
 
     def markAsNeedsGeneratorReturnHandling(self, value):
-        self.needs_generator_return_exit = max(
-            self.needs_generator_return_exit,
-            value
-        )
+        self.needs_generator_return_exit = max(self.needs_generator_return_exit, value)
 
     def needsGeneratorReturnHandling(self):
         return self.needs_generator_return_exit == 2
