@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -28,6 +28,8 @@ from .ExpressionBases import (
 
 
 class ExpressionBuiltinNext1(ExpressionBuiltinSingleArgBase):
+    __slots__ = ("may_not_raise",)
+
     kind = "EXPRESSION_BUILTIN_NEXT1"
 
     def __init__(self, value, source_ref):
@@ -35,13 +37,28 @@ class ExpressionBuiltinNext1(ExpressionBuiltinSingleArgBase):
             self, value=value, source_ref=source_ref
         )
 
+        self.may_not_raise = False
+
     def computeExpression(self, trace_collection):
-        return self.getValue().computeExpressionNext1(
+        self.may_not_raise, result = self.subnode_value.computeExpressionNext1(
             next_node=self, trace_collection=trace_collection
         )
 
+        return result
+
+    def mayRaiseExceptionOperation(self):
+        return not self.may_not_raise
+
+    def mayRaiseException(self, exception_type):
+        return (
+            self.subnode_value.mayRaiseException(exception_type)
+            or self.mayRaiseExceptionOperation()
+        )
+
 
 class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
+    __slots__ = ("count", "expected", "starred")
+
     kind = "EXPRESSION_SPECIAL_UNPACK"
 
     def __init__(self, value, count, expected, starred, source_ref):
@@ -49,7 +66,7 @@ class ExpressionSpecialUnpack(ExpressionBuiltinNext1):
 
         self.count = int(count)
 
-        # TODO: Unused before 3.5 or higher, maybe specialize for it.
+        # TODO: Unused before 3.5 or higher, and even then starred is rare, maybe specialize for it.
         self.expected = int(expected)
         self.starred = starred
 
@@ -74,8 +91,6 @@ class ExpressionBuiltinNext2(ExpressionChildrenHavingBase):
     kind = "EXPRESSION_BUILTIN_NEXT2"
 
     named_children = ("iterator", "default")
-    getIterator = ExpressionChildrenHavingBase.childGetter("iterator")
-    getDefault = ExpressionChildrenHavingBase.childGetter("default")
 
     def __init__(self, iterator, default, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -87,4 +102,11 @@ class ExpressionBuiltinNext2(ExpressionChildrenHavingBase):
     def computeExpression(self, trace_collection):
         # TODO: The "iterator" should be investigated here, if it is iterable,
         # or if the default is raising.
+
+        # Any code could be run, note that.
+        trace_collection.onControlFlowEscape(self)
+
+        # Any exception may be raised.
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
