@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -32,24 +32,24 @@ from .ExpressionBases import (
     ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
 )
+from .ExpressionShapeMixins import ExpressionListShapeExactMixin
 from .IterationHandles import (
-    ConstantIterationHandleRange1,
-    ConstantIterationHandleRange2,
-    ConstantIterationHandleRange3,
+    IterationHandleRange1,
+    IterationHandleRange2,
+    IterationHandleRange3,
 )
 from .NodeMakingHelpers import makeConstantReplacementNode
-from .shapes.BuiltinTypeShapes import tshape_list, tshape_xrange
+from .shapes.BuiltinTypeShapes import tshape_xrange
 
 
-class ExpressionBuiltinRangeMixin(object):
-    """ Mixin class for range nodes with 1/2/3 arguments. """
+class ExpressionBuiltinRangeMixin(ExpressionListShapeExactMixin):
+    """Mixin class for range nodes with 1/2/3 arguments."""
+
+    # Mixins are required to slots
+    __slots__ = ()
 
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_list
-
     def getTruthValue(self):
         length = self.getIterationLength()
 
@@ -66,11 +66,7 @@ class ExpressionBuiltinRangeMixin(object):
             if child.getIntegerValue() is None:
                 return True
 
-            if (
-                python_version >= 270
-                and child.isExpressionConstantRef()
-                and type(child.getConstant()) is float
-            ):
+            if python_version >= 0x270 and child.isExpressionConstantFloatRef():
                 return True
 
         return False
@@ -84,14 +80,10 @@ class ExpressionBuiltinRangeMixin(object):
             if child.getIntegerValue() is None:
                 return True
 
-            if (
-                python_version >= 270
-                and child.isExpressionConstantRef()
-                and type(child.getConstant()) is float
-            ):
+            if python_version >= 0x270 and child.isExpressionConstantFloatRef():
                 return True
 
-        step = self.getStep()
+        step = self.subnode_step
 
         # A step of 0 will raise.
         if step is not None and step.getIntegerValue() == 0:
@@ -117,68 +109,54 @@ class ExpressionBuiltinRangeMixin(object):
         )
 
     def computeExpressionIter1(self, iter_node, trace_collection):
-        assert python_version < 300
+        assert python_version < 0x300
 
-        iteration_length = self.getIterationLength()
+        # TODO: The xrange is always faster and more memory usage than range, so this makes no sense, to
+        # use it as a source for any iteration, esp. as xrange is the Python3 only type that will be
+        # best optimized.
+        result = makeExpressionBuiltinXrange(
+            low=self.subnode_low,
+            high=self.subnode_high,
+            step=self.subnode_step,
+            source_ref=self.source_ref,
+        )
 
-        if iteration_length is not None and iteration_length > 256:
-            result = makeExpressionBuiltinXrange(
-                low=self.getLow(),
-                high=self.getHigh(),
-                step=self.getStep(),
-                source_ref=self.getSourceReference(),
-            )
+        self.parent.replaceChild(self, result)
+        del self.parent
 
-            self.parent.replaceChild(self, result)
-            del self.parent
-
-            return (
-                iter_node,
-                "new_expression",
-                "Replaced 'range' with 'xrange' built-in call for iteration.",
-            )
-
-        # No exception will be raised on ranges.
-
-        return iter_node, None, None
+        return (
+            iter_node,
+            "new_expression",
+            "Replaced 'range' with 'xrange' built-in call for iteration.",
+        )
 
     def canPredictIterationValues(self):
         return self.getIterationLength() is not None
 
-    @staticmethod
-    def getLow():
-        return None
-
-    @staticmethod
-    def getHigh():
-        return None
-
-    @staticmethod
-    def getStep():
-        return None
-
 
 class ExpressionBuiltinRange1(ExpressionBuiltinRangeMixin, ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_RANGE1"
 
     named_child = "low"
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
+
+    subnode_high = None
+    subnode_step = None
 
     def __init__(self, low, source_ref):
         assert low is not None
-        assert python_version < 300
+        assert python_version < 0x300
 
         ExpressionChildHavingBase.__init__(self, value=low, source_ref=source_ref)
 
     def computeExpression(self, trace_collection):
-        low = self.getLow()
+        low = self.subnode_low
 
         return self.computeBuiltinSpec(
             trace_collection=trace_collection, given_values=(low,)
         )
 
     def getIterationLength(self):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
 
         if low is None:
             return None
@@ -186,11 +164,11 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeMixin, ExpressionChildHaving
         return max(0, low)
 
     def getIterationHandle(self):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
         if low is None:
             return None
 
-        return ConstantIterationHandleRange1(low, self.source_ref)
+        return IterationHandleRange1(low, self.source_ref)
 
     def getIterationValue(self, element_index):
         length = self.getIterationLength()
@@ -203,7 +181,9 @@ class ExpressionBuiltinRange1(ExpressionBuiltinRangeMixin, ExpressionChildHaving
 
         # TODO: Make sure to cast element_index to what CPython will give, for
         # now a downcast will do.
-        return makeConstantReplacementNode(constant=int(element_index), node=self)
+        return makeConstantReplacementNode(
+            constant=int(element_index), node=self, user_provided=False
+        )
 
     def isKnownToBeIterable(self, count):
         return count is None or count == self.getIterationLength()
@@ -215,8 +195,8 @@ class ExpressionBuiltinRange2(
     kind = "EXPRESSION_BUILTIN_RANGE2"
 
     named_children = ("low", "high")
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
+
+    subnode_step = None
 
     def __init__(self, low, high, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -226,18 +206,18 @@ class ExpressionBuiltinRange2(
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
     def computeExpression(self, trace_collection):
-        assert python_version < 300
+        assert python_version < 0x300
 
-        low = self.getLow()
-        high = self.getHigh()
+        low = self.subnode_low
+        high = self.subnode_high
 
         return self.computeBuiltinSpec(
             trace_collection=trace_collection, given_values=(low, high)
         )
 
     def getIterationLength(self):
-        low = self.getLow()
-        high = self.getHigh()
+        low = self.subnode_low
+        high = self.subnode_high
 
         low = low.getIntegerValue()
 
@@ -252,19 +232,19 @@ class ExpressionBuiltinRange2(
         return max(0, high - low)
 
     def getIterationHandle(self):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
         if low is None:
             return None
 
-        high = self.getHigh().getIntegerValue()
+        high = self.subnode_high.getIntegerValue()
         if high is None:
             return None
 
-        return ConstantIterationHandleRange2(low, high, self.source_ref)
+        return IterationHandleRange2(low, high, self.source_ref)
 
     def getIterationValue(self, element_index):
-        low = self.getLow()
-        high = self.getHigh()
+        low = self.subnode_low
+        high = self.subnode_high
 
         low = low.getIntegerValue()
 
@@ -281,7 +261,9 @@ class ExpressionBuiltinRange2(
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(constant=result, node=self)
+            return makeConstantReplacementNode(
+                constant=result, node=self, user_provided=False
+            )
 
     def isKnownToBeIterable(self, count):
         return count is None or count == self.getIterationLength()
@@ -293,11 +275,10 @@ class ExpressionBuiltinRange3(
     kind = "EXPRESSION_BUILTIN_RANGE3"
 
     named_children = ("low", "high", "step")
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
-    getStep = ExpressionChildrenHavingBase.childGetter("step")
 
     def __init__(self, low, high, step, source_ref):
+        assert python_version < 0x300
+
         ExpressionChildrenHavingBase.__init__(
             self, values={"low": low, "high": high, "step": step}, source_ref=source_ref
         )
@@ -305,20 +286,18 @@ class ExpressionBuiltinRange3(
     builtin_spec = BuiltinParameterSpecs.builtin_range_spec
 
     def computeExpression(self, trace_collection):
-        assert python_version < 300
-
-        low = self.getLow()
-        high = self.getHigh()
-        step = self.getStep()
+        low = self.subnode_low
+        high = self.subnode_high
+        step = self.subnode_step
 
         return self.computeBuiltinSpec(
             trace_collection=trace_collection, given_values=(low, high, step)
         )
 
     def getIterationLength(self):
-        low = self.getLow()
-        high = self.getHigh()
-        step = self.getStep()
+        low = self.subnode_low
+        high = self.subnode_high
+        step = self.subnode_step
 
         low = low.getIntegerValue()
 
@@ -360,15 +339,15 @@ class ExpressionBuiltinRange3(
         return self.getIterationLength() is not None
 
     def getIterationHandle(self):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
         if low is None:
             return None
 
-        high = self.getHigh().getIntegerValue()
+        high = self.subnode_high.getIntegerValue()
         if high is None:
             return None
 
-        step = self.getStep().getIntegerValue()
+        step = self.subnode_step.getIntegerValue()
         if step is None:
             return None
 
@@ -376,34 +355,39 @@ class ExpressionBuiltinRange3(
         if step == 0:
             return None
 
-        return ConstantIterationHandleRange3(low, high, step, self.source_ref)
+        return IterationHandleRange3(low, high, step, self.source_ref)
 
     def getIterationValue(self, element_index):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
 
         if low is None:
             return None
 
-        high = self.getHigh().getIntegerValue()
+        high = self.subnode_high.getIntegerValue()
 
         if high is None:
             return None
 
-        step = self.getStep().getIntegerValue()
+        step = self.subnode_step.getIntegerValue()
 
         result = low + step * element_index
 
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(constant=result, node=self)
+            return makeConstantReplacementNode(
+                constant=result, node=self, user_provided=False
+            )
 
     def isKnownToBeIterable(self, count):
         return count is None or count == self.getIterationLength()
 
 
 class ExpressionBuiltinXrangeMixin(object):
-    """ Mixin class for xrange nodes with 1/2/3 arguments. """
+    """Mixin class for xrange nodes with 1/2/3 arguments."""
+
+    # Mixins are required to slots
+    __slots__ = ()
 
     builtin_spec = BuiltinParameterSpecs.builtin_xrange_spec
 
@@ -441,7 +425,7 @@ class ExpressionBuiltinXrangeMixin(object):
             if child.getIntegerValue() is None:
                 return True
 
-        step = self.getStep()
+        step = self.subnode_step
 
         # A step of 0 will raise.
         if step is not None and step.getIntegerValue() == 0:
@@ -472,38 +456,28 @@ class ExpressionBuiltinXrangeMixin(object):
 
         return iter_node, None, None
 
-    @staticmethod
-    def getLow():
-        return None
-
-    @staticmethod
-    def getHigh():
-        return None
-
-    @staticmethod
-    def getStep():
-        return None
-
 
 class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeMixin, ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_XRANGE1"
 
     named_child = "low"
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
+
+    subnode_high = None
+    subnode_step = None
 
     def __init__(self, low, source_ref):
         ExpressionChildHavingBase.__init__(self, value=low, source_ref=source_ref)
 
     def computeExpression(self, trace_collection):
-        low = self.getLow()
+        low = self.subnode_low
 
-        # TODO: Optimize this if self.getLow().getIntegerValue() is Not None
+        # TODO: Optimize this if self.subnode_low.getIntegerValue() is Not None
         return self.computeBuiltinSpec(
             trace_collection=trace_collection, given_values=(low,)
         )
 
     def getIterationLength(self):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
 
         if low is None:
             return None
@@ -521,7 +495,9 @@ class ExpressionBuiltinXrange1(ExpressionBuiltinXrangeMixin, ExpressionChildHavi
 
         # TODO: Make sure to cast element_index to what CPython will give, for
         # now a downcast will do.
-        return makeConstantReplacementNode(constant=int(element_index), node=self)
+        return makeConstantReplacementNode(
+            constant=int(element_index), node=self, user_provided=False
+        )
 
 
 class ExpressionBuiltinXrange2(
@@ -530,8 +506,8 @@ class ExpressionBuiltinXrange2(
     kind = "EXPRESSION_BUILTIN_XRANGE2"
 
     named_children = ("low", "high")
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
+
+    subnode_step = None
 
     def __init__(self, low, high, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -539,16 +515,16 @@ class ExpressionBuiltinXrange2(
         )
 
     def computeExpression(self, trace_collection):
-        low = self.getLow()
-        high = self.getHigh()
+        low = self.subnode_low
+        high = self.subnode_high
 
         return self.computeBuiltinSpec(
             trace_collection=trace_collection, given_values=(low, high)
         )
 
     def getIterationLength(self):
-        low = self.getLow()
-        high = self.getHigh()
+        low = self.subnode_low
+        high = self.subnode_high
 
         low = low.getIntegerValue()
 
@@ -563,8 +539,8 @@ class ExpressionBuiltinXrange2(
         return max(0, high - low)
 
     def getIterationValue(self, element_index):
-        low = self.getLow()
-        high = self.getHigh()
+        low = self.subnode_low
+        high = self.subnode_high
 
         low = low.getIntegerValue()
 
@@ -581,7 +557,9 @@ class ExpressionBuiltinXrange2(
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(constant=result, node=self)
+            return makeConstantReplacementNode(
+                constant=result, node=self, user_provided=False
+            )
 
 
 class ExpressionBuiltinXrange3(
@@ -590,9 +568,6 @@ class ExpressionBuiltinXrange3(
     kind = "EXPRESSION_BUILTIN_XRANGE3"
 
     named_children = ("low", "high", "step")
-    getLow = ExpressionChildrenHavingBase.childGetter("low")
-    getHigh = ExpressionChildrenHavingBase.childGetter("high")
-    getStep = ExpressionChildrenHavingBase.childGetter("step")
 
     def __init__(self, low, high, step, source_ref):
         ExpressionChildrenHavingBase.__init__(
@@ -600,18 +575,18 @@ class ExpressionBuiltinXrange3(
         )
 
     def computeExpression(self, trace_collection):
-        low = self.getLow()
-        high = self.getHigh()
-        step = self.getStep()
+        low = self.subnode_low
+        high = self.subnode_high
+        step = self.subnode_step
 
         return self.computeBuiltinSpec(
             trace_collection=trace_collection, given_values=(low, high, step)
         )
 
     def getIterationLength(self):
-        low = self.getLow()
-        high = self.getHigh()
-        step = self.getStep()
+        low = self.subnode_low
+        high = self.subnode_high
+        step = self.subnode_step
 
         low = low.getIntegerValue()
 
@@ -650,24 +625,26 @@ class ExpressionBuiltinXrange3(
         return int(estimate)
 
     def getIterationValue(self, element_index):
-        low = self.getLow().getIntegerValue()
+        low = self.subnode_low.getIntegerValue()
 
         if low is None:
             return None
 
-        high = self.getHigh().getIntegerValue()
+        high = self.subnode_high.getIntegerValue()
 
         if high is None:
             return None
 
-        step = self.getStep().getIntegerValue()
+        step = self.subnode_step.getIntegerValue()
 
         result = low + step * element_index
 
         if result >= high:
             return None
         else:
-            return makeConstantReplacementNode(constant=result, node=self)
+            return makeConstantReplacementNode(
+                constant=result, node=self, user_provided=False
+            )
 
 
 def makeExpressionBuiltinXrange(low, high, step, source_ref):
