@@ -23,27 +23,31 @@ that should allow some important optimizations.
 
 from nuitka.specs import BuiltinParameterSpecs
 
+from .ConstantRefNodes import makeConstantRefNode
 from .ExpressionBases import (
+    CompileTimeConstantExpressionBase,
     ExpressionBuiltinSingleArgBase,
     ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
     ExpressionSpecBasedComputationMixin,
 )
+from .ExpressionShapeMixins import (
+    ExpressionBoolShapeExactMixin,
+    ExpressionBytearrayShapeExactMixin,
+    ExpressionBytesShapeExactMixin,
+    ExpressionFrozensetShapeExactMixin,
+    ExpressionListShapeExactMixin,
+    ExpressionSetShapeExactMixin,
+    ExpressionTupleShapeExactMixin,
+)
 from .NodeMakingHelpers import (
     makeConstantReplacementNode,
     wrapExpressionWithNodeSideEffects,
 )
 from .shapes.BuiltinTypeShapes import (
-    tshape_bool,
-    tshape_bytearray,
-    tshape_bytes,
     tshape_bytes_derived,
     tshape_float_derived,
-    tshape_frozenset,
-    tshape_list,
-    tshape_set,
     tshape_str_derived,
-    tshape_tuple,
     tshape_unicode_derived,
 )
 
@@ -83,45 +87,37 @@ class ExpressionBuiltinContainerBase(
             )
 
 
-class ExpressionBuiltinTuple(ExpressionBuiltinContainerBase):
+class ExpressionBuiltinTuple(
+    ExpressionTupleShapeExactMixin, ExpressionBuiltinContainerBase
+):
     kind = "EXPRESSION_BUILTIN_TUPLE"
 
     builtin_spec = BuiltinParameterSpecs.builtin_tuple_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_tuple
 
-
-class ExpressionBuiltinList(ExpressionBuiltinContainerBase):
+class ExpressionBuiltinList(
+    ExpressionListShapeExactMixin, ExpressionBuiltinContainerBase
+):
     kind = "EXPRESSION_BUILTIN_LIST"
 
     builtin_spec = BuiltinParameterSpecs.builtin_list_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_list
 
-
-class ExpressionBuiltinSet(ExpressionBuiltinContainerBase):
+class ExpressionBuiltinSet(
+    ExpressionSetShapeExactMixin, ExpressionBuiltinContainerBase
+):
     kind = "EXPRESSION_BUILTIN_SET"
 
     builtin_spec = BuiltinParameterSpecs.builtin_set_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_set
 
-
-class ExpressionBuiltinFrozenset(ExpressionBuiltinContainerBase):
+class ExpressionBuiltinFrozenset(
+    ExpressionFrozensetShapeExactMixin, ExpressionBuiltinContainerBase
+):
     kind = "EXPRESSION_BUILTIN_FROZENSET"
 
     builtin_spec = BuiltinParameterSpecs.builtin_frozenset_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_frozenset
-
 
 class ExpressionBuiltinFloat(ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_FLOAT"
@@ -145,7 +141,7 @@ class ExpressionBuiltinFloat(ExpressionChildHavingBase):
         return self.subnode_value.mayRaiseExceptionFloat(exception_type)
 
 
-class ExpressionBuiltinBool(ExpressionBuiltinTypeBase):
+class ExpressionBuiltinBool(ExpressionBoolShapeExactMixin, ExpressionBuiltinTypeBase):
     kind = "EXPRESSION_BUILTIN_BOOL"
 
     builtin_spec = BuiltinParameterSpecs.builtin_bool_spec
@@ -171,11 +167,6 @@ class ExpressionBuiltinBool(ExpressionBuiltinTypeBase):
 
         return ExpressionBuiltinTypeBase.computeExpression(self, trace_collection)
 
-    @staticmethod
-    def getTypeShape():
-        # Note: Not allowed to subclass bool.
-        return tshape_bool
-
 
 class ExpressionBuiltinUnicodeBase(
     ExpressionSpecBasedComputationMixin, ExpressionChildrenHavingBase
@@ -208,7 +199,7 @@ class ExpressionBuiltinUnicodeBase(
 
 
 class ExpressionBuiltinStrP2(ExpressionBuiltinTypeBase):
-    """ Python2 built-in str call. """
+    """Python2 built-in str call."""
 
     kind = "EXPRESSION_BUILTIN_STR_P2"
 
@@ -240,7 +231,7 @@ class ExpressionBuiltinStrP2(ExpressionBuiltinTypeBase):
 
 
 class ExpressionBuiltinUnicodeP2(ExpressionBuiltinUnicodeBase):
-    """ Python2 built-in unicode call. """
+    """Python2 built-in unicode call."""
 
     kind = "EXPRESSION_BUILTIN_UNICODE_P2"
 
@@ -252,7 +243,7 @@ class ExpressionBuiltinUnicodeP2(ExpressionBuiltinUnicodeBase):
 
 
 class ExpressionBuiltinStrP3(ExpressionBuiltinUnicodeBase):
-    """ Python3 built-in str call. """
+    """Python3 built-in str call."""
 
     kind = "EXPRESSION_BUILTIN_STR_P3"
 
@@ -263,15 +254,13 @@ class ExpressionBuiltinStrP3(ExpressionBuiltinUnicodeBase):
         return tshape_str_derived
 
 
-class ExpressionBuiltinBytes3(ExpressionBuiltinUnicodeBase):
+class ExpressionBuiltinBytes3(
+    ExpressionBytesShapeExactMixin, ExpressionBuiltinUnicodeBase
+):
     kind = "EXPRESSION_BUILTIN_BYTES3"
 
     builtin_spec = BuiltinParameterSpecs.builtin_bytes_p3_spec
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_bytes
-
 
 class ExpressionBuiltinBytes1(ExpressionChildHavingBase):
     kind = "EXPRESSION_BUILTIN_BYTES1"
@@ -295,7 +284,9 @@ class ExpressionBuiltinBytes1(ExpressionChildHavingBase):
         return self.subnode_value.mayRaiseExceptionBytes(exception_type)
 
 
-class ExpressionBuiltinBytearray1(ExpressionBuiltinTypeBase):
+class ExpressionBuiltinBytearray1(
+    ExpressionBytearrayShapeExactMixin, ExpressionBuiltinTypeBase
+):
     kind = "EXPRESSION_BUILTIN_BYTEARRAY1"
 
     builtin_spec = BuiltinParameterSpecs.builtin_bytearray_spec
@@ -303,12 +294,10 @@ class ExpressionBuiltinBytearray1(ExpressionBuiltinTypeBase):
     def __init__(self, value, source_ref):
         ExpressionBuiltinTypeBase.__init__(self, value=value, source_ref=source_ref)
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_bytearray
 
-
-class ExpressionBuiltinBytearray3(ExpressionChildrenHavingBase):
+class ExpressionBuiltinBytearray3(
+    ExpressionBytearrayShapeExactMixin, ExpressionChildrenHavingBase
+):
     kind = "EXPRESSION_BUILTIN_BYTEARRAY3"
 
     named_children = ("string", "encoding", "errors")
@@ -327,6 +316,64 @@ class ExpressionBuiltinBytearray3(ExpressionChildrenHavingBase):
 
         return self, None, None
 
-    @staticmethod
-    def getTypeShape():
-        return tshape_bytearray
+
+class ExpressionConstantGenericAlias(CompileTimeConstantExpressionBase):
+    kind = "EXPRESSION_CONSTANT_GENERIC_ALIAS"
+
+    __slots__ = ("generic_alias",)
+
+    def __init__(self, generic_alias, source_ref):
+        CompileTimeConstantExpressionBase.__init__(self, source_ref=source_ref)
+
+        self.generic_alias = generic_alias
+
+    def finalize(self):
+        del self.parent
+
+    def getDetails(self):
+        return {"generic_alias": self.generic_alias}
+
+    def getCompileTimeConstant(self):
+        return self.generic_alias
+
+    def getStrValue(self):
+        return makeConstantRefNode(
+            constant=str(self.getCompileTimeConstant()),
+            user_provided=True,
+            source_ref=self.source_ref,
+        )
+
+    def computeExpressionRaw(self, trace_collection):
+        # Nothing much to do.
+        return self, None, None
+
+
+class ExpressionConstantUnionType(CompileTimeConstantExpressionBase):
+    kind = "EXPRESSION_CONSTANT_UNION_TYPE"
+
+    __slots__ = ("union_type",)
+
+    def __init__(self, union_type, source_ref):
+        CompileTimeConstantExpressionBase.__init__(self, source_ref=source_ref)
+
+        self.union_type = union_type
+
+    def finalize(self):
+        del self.parent
+
+    def getDetails(self):
+        return {"union_type": self.union_type}
+
+    def getCompileTimeConstant(self):
+        return self.union_type
+
+    def getStrValue(self):
+        return makeConstantRefNode(
+            constant=str(self.getCompileTimeConstant()),
+            user_provided=True,
+            source_ref=self.source_ref,
+        )
+
+    def computeExpressionRaw(self, trace_collection):
+        # Nothing much to do.
+        return self, None, None
