@@ -21,14 +21,18 @@
 
 from .ExpressionBases import (
     ExpressionBase,
-    ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
+    ExpressionChildTupleHavingBase,
+    ExpressionNoSideEffectsMixin,
 )
 from .NodeBases import StatementBase, StatementChildrenHavingBase
 from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
 
 
 class StatementRaiseExceptionMixin(object):
+    # Mixins are required to also specify slots
+    __slots__ = ()
+
     @staticmethod
     def isStatementAborting():
         return True
@@ -50,6 +54,8 @@ class StatementRaiseException(
         "exception_cause",
     )
 
+    __slots__ = ("reraise_finally",)
+
     def __init__(
         self,
         exception_type,
@@ -228,6 +234,8 @@ class ExpressionRaiseException(ExpressionChildrenHavingBase):
         return exception_type is BaseException
 
     def computeExpression(self, trace_collection):
+        trace_collection.onExceptionRaiseExit(BaseException)
+
         return self, None, None
 
     def computeExpressionDrop(self, statement, trace_collection):
@@ -252,7 +260,7 @@ Propagated implicit raise expression to raise statement.""",
         )
 
 
-class ExpressionBuiltinMakeException(ExpressionChildHavingBase):
+class ExpressionBuiltinMakeException(ExpressionChildTupleHavingBase):
     kind = "EXPRESSION_BUILTIN_MAKE_EXCEPTION"
 
     named_child = "args"
@@ -260,7 +268,7 @@ class ExpressionBuiltinMakeException(ExpressionChildHavingBase):
     __slots__ = ("exception_name",)
 
     def __init__(self, exception_name, args, source_ref):
-        ExpressionChildHavingBase.__init__(
+        ExpressionChildTupleHavingBase.__init__(
             self, value=tuple(args), source_ref=source_ref
         )
 
@@ -284,7 +292,7 @@ class ExpressionBuiltinMakeException(ExpressionChildHavingBase):
 
 
 class ExpressionBuiltinMakeExceptionImportError(ExpressionChildrenHavingBase):
-    """ Python3 ImportError dedicated node with extra arguments. """
+    """Python3 ImportError dedicated node with extra arguments."""
 
     kind = "EXPRESSION_BUILTIN_MAKE_EXCEPTION_IMPORT_ERROR"
 
@@ -318,54 +326,42 @@ class ExpressionBuiltinMakeExceptionImportError(ExpressionChildrenHavingBase):
         return False
 
 
-class ExpressionCaughtExceptionTypeRef(ExpressionBase):
+class ExpressionCaughtMixin(ExpressionNoSideEffectsMixin):
+    """Common things for all caught exception references."""
+
+    __slots__ = ()
+
+    def finalize(self):
+        del self.parent
+
+
+class ExpressionCaughtExceptionTypeRef(ExpressionCaughtMixin, ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TYPE_REF"
 
     def __init__(self, source_ref):
         ExpressionBase.__init__(self, source_ref=source_ref)
 
-    def finalize(self):
-        del self.parent
-
     def computeExpressionRaw(self, trace_collection):
         # TODO: Might be predictable based on the exception handler this is in.
         return self, None, None
 
-    def mayHaveSideEffects(self):
-        # Referencing the expression type has no side effect
-        return False
 
-
-class ExpressionCaughtExceptionValueRef(ExpressionBase):
+class ExpressionCaughtExceptionValueRef(ExpressionCaughtMixin, ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_VALUE_REF"
 
     def __init__(self, source_ref):
         ExpressionBase.__init__(self, source_ref=source_ref)
 
-    def finalize(self):
-        del self.parent
-
     def computeExpressionRaw(self, trace_collection):
         # TODO: Might be predictable based on the exception handler this is in.
         return self, None, None
 
-    def mayHaveSideEffects(self):
-        # Referencing the expression type has no side effect
-        return False
 
-
-class ExpressionCaughtExceptionTracebackRef(ExpressionBase):
+class ExpressionCaughtExceptionTracebackRef(ExpressionCaughtMixin, ExpressionBase):
     kind = "EXPRESSION_CAUGHT_EXCEPTION_TRACEBACK_REF"
 
     def __init__(self, source_ref):
         ExpressionBase.__init__(self, source_ref=source_ref)
 
-    def finalize(self):
-        del self.parent
-
     def computeExpressionRaw(self, trace_collection):
         return self, None, None
-
-    def mayHaveSideEffects(self):
-        # Referencing the expression type has no side effect
-        return False
