@@ -24,13 +24,18 @@ whose implementation lives here. The creation itself also lives here.
 
 from nuitka.PythonVersions import python_version
 
-from .ExpressionBases import ExpressionChildHavingBase
+from .ExpressionBases import (
+    ExpressionChildHavingBase,
+    ExpressionNoSideEffectsMixin,
+)
 from .FunctionNodes import ExpressionFunctionEntryPointBase
 from .IndicatorMixins import MarkUnoptimizedFunctionIndicatorMixin
 from .ReturnNodes import StatementReturn, StatementReturnNone
 
 
-class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
+class ExpressionMakeGeneratorObject(
+    ExpressionNoSideEffectsMixin, ExpressionChildHavingBase
+):
     kind = "EXPRESSION_MAKE_GENERATOR_OBJECT"
 
     named_child = "generator_ref"
@@ -65,14 +70,6 @@ class ExpressionMakeGeneratorObject(ExpressionChildHavingBase):
         # TODO: Generator body may know something too.
         return self, None, None
 
-    @staticmethod
-    def mayRaiseException(exception_type):
-        return False
-
-    @staticmethod
-    def mayHaveSideEffects():
-        return False
-
     def getClosureVariableVersions(self):
         return self.variable_closure_traces
 
