@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -22,7 +22,6 @@ fallback can be optimized to no fallback variants.
 
 """
 
-from nuitka import Variables
 from nuitka.optimizations.TraceCollections import TraceCollectionBranch
 from nuitka.PythonVersions import python_version
 from nuitka.tree.TreeHelpers import makeStatementsSequence
@@ -30,28 +29,24 @@ from nuitka.tree.TreeHelpers import makeStatementsSequence
 from .AssignNodes import (
     StatementAssignmentVariable,
     StatementDelVariable,
-    StatementReleaseVariable
+    StatementReleaseVariable,
 )
 from .ConditionalNodes import ExpressionConditional
 from .ConstantRefNodes import ExpressionConstantDictEmptyRef
-from .ExpressionBases import ExpressionBase, ExpressionChildrenHavingBase
+from .ExpressionBases import ExpressionBase, ExpressionChildHavingBase
 from .NodeBases import StatementBase, StatementChildHavingBase
 from .VariableRefNodes import ExpressionTempVariableRef
 
 
-class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
+class ExpressionLocalsVariableRefOrFallback(ExpressionChildHavingBase):
     kind = "EXPRESSION_LOCALS_VARIABLE_REF_OR_FALLBACK"
 
-    named_children = ("fallback",)
+    named_child = "fallback"
+
+    __slots__ = ("locals_scope", "variable", "variable_trace")
 
     def __init__(self, locals_scope, variable_name, fallback, source_ref):
-        ExpressionChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "fallback" : fallback
-            },
-            source_ref = source_ref
-        )
+        ExpressionChildHavingBase.__init__(self, value=fallback, source_ref=source_ref)
 
         assert locals_scope is not None
 
@@ -62,8 +57,8 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
 
     def getDetails(self):
         return {
-            "locals_scope"  : self.locals_scope,
-            "variable_name" : self.getVariableName(),
+            "locals_scope": self.locals_scope,
+            "variable_name": self.getVariableName(),
         }
 
     def getVariableName(self):
@@ -74,7 +69,7 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
 
     def computeExpressionRaw(self, trace_collection):
         self.variable_trace = trace_collection.getVariableCurrentTrace(
-            variable = self.variable
+            variable=self.variable
         )
 
         replacement = self.variable_trace.getReplacementNode(self)
@@ -83,28 +78,39 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
             trace_collection.signalChange(
                 "new_expression",
                 self.source_ref,
-                "Value propagated for '%s' from '%s'." % (
+                "Value propagated for '%s' from '%s'."
+                % (
                     self.variable.getName(),
-                    replacement.getSourceReference().getAsString()
-                )
+                    replacement.getSourceReference().getAsString(),
+                ),
             )
 
             # Need to compute the replacement still.
             return replacement.computeExpressionRaw(trace_collection)
 
+        no_exec = not self.locals_scope.isUnoptimizedFunctionScope()
+
+        # if we can be sure if doesn't have a value set, go to the fallback directly.
+        if no_exec and self.variable_trace.mustNotHaveValue():
+            return trace_collection.computedExpressionResultRaw(
+                self.subnode_fallback,
+                "new_expression",
+                "Name '%s' cannot be in locals dict." % self.variable.getName(),
+            )
+
         # If we cannot be sure if the value is set, then we need the fallback,
         # otherwise we could remove it simply.
-        if self.variable_trace.mustHaveValue():
+        if no_exec and self.variable_trace.mustHaveValue():
             trace_collection.signalChange(
                 "new_expression",
                 self.source_ref,
-                "Name '%s' must be in locals dict." % self.variable.getName()
+                "Name '%s' must be in locals dict." % self.variable.getName(),
             )
 
             result = ExpressionLocalsVariableRef(
-                locals_scope  = self.locals_scope,
-                variable_name = self.variable.getName(),
-                source_ref    = self.source_ref
+                locals_scope=self.locals_scope,
+                variable_name=self.variable.getName(),
+                source_ref=self.source_ref,
             )
 
             # Need to compute the replacement still.
@@ -113,80 +119,83 @@ class ExpressionLocalsVariableRefORFallback(ExpressionChildrenHavingBase):
             trace_collection.onExceptionRaiseExit(BaseException)
 
             branch_fallback = TraceCollectionBranch(
-                parent = trace_collection,
-                name   = "fallback node usage"
+                parent=trace_collection, name="fallback node usage"
             )
 
             branch_fallback.computeBranch(self.subnode_fallback)
 
-            trace_collection.mergeBranches(
-                branch_fallback,
-                None
-            )
+            trace_collection.mergeBranches(branch_fallback, None)
 
-        # if we can be sure if doesn't have a value set, go to the fallback directly.
-        if self.variable_trace.mustNotHaveValue():
-            return (
-                self.subnode_fallback,
-                "new_expression",
-                "Name '%s' cannot be in locals dict." % self.variable.getName()
-            )
-        else:
             return self, None, None
 
-    def computeExpressionCall(self, call_node, call_args, call_kw,
-                              trace_collection):
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
         trace_collection.onControlFlowEscape(self)
 
-        if not Variables.complete and \
-           self.variable.getName() in ("dir", "eval", "exec", "execfile", "locals", "vars") and \
-           self.subnode_fallback.isExpressionVariableRef() and \
-           self.subnode_fallback.getVariable().isModuleVariable():
+        if (
+            self.variable.getName()
+            in ("dir", "eval", "exec", "execfile", "locals", "vars")
+            and self.subnode_fallback.isExpressionVariableRef()
+            and self.subnode_fallback.getVariable().isIncompleteModuleVariable()
+        ):
             # Just inform the collection that all escaped.
-            trace_collection.onLocalsUsage(self.getParentVariableProvider())
+            trace_collection.onLocalsUsage(self.getLocalsDictScope())
 
-        if self.subnode_fallback.isExpressionBuiltinRef() or \
-           self.subnode_fallback.isExpressionConstantTypeRef():
+        if (
+            self.subnode_fallback.isExpressionBuiltinRef()
+            or self.subnode_fallback.isExpressionConstantTypeRef()
+        ):
             variable_name = self.variable.getName()
 
             # Create a cloned node with the locals variable.
             call_node_clone = call_node.makeClone()
-            call_node_clone.setCalled(
+            call_node_clone.setChild(
+                "called",
                 ExpressionLocalsVariableRef(
-                    locals_scope  = self.locals_scope,
-                    variable_name = variable_name,
-                    source_ref    = self.source_ref
-                )
+                    locals_scope=self.locals_scope,
+                    variable_name=variable_name,
+                    source_ref=self.source_ref,
+                ),
             )
 
             # Make the original one for the fallback
             call_node = call_node.makeCloneShallow()
-            call_node.setCalled(self.subnode_fallback)
+            call_node.setChild("called", self.subnode_fallback)
 
             result = ExpressionConditional(
-                condition      = ExpressionLocalsVariableCheck(
-                    locals_scope  = self.locals_scope,
-                    variable_name = variable_name,
-                    source_ref    = self.source_ref
-
+                condition=ExpressionLocalsVariableCheck(
+                    locals_scope=self.locals_scope,
+                    variable_name=variable_name,
+                    source_ref=self.source_ref,
                 ),
-                expression_yes = call_node_clone,
-                expression_no  = call_node,
-                source_ref     = self.source_ref
+                expression_yes=call_node_clone,
+                expression_no=call_node,
+                source_ref=self.source_ref,
             )
 
-            return result, "new_expression", "Moved call of uncertain dict variable to inside."
+            return (
+                result,
+                "new_expression",
+                "Moved call of uncertain dict variable '%s' to inside." % variable_name,
+            )
 
         return call_node, None, None
 
+    # TODO: Specialize for Python3 maybe to save attribute for Python2.
+    may_raise_access = python_version >= 0x300
+
     def mayRaiseException(self, exception_type):
-        return python_version >= 300 or \
-               self.subnode_fallback.mayRaiseException(exception_type)
+        if self.may_raise_access and self.locals_scope.hasShapeDictionaryExact():
+            return True
+
+        return self.subnode_fallback.mayRaiseException(exception_type)
 
 
-class ExpressionLocalsMappingVariableRefORFallback(ExpressionLocalsVariableRefORFallback):
+# TODO: Why is this unused.
+class ExpressionLocalsMappingVariableRefOrFallback(
+    ExpressionLocalsVariableRefOrFallback
+):
     kind = "EXPRESSION_LOCALS_MAPPING_VARIABLE_REF_OR_FALLBACK"
 
 
@@ -197,10 +206,7 @@ class ExpressionLocalsVariableRef(ExpressionBase):
 
     def __init__(self, locals_scope, variable_name, source_ref):
 
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
         self.locals_scope = locals_scope
         self.variable = locals_scope.getLocalsDictVariable(variable_name)
@@ -214,14 +220,12 @@ class ExpressionLocalsVariableRef(ExpressionBase):
 
     def getDetails(self):
         return {
-            "variable_name" : self.getVariableName(),
-            "locals_scope"  : self.locals_scope
+            "variable_name": self.getVariableName(),
+            "locals_scope": self.locals_scope,
         }
 
     def getDetailsForDisplay(self):
-        return {
-            "variable_name" : self.getVariableName(),
-        }
+        return {"variable_name": self.getVariableName()}
 
     def getVariableName(self):
         return self.variable.getName()
@@ -234,13 +238,11 @@ class ExpressionLocalsVariableRef(ExpressionBase):
             variable_name = self.getVariableName()
 
             variable = self.locals_scope.allocateTempReplacementVariable(
-                trace_collection = trace_collection,
-                variable_name    = variable_name
+                trace_collection=trace_collection, variable_name=variable_name
             )
 
             result = ExpressionTempVariableRef(
-                variable   = variable,
-                source_ref = self.source_ref
+                variable=variable, source_ref=self.source_ref
             )
             result.parent = self.parent
 
@@ -251,11 +253,14 @@ class ExpressionLocalsVariableRef(ExpressionBase):
             if new_result[0] is not result:
                 assert False, (new_result, result)
 
-            return result, "new_expression", "Replaced dictionary ref with temporary variable."
-
+            return (
+                result,
+                "new_expression",
+                "Replaced dictionary ref with temporary variable.",
+            )
 
         self.variable_trace = trace_collection.getVariableCurrentTrace(
-            variable = self.variable
+            variable=self.variable
         )
 
         replacement = self.variable_trace.getReplacementNode(self)
@@ -264,10 +269,11 @@ class ExpressionLocalsVariableRef(ExpressionBase):
             trace_collection.signalChange(
                 "new_expression",
                 self.source_ref,
-                "Value propagated for '%s' from '%s'." % (
+                "Value propagated for '%s' from '%s'."
+                % (
                     self.variable.getName(),
-                    replacement.getSourceReference().getAsString()
-                )
+                    replacement.getSourceReference().getAsString(),
+                ),
             )
 
             # Need to compute the replacement still.
@@ -278,17 +284,14 @@ class ExpressionLocalsVariableRef(ExpressionBase):
 
         return self, None, None
 
-
-    def computeExpressionCall(self, call_node, call_args, call_kw,
-                              trace_collection):
+    def computeExpressionCall(self, call_node, call_args, call_kw, trace_collection):
         trace_collection.onExceptionRaiseExit(BaseException)
 
         trace_collection.onControlFlowEscape(self)
         return call_node, None, None
 
     def mayRaiseException(self, exception_type):
-        return self.variable_trace is None or \
-               not self.variable_trace.mustHaveValue()
+        return self.variable_trace is None or not self.variable_trace.mustHaveValue()
 
 
 class ExpressionLocalsVariableCheck(ExpressionBase):
@@ -299,10 +302,7 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
     def __init__(self, locals_scope, variable_name, source_ref):
         self.variable_name = variable_name
 
-        ExpressionBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        ExpressionBase.__init__(self, source_ref=source_ref)
 
         self.locals_scope = locals_scope
 
@@ -311,10 +311,7 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
         del self.locals_scope
 
     def getDetails(self):
-        return {
-            "locals_scope"  : self.locals_scope,
-            "variable_name" : self.variable_name,
-        }
+        return {"locals_scope": self.locals_scope, "variable_name": self.variable_name}
 
     def getVariableName(self):
         return self.variable_name
@@ -330,28 +327,22 @@ class ExpressionLocalsVariableCheck(ExpressionBase):
 class StatementLocalsDictOperationSet(StatementChildHavingBase):
     kind = "STATEMENT_LOCALS_DICT_OPERATION_SET"
 
-    named_child = "value"
+    named_child = "source"
 
     __slots__ = ("variable", "variable_version", "variable_trace", "locals_scope")
 
     # TODO: Specialize for Python3 maybe to save attribute for Python2.
-    may_raise_set = python_version >= 300
+    may_raise_set = python_version >= 0x300
 
     def __init__(self, locals_scope, variable_name, value, source_ref):
         assert type(variable_name) is str
         assert value is not None
 
-        StatementChildHavingBase.__init__(
-            self,
-            value      = value,
-            source_ref = source_ref
-        )
+        StatementChildHavingBase.__init__(self, value=value, source_ref=source_ref)
 
         assert locals_scope is not None
 
-        self.variable = locals_scope.getLocalsDictVariable(
-            variable_name = variable_name
-        )
+        self.variable = locals_scope.getLocalsDictVariable(variable_name=variable_name)
 
         self.variable_version = self.variable.allocateTargetNumber()
         self.variable_trace = None
@@ -365,8 +356,8 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
 
     def getDetails(self):
         return {
-            "locals_scope"  : self.locals_scope,
-            "variable_name" : self.getVariableName()
+            "locals_scope": self.locals_scope,
+            "variable_name": self.getVariableName(),
         }
 
     def getVariableName(self):
@@ -375,22 +366,21 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
     def getLocalsDictScope(self):
         return self.locals_scope
 
-    # TODO: Child name inconsistent with accessor name.
-    getAssignSource = StatementChildHavingBase.childGetter("value")
+    def getTypeShape(self):
+        return self.locals_scope.getMappingValueShape(self.variable)
 
     def computeStatement(self, trace_collection):
         if self.locals_scope.isMarkedForPropagation():
             variable_name = self.getVariableName()
 
             variable = self.locals_scope.allocateTempReplacementVariable(
-                trace_collection = trace_collection,
-                variable_name    = variable_name
+                trace_collection=trace_collection, variable_name=variable_name
             )
 
             result = StatementAssignmentVariable(
-                source     = self.subnode_value,
-                variable   = variable,
-                source_ref = self.source_ref
+                source=self.subnode_source,
+                variable=variable,
+                source_ref=self.source_ref,
             )
             result.parent = self.parent
 
@@ -400,19 +390,21 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
                 assert False, (new_result, result)
 
             self.finalize()
-            return result, "new_statements", "Replaced dictionary assignment with temporary variable."
+            return (
+                result,
+                "new_statements",
+                "Replaced dictionary assignment with temporary variable.",
+            )
 
         result, change_tags, change_desc = self.computeStatementSubExpressions(
-            trace_collection = trace_collection
+            trace_collection=trace_collection
         )
 
         if result is not self:
             return result, change_tags, change_desc
 
         self.variable_trace = trace_collection.onVariableSet(
-            variable    = self.variable,
-            version     = self.variable_version,
-            assign_node = self
+            variable=self.variable, version=self.variable_version, assign_node=self
         )
 
         if self.may_raise_set:
@@ -421,27 +413,33 @@ class StatementLocalsDictOperationSet(StatementChildHavingBase):
         return self, None, None
 
     def mayRaiseException(self, exception_type):
-        return self.may_raise_set or self.subnode_value.mayRaiseException(exception_type)
+        return self.may_raise_set or self.subnode_source.mayRaiseException(
+            exception_type
+        )
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "locals dictionary value set statement"
 
 
 class StatementLocalsDictOperationDel(StatementBase):
     kind = "STATEMENT_LOCALS_DICT_OPERATION_DEL"
 
-    __slots__ = "variable", "variable_version", "previous_trace", "locals_scope"
+    __slots__ = (
+        "variable",
+        "variable_version",
+        "previous_trace",
+        "locals_scope",
+        "tolerant",
+    )
 
     # TODO: Specialize for Python3 maybe to save attribute for Python2.
-    may_raise_del = python_version >= 300
+    may_raise_del = python_version >= 0x300
 
-    def __init__(self, locals_scope, variable_name, source_ref):
+    def __init__(self, locals_scope, variable_name, tolerant, source_ref):
         assert type(variable_name) is str
 
-        StatementBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         assert locals_scope is not None
 
@@ -450,6 +448,8 @@ class StatementLocalsDictOperationDel(StatementBase):
         self.variable = locals_scope.getLocalsDictVariable(variable_name)
         self.variable_version = self.variable.allocateTargetNumber()
 
+        self.tolerant = tolerant
+
         self.previous_trace = None
 
     def finalize(self):
@@ -460,8 +460,8 @@ class StatementLocalsDictOperationDel(StatementBase):
 
     def getDetails(self):
         return {
-            "variable_name" : self.getVariableName(),
-            "locals_scope"  : self.locals_scope
+            "variable_name": self.getVariableName(),
+            "locals_scope": self.locals_scope,
         }
 
     def getVariableName(self):
@@ -471,18 +471,16 @@ class StatementLocalsDictOperationDel(StatementBase):
         return self.locals_scope
 
     def computeStatement(self, trace_collection):
+        # Conversion from dictionary to normal nodes is done here.
         if self.locals_scope.isMarkedForPropagation():
             variable_name = self.getVariableName()
 
             variable = self.locals_scope.allocateTempReplacementVariable(
-                trace_collection = trace_collection,
-                variable_name    = variable_name
+                trace_collection=trace_collection, variable_name=variable_name
             )
 
             result = StatementDelVariable(
-                variable   = variable,
-                tolerant   = False,
-                source_ref = self.source_ref
+                variable=variable, tolerant=False, source_ref=self.source_ref
             )
             result.parent = self.parent
 
@@ -491,35 +489,38 @@ class StatementLocalsDictOperationDel(StatementBase):
             if new_result[0] is not result:
                 assert False, (new_result, result)
 
-            return result, "new_statements", "Replaced dictionary del with temporary variable."
+            return (
+                result,
+                "new_statements",
+                "Replaced dictionary del with temporary variable.",
+            )
 
         self.previous_trace = trace_collection.getVariableCurrentTrace(self.variable)
 
-        # The "del" is a potential use of a value. TODO: This could be made more
-        # beautiful indication, as it's not any kind of usage.
-        self.previous_trace.addPotentialUsage()
+        # Deleting is usage of the value, and may call code on it. This is to inhibit
+        # just removing it.
+        self.previous_trace.addUsage()
 
         # We may not exception exit now during the __del__ unless there is no value.
+        # TODO: In which case, there is doing to be a NameError or UnboundLocalError.
         if not self.previous_trace.mustHaveValue():
             trace_collection.onExceptionRaiseExit(BaseException)
 
         # Record the deletion, needs to start a new version then.
         _variable_trace = trace_collection.onVariableDel(
-            variable = self.variable,
-            version  = self.variable_version
+            variable=self.variable, version=self.variable_version, del_node=self
         )
 
-        trace_collection.onVariableContentEscapes(self.variable)
-
         # Any code could be run, note that.
         trace_collection.onControlFlowEscape(self)
 
         return self, None, None
 
     def mayRaiseException(self, exception_type):
-        return self.may_raise_del
+        return self.may_raise_del and not self.tolerant
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "locals dictionary value del statement"
 
 
@@ -531,11 +532,7 @@ class StatementSetLocals(StatementChildHavingBase):
     __slots__ = ("locals_scope",)
 
     def __init__(self, locals_scope, new_locals, source_ref):
-        StatementChildHavingBase.__init__(
-            self,
-            value      = new_locals,
-            source_ref = source_ref
-        )
+        StatementChildHavingBase.__init__(self, value=new_locals, source_ref=source_ref)
 
         self.locals_scope = locals_scope
 
@@ -545,49 +542,50 @@ class StatementSetLocals(StatementChildHavingBase):
         del self.subnode_new_locals
 
     def getDetailsForDisplay(self):
-        return {
-            "locals_scope" : self.locals_scope.getCodeName()
-        }
+        return {"locals_scope": self.locals_scope.getCodeName()}
 
     def getDetails(self):
-        return {
-            "locals_scope" : self.locals_scope
-        }
+        return {"locals_scope": self.locals_scope}
 
     def getLocalsScope(self):
         return self.locals_scope
 
     def mayRaiseException(self, exception_type):
-        return self.getNewLocals().mayRaiseException(exception_type)
-
-    getNewLocals = StatementChildHavingBase.childGetter("new_locals")
-    getAssignSource = getNewLocals
+        return self.subnode_new_locals.mayRaiseException(exception_type)
 
     def computeStatement(self, trace_collection):
-        trace_collection.onExpression(self.getNewLocals())
-        new_locals = self.getNewLocals()
+        new_locals = trace_collection.onExpression(self.subnode_new_locals)
 
         if new_locals.willRaiseException(BaseException):
-            from .NodeMakingHelpers import \
-               makeStatementExpressionOnlyReplacementNode
-
-            result = makeStatementExpressionOnlyReplacementNode(
-                expression = new_locals,
-                node       = self
+            from .NodeMakingHelpers import (
+                makeStatementExpressionOnlyReplacementNode,
             )
 
-            return result, "new_raise", """\
-Setting locals already raises implicitly building new locals."""
+            result = makeStatementExpressionOnlyReplacementNode(
+                expression=new_locals, node=self
+            )
+
+            return (
+                result,
+                "new_raise",
+                """\
+Setting locals already raises implicitly building new locals.""",
+            )
 
         if self.locals_scope.isMarkedForPropagation():
             self.finalize()
 
-            return None, "new_statements", """\
-Forward propagating locals."""
+            return (
+                None,
+                "new_statements",
+                """\
+Forward propagating locals.""",
+            )
 
         return self, None, None
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "locals mapping init statement"
 
 
@@ -597,17 +595,19 @@ class StatementSetLocalsDictionary(StatementSetLocals):
     def __init__(self, locals_scope, source_ref):
         StatementSetLocals.__init__(
             self,
-            locals_scope = locals_scope,
-            new_locals   = ExpressionConstantDictEmptyRef(
-                source_ref = source_ref
+            locals_scope=locals_scope,
+            new_locals=ExpressionConstantDictEmptyRef(
+                source_ref=source_ref, user_provided=True
             ),
-            source_ref   = source_ref
+            source_ref=source_ref,
         )
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "locals dictionary init statement"
 
 
@@ -617,10 +617,7 @@ class StatementReleaseLocals(StatementBase):
     __slots__ = ("locals_scope",)
 
     def __init__(self, locals_scope, source_ref):
-        StatementBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         self.locals_scope = locals_scope
 
@@ -631,36 +628,34 @@ class StatementReleaseLocals(StatementBase):
     def computeStatement(self, trace_collection):
         if self.locals_scope.isMarkedForPropagation():
             statements = [
-                StatementReleaseVariable(
-                    variable   = variable,
-                    source_ref = self.source_ref
-                )
-                for variable
-                in self.locals_scope.getPropagationVariables().values()
+                StatementReleaseVariable(variable=variable, source_ref=self.source_ref)
+                for variable in self.locals_scope.getPropagationVariables().values()
             ]
 
             result = makeStatementsSequence(
-                statements = statements,
-                allow_none = False,
-                source_ref = self.source_ref
+                statements=statements, allow_none=False, source_ref=self.source_ref
             )
 
             self.finalize()
 
-            return result, "new_statements", "Releasing temp variables instead of locals dict."
+            return (
+                result,
+                "new_statements",
+                "Releasing temp variables instead of locals dict.",
+            )
 
         return self, None, None
 
     def getDetails(self):
-        return {
-            "locals_scope": self.locals_scope
-        }
+        return {"locals_scope": self.locals_scope}
 
     def getLocalsScope(self):
         return self.locals_scope
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
-    def getStatementNiceName(self):
+    @staticmethod
+    def getStatementNiceName():
         return "locals dictionary release statement"
