@@ -19,7 +19,11 @@
 
 from nuitka import Variables
 from nuitka.containers.odict import OrderedDict
-from nuitka.utils.InstanceCounters import counted_del, counted_init
+from nuitka.utils.InstanceCounters import (
+    counted_del,
+    counted_init,
+    isCountingInstances,
+)
 
 from .shapes.BuiltinTypeShapes import tshape_dict
 from .shapes.StandardShapes import tshape_unknown
@@ -54,11 +58,9 @@ def getLocalsDictHandle(locals_name, kind, owner):
     return locals_dict_handles[locals_name]
 
 
-def getLocalsDictHandles():
-    return locals_dict_handles
-
-
 class LocalsDictHandleBase(object):
+    # TODO: Might remove some of these later, pylint: disable=too-many-instance-attributes
+
     __slots__ = (
         "locals_name",
         # TODO: Specialize what the kinds really use.
@@ -68,6 +70,7 @@ class LocalsDictHandleBase(object):
         "mark_for_propagation",
         "propagation",
         "owner",
+        "complete",
     )
 
     @counted_init
@@ -87,7 +90,10 @@ class LocalsDictHandleBase(object):
 
         self.propagation = None
 
-    __del__ = counted_del()
+        self.complete = False
+
+    if isCountingInstances():
+        __del__ = counted_del()
 
     def __repr__(self):
         return "<%s of %s>" % (self.__class__.__name__, self.locals_name)
@@ -141,6 +147,10 @@ class LocalsDictHandleBase(object):
     def getTypeShape():
         return tshape_dict
 
+    @staticmethod
+    def hasShapeDictionaryExact():
+        return True
+
     def getCodeName(self):
         return self.locals_name
 
@@ -156,6 +166,10 @@ class LocalsDictHandleBase(object):
     def isFunctionScope():
         return False
 
+    @staticmethod
+    def isUnoptimizedFunctionScope():
+        return False
+
     def getProvidedVariables(self):
         return self.providing.values()
 
@@ -165,7 +179,7 @@ class LocalsDictHandleBase(object):
         self.providing[variable_name] = variable
 
     def unregisterProvidedVariable(self, variable):
-        """ Remove provided variable, e.g. because it became unused. """
+        """Remove provided variable, e.g. because it became unused."""
 
         variable_name = variable.getName()
 
@@ -176,17 +190,17 @@ class LocalsDictHandleBase(object):
     unregisterClosureVariable = unregisterProvidedVariable
 
     def hasProvidedVariable(self, variable_name):
-        """ Test if a variable is provided. """
+        """Test if a variable is provided."""
 
         return variable_name in self.providing
 
     def getProvidedVariable(self, variable_name):
-        """ Test if a variable is provided. """
+        """Test if a variable is provided."""
 
         return self.providing[variable_name]
 
     def getLocalsRelevantVariables(self):
-        """ The variables relevant to locals. """
+        """The variables relevant to locals."""
 
         return self.providing.values()
 
@@ -251,9 +265,47 @@ class LocalsDictHandleBase(object):
         del self.variables
         del self.providing
 
+    def markAsComplete(self, trace_collection):
+        self.complete = True
+
+        self._considerUnusedUserLocalVariables(trace_collection)
+        self._considerPropagation(trace_collection)
+
+    # TODO: Limited to Python2 classes for now, more overloads need to be added, this
+    # ought to be abstract and have variants with TODOs for each of them.
+    @staticmethod
+    def _considerPropagation(trace_collection):
+        """For overload by scope type. Check if this can be replaced."""
+
+    def _considerUnusedUserLocalVariables(self, trace_collection):
+        """Check scope for unused variables."""
+
+        provided = self.getProvidedVariables()
+        removals = []
+
+        for variable in provided:
+            if (
+                variable.isLocalVariable()
+                and not variable.isParameterVariable()
+                and variable.getOwner() is self.owner
+            ):
+                empty = trace_collection.hasEmptyTraces(variable)
+
+                if empty:
+                    removals.append(variable)
+
+        for variable in removals:
+            self.unregisterProvidedVariable(variable)
+
+            trace_collection.signalChange(
+                "var_usage",
+                self.owner.getSourceReference(),
+                message="Remove unused local variable '%s'." % variable.getName(),
+            )
+
 
 class LocalsDictHandle(LocalsDictHandleBase):
-    """ Locals dict for a Python class with mere dict. """
+    """Locals dict for a Python class with mere dict."""
 
     __slots__ = ()
 
@@ -267,6 +319,51 @@ class LocalsDictHandle(LocalsDictHandleBase):
         # pylint: disable=unused-argument
         return tshape_unknown
 
+    def _considerPropagation(self, trace_collection):
+        self.complete = True
+
+        propagate = True
+
+        for variable in self.variables.values():
+            for variable_trace in variable.traces:
+                if variable_trace.isAssignTrace():
+                    # For assign traces we want the value to not have a side effect,
+                    # then we can push it down the line. TODO: Once temporary
+                    # variables and dictionary building allows for unset values
+                    # remove this
+                    if (
+                        variable_trace.getAssignNode().subnode_source.mayHaveSideEffects()
+                    ):
+                        propagate = False
+                        break
+                elif variable_trace.isDeletedTrace():
+                    propagate = False
+                    break
+                elif variable_trace.isMergeTrace():
+                    propagate = False
+                    break
+                elif variable_trace.isUninitTrace():
+                    pass
+                elif variable_trace.isUnknownTrace():
+                    propagate = False
+                    break
+                elif variable_trace.isEscapeTrace():
+                    propagate = False
+                    break
+                else:
+                    assert False, (variable, variable_trace)
+
+        if propagate:
+            trace_collection.signalChange(
+                "var_usage",
+                self.owner.getSourceReference(),
+                message="Forward propagage locals dictionary.",
+            )
+
+            self.markForLocalsDictPropagation()
+
+        return propagate
+
 
 class LocalsMappingHandle(LocalsDictHandle):
     """Locals dict of a Python3 class with a mapping."""
@@ -278,6 +375,11 @@ class LocalsMappingHandle(LocalsDictHandle):
         # TODO: Make mapping available for this.
         return tshape_unknown
 
+    @staticmethod
+    def hasShapeDictionaryExact():
+        # TODO: Keep in sync with getTypeShape being calculated eventually.
+        return False
+
     @staticmethod
     def isClassScope():
         return True
@@ -339,10 +441,6 @@ class LocalsDictFunctionHandle(LocalsDictHandleBase):
     def isFunctionScope():
         return True
 
-    @staticmethod
-    def isUnoptimizedFunctionScope():
-        return False
-
 
 class GlobalsDictHandle(LocalsDictHandleBase):
     __slots__ = ("escaped",)
