@@ -32,7 +32,11 @@ from nuitka.__past__ import iterItems
 from nuitka.Errors import NuitkaNodeDesignError, NuitkaNodeError
 from nuitka.PythonVersions import python_version
 from nuitka.SourceCodeReferences import SourceCodeReference
-from nuitka.utils.InstanceCounters import counted_del, counted_init
+from nuitka.utils.InstanceCounters import (
+    counted_del,
+    counted_init,
+    isCountingInstances,
+)
 
 from .FutureSpecs import fromFlags
 from .NodeMakingHelpers import makeStatementOnlyNodesFromExpressions
@@ -42,6 +46,9 @@ from .NodeMetaClasses import NodeCheckMetaClass, NodeMetaClassBase
 class NodeBase(NodeMetaClassBase):
     __slots__ = "parent", "source_ref"
 
+    # This can trigger if this is included to early.
+    assert Options.is_fullcompat is not None
+
     # Avoid the attribute unless it's really necessary.
     if Options.is_fullcompat:
         __slots__ += ("effective_source_ref",)
@@ -62,7 +69,8 @@ class NodeBase(NodeMetaClassBase):
 
         self.source_ref = source_ref
 
-    __del__ = counted_del()
+    if isCountingInstances():
+        __del__ = counted_del()
 
     @abstractmethod
     def finalize(self):
@@ -136,10 +144,9 @@ class NodeBase(NodeMetaClassBase):
         return result
 
     def getParent(self):
-        """Parent of the node. Every node except modules have to have a parent."""
+        """Parent of the node. Every node except modules has to have a parent."""
 
         if self.parent is None and not self.isCompiledPythonModule():
-            # print self.getVisitableNodesNamed()
             assert False, (self, self.source_ref)
 
         return self.parent
@@ -269,7 +276,7 @@ class NodeBase(NodeMetaClassBase):
     def asXml(self):
         line = self.source_ref.getLineNumber()
 
-        result = TreeXML.Element("node", kind=self.__class__.__name__, line="%s" % line)
+        result = TreeXML.Element("node", kind=self.__class__.__name__, line=str(line))
 
         compat_line = self.getCompatibleSourceReference().getLineNumber()
 
@@ -337,6 +344,14 @@ class NodeBase(NodeMetaClassBase):
     def isExpressionConstantRef():
         return False
 
+    @staticmethod
+    def isExpressionConstantBoolRef():
+        return False
+
+    @staticmethod
+    def isExpressionOperationUnary():
+        return False
+
     @staticmethod
     def isExpressionOperationBinary():
         return False
@@ -345,6 +360,10 @@ class NodeBase(NodeMetaClassBase):
     def isExpressionOperationInplace():
         return False
 
+    @staticmethod
+    def isExpressionComparison():
+        return False
+
     @staticmethod
     def isExpressionSideEffects():
         return False
@@ -391,13 +410,13 @@ class NodeBase(NodeMetaClassBase):
 
     @staticmethod
     def getName():
-        """ Name of the node if any. """
+        """Name of the node if any."""
 
         return None
 
     @staticmethod
     def mayHaveSideEffects():
-        """ Unless we are told otherwise, everything may have a side effect. """
+        """Unless we are told otherwise, everything may have a side effect."""
 
         return True
 
@@ -405,20 +424,20 @@ class NodeBase(NodeMetaClassBase):
         return self.mayHaveSideEffects()
 
     def extractSideEffects(self):
-        """ Unless defined otherwise, the expression is the side effect. """
+        """Unless defined otherwise, the expression is the side effect."""
 
         return (self,)
 
     @staticmethod
     def mayRaiseException(exception_type):
-        """ Unless we are told otherwise, everything may raise everything. """
+        """Unless we are told otherwise, everything may raise everything."""
         # Virtual method, pylint: disable=unused-argument
 
         return True
 
     @staticmethod
     def mayReturn():
-        """ May this node do a return exit, to be overloaded for things that might. """
+        """May this node do a return exit, to be overloaded for things that might."""
         return False
 
     @staticmethod
@@ -430,19 +449,19 @@ class NodeBase(NodeMetaClassBase):
         return False
 
     def needsFrame(self):
-        """ Unless we are tolder otherwise, this depends on exception raise. """
+        """Unless we are tolder otherwise, this depends on exception raise."""
 
         return self.mayRaiseException(BaseException)
 
     @staticmethod
     def willRaiseException(exception_type):
-        """ Unless we are told otherwise, nothing may raise anything. """
+        """Unless we are told otherwise, nothing may raise anything."""
         # Virtual method, pylint: disable=unused-argument
         return False
 
     @staticmethod
     def isStatementAborting():
-        """ Is the node aborting, control flow doesn't continue after this node.  """
+        """Is the node aborting, control flow doesn't continue after this node."""
         return False
 
 
@@ -688,7 +707,7 @@ class ChildrenHavingMixin(object):
 
 
 class ClosureGiverNodeMixin(CodeNodeMixin):
-    """Base class for nodes that provide variables for closure takers. """
+    """Base class for nodes that provide variables for closure takers."""
 
     # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
@@ -750,7 +769,7 @@ class ClosureGiverNodeMixin(CodeNodeMixin):
         elif temp_type == "bool":
             temp_class = Variables.TempVariableBool
         else:
-            assert False, temp_class
+            assert False, temp_type
 
         result = temp_class(owner=self, variable_name=temp_name)
 
@@ -769,9 +788,21 @@ class ClosureGiverNodeMixin(CodeNodeMixin):
     def getTempVariables(self):
         return self.temp_variables.values()
 
-    def removeTempVariable(self, variable):
+    def _removeTempVariable(self, variable):
         del self.temp_variables[variable.getName()]
 
+    def optimizeUnusedTempVariables(self):
+        remove = []
+
+        for temp_variable in self.getTempVariables():
+            empty = self.trace_collection.hasEmptyTraces(variable=temp_variable)
+
+            if empty:
+                remove.append(temp_variable)
+
+        for temp_variable in remove:
+            self._removeTempVariable(temp_variable)
+
     def allocatePreserverId(self):
         if python_version >= 0x300:
             self.preserver_id += 1
@@ -780,7 +811,7 @@ class ClosureGiverNodeMixin(CodeNodeMixin):
 
 
 class ClosureTakerMixin(object):
-    """ Mixin for nodes that accept variables from closure givers. """
+    """Mixin for nodes that accept variables from closure givers."""
 
     # Mixins are not allow to specify slots, pylint: disable=assigning-non-slot
     __slots__ = ()
@@ -896,8 +927,6 @@ class StatementChildHavingBase(StatementBase):
     def __init__(self, value, source_ref):
         StatementBase.__init__(self, source_ref=source_ref)
 
-        assert type(self.named_child) is str and self.named_child
-
         if self.checker is not None:
             value = self.checker(value)  # False alarm, pylint: disable=not-callable
 
@@ -1059,7 +1088,6 @@ class SideEffectsFromChildrenMixin(object):
 
     def extractSideEffects(self):
         # No side effects at all but from the children.
-
         result = []
 
         for child in self.getVisitableNodes():
