@@ -1,4 +1,4 @@
-#     Copyright 2020, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -37,9 +37,6 @@ class StatementPrintValue(StatementChildrenHavingBase):
     kind = "STATEMENT_PRINT_VALUE"
 
     named_children = ("dest", "value")
-    getDestination = StatementChildrenHavingBase.childGetter("dest")
-    getValue = StatementChildrenHavingBase.childGetter("value")
-    setValue = StatementChildrenHavingBase.childSetter("value")
 
     def __init__(self, dest, value, source_ref):
         StatementChildrenHavingBase.__init__(
@@ -50,7 +47,7 @@ class StatementPrintValue(StatementChildrenHavingBase):
 
     def computeStatement(self, trace_collection):
         dest = trace_collection.onExpression(
-            expression=self.getDestination(), allow_none=True
+            expression=self.subnode_dest, allow_none=True
         )
 
         if dest is not None and dest.mayRaiseException(BaseException):
@@ -68,7 +65,7 @@ class StatementPrintValue(StatementChildrenHavingBase):
 Exception raise in 'print' statement destination converted to explicit raise.""",
             )
 
-        value = trace_collection.onExpression(expression=self.getValue())
+        value = trace_collection.onExpression(expression=self.subnode_value)
 
         if value.mayRaiseException(BaseException):
             trace_collection.onExceptionRaiseExit(BaseException)
@@ -97,11 +94,11 @@ Exception raise in 'print' statement arguments converted to explicit raise.""",
 
         if dest is None:
             if value.isExpressionSideEffects():
-                self.setValue(value.getExpression())
+                self.setChild("value", value.subnode_expression)
 
                 statements = [
                     makeStatementExpressionOnlyReplacementNode(side_effect, self)
-                    for side_effect in value.getSideEffects()
+                    for side_effect in value.subnode_side_effects
                 ]
 
                 statements.append(self)
@@ -118,16 +115,19 @@ Side effects printed item promoted to statements.""",
                 )
 
         if value.isCompileTimeConstant():
-            if not (value.isExpressionConstantRef() and value.isUnicodeConstant()):
+            # Avoid unicode encoding issues.
+            if not value.isExpressionConstantUnicodeRef():
                 new_value = value.getStrValue()
                 assert new_value is not None, value
 
                 if value is not new_value:
-                    self.setValue(new_value)
+                    self.setChild("value", new_value)
 
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
+        # Output may always fail due to external reasons.
         return True
 
 
@@ -135,14 +135,13 @@ class StatementPrintNewline(StatementChildHavingBase):
     kind = "STATEMENT_PRINT_NEWLINE"
 
     named_child = "dest"
-    getDestination = StatementChildHavingBase.childGetter("dest")
 
     def __init__(self, dest, source_ref):
         StatementChildHavingBase.__init__(self, value=dest, source_ref=source_ref)
 
     def computeStatement(self, trace_collection):
         dest = trace_collection.onExpression(
-            expression=self.getDestination(), allow_none=True
+            expression=self.subnode_dest, allow_none=True
         )
 
         if dest is not None and dest.mayRaiseException(BaseException):
@@ -164,5 +163,7 @@ Exception raise in 'print' statement destination converted to explicit raise."""
 
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
+        # Output may always fail due to external reasons.
         return True
