@@ -1,4 +1,4 @@
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -19,11 +19,11 @@
 
 """
 
-from .NodeBases import NodeBase, StatementChildrenHavingBase
+from .NodeBases import StatementBase, StatementChildHavingBase
 
 
 def checkStatements(value):
-    """ Check that statements list value property.
+    """Check that statements list value property.
 
     Must not be None, must not contain None, and of course only statements,
     may be empty.
@@ -33,34 +33,30 @@ def checkStatements(value):
     assert None not in value
 
     for statement in value:
-        assert statement.isStatement() or statement.isStatementsFrame(), \
-          statement.asXmlText()
+        assert (
+            statement.isStatement() or statement.isStatementsFrame()
+        ), statement.asXmlText()
 
     return tuple(value)
 
 
-class StatementsSequence(StatementChildrenHavingBase):
+class StatementsSequence(StatementChildHavingBase):
     kind = "STATEMENTS_SEQUENCE"
 
-    named_children = (
-        "statements",
-    )
+    named_child = "statements"
 
-    checkers = {
-        "statements" : checkStatements
-    }
+    checker = checkStatements
 
     def __init__(self, statements, source_ref):
-        StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "statements" : statements
-            },
-            source_ref = source_ref
+        StatementChildHavingBase.__init__(
+            self, value=tuple(statements), source_ref=source_ref
         )
 
-    getStatements = StatementChildrenHavingBase.childGetter("statements")
-    setStatements = StatementChildrenHavingBase.childSetter("statements")
+    def finalize(self):
+        del self.parent
+
+        for s in self.subnode_statements:
+            s.finalize()
 
     # Overloading name based automatic check, so that derived ones know it too.
     def isStatementsSequence(self):
@@ -71,82 +67,85 @@ class StatementsSequence(StatementChildrenHavingBase):
     def trimStatements(self, statement):
         assert statement.parent is self
 
-        old_statements = list(self.getStatements())
-        assert statement in old_statements, \
-          (statement, self)
+        old_statements = list(self.subnode_statements)
+        assert statement in old_statements, (statement, self)
 
-        new_statements = old_statements[ : old_statements.index(statement)+1 ]
+        new_statements = old_statements[: old_statements.index(statement) + 1]
 
         self.setChild("statements", new_statements)
 
     def removeStatement(self, statement):
         assert statement.parent is self
 
-        statements = list(self.getStatements())
+        statements = list(self.subnode_statements)
         statements.remove(statement)
-
         self.setChild("statements", statements)
 
-    def mergeStatementsSequence(self, statement_sequence):
-        assert statement_sequence.parent is self
+        if statements:
+            return self
+        else:
+            return None
 
-        old_statements = list(self.getStatements())
-        assert statement_sequence in old_statements, \
-          (statement_sequence, self)
+    def replaceStatement(self, statement, statements):
+        old_statements = list(self.subnode_statements)
 
-        merge_index =  old_statements.index(statement_sequence)
+        merge_index = old_statements.index(statement)
 
-        new_statements = tuple(old_statements[ : merge_index ])     + \
-                         statement_sequence.getStatements()         + \
-                         tuple(old_statements[ merge_index+1 : ])
+        new_statements = (
+            tuple(old_statements[:merge_index])
+            + tuple(statements)
+            + tuple(old_statements[merge_index + 1 :])
+        )
 
         self.setChild("statements", new_statements)
 
     def mayHaveSideEffects(self):
         # Statement sequences have a side effect if one of the statements does.
-        for statement in self.getStatements():
+        for statement in self.subnode_statements:
             if statement.mayHaveSideEffects():
                 return True
         return False
 
     def mayRaiseException(self, exception_type):
-        for statement in self.getStatements():
+        for statement in self.subnode_statements:
             if statement.mayRaiseException(exception_type):
                 return True
         return False
 
     def needsFrame(self):
-        for statement in self.getStatements():
+        for statement in self.subnode_statements:
             if statement.needsFrame():
                 return True
         return False
 
     def mayReturn(self):
-        for statement in self.getStatements():
+        for statement in self.subnode_statements:
             if statement.mayReturn():
                 return True
         return False
 
     def mayBreak(self):
-        for statement in self.getStatements():
+        for statement in self.subnode_statements:
             if statement.mayBreak():
                 return True
         return False
 
     def mayContinue(self):
-        for statement in self.getStatements():
+        for statement in self.subnode_statements:
             if statement.mayContinue():
                 return True
         return False
 
     def mayRaiseExceptionOrAbort(self, exception_type):
-        return self.mayRaiseException(exception_type) or \
-               self.mayReturn() or \
-               self.mayBreak() or \
-               self.mayContinue()
+        return (
+            self.mayRaiseException(exception_type)
+            or self.mayReturn()
+            or self.mayBreak()
+            or self.mayContinue()
+        )
 
     def isStatementAborting(self):
-        return self.getStatements()[-1].isStatementAborting()
+        return self.subnode_statements[-1].isStatementAborting()
 
     def computeStatement(self, trace_collection):
         # Don't want to be called like this.
@@ -155,44 +154,43 @@ class StatementsSequence(StatementChildrenHavingBase):
     def computeStatementsSequence(self, trace_collection):
         new_statements = []
 
-        statements = self.getStatements()
+        statements = self.subnode_statements
         assert statements, self
 
         for count, statement in enumerate(statements):
             # May be frames embedded.
             if statement.isStatementsFrame():
-                new_statement = statement.computeStatementsSequence(
-                    trace_collection
-                )
+                new_statement = statement.computeStatementsSequence(trace_collection)
             else:
-                new_statement = trace_collection.onStatement(
-                    statement = statement
-                )
+                new_statement = trace_collection.onStatement(statement=statement)
 
             if new_statement is not None:
-                if new_statement.isStatementsSequence() and \
-                   not new_statement.isStatementsFrame():
-                    new_statements.extend(
-                        new_statement.getStatements()
-                    )
+                if (
+                    new_statement.isStatementsSequence()
+                    and not new_statement.isStatementsFrame()
+                ):
+                    new_statements.extend(new_statement.subnode_statements)
                 else:
-                    new_statements.append(
-                        new_statement
-                    )
+                    new_statements.append(new_statement)
 
-                if statement is not statements[-1] and \
-                   new_statement.isStatementAborting():
+                if (
+                    statement is not statements[-1]
+                    and new_statement.isStatementAborting()
+                ):
                     trace_collection.signalChange(
                         "new_statements",
-                        statements[count+1].getSourceReference(),
-                        "Removed dead statements."
+                        statements[count + 1].getSourceReference(),
+                        "Removed dead statements.",
                     )
 
+                    for s in statements[statements.index(statement) + 1 :]:
+                        s.finalize()
+
                     break
 
         if statements != new_statements:
             if new_statements:
-                self.setStatements(new_statements)
+                self.setChild("statements", new_statements)
 
                 return self
             else:
@@ -200,75 +198,57 @@ class StatementsSequence(StatementChildrenHavingBase):
         else:
             return self
 
+    @staticmethod
+    def getStatementNiceName():
+        return "statements sequence"
 
-class StatementExpressionOnly(StatementChildrenHavingBase):
+
+class StatementExpressionOnly(StatementChildHavingBase):
     kind = "STATEMENT_EXPRESSION_ONLY"
 
-    named_children = (
-        "expression",
-    )
+    named_child = "expression"
 
     def __init__(self, expression, source_ref):
         assert expression.isExpression()
 
-        StatementChildrenHavingBase.__init__(
-            self,
-            values     = {
-                "expression" : expression
-            },
-            source_ref = source_ref
-        )
-
-    def getDetail(self):
-        return "expression %s" % self.getExpression()
+        StatementChildHavingBase.__init__(self, value=expression, source_ref=source_ref)
 
     def mayHaveSideEffects(self):
-        return self.getExpression().mayHaveSideEffects()
+        return self.subnode_expression.mayHaveSideEffects()
 
     def mayRaiseException(self, exception_type):
-        return self.getExpression().mayRaiseException(exception_type)
-
-    getExpression = StatementChildrenHavingBase.childGetter(
-        "expression"
-    )
+        return self.subnode_expression.mayRaiseException(exception_type)
 
     def computeStatement(self, trace_collection):
-        trace_collection.onExpression(
-            expression = self.getExpression()
-        )
-        expression = self.getExpression()
+        expression = trace_collection.onExpression(expression=self.subnode_expression)
 
-        if expression.mayRaiseException(BaseException):
-            trace_collection.onExceptionRaiseExit(BaseException)
-
-        result, change_tags, change_desc = expression.computeExpressionDrop(
-            statement        = self,
-            trace_collection = trace_collection
+        return expression.computeExpressionDrop(
+            statement=self, trace_collection=trace_collection
         )
 
-        if result is not self:
-            return result, change_tags, change_desc
+    @staticmethod
+    def getStatementNiceName():
+        return "expression only statement"
 
-        return self, None, None
+    def getDetailsForDisplay(self):
+        return {"expression": self.subnode_expression.kind}
 
 
-class StatementPreserveFrameException(NodeBase):
+class StatementPreserveFrameException(StatementBase):
     kind = "STATEMENT_PRESERVE_FRAME_EXCEPTION"
 
     __slots__ = ("preserver_id",)
 
     def __init__(self, preserver_id, source_ref):
-        NodeBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         self.preserver_id = preserver_id
 
+    def finalize(self):
+        del self.parent
+
     def getDetails(self):
-        return {
-            "preserver_id" : self.preserver_id
-        }
+        return {"preserver_id": self.preserver_id}
 
     def getPreserverId(self):
         return self.preserver_id
@@ -283,33 +263,33 @@ class StatementPreserveFrameException(NodeBase):
             return (
                 None,
                 "new_statements",
-                "Removed frame preservation for generators."
+                "Removed frame preservation for generators.",
             )
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
-    def needsFrame(self):
+    @staticmethod
+    def needsFrame():
         return True
 
 
-class StatementRestoreFrameException(NodeBase):
+class StatementRestoreFrameException(StatementBase):
     kind = "STATEMENT_RESTORE_FRAME_EXCEPTION"
 
     __slots__ = ("preserver_id",)
 
     def __init__(self, preserver_id, source_ref):
-        NodeBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
 
         self.preserver_id = preserver_id
 
+    def finalize(self):
+        del self.parent
+
     def getDetails(self):
-        return {
-            "preserver_id" : self.preserver_id
-        }
+        return {"preserver_id": self.preserver_id}
 
     def getPreserverId(self):
         return self.preserver_id
@@ -317,22 +297,24 @@ class StatementRestoreFrameException(NodeBase):
     def computeStatement(self, trace_collection):
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
 
 
-class StatementPublishException(NodeBase):
+class StatementPublishException(StatementBase):
     kind = "STATEMENT_PUBLISH_EXCEPTION"
 
     def __init__(self, source_ref):
-        NodeBase.__init__(
-            self,
-            source_ref = source_ref
-        )
+        StatementBase.__init__(self, source_ref=source_ref)
+
+    def finalize(self):
+        del self.parent
 
     def computeStatement(self, trace_collection):
         # TODO: Determine the need for it.
         return self, None, None
 
-    def mayRaiseException(self, exception_type):
+    @staticmethod
+    def mayRaiseException(exception_type):
         return False
