@@ -33,8 +33,11 @@ from .BuiltinRefNodes import (
 )
 from .ExpressionBases import (
     ExpressionBuiltinSingleArgBase,
+    ExpressionChildHavingBase,
     ExpressionChildrenHavingBase,
 )
+from .ExpressionShapeMixins import ExpressionBoolShapeExactMixin
+from .NodeBases import SideEffectsFromChildrenMixin
 from .NodeMakingHelpers import wrapExpressionWithNodeSideEffects
 from .shapes.BuiltinTypeShapes import tshape_type
 
@@ -119,15 +122,38 @@ Removed type taking for unused result.""",
         return self.subnode_value.mayHaveSideEffects()
 
 
-class ExpressionBuiltinSuper(ExpressionChildrenHavingBase):
-    kind = "EXPRESSION_BUILTIN_SUPER"
+class ExpressionBuiltinSuper2(ExpressionChildrenHavingBase):
+    """Two arguments form of super."""
 
-    named_children = ("type", "object")
+    kind = "EXPRESSION_BUILTIN_SUPER2"
 
-    def __init__(self, super_type, super_object, source_ref):
+    named_children = ("type_arg", "object_arg")
+
+    def __init__(self, type_arg, object_arg, source_ref):
         ExpressionChildrenHavingBase.__init__(
             self,
-            values={"type": super_type, "object": super_object},
+            values={"type_arg": type_arg, "object_arg": object_arg},
+            source_ref=source_ref,
+        )
+
+    def computeExpression(self, trace_collection):
+        trace_collection.onExceptionRaiseExit(BaseException)
+
+        # TODO: Quite some cases should be possible to predict.
+        return self, None, None
+
+
+class ExpressionBuiltinSuper0(ExpressionChildrenHavingBase):
+    """Python3 form of super, arguments determined from cells and function arguments."""
+
+    kind = "EXPRESSION_BUILTIN_SUPER0"
+
+    named_children = ("type_arg", "object_arg")
+
+    def __init__(self, type_arg, object_arg, source_ref):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values={"type_arg": type_arg, "object_arg": object_arg},
             source_ref=source_ref,
         )
 
@@ -162,9 +188,9 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
 
             return self, None, None
 
-        cls = self.subnode_classes
+        classes = self.subnode_classes
 
-        if not cls.isCompileTimeConstant():
+        if not classes.isCompileTimeConstant():
             trace_collection.onExceptionRaiseExit(BaseException)
 
             return self, None, None
@@ -173,7 +199,70 @@ class ExpressionBuiltinIsinstance(ExpressionChildrenHavingBase):
         return trace_collection.getCompileTimeComputationResult(
             node=self,
             computation=lambda: isinstance(
-                instance.getCompileTimeConstant(), cls.getCompileTimeConstant()
+                instance.getCompileTimeConstant(), classes.getCompileTimeConstant()
             ),
             description="Built-in call to 'isinstance' computed.",
         )
+
+
+class ExpressionBuiltinIssubclass(ExpressionChildrenHavingBase):
+    kind = "EXPRESSION_BUILTIN_ISSUBCLASS"
+
+    named_children = ("cls", "classes")
+
+    def __init__(self, cls, classes, source_ref):
+        ExpressionChildrenHavingBase.__init__(
+            self,
+            values={"cls": cls, "classes": classes},
+            source_ref=source_ref,
+        )
+
+    def computeExpression(self, trace_collection):
+        # TODO: Quite some cases should be possible to predict.
+
+        cls = self.subnode_cls
+
+        # TODO: Should be possible to query run time type instead, but we don't
+        # have that method yet. Later this will be essential.
+        if not cls.isCompileTimeConstant():
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        classes = self.subnode_classes
+
+        if not classes.isCompileTimeConstant():
+            trace_collection.onExceptionRaiseExit(BaseException)
+
+            return self, None, None
+
+        # So if both are compile time constant, we are able to compute it.
+        return trace_collection.getCompileTimeComputationResult(
+            node=self,
+            computation=lambda: issubclass(
+                cls.getCompileTimeConstant(), classes.getCompileTimeConstant()
+            ),
+            description="Built-in call to 'issubclass' computed.",
+        )
+
+
+class ExpressionTypeCheck(
+    ExpressionBoolShapeExactMixin,
+    SideEffectsFromChildrenMixin,
+    ExpressionChildHavingBase,
+):
+    kind = "EXPRESSION_TYPE_CHECK"
+
+    named_child = "cls"
+
+    def __init__(self, cls, source_ref):
+        ExpressionChildHavingBase.__init__(
+            self,
+            value=cls,
+            source_ref=source_ref,
+        )
+
+    def computeExpression(self, trace_collection):
+        # TODO: Quite some cases should be possible to predict, but I am not aware of
+        # 100% true Python equivalent at this time.
+        return self, None, None
