@@ -26,8 +26,8 @@ make others possible.
 import inspect
 
 from nuitka import ModuleRegistry, Options, Variables
-from nuitka.Errors import NuitkaForbiddenImportEncounter
 from nuitka.importing import ImportCache
+from nuitka.importing.Recursion import considerUsedModules
 from nuitka.plugins.Plugins import Plugins
 from nuitka.Progress import (
     closeProgressBar,
@@ -101,8 +101,6 @@ def optimizeCompiledPythonModule(module):
             # print("Compute module")
             with withChangeIndicationsTo(signalChange):
                 scopes_were_incomplete = module.computeModule()
-        except NuitkaForbiddenImportEncounter as e:
-            optimization_logger.sysexit("Error, forbidden import '%s' encountered." % e)
         except SystemExit:
             raise
         except BaseException:
@@ -153,7 +151,7 @@ def optimizeCompiledPythonModule(module):
             % (module.getFullName(), memory_watch.asStr())
         )
 
-    Plugins.considerImplicitImports(module=module, signal_change=signalChange)
+    considerUsedModules(module=module, signal_change=signalChange)
 
     return touched
 
@@ -166,11 +164,7 @@ def optimizeUncompiledPythonModule(module):
         )
     )
 
-    for used_module_name, used_module_path in module.getUsedModules():
-        used_module = ImportCache.getImportedModuleByNameAndPath(
-            used_module_name, used_module_path
-        )
-        ModuleRegistry.addUsedModule(used_module)
+    considerUsedModules(module=module, signal_change=signalChange)
 
     package_name = full_name.getPackageName()
 
@@ -182,7 +176,13 @@ def optimizeUncompiledPythonModule(module):
         except KeyError:
             pass
         else:
-            ModuleRegistry.addUsedModule(used_module)
+            ModuleRegistry.addUsedModule(
+                module=used_module,
+                using_module=module,
+                usage_tag="package",
+                reason="Package of %s" % module.getFullName(),
+                source_ref=module.source_ref,
+            )
 
     Plugins.considerImplicitImports(module=module, signal_change=signalChange)
 
@@ -217,7 +217,7 @@ last_total = 0
 
 
 def _restartProgress():
-    global pass_count, last_total  # Singleton, pylint: disable=global-statement
+    global pass_count  # Singleton, pylint: disable=global-statement
 
     closeProgressBar()
     pass_count += 1
@@ -329,6 +329,7 @@ def makeOptimizationPass():
                 )
 
                 unused_function.trace_collection = None
+                unused_function.finalize()
 
             used_functions = tuple(
                 function
@@ -343,7 +344,7 @@ def makeOptimizationPass():
     return finished
 
 
-def optimize(output_filename):
+def optimizeModules(output_filename):
     Graphs.startGraph()
 
     finished = makeOptimizationPass()
@@ -357,8 +358,6 @@ def optimize(output_filename):
         ):
             demoteCompiledModuleToBytecode(module)
 
-    global pass_count  # Singleton, pylint: disable=global-statement
-
     # Second, "endless" pass.
     while not finished:
         finished = makeOptimizationPass()
