@@ -21,17 +21,17 @@ For built-in name references, we check if it's one of the supported built-in
 types, and then specialize for the ones, where it makes sense.
 """
 
-from nuitka.__past__ import xrange  # pylint: disable=I0021,redefined-builtin
+from nuitka.__past__ import xrange
 from nuitka.Errors import NuitkaAssumptionError
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
     StatementDelVariable,
 )
 from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
     ExpressionBuiltinGetattr,
     ExpressionBuiltinHasattr,
     ExpressionBuiltinSetattr,
+    makeExpressionAttributeLookup,
 )
 from nuitka.nodes.BuiltinAllNodes import ExpressionBuiltinAll
 from nuitka.nodes.BuiltinAnyNodes import ExpressionBuiltinAny
@@ -124,20 +124,21 @@ from nuitka.nodes.NodeMakingHelpers import (
     makeExpressionBuiltinLocals,
     makeRaiseExceptionReplacementExpression,
     makeRaiseExceptionReplacementExpressionFromInstance,
-    wrapExpressionWithSideEffects,
 )
-from nuitka.nodes.OperatorNodes import (
-    ExpressionOperationAbs,
-    ExpressionOperationBinaryDivmod,
+from nuitka.nodes.OperatorNodes import ExpressionOperationBinaryDivmod
+from nuitka.nodes.OperatorNodesUnary import (
     ExpressionOperationNot,
-    ExpressionOperationUnary,
+    ExpressionOperationUnaryAbs,
+    ExpressionOperationUnaryRepr,
 )
 from nuitka.nodes.OutlineNodes import ExpressionOutlineBody
 from nuitka.nodes.ReturnNodes import makeStatementReturn
 from nuitka.nodes.SliceNodes import makeExpressionBuiltinSlice
 from nuitka.nodes.TypeNodes import (
     ExpressionBuiltinIsinstance,
-    ExpressionBuiltinSuper,
+    ExpressionBuiltinIssubclass,
+    ExpressionBuiltinSuper0,
+    ExpressionBuiltinSuper2,
     ExpressionBuiltinType1,
 )
 from nuitka.nodes.VariableRefNodes import (
@@ -146,7 +147,6 @@ from nuitka.nodes.VariableRefNodes import (
 )
 from nuitka.PythonVersions import python_version
 from nuitka.specs import BuiltinParameterSpecs
-from nuitka.Tracing import optimization_logger
 from nuitka.tree.ReformulationExecStatements import wrapEvalGlobalsAndLocals
 from nuitka.tree.ReformulationTryFinallyStatements import (
     makeTryFinallyStatement,
@@ -167,7 +167,7 @@ def dir_extractor(node):
         )
 
         result = makeCallNode(
-            ExpressionAttributeLookup(
+            makeExpressionAttributeLookup(
                 expression=source, attribute_name="keys", source_ref=source_ref
             ),
             source_ref,
@@ -308,28 +308,14 @@ def dict_extractor(node):
     # The "dict" built-in is a bit strange in that it accepts a position
     # parameter, or not, but won't have a default value.
     def wrapExpressionBuiltinDictCreation(positional_args, dict_star_arg, source_ref):
-        if len(positional_args) > 1:
-
-            result = makeRaiseExceptionReplacementExpressionFromInstance(
-                expression=node,
-                exception=TypeError(
-                    "dict expected at most 1 arguments, got %d" % (len(positional_args))
-                ),
-            )
-
-            result = wrapExpressionWithSideEffects(
-                side_effects=positional_args, old_node=node, new_node=result
-            )
-
-            if dict_star_arg:
-                result = wrapExpressionWithSideEffects(
-                    side_effects=dict_star_arg, old_node=node, new_node=result
-                )
-
-            return result
+        if positional_args:
+            # Only one allowed, the spec converted too many into an exception.
+            (pos_arg,) = positional_args
+        else:
+            pos_arg = None
 
         return ExpressionBuiltinDict(
-            pos_arg=positional_args[0] if positional_args else None,
+            pos_arg=pos_arg,
             pairs=dict_star_arg,
             source_ref=source_ref,
         )
@@ -399,14 +385,9 @@ def id_extractor(node):
 
 
 def repr_extractor(node):
-    def makeReprOperator(operand, source_ref):
-        return ExpressionOperationUnary(
-            operator="Repr", operand=operand, source_ref=source_ref
-        )
-
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
-        builtin_class=makeReprOperator,
+        builtin_class=ExpressionOperationUnaryRepr,
         builtin_spec=BuiltinParameterSpecs.builtin_repr_spec,
     )
 
@@ -509,7 +490,7 @@ def all_extractor(node):
 def abs_extractor(node):
     return BuiltinParameterSpecs.extractBuiltinArgs(
         node=node,
-        builtin_class=ExpressionOperationAbs,
+        builtin_class=ExpressionOperationUnaryAbs,
         builtin_spec=BuiltinParameterSpecs.builtin_abs_spec,
     )
 
@@ -622,7 +603,6 @@ if python_version < 0x300:
             builtin_spec=ExpressionBuiltinUnicodeP2.builtin_spec,
         )
 
-
 else:
     from nuitka.nodes.BuiltinTypeNodes import (
         ExpressionBuiltinBytes1,
@@ -775,7 +755,7 @@ if python_version < 0x300:
                     makeStatementReturn(
                         expression=ExpressionBuiltinExecfile(
                             source_code=makeCallNode(
-                                ExpressionAttributeLookup(
+                                makeExpressionAttributeLookup(
                                     expression=ExpressionBuiltinOpen(
                                         filename=filename,
                                         mode=makeConstantRefNode(
@@ -891,7 +871,7 @@ def eval_extractor(node):
         string_fixup = StatementAssignmentVariable(
             variable=source_variable,
             source=makeExpressionCall(
-                called=ExpressionAttributeLookup(
+                called=makeExpressionAttributeLookup(
                     expression=ExpressionTempVariableRef(
                         variable=source_variable, source_ref=source_ref
                     ),
@@ -1064,6 +1044,8 @@ def open_extractor(node):
     def makeOpen0(source_ref):
         # pylint: disable=unused-argument
         try:
+            # Not giving arguments or context on purpose
+            # pylint: disable=consider-using-with,unspecified-encoding
             open()
         except Exception as e:  # We want to broad here, pylint: disable=broad-except
             return makeRaiseExceptionReplacementExpressionFromInstance(
@@ -1103,14 +1085,11 @@ def super_extractor(node):
 
             # If we already have this as a local variable, then use that
             # instead.
-            type_arg_owner = type_arg.getVariable().getOwner()
+            type_arg_owner = class_variable.getOwner()
             if type_arg_owner is provider or not (
                 type_arg_owner.isExpressionFunctionBody()
                 or type_arg_owner.isExpressionClassBody()
             ):
-                type_arg = None
-
-            if type_arg is None:
                 return makeRaiseExceptionReplacementExpression(
                     expression=node,
                     exception_type="SystemError"
@@ -1129,7 +1108,13 @@ def super_extractor(node):
                 else:
                     parameter_provider = provider
 
-                if parameter_provider.getParameters().getArgumentCount() > 0:
+                if parameter_provider.getParameters().getArgumentCount() == 0:
+                    return makeRaiseExceptionReplacementExpression(
+                        expression=node,
+                        exception_type="RuntimeError",
+                        exception_value="super(): no arguments",
+                    )
+                else:
                     par1_name = parameter_provider.getParameters().getArgumentNames()[0]
 
                     object_variable = provider.getVariableForReference(
@@ -1152,15 +1137,13 @@ def super_extractor(node):
                             else "RuntimeError",
                             exception_value="super(): __class__ cell not found",
                         )
-                else:
-                    return makeRaiseExceptionReplacementExpression(
-                        expression=node,
-                        exception_type="RuntimeError",
-                        exception_value="super(): no arguments",
-                    )
 
-        return ExpressionBuiltinSuper(
-            super_type=type_arg, super_object=object_arg, source_ref=source_ref
+            return ExpressionBuiltinSuper0(
+                type_arg=type_arg, object_arg=object_arg, source_ref=source_ref
+            )
+
+        return ExpressionBuiltinSuper2(
+            type_arg=type_arg, object_arg=object_arg, source_ref=source_ref
         )
 
     provider = node.getParentVariableProvider().getEntryPoint()
@@ -1225,6 +1208,14 @@ def isinstance_extractor(node):
     )
 
 
+def issubclass_extractor(node):
+    return BuiltinParameterSpecs.extractBuiltinArgs(
+        node=node,
+        builtin_class=ExpressionBuiltinIssubclass,
+        builtin_spec=BuiltinParameterSpecs.builtin_isinstance_spec,
+    )
+
+
 def bytearray_extractor(node):
     def makeBytearray0(source_ref):
         return makeConstantRefNode(constant=bytearray(), source_ref=source_ref)
@@ -1369,6 +1360,7 @@ _dispatch_dict = {
     "getattr": getattr_extractor,
     "setattr": setattr_extractor,
     "isinstance": isinstance_extractor,
+    "issubclass": issubclass_extractor,
     "bytearray": bytearray_extractor,
     "slice": slice_extractor,
     "hash": hash_extractor,
@@ -1406,7 +1398,7 @@ def check():
 
 check()
 
-_builtin_white_list = (
+_builtin_ignore_list = (
     # Not supporting 'print', because it could be replaced, and is not
     # worth the effort yet.
     "print",
@@ -1503,9 +1495,10 @@ def computeBuiltinCall(builtin_name, call_node):
 
         return new_node, tags, message
     else:
-        if False and builtin_name not in _builtin_white_list:
-            optimization_logger.warning(
-                "Not handling built-in '%s', consider support." % builtin_name
-            )
+        # TODO: Achieve coverage of all built-ins in at least the ignore list.
+        # if False and builtin_name not in _builtin_ignore_list:
+        #     optimization_logger.warning(
+        #         "Not handling built-in %r, consider support." % builtin_name
+        #     )
 
         return call_node, None, None
