@@ -24,25 +24,31 @@ This is about collecting these constraints and to manage them.
 """
 
 import contextlib
+from collections import defaultdict
+from contextlib import contextmanager
 
 from nuitka import Tracing, Variables
 from nuitka.__past__ import iterItems  # Python3 compatibility.
 from nuitka.containers.oset import OrderedSet
-from nuitka.importing.ImportCache import getImportedModuleByNameAndPath
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.nodes.NodeMakingHelpers import getComputationResult
 from nuitka.nodes.shapes.BuiltinTypeShapes import tshape_dict
 from nuitka.nodes.shapes.StandardShapes import tshape_uninit
 from nuitka.tree.SourceReading import readSourceLine
-from nuitka.utils.FileOperations import relpath
-from nuitka.utils.InstanceCounters import counted_del, counted_init
-from nuitka.utils.ModuleNames import ModuleName
+from nuitka.utils.InstanceCounters import (
+    counted_del,
+    counted_init,
+    isCountingInstances,
+)
+from nuitka.utils.Timing import TimerReport
 
 from .ValueTraces import (
     ValueTraceAssign,
     ValueTraceDeleted,
     ValueTraceEscaped,
     ValueTraceInit,
+    ValueTraceInitStarArgs,
+    ValueTraceInitStarDict,
     ValueTraceLoopComplete,
     ValueTraceLoopIncomplete,
     ValueTraceMerge,
@@ -53,7 +59,61 @@ from .ValueTraces import (
 signalChange = None
 
 
-class CollectionStartpointMixin(object):
+@contextmanager
+def withChangeIndicationsTo(signal_change):
+    """Decide where change indications should go to."""
+
+    global signalChange  # Singleton, pylint: disable=global-statement
+
+    old = signalChange
+    signalChange = signal_change
+    yield
+    signalChange = old
+
+
+class CollectionUpdateMixin(object):
+    """Mixin to use in every collection to add traces."""
+
+    # Mixins are not allow to specify slots.
+    __slots__ = ()
+
+    def hasVariableTrace(self, variable, version):
+        return (variable, version) in self.variable_traces
+
+    def getVariableTrace(self, variable, version):
+        return self.variable_traces[(variable, version)]
+
+    def getVariableTraces(self, variable):
+        result = []
+
+        for key, variable_trace in iterItems(self.variable_traces):
+            candidate = key[0]
+
+            if variable is candidate:
+                result.append(variable_trace)
+
+        return result
+
+    def getVariableTracesAll(self):
+        return self.variable_traces
+
+    def addVariableTrace(self, variable, version, trace):
+        key = variable, version
+
+        assert key not in self.variable_traces, (key, self)
+        self.variable_traces[key] = trace
+
+    def addVariableMergeMultipleTrace(self, variable, traces):
+        version = variable.allocateTargetNumber()
+
+        trace_merge = ValueTraceMerge(traces)
+
+        self.addVariableTrace(variable, version, trace_merge)
+
+        return version
+
+
+class CollectionStartpointMixin(CollectionUpdateMixin):
     """Mixin to use in start points of collections.
 
     These are modules, functions, etc. typically entry points.
@@ -148,59 +208,55 @@ class CollectionStartpointMixin(object):
     def getExceptionRaiseCollections(self):
         return self.exception_collections
 
-    def hasVariableTrace(self, variable, version):
-        return (variable, version) in self.variable_traces
+    def hasEmptyTraces(self, variable):
+        # TODO: Combine these steps into one for performance gains.
+        traces = self.getVariableTraces(variable)
+        return areEmptyTraces(traces)
 
-    def getVariableTrace(self, variable, version):
-        return self.variable_traces[(variable, version)]
-
-    def getVariableTraces(self, variable):
-        result = []
-
-        for key, variable_trace in iterItems(self.variable_traces):
-            candidate = key[0]
-
-            if variable is candidate:
-                result.append(variable_trace)
-
-        return result
-
-    def getVariableTracesAll(self):
-        return self.variable_traces
-
-    def addVariableTrace(self, variable, version, trace):
-        key = variable, version
-
-        assert key not in self.variable_traces, (key, self)
-        self.variable_traces[key] = trace
-
-    def addVariableMergeMultipleTrace(self, variable, traces):
-        version = variable.allocateTargetNumber()
-
-        trace_merge = ValueTraceMerge(traces)
-
-        self.addVariableTrace(variable, version, trace_merge)
-
-        return version
+    def hasReadOnlyTraces(self, variable):
+        # TODO: Combine these steps into one for performance gains.
+        traces = self.getVariableTraces(variable)
+        return areReadOnlyTraces(traces)
 
     def initVariableUnknown(self, variable):
         trace = ValueTraceUnknown(owner=self.owner, previous=None)
 
-        self.addVariableTrace(variable=variable, version=0, trace=trace)
+        self.addVariableTrace(variable, 0, trace)
+
+        return trace
+
+    def initVariableModule(self, variable):
+        trace = ValueTraceUnknown(owner=self.owner, previous=None)
+
+        self.addVariableTrace(variable, 0, trace)
 
         return trace
 
     def _initVariableInit(self, variable):
         trace = ValueTraceInit(self.owner)
 
-        self.addVariableTrace(variable=variable, version=0, trace=trace)
+        self.addVariableTrace(variable, 0, trace)
+
+        return trace
+
+    def _initVariableInitStarArgs(self, variable):
+        trace = ValueTraceInitStarArgs(self.owner)
+
+        self.addVariableTrace(variable, 0, trace)
+
+        return trace
+
+    def _initVariableInitStarDict(self, variable):
+        trace = ValueTraceInitStarDict(self.owner)
+
+        self.addVariableTrace(variable, 0, trace)
 
         return trace
 
     def _initVariableUninit(self, variable):
         trace = ValueTraceUninit(owner=self.owner, previous=None)
 
-        self.addVariableTrace(variable=variable, version=0, trace=trace)
+        self.addVariableTrace(variable, 0, trace)
 
         return trace
 
@@ -237,11 +293,12 @@ class CollectionStartpointMixin(object):
 
     def initVariable(self, variable):
         if variable.isParameterVariable():
+            # TODO: That's not happening, maybe just assert against it.
             result = self._initVariableInit(variable)
         elif variable.isLocalVariable():
             result = self._initVariableUninit(variable)
         elif variable.isModuleVariable():
-            result = self.initVariableUnknown(variable)
+            result = self.initVariableModule(variable)
         elif variable.isTempVariable():
             result = self._initVariableUninit(variable)
         elif variable.isLocalsDictVariable():
@@ -284,7 +341,8 @@ class TraceCollectionBase(object):
 
     __slots__ = ("owner", "parent", "name", "value_states", "variable_actives")
 
-    __del__ = counted_del()
+    if isCountingInstances():
+        __del__ = counted_del()
 
     @counted_init
     def __init__(self, owner, name, parent):
@@ -334,15 +392,29 @@ class TraceCollectionBase(object):
         return self.variable_actives.keys()
 
     def markActiveVariableAsEscaped(self, variable):
-        current = self.getVariableCurrentTrace(variable=variable)
+        current = self.getVariableCurrentTrace(variable)
+
+        if not current.isEscapeOrUnknownOrUninitTrace():
+            version = variable.allocateTargetNumber()
+
+            self.addVariableTrace(
+                variable,
+                version,
+                ValueTraceEscaped(owner=self.owner, previous=current),
+            )
+
+            self.markCurrentVariableTrace(variable, version)
+
+    def markActiveVariableAsUnknown(self, variable):
+        current = self.getVariableCurrentTrace(variable)
 
         if not current.isUnknownTrace():
             version = variable.allocateTargetNumber()
 
             self.addVariableTrace(
-                variable=variable,
-                version=version,
-                trace=ValueTraceEscaped(owner=self.owner, previous=current),
+                variable,
+                version,
+                ValueTraceUnknown(owner=self.owner, previous=current),
             )
 
             self.markCurrentVariableTrace(variable, version)
@@ -363,26 +435,49 @@ class TraceCollectionBase(object):
             result = ValueTraceLoopComplete(loop_node, current, shapes)
 
         version = variable.allocateTargetNumber()
-        self.addVariableTrace(variable=variable, version=version, trace=result)
+        self.addVariableTrace(variable, version, result)
 
         self.markCurrentVariableTrace(variable, version)
 
         return result
 
-    def markActiveVariablesAsUnknown(self):
+    def markActiveVariablesAsEscaped(self):
         for variable in self.getActiveVariables():
             if variable.isTempVariable():
                 continue
 
             self.markActiveVariableAsEscaped(variable)
 
+    def markActiveVariablesAsUnknown(self):
+        for variable in self.getActiveVariables():
+            if variable.isTempVariable():
+                continue
+
+            self.markActiveVariableAsUnknown(variable)
+
     @staticmethod
     def signalChange(tags, source_ref, message):
         # This is monkey patched from another module. pylint: disable=I0021,not-callable
         signalChange(tags, source_ref, message)
 
-    def onUsedModule(self, module_name, module_relpath):
-        return self.parent.onUsedModule(module_name, module_relpath)
+    def onUsedFunction(self, function_body):
+        owning_module = function_body.getParentModule()
+
+        # Make sure the owning module is added to the used set. This is most
+        # important for helper functions, or modules, which otherwise have
+        # become unused.
+        addUsedModule(
+            module=owning_module,
+            using_module=None,
+            usage_tag="function",
+            reason="Function %s" % self.name,
+            source_ref=owning_module.source_ref,
+        )
+
+        needs_visit = owning_module.addUsedFunction(function_body)
+
+        if needs_visit:
+            function_body.computeFunctionRaw(self)
 
     @staticmethod
     def mustAlias(a, b):
@@ -405,18 +500,26 @@ class TraceCollectionBase(object):
         # to be considered escaped, pylint: disable=unused-argument
 
         for variable in self.getActiveVariables():
+            # TODO: Move this to the variable, and prepare and cache it better for
+            # compile time savings.
             if variable.isModuleVariable():
-                # print variable
-
-                self.markActiveVariableAsEscaped(variable)
+                self.markActiveVariableAsUnknown(variable)
 
             elif variable.isLocalVariable():
-                if variable.hasAccessesOutsideOf(self.owner) is not False:
+                if (
+                    str is not bytes
+                    and variable.hasWritersOutsideOf(self.owner) is not False
+                ):
+                    self.markActiveVariableAsUnknown(variable)
+                elif variable.hasAccessesOutsideOf(self.owner) is not False:
                     self.markActiveVariableAsEscaped(variable)
 
     def removeKnowledge(self, node):
         if node.isExpressionVariableRef():
-            self.markActiveVariableAsEscaped(node.variable)
+            if node.variable.isModuleVariable():
+                self.markActiveVariableAsUnknown(node.variable)
+            else:
+                self.markActiveVariableAsEscaped(node.variable)
 
     def onValueEscapeStr(self, node):
         # TODO: We can ignore these for now.
@@ -425,26 +528,14 @@ class TraceCollectionBase(object):
     def removeAllKnowledge(self):
         self.markActiveVariablesAsUnknown()
 
-    def getVariableTrace(self, variable, version):
-        return self.parent.getVariableTrace(variable, version)
-
-    def hasVariableTrace(self, variable, version):
-        return self.parent.hasVariableTrace(variable, version)
-
-    def addVariableTrace(self, variable, version, trace):
-        self.parent.addVariableTrace(variable, version, trace)
-
-    def addVariableMergeMultipleTrace(self, variable, traces):
-        return self.parent.addVariableMergeMultipleTrace(variable, traces)
-
     def onVariableSet(self, variable, version, assign_node):
         variable_trace = ValueTraceAssign(
             owner=self.owner,
             assign_node=assign_node,
-            previous=self.getVariableCurrentTrace(variable=variable),
+            previous=self.getVariableCurrentTrace(variable),
         )
 
-        self.addVariableTrace(variable=variable, version=version, trace=variable_trace)
+        self.addVariableTrace(variable, version, variable_trace)
 
         # Make references point to it.
         self.markCurrentVariableTrace(variable, version)
@@ -456,12 +547,14 @@ class TraceCollectionBase(object):
         # remember the delete of the current
         old_trace = self.getVariableCurrentTrace(variable)
 
+        # TODO: Annotate value content as escaped.
+
         variable_trace = ValueTraceDeleted(
             owner=self.owner, del_node=del_node, previous=old_trace
         )
 
         # Assign to not initialized again.
-        self.addVariableTrace(variable=variable, version=version, trace=variable_trace)
+        self.addVariableTrace(variable, version, variable_trace)
 
         # Make references point to it.
         self.markCurrentVariableTrace(variable, version)
@@ -579,6 +672,30 @@ class TraceCollectionBase(object):
         else:
             return expression, change_tags, change_desc
 
+    def computedExpressionResultRaw(self, expression, change_tags, change_desc):
+        """Make sure the replacement expression is computed.
+
+        Use this when a replacement expression needs to be seen by the trace
+        collection and be computed, without causing any duplication, but where
+        otherwise there would be loss of annotated effects.
+
+        This may e.g. be true for nodes that need an initial run to know their
+        exception result and type shape.
+
+        This is for raw, i.e. subnodes are not yet computed automatically.
+        """
+
+        # Need to compute the replacement still.
+        new_expression = expression.computeExpressionRaw(self)
+
+        if new_expression[0] is not expression:
+            # Signal intermediate result as well.
+            self.signalChange(change_tags, expression.getSourceReference(), change_desc)
+
+            return new_expression
+        else:
+            return expression, change_tags, change_desc
+
     def mergeBranches(self, collection_yes, collection_no):
         """Merge two alternative branches into this trace.
 
@@ -633,55 +750,94 @@ class TraceCollectionBase(object):
 
         for variable, versions in iterItems(variable_versions):
             if type(versions) is tuple:
-                version = self.addVariableMergeMultipleTrace(
-                    variable=variable,
-                    traces=(
-                        self.getVariableTrace(variable, versions[0]),
-                        self.getVariableTrace(variable, versions[1]),
-                    ),
-                )
+                trace1 = self.getVariableTrace(variable, versions[0])
+                trace2 = self.getVariableTrace(variable, versions[1])
+
+                if trace1.isEscapeTrace() and trace1.previous is trace2:
+                    version = versions[0]
+                elif trace2 is trace1.isEscapeTrace() and trace2.previous is trace1:
+                    version = versions[1]
+                else:
+                    version = self.addVariableMergeMultipleTrace(
+                        variable=variable,
+                        traces=(
+                            trace1,
+                            trace2,
+                        ),
+                    )
             else:
                 version = versions
 
             self.markCurrentVariableTrace(variable, version)
 
     def mergeMultipleBranches(self, collections):
+        # This one is really complex, pylint: disable=too-many-branches
+
         assert collections
 
         # Optimize for length 1, which is trivial merge and needs not a
-        # lot of work.
-        if len(collections) == 1:
+        # lot of work, and length 2 has dedicated code as it's so frequent.
+        merge_size = len(collections)
+
+        if merge_size == 1:
             self.replaceBranch(collections[0])
-            return None
+            return
+        elif merge_size == 2:
+            return self.mergeBranches(*collections)
 
-        variable_versions = {}
+        # print("Enter mergeMultipleBranches", len(collections))
+        with TimerReport(
+            message="Running merge for %s took %%.2f seconds" % collections,
+            decider=lambda: 0,
+        ):
+            variable_versions = defaultdict(OrderedSet)
 
-        for collection in collections:
-            for variable, version in iterItems(collection.variable_actives):
-                if variable not in variable_versions:
-                    variable_versions[variable] = OrderedSet((version,))
-                else:
+            for collection in collections:
+                for variable, version in iterItems(collection.variable_actives):
                     variable_versions[variable].add(version)
 
-        for collection in collections:
+            for collection in collections:
+                for variable, versions in iterItems(variable_versions):
+                    if variable not in collection.variable_actives:
+                        versions.add(0)
+
+            self.variable_actives = {}
+
             for variable, versions in iterItems(variable_versions):
-                if variable not in collection.variable_actives:
-                    versions.add(0)
+                if len(versions) == 1:
+                    (version,) = versions
+                else:
+                    traces = []
+                    escaped = []
+                    winner_version = None
 
-        self.variable_actives = {}
+                    for version in versions:
+                        trace = self.getVariableTrace(variable, version)
 
-        for variable, versions in iterItems(variable_versions):
-            if len(versions) == 1:
-                (version,) = versions
-            else:
-                version = self.addVariableMergeMultipleTrace(
-                    variable=variable,
-                    traces=tuple(
-                        self.getVariableTrace(variable, version) for version in versions
-                    ),
-                )
+                        if trace.isEscapeTrace():
+                            winner_version = version
+                            escaped_trace = trace.previous
 
-            self.markCurrentVariableTrace(variable, version)
+                            if escaped_trace in traces:
+                                traces.remove(trace.previous)
+
+                            escaped.append(escaped)
+                            traces.append(trace)
+                        else:
+                            if trace not in escaped:
+                                traces.append(trace)
+
+                    if len(traces) == 1:
+                        version = winner_version
+                        assert winner_version is not None
+                    else:
+                        version = self.addVariableMergeMultipleTrace(
+                            variable=variable, traces=tuple(traces)
+                        )
+
+                self.markCurrentVariableTrace(variable, version)
+
+            # print("Leave mergeMultipleBranches", len(collections))
 
     def replaceBranch(self, collection_replace):
         self.variable_actives.update(collection_replace.variable_actives)
@@ -736,12 +892,14 @@ class TraceCollectionBase(object):
     def onLocalsDictEscaped(self, locals_scope):
         self.parent.onLocalsDictEscaped(locals_scope)
 
-    def getCompileTimeComputationResult(self, node, computation, description):
+    def getCompileTimeComputationResult(
+        self, node, computation, description, user_provided=False
+    ):
         new_node, change_tags, message = getComputationResult(
             node=node,
             computation=computation,
             description=description,
-            user_provided=False,
+            user_provided=user_provided,
         )
 
         if change_tags == "new_raise":
@@ -767,9 +925,12 @@ class TraceCollectionBase(object):
     def addOutlineFunction(self, outline):
         self.parent.addOutlineFunction(outline)
 
+    def getVeryTrustedModuleVariables(self):
+        return self.parent.getVeryTrustedModuleVariables()
 
-class TraceCollectionBranch(TraceCollectionBase):
-    __slots__ = ()
+
+class TraceCollectionBranch(CollectionUpdateMixin, TraceCollectionBase):
+    __slots__ = ("variable_traces",)
 
     def __init__(self, name, parent):
         TraceCollectionBase.__init__(self, owner=parent.owner, name=name, parent=parent)
@@ -777,6 +938,9 @@ class TraceCollectionBranch(TraceCollectionBase):
         # Detach from others
         self.variable_actives = dict(parent.variable_actives)
 
+        # For quick access without going to parent.
+        self.variable_traces = parent.variable_traces
+
     def computeBranch(self, branch):
         if branch.isStatementsSequence():
             result = branch.computeStatementsSequence(trace_collection=self)
@@ -793,9 +957,6 @@ class TraceCollectionBranch(TraceCollectionBase):
 
         return variable_trace
 
-    def onLocalsDictEscaped(self, locals_scope):
-        return self.parent.onLocalsDictEscaped(locals_scope)
-
     def dumpTraces(self):
         Tracing.printSeparator()
         self.parent.dumpTraces()
@@ -819,6 +980,7 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
         "return_collections",
         "exception_collections",
         "outline_functions",
+        "very_trusted_module_variables",
     )
 
     def __init__(self, parent, function_body):
@@ -838,11 +1000,28 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
             parent=parent,
         )
 
+        if parent is not None:
+            self.very_trusted_module_variables = parent.getVeryTrustedModuleVariables()
+        else:
+            self.very_trusted_module_variables = ()
+
         if function_body.isExpressionFunctionBody():
-            for parameter_variable in function_body.getParameters().getAllVariables():
+            parameters = function_body.getParameters()
+
+            for parameter_variable in parameters.getTopLevelVariables():
                 self._initVariableInit(parameter_variable)
                 self.variable_actives[parameter_variable] = 0
 
+            list_star_variable = parameters.getListStarArgVariable()
+            if list_star_variable is not None:
+                self._initVariableInitStarArgs(list_star_variable)
+                self.variable_actives[list_star_variable] = 0
+
+            dict_star_variable = parameters.getDictStarArgVariable()
+            if dict_star_variable is not None:
+                self._initVariableInitStarDict(dict_star_variable)
+                self.variable_actives[dict_star_variable] = 0
+
         for closure_variable in function_body.getClosureVariables():
             self.initVariableUnknown(closure_variable)
             self.variable_actives[closure_variable] = 0
@@ -857,6 +1036,39 @@ class TraceCollectionFunction(CollectionStartpointMixin, TraceCollectionBase):
             else:
                 function_body.locals_scope = None
 
+    def initVariableModule(self, variable):
+        trusted_node = self.very_trusted_module_variables.get(variable)
+
+        if trusted_node is None:
+            return CollectionStartpointMixin.initVariableModule(self, variable)
+
+        assign_trace = ValueTraceAssign(
+            self.owner, assign_node=trusted_node.getParent(), previous=None
+        )
+
+        # This is rare enough to not need a more optimized code.
+        self.addVariableTrace(variable, 0, assign_trace)
+        self.markActiveVariableAsEscaped(variable)
+
+        return self.getVariableCurrentTrace(variable)
+
+
+class TraceCollectionPureFunction(TraceCollectionFunction):
+    """Pure functions don't feed their parent."""
+
+    __slots__ = ("used_functions",)
+
+    def __init__(self, function_body):
+        TraceCollectionFunction.__init__(self, parent=None, function_body=function_body)
+
+        self.used_functions = OrderedSet()
+
+    def getUsedFunctions(self):
+        return self.used_functions
+
+    def onUsedFunction(self, function_body):
+        self.used_functions.add(function_body)
+
 
 class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
     __slots__ = (
@@ -867,9 +1079,10 @@ class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
         "return_collections",
         "exception_collections",
         "outline_functions",
+        "very_trusted_module_variables",
     )
 
-    def __init__(self, module):
+    def __init__(self, module, very_trusted_module_variables):
         assert module.isCompiledPythonModule(), module
 
         CollectionStartpointMixin.__init__(self)
@@ -878,14 +1091,88 @@ class TraceCollectionModule(CollectionStartpointMixin, TraceCollectionBase):
             self, owner=module, name="module:" + module.getFullName(), parent=None
         )
 
-    def onUsedModule(self, module_name, module_relpath):
-        assert type(module_name) is ModuleName, module_name
+        self.very_trusted_module_variables = very_trusted_module_variables
 
-        # TODO: Make users provide this through a method that has already
-        # done this.
-        module_relpath = relpath(module_relpath)
+    def getVeryTrustedModuleVariables(self):
+        return self.very_trusted_module_variables
 
-        self.owner.addUsedModule((module_name, module_relpath))
+    def updateVeryTrustedModuleVariables(self, very_trusted_module_variables):
+        result = self.very_trusted_module_variables != very_trusted_module_variables
 
-        module = getImportedModuleByNameAndPath(module_name, module_relpath)
-        addUsedModule(module)
+        self.very_trusted_module_variables = very_trusted_module_variables
+
+        return result
+
+
+# TODO: This should not exist, but be part of decision at the time these are collected.
+def areEmptyTraces(variable_traces):
+    """Do these traces contain any writes or accesses."""
+    # Many cases immediately return, that is how we do it here,
+    # pylint: disable=too-many-branches,too-many-return-statements
+
+    for variable_trace in variable_traces:
+        if variable_trace.isAssignTrace():
+            return False
+        elif variable_trace.isInitTrace():
+            return False
+        elif variable_trace.isDeletedTrace():
+            # A "del" statement can do this, and needs to prevent variable
+            # from being removed.
+
+            return False
+        elif variable_trace.isUninitTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isUnknownTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isEscapeTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isMergeTrace():
+            if variable_trace.getUsageCount():
+                # Checking definite is enough, the merges, we shall see
+                # them as well.
+                return False
+        elif variable_trace.isLoopTrace():
+            return False
+        else:
+            assert False, variable_trace
+
+    return True
+
+
+def areReadOnlyTraces(variable_traces):
+    """Do these traces contain any writes."""
+
+    # Many cases immediately return, that is how we do it here,
+    for variable_trace in variable_traces:
+        if variable_trace.isAssignTrace():
+            return False
+        elif variable_trace.isInitTrace():
+            pass
+        elif variable_trace.isDeletedTrace():
+            # A "del" statement can do this, and needs to prevent variable
+            # from being not released.
+
+            return False
+        elif variable_trace.isUninitTrace():
+            pass
+        elif variable_trace.isUnknownTrace():
+            return False
+        elif variable_trace.isEscapeTrace():
+            pass
+        elif variable_trace.isMergeTrace():
+            pass
+        elif variable_trace.isLoopTrace():
+            pass
+        else:
+            assert False, variable_trace
+
+    return True
