@@ -18,7 +18,7 @@
 """
 Plugins: Welcome to Nuitka! This is your shortest way to become part of it.
 
-This is to provide the base class for all plug-ins. Some of which are part of
+This is to provide the base class for all plugins. Some of which are part of
 proper Nuitka, and some of which are waiting to be created and submitted for
 inclusion by you.
 
@@ -26,16 +26,20 @@ The base class will serve as documentation. And it will point to examples of
 it being used.
 """
 
+import inspect
 import os
 import shutil
+import sys
+from collections import namedtuple
 
-from nuitka import Options, OutputDirectories
-from nuitka.Errors import NuitkaPluginError
-from nuitka.ModuleRegistry import addUsedModule
-from nuitka.SourceCodeReferences import fromFilename
+from nuitka.__past__ import getMetaClassBase
+from nuitka.freezer.IncludedEntryPoints import makeDllEntryPoint
+from nuitka.Options import isStandaloneMode
 from nuitka.Tracing import plugins_logger
-from nuitka.utils.FileOperations import putTextFileContents, relpath
+from nuitka.utils.Execution import NuitkaCalledProcessError, check_output
+from nuitka.utils.FileOperations import makePath
 from nuitka.utils.ModuleNames import ModuleName
+from nuitka.utils.SharedLibraries import locateDLL, locateDLLsInDirectory
 
 pre_modules = {}
 post_modules = {}
@@ -43,8 +47,8 @@ post_modules = {}
 warned_unused_plugins = set()
 
 
-class NuitkaPluginBase(object):
-    """Nuitka base class for all plug-ins.
+class NuitkaPluginBase(getMetaClassBase("Plugin")):
+    """Nuitka base class for all plugins.
 
     Derive your plugin from "NuitkaPluginBase" please.
     For instructions, see https://github.com/Nuitka/Nuitka/blob/orsiris/UserPlugin-Creation.rst
@@ -58,7 +62,7 @@ class NuitkaPluginBase(object):
     certain situations.
 
     A plugin in general must be enabled to be used by Nuitka. This happens by
-    specifying "--plugin-enable" (standard plugins) or by "--user-plugin" (user
+    specifying "--enable-plugin" (standard plugins) or by "--user-plugin" (user
     plugins) in the Nuitka command line. However, some plugins are always enabled
     and invisible to the user.
 
@@ -116,6 +120,11 @@ class NuitkaPluginBase(object):
         # Call group.add_option() here.
         pass
 
+    @classmethod
+    def getTagDataFileTagOptions(cls):
+        # Return tag_name, description tuples
+        return ()
+
     @classmethod
     def getPluginDefaultOptionValues(cls):
         """This method is used to get a values to use as defaults.
@@ -136,72 +145,6 @@ class NuitkaPluginBase(object):
 
         return result
 
-    def considerImplicitImports(self, module, signal_change):
-        """Provide additional modules to import implicitly when encountering the module.
-
-        Notes:
-            Better do not overload this method.
-            The standard plugin 'ImplicitImports.py' already contains MANY of these.
-            If you do have a new candidate, consider a PR to get it included there.
-
-        Args:
-            module: the module object
-            signal_change: bool
-        Returns:
-            None
-        """
-        from nuitka.importing.Importing import getModuleNameAndKindFromFilename
-
-        for full_name in self.getImplicitImports(module):
-            if type(full_name) in (tuple, list):
-                raise NuitkaPluginError(
-                    "Plugin %s needs to be change to only return modules names, not %r"
-                    % (self, full_name)
-                )
-
-            full_name = ModuleName(full_name)
-
-            try:
-                module_filename = self.locateModule(
-                    importing=module, module_name=full_name
-                )
-            except Exception:
-                self.warning(
-                    "Problem locating '%s' implicit imports '%s'."
-                    % (module.getFullName(), full_name)
-                )
-                raise
-
-            if module_filename is None:
-                if Options.isShowInclusion():
-                    self.info(
-                        "Implicit module '%s' suggested by '%s' not found."
-                        % (full_name, module.getFullName())
-                    )
-
-                continue
-
-            _module_name2, module_kind = getModuleNameAndKindFromFilename(
-                module_filename
-            )
-
-            # TODO: This should get back to plug-ins, they should be allowed to
-            # preempt or override the decision.
-            decision, reason = self.decideRecursion(
-                module_filename=module_filename,
-                module_name=full_name,
-                module_kind=module_kind,
-            )
-
-            if decision:
-                self.recurseTo(
-                    module_package=full_name.getPackageName(),
-                    module_filename=module_filename,
-                    module_kind=module_kind,
-                    reason=reason,
-                    signal_change=signal_change,
-                )
-
     def isRequiredImplicitImport(self, module, full_name):
         """Indicate whether an implicitly imported module should be accepted.
 
@@ -229,19 +172,6 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
-    # Provide fall-back for failed imports here.
-    module_aliases = {}
-
-    def considerFailedImportReferrals(self, module_name):
-        """Provide a dictionary of fallback imports for modules that failed to import.
-
-        Args:
-            module_name: name of module
-        Returns:
-            dict
-        """
-        return self.module_aliases.get(module_name)
-
     def onModuleSourceCode(self, module_name, source_code):
         """Inspect or modify source code.
 
@@ -295,62 +225,9 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return bytecode
 
-    @staticmethod
-    def _createTriggerLoadedModule(module, trigger_name, code):
-        """Create a "trigger" for a module to be imported.
-
-        Notes:
-            The trigger will incorpaorate the code to be prepended / appended.
-            Called by @onModuleDiscovered.
-
-        Args:
-            module: the module object (serves as dict key)
-            trigger_name: string ("-preload"/"-postload")
-            code: the code string
-
-        Returns
-            trigger_module
-        """
-        from nuitka.nodes.ModuleNodes import CompiledPythonModule
-        from nuitka.tree.Building import createModuleTree
-
-        from .Plugins import Plugins
-
-        module_name = ModuleName(module.getFullName() + trigger_name)
-        source_ref = fromFilename(module.getCompileTimeFilename() + trigger_name)
-
-        mode = Plugins.decideCompilation(module_name, source_ref)
-
-        trigger_module = CompiledPythonModule(
-            module_name=module_name,
-            is_top=False,
-            mode=mode,
-            future_spec=None,
-            source_ref=source_ref,
-        )
-
-        createModuleTree(
-            module=trigger_module,
-            source_ref=module.getSourceReference(),
-            source_code=code,
-            is_main=False,
-        )
-
-        if mode == "bytecode":
-            trigger_module.setSourceCode(code)
-
-        if Options.is_debug:
-            source_path = os.path.join(
-                OutputDirectories.getSourceDirectoryPath(), module_name + ".py"
-            )
-
-            putTextFileContents(filename=source_path, contents=code)
-
-        return trigger_module
-
     @staticmethod
     def createPreModuleLoadCode(module):
-        """Create code to prepend to a module.
+        """Create code to execute before importing a module.
 
         Notes:
             Called by @onModuleDiscovered.
@@ -358,25 +235,30 @@ class NuitkaPluginBase(object):
         Args:
             module: the module object
         Returns:
+            None (does not apply, default)
             tuple (code, documentary string)
+            tuple (code, documentary string, flags)
         """
         # Virtual method, pylint: disable=unused-argument
-        return None, None
+        return None
 
     @staticmethod
     def createPostModuleLoadCode(module):
-        """Create code to append to a module.
+        """Create code to execute after loading to a module.
 
         Notes:
             Called by @onModuleDiscovered.
 
         Args:
             module: the module object
+
         Returns:
+            None (does not apply, default)
             tuple (code, documentary string)
+            tuple (code, documentary string, flags)
         """
         # Virtual method, pylint: disable=unused-argument
-        return None, None
+        return None
 
     def onModuleDiscovered(self, module):
         """Called with a module to be loaded.
@@ -392,44 +274,8 @@ class NuitkaPluginBase(object):
         Returns:
             None
         """
-
-        full_name = module.getFullName()
-
-        pre_code, reason = self.createPreModuleLoadCode(module)
-
-        if pre_code:
-            # Note: We could find a way to handle this if needed.
-            if full_name in pre_modules:
-                plugins_logger.sysexit(
-                    "Error, conflicting pre module code from plug-ins for %s"
-                    % full_name
-                )
-
-            self.info("Injecting pre-module load code for module '%s':" % full_name)
-            for line in reason.split("\n"):
-                self.info("    " + line)
-
-            pre_modules[full_name] = self._createTriggerLoadedModule(
-                module=module, trigger_name="-preLoad", code=pre_code
-            )
-
-        post_code, reason = self.createPostModuleLoadCode(module)
-
-        if post_code:
-            # Note: We could find a way to handle this if needed.
-            if full_name is post_modules:
-                plugins_logger.sysexit(
-                    "Error, conflicting post module code from plug-ins for %s"
-                    % full_name
-                )
-
-            self.info("Injecting post-module load code for module '%s':" % full_name)
-            for line in reason.split("\n"):
-                self.info("    " + line)
-
-            post_modules[full_name] = self._createTriggerLoadedModule(
-                module=module, trigger_name="-postLoad", code=post_code
-            )
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
 
     def onModuleEncounter(self, module_filename, module_name, module_kind):
         """Help decide whether to include a module.
@@ -455,69 +301,84 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use
         return ()
 
+    def onModuleCompleteSet(self, module_set):
+        """Provide extra modules to the initial root module set.
+
+        Args:
+            module_set - tuple of module objects
+        Returns:
+            None
+        Notes:
+            You must not change anything, this is purely for warning
+            and error checking, and potentially for later stages to
+            prepare.
+        """
+
     @staticmethod
-    def locateModule(importing, module_name):
+    def locateModule(module_name):
         """Provide a filename / -path for a to-be-imported module.
 
         Args:
-            importing: module object
+            importing: module object that asked for it (tracing only)
             module_name: (str or ModuleName) full name of module
-            warn: (bool) True if required module
         Returns:
             filename for module
         """
-        from nuitka.importing import Importing
 
-        _module_package, module_filename, _finding = Importing.findModule(
-            importing=importing,
-            module_name=ModuleName(module_name),
-            parent_package=None,
-            level=-1,
-            warn=False,
+        from nuitka.importing.Importing import locateModule
+
+        _module_name, module_filename, _finding = locateModule(
+            module_name=ModuleName(module_name), parent_package=None, level=0
         )
 
         return module_filename
 
     @staticmethod
-    def decideRecursion(module_filename, module_name, module_kind):
-        """Decide whether Nuitka should recurse down to a given module.
+    def locateModules(module_name):
+        """Provide a filename / -path for a to-be-imported module.
 
         Args:
-            module_filename: filename
-            module_name: full module name
-            module_kind: one of "py" or "shlib" (shared library)
+            module_name: (str or ModuleName) full name of module
         Returns:
-            (decision, reason) where decision is either a bool or None, and reason is a string message.
+            list of ModuleName
         """
-        from nuitka.importing import Recursion
 
-        decision, reason = Recursion.decideRecursion(
-            module_filename=module_filename,
-            module_name=module_name,
-            module_kind=module_kind,
+        from nuitka.importing.Importing import locateModules
+
+        return locateModules(module_name)
+
+    @classmethod
+    def locateDLL(cls, dll_name):
+        """Locate a DLL by name."""
+        return locateDLL(dll_name)
+
+    @classmethod
+    def locateDLLsInDirectory(cls, directory):
+        """Locate all DLLs in a folder
+
+        Returns:
+            list of (filename, filename_relative, dll_extension)
+        """
+        return locateDLLsInDirectory(directory)
+
+    @classmethod
+    def makeDllEntryPoint(cls, source_path, dest_path, package_name):
+        """Create an entry point, as expected to be provided by getExtraDlls."""
+        return makeDllEntryPoint(
+            source_path=source_path, dest_path=dest_path, package_name=package_name
         )
 
-        return decision, reason
-
-    @staticmethod
-    def recurseTo(module_package, module_filename, module_kind, reason, signal_change):
-        from nuitka.importing import Recursion
-
-        imported_module, added_flag = Recursion.recurseTo(
-            module_package=module_package,
-            module_filename=module_filename,
-            module_relpath=relpath(module_filename),
-            module_kind=module_kind,
-            reason=reason,
-        )
-
-        addUsedModule(imported_module)
-
-        if added_flag:
-            signal_change(
-                "new_code", imported_module.getSourceReference(), "Recursed to module."
+    def reportFileCount(self, module_name, count, section=None):
+        if count:
+            msg = "Found %d %s DLLs from '%s' %sinstallation." % (
+                count,
+                "file" if count < 2 else "files",
+                "" if not section else section,
+                module_name.asString(),
             )
 
+            self.info(msg)
+
     def considerExtraDlls(self, dist_dir, module):
         """Provide a tuple of names of binaries to be included.
 
@@ -528,11 +389,13 @@ class NuitkaPluginBase(object):
             tuple
         """
         # TODO: This should no longer be here, as this API is obsolete, pylint: disable=unused-argument
-
         for included_entry_point in self.getExtraDlls(module):
-            # Copy to the dist directory.
-            target_path = (included_entry_point.dest_path,)
-            shutil.copyfile(included_entry_point.source_path, target_path)
+            # Copy to the dist directory, which normally should not be a plugin task, but is for now.
+            makePath(os.path.dirname(included_entry_point.dest_path))
+
+            shutil.copyfile(
+                included_entry_point.source_path, included_entry_point.dest_path
+            )
 
             yield included_entry_point
 
@@ -548,6 +411,17 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return ()
 
+    def getModuleSpecificDllPaths(self, module_name):
+        """Provide a list of directories, where DLLs should be searched for this package (or module).
+
+        Args:
+            module_name: name of a package or module, for which the DLL path addition applies.
+        Returns:
+            iterable of paths
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return ()
+
     def removeDllDependencies(self, dll_filename, dll_filenames):
         """Yield any DLLs / shared libraries not to be included in distribution.
 
@@ -639,7 +513,7 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return False
 
-    def decideCompilation(self, module_name, source_ref):
+    def decideCompilation(self, module_name):
         """Decide whether to compile a module (or just use its bytecode).
 
         Notes:
@@ -649,10 +523,9 @@ class NuitkaPluginBase(object):
 
         Args:
             module_name: name of module
-            source_ref: ???
 
         Returns:
-            "compiled" or "bytecode" or None (default)
+            "compiled" or "bytecode" or None (no opinion, use by default)
         """
         # Virtual method, pylint: disable=no-self-use,unused-argument
         return None
@@ -702,6 +575,21 @@ class NuitkaPluginBase(object):
         # Virtual method, pylint: disable=no-self-use
         return None
 
+    def getExtraLinkDirectories(self):
+        """Decide which link directories should be added.
+
+        Notes:
+            Directories provided multiple times, e.g. by multiple plugins are
+            only added once.
+
+        Returns:
+            None for no extra link directory, otherwise the name as a **str**
+            or an iterable of names of link directories.
+        """
+
+        # Virtual method, pylint: disable=no-self-use
+        return None
+
     def warnUnusedPlugin(self, message):
         """An inactive plugin may issue a warning if it believes this may be wrong.
 
@@ -712,9 +600,102 @@ class NuitkaPluginBase(object):
             warned_unused_plugins.add(self.plugin_name)
 
             plugins_logger.warning(
-                "Use '--plugin-enable=%s' for: %s" % (self.plugin_name, message)
+                "Use '--enable-plugin=%s' for: %s" % (self.plugin_name, message)
             )
 
+    def onDataComposerResult(self, blob_filename):
+        """Internal use only.
+
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
+    def encodeDataComposerName(self, data_name):
+        """Internal use only.
+
+        Returns:
+            None
+        """
+        # Virtual method, pylint: disable=no-self-use,unused-argument
+        return None
+
+    _runtime_information_cache = {}
+
+    def queryRuntimeInformationMultiple(self, info_name, setup_codes, values):
+        info_name = self.plugin_name + "_" + info_name
+
+        if info_name in self._runtime_information_cache:
+            return self._runtime_information_cache[info_name]
+
+        keys = []
+        query_codes = []
+
+        for key, value_expression in values:
+            keys.append(key)
+
+            query_codes.append("print(repr(%s))" % value_expression)
+            query_codes.append('print("-" * 27)')
+
+        if type(setup_codes) is str:
+            setup_codes = setup_codes.split("\n")
+
+        cmd = r"""\
+from __future__ import print_function
+from __future__ import absolute_import
+
+try:
+    %(setup_codes)s
+except ImportError:
+    import sys
+    sys.exit(38)
+%(query_codes)s
+""" % {
+            "setup_codes": "\n   ".join(setup_codes),
+            "query_codes": "\n".join(query_codes),
+        }
+
+        try:
+            feedback = check_output([sys.executable, "-c", cmd])
+        except NuitkaCalledProcessError as e:
+            if e.returncode == 38:
+                return None
+            raise
+
+        if str is not bytes:  # We want to work with strings, that's hopefully OK.
+            feedback = feedback.decode("utf8")
+
+        # Ignore Windows newlines difference.
+        feedback = [line.strip() for line in feedback.splitlines()]
+
+        if feedback.count("-" * 27) != len(keys):
+            self.sysexit(
+                "Error, mismatch in output retrieving %r information." % info_name
+            )
+
+        feedback = [line for line in feedback if line != "-" * 27]
+
+        NamedTupleResult = namedtuple(info_name, keys)
+
+        # We are being lazy here, the code is trusted, pylint: disable=eval-used
+        self._runtime_information_cache[info_name] = NamedTupleResult(
+            *(eval(value) for value in feedback)
+        )
+
+        return self._runtime_information_cache[info_name]
+
+    def queryRuntimeInformationSingle(self, setup_codes, value):
+        return self.queryRuntimeInformationMultiple(
+            info_name="temp_info_for_" + self.plugin_name.replace("-", "_"),
+            setup_codes=setup_codes,
+            values=(("key", value),),
+        ).key
+
+    @staticmethod
+    def onFunctionBodyParsing(module_name, function_name, body):
+        pass
+
     @classmethod
     def warning(cls, message):
         plugins_logger.warning(cls.plugin_name + ": " + message)
@@ -750,3 +731,22 @@ def replaceTriggerModule(old, new):
 
     if found is not None:
         post_modules[found] = new
+
+
+import functools
+
+
+def standalone_only(func):
+    """For plugins that have functionality that should be done in standalone mode only."""
+
+    @functools.wraps(func)
+    def wrapped(*args, **kwargs):
+        if isStandaloneMode():
+            return func(*args, **kwargs)
+        else:
+            if inspect.isgeneratorfunction(func):
+                return ()
+            else:
+                return None
+
+    return wrapped
