@@ -19,7 +19,7 @@
 
 Plugins: Welcome to Nuitka! This is your shortest way to become part of it.
 
-This is to provide the base class for all plug-ins. Some of which are part of
+This is to provide the base class for all plugins. Some of which are part of
 proper Nuitka, and some of which are waiting to be created and submitted for
 inclusion by you.
 
@@ -29,24 +29,26 @@ The base class in PluginBase will serve as documentation of available.
 
 import inspect
 import os
-import pkgutil
 import shutil
 from optparse import OptionConflictError, OptionGroup
 
 import nuitka.plugins.commercial
 import nuitka.plugins.standard
 from nuitka import Options, OutputDirectories
-from nuitka.__past__ import basestring  # pylint: disable=I0021,redefined-builtin
+from nuitka.__past__ import basestring, iter_modules
 from nuitka.build.DataComposerInterface import deriveModuleConstantsBlobName
 from nuitka.containers.odict import OrderedDict
 from nuitka.containers.oset import OrderedSet
-from nuitka.Errors import NuitkaPluginError
 from nuitka.freezer.IncludedEntryPoints import makeDllEntryPointOld
 from nuitka.ModuleRegistry import addUsedModule
 from nuitka.Tracing import plugins_logger, printLine
-from nuitka.utils.FileOperations import makePath, putTextFileContents, relpath
+from nuitka.utils.FileOperations import (
+    addFileExecutablePermission,
+    makePath,
+    putTextFileContents,
+)
 from nuitka.utils.Importing import importFileAsModule
-from nuitka.utils.ModuleNames import ModuleName
+from nuitka.utils.ModuleNames import ModuleName, checkModuleName
 
 from .PluginBase import NuitkaPluginBase, post_modules, pre_modules
 
@@ -136,12 +138,29 @@ def getPluginClass(plugin_name):
     return plugin_name2plugin_classes[plugin_name][0]
 
 
-def _loadPluginClassesFromPath(scan_path):
-    for loader, name, is_pkg in pkgutil.iter_modules(scan_path):
-        if is_pkg:
+def _addPluginClass(plugin_class, detector):
+    plugin_name = plugin_class.plugin_name
+
+    if plugin_name in plugin_name2plugin_classes:
+        plugins_logger.sysexit(
+            "Error, plugins collide by name %s: %s <-> %s"
+            % (plugin_name, plugin_class, plugin_name2plugin_classes[plugin_name])
+        )
+
+    plugin_name2plugin_classes[plugin_name] = (
+        plugin_class,
+        detector,
+    )
+
+
+def _loadPluginClassesFromPackage(scan_package):
+    scan_path = scan_package.__path__
+
+    for item in iter_modules(scan_path):
+        if item.ispkg:
             continue
 
-        module_loader = loader.find_module(name)
+        module_loader = item.module_finder.find_module(item.name)
 
         # Ignore bytecode only left overs.
         try:
@@ -153,17 +172,21 @@ def _loadPluginClassesFromPath(scan_path):
             pass
 
         try:
-            plugin_module = module_loader.load_module(name)
+            plugin_module = module_loader.load_module(item.name)
         except Exception:
             if Options.is_nondebug:
                 plugins_logger.warning(
                     "Problem loading plugin %r (%s), ignored. Use --debug to make it visible."
-                    % (name, module_loader.get_filename())
+                    % (item.name, module_loader.get_filename())
                 )
                 continue
 
             raise
 
+        # At least for Python2, this is not set properly, but we use it for package
+        # data loading.
+        plugin_module.__package__ = scan_package.__name__
+
         plugin_classes = set(
             obj
             for obj in plugin_module.__dict__.values()
@@ -176,8 +199,18 @@ def _loadPluginClassesFromPath(scan_path):
             if hasattr(plugin_class, "detector_for")
         ]
 
+        # First the ones with detectors.
         for detector in detectors:
             plugin_class = detector.detector_for
+
+            if detector.__name__.replace(
+                "NuitkaPluginDetector", ""
+            ) != plugin_class.__name__.replace("NuitkaPlugin", ""):
+                plugins_logger.warning(
+                    "Class names %r and %r do not match NuitkaPlugin* and NuitkaPluginDetector* naming convention."
+                    % (plugin_class.__name__, detector.__name__)
+                )
+
             assert detector.plugin_name is None, detector
             detector.plugin_name = plugin_class.plugin_name
 
@@ -190,13 +223,14 @@ def _loadPluginClassesFromPath(scan_path):
             plugin_classes.remove(detector)
             plugin_classes.remove(plugin_class)
 
-            plugin_name2plugin_classes[plugin_class.plugin_name] = (
-                plugin_class,
-                detector,
+            _addPluginClass(
+                plugin_class=plugin_class,
+                detector=detector,
             )
 
+        # Remaining ones have no detector.
         for plugin_class in plugin_classes:
-            plugin_name2plugin_classes[plugin_class.plugin_name] = plugin_class, None
+            _addPluginClass(plugin_class=plugin_class, detector=None)
 
 
 def loadStandardPluginClasses():
@@ -209,8 +243,8 @@ def loadStandardPluginClasses():
     Returns:
         None
     """
-    _loadPluginClassesFromPath(nuitka.plugins.standard.__path__)
-    _loadPluginClassesFromPath(nuitka.plugins.commercial.__path__)
+    _loadPluginClassesFromPackage(nuitka.plugins.standard)
+    _loadPluginClassesFromPackage(nuitka.plugins.commercial)
 
 
 class Plugins(object):
@@ -222,26 +256,34 @@ class Plugins(object):
 
         result = []
 
-        for full_name in plugin.getImplicitImports(module):
-            if type(full_name) in (tuple, list):
-                raise NuitkaPluginError(
-                    "Plugin %r needs to be change to only return modules names, not %r"
-                    % (plugin, full_name)
-                )
+        def iterateModuleNames(value):
+            for v in value:
+                if type(v) in (tuple, list):
+                    plugin.sysexit(
+                        "Plugin %r needs to be change to only return modules names, not %r (for %s)"
+                        % (plugin.plugin_name, v, module.getFullName().asString())
+                    )
 
-            full_name = ModuleName(full_name)
+                if inspect.isgenerator(v):
+                    for w in iterateModuleNames(v):
+                        yield w
 
+                    return
+
+                if not checkModuleName(v):
+                    plugin.sysexit(
+                        "Plugin %r returned an invalid module name, not %r (for %s)"
+                        % (plugin, v, module.getFullName().asString())
+                    )
+
+                yield ModuleName(v)
+
+        for full_name in iterateModuleNames(plugin.getImplicitImports(module)):
             try:
-                _module_package, module_filename, _finding = Importing.findModule(
-                    importing=module,
+                _module_name, module_filename, _finding = Importing.locateModule(
                     module_name=full_name,
                     parent_package=None,
                     level=-1,
-                    warn=False,
-                )
-
-                module_filename = plugin.locateModule(
-                    importing=module, module_name=full_name
                 )
             except Exception:
                 plugin.warning(
@@ -270,7 +312,7 @@ class Plugins(object):
         return result
 
     @staticmethod
-    def _reportImplicitImports(implicit_imports, signal_change):
+    def _reportImplicitImports(plugin, module, implicit_imports, signal_change):
         from nuitka.importing import Recursion
         from nuitka.importing.Importing import getModuleNameAndKindFromFilename
 
@@ -289,14 +331,19 @@ class Plugins(object):
             if decision:
                 imported_module = Recursion.recurseTo(
                     signal_change=signal_change,
-                    module_package=full_name.getPackageName(),
+                    module_name=full_name,
                     module_filename=module_filename,
-                    module_relpath=relpath(module_filename),
                     module_kind=module_kind,
                     reason=reason,
                 )
 
-                addUsedModule(imported_module)
+                addUsedModule(
+                    module=imported_module,
+                    using_module=module,
+                    usage_tag="plugin:" + plugin.plugin_name,
+                    reason=reason,
+                    source_ref=module.source_ref,
+                )
 
     @classmethod
     def considerImplicitImports(cls, module, signal_change):
@@ -309,6 +356,8 @@ class Plugins(object):
                 )
 
             cls._reportImplicitImports(
+                plugin=plugin,
+                module=module,
                 implicit_imports=cls.implicit_imports_cache[key],
                 signal_change=signal_change,
             )
@@ -317,10 +366,22 @@ class Plugins(object):
         full_name = module.getFullName()
 
         if full_name in pre_modules:
-            addUsedModule(pre_modules[full_name])
+            addUsedModule(
+                pre_modules[full_name],
+                using_module=module,
+                usage_tag="plugins",
+                reason="Not yet propagated by plugins.",
+                source_ref=module.source_ref,
+            )
 
         if full_name in post_modules:
-            addUsedModule(post_modules[full_name])
+            addUsedModule(
+                module=post_modules[full_name],
+                using_module=module,
+                usage_tag="plugins",
+                reason="Not yet propagated by plugins.",
+                source_ref=module.source_ref,
+            )
 
     @staticmethod
     def onStandaloneDistributionFinished(dist_dir):
@@ -380,6 +441,9 @@ class Plugins(object):
 
                     shutil.copyfile(extra_dll.source_path, extra_dll.dest_path)
 
+                    if extra_dll.executable:
+                        addFileExecutablePermission(extra_dll.dest_path)
+
                 result.append(extra_dll)
 
         return result
@@ -480,7 +544,7 @@ class Plugins(object):
                     os.path.dirname(module.getCompileTimeFilename()),
                     module_name.asPath() + ".py",
                 ),
-                module_package=module_name.getPackageName(),
+                module_name=module_name,
                 source_code=code,
                 is_top=False,
                 is_main=False,
@@ -649,6 +713,8 @@ class Plugins(object):
 
     @staticmethod
     def onModuleInitialSet():
+        """The initial set of root modules is complete, plugins may add more."""
+
         from nuitka.ModuleRegistry import addRootModule
 
         for plugin in getActivePlugins():
@@ -656,44 +722,39 @@ class Plugins(object):
                 addRootModule(module)
 
     @staticmethod
-    def considerFailedImportReferrals(module_name):
+    def onModuleCompleteSet():
+        """The final set of modules is determined, this is only for inspection, cannot change."""
+        from nuitka.ModuleRegistry import getDoneModules
+
+        # Make sure it's immutable.
+        module_set = tuple(getDoneModules())
+
         for plugin in getActivePlugins():
-            new_module_name = plugin.considerFailedImportReferrals(module_name)
-
-            if new_module_name is not None:
-                return ModuleName(new_module_name)
-
-        return None
+            plugin.onModuleCompleteSet(module_set)
 
     @staticmethod
-    def suppressUnknownImportWarning(importing, module_name):
+    def suppressUnknownImportWarning(importing, source_ref, module_name):
         """Let plugins decide whether to suppress import warnings for an unknown module.
 
         Notes:
             If all plugins return False or None, the return will be False, else True.
         Args:
             importing: the module which is importing "module_name"
+            source_ref: pointer to file source code or bytecode
             module_name: the module to be imported
         returns:
             True or False (default)
         """
-        if importing.isCompiledPythonModule() or importing.isPythonShlibModule():
-            importing_module = importing
-        else:
-            importing_module = importing.getParentModule()
-
         source_ref = importing.getSourceReference()
 
         for plugin in getActivePlugins():
-            if plugin.suppressUnknownImportWarning(
-                importing_module, module_name, source_ref
-            ):
+            if plugin.suppressUnknownImportWarning(importing, module_name, source_ref):
                 return True
 
         return False
 
     @staticmethod
-    def decideCompilation(module_name, source_ref):
+    def decideCompilation(module_name):
         """Let plugins decide whether to C compile a module or include as bytecode.
 
         Notes:
@@ -703,13 +764,13 @@ class Plugins(object):
             "compiled" (default) or "bytecode".
         """
         for plugin in getActivePlugins():
-            value = plugin.decideCompilation(module_name, source_ref)
+            value = plugin.decideCompilation(module_name)
 
             if value is not None:
                 assert value in ("compiled", "bytecode")
                 return value
 
-        return "compiled"
+        return None
 
     preprocessor_symbols = None
 
@@ -783,6 +844,25 @@ class Plugins(object):
 
         return cls.extra_link_libraries
 
+    extra_link_directories = None
+
+    @classmethod
+    def getExtraLinkDirectories(cls):
+        if cls.extra_link_directories is None:
+            cls.extra_link_directories = OrderedSet()
+
+            for plugin in getActivePlugins():
+                value = plugin.getExtraLinkDirectories()
+
+                if value is not None:
+                    if isinstance(value, basestring):
+                        cls.extra_link_directories.add(value)
+                    else:
+                        for dir_name in value:
+                            cls.extra_link_directories.add(dir_name)
+
+        return cls.extra_link_directories
+
     @classmethod
     def onDataComposerResult(cls, blob_filename):
         for plugin in getActivePlugins():
@@ -810,14 +890,14 @@ class Plugins(object):
         return name
 
     @classmethod
-    def onFunctionAssignmentParsed(cls, function_body, assign_node):
-        module_name = function_body.getParentModule().getFullName()
+    def onFunctionBodyParsing(cls, provider, function_name, body):
+        module_name = provider.getParentModule().getFullName()
 
         for plugin in getActivePlugins():
-            plugin.onFunctionAssignmentParsed(
+            plugin.onFunctionBodyParsing(
                 module_name=module_name,
-                function_body=function_body,
-                assign_node=assign_node,
+                function_name=function_name,
+                body=body,
             )
 
 
@@ -849,6 +929,7 @@ def isObjectAUserPluginBaseClass(obj):
             obj is not NuitkaPluginBase
             and issubclass(obj, NuitkaPluginBase)
             and not inspect.isabstract(obj)
+            and not obj.__name__.endswith("PluginBase")
         )
     except TypeError:
         return False
