@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -21,12 +21,11 @@
 
 import os
 import re
-import sys
 
 from nuitka import Options
 from nuitka.__past__ import StringIO
 from nuitka.plugins.PluginBase import NuitkaPluginBase
-from nuitka.utils.FileOperations import listDir
+from nuitka.utils.FileOperations import getFileContents, listDir
 
 # The main logic of this is from StackOverflow answer:
 # http://stackoverflow.com/questions/6772916/python-pmw-and-cx-freeze
@@ -36,36 +35,83 @@ from nuitka.utils.FileOperations import listDir
 # other files must appear later.  Files may be deleted if they are not
 # used.
 files = [
-    "Dialog", "TimeFuncs", "Balloon", "ButtonBox", "EntryField",
-    "Group", "LabeledWidget", "MainMenuBar", "MenuBar", "MessageBar",
-    "MessageDialog", "NoteBook", "OptionMenu", "PanedWidget", "PromptDialog",
-    "RadioSelect", "ScrolledCanvas", "ScrolledField", "ScrolledFrame",
-    "ScrolledListBox", "ScrolledText", "HistoryText", "SelectionDialog",
-    "TextDialog", "TimeCounter", "AboutDialog", "ComboBox", "ComboBoxDialog",
-    "Counter", "CounterDialog",
+    "Dialog",
+    "TimeFuncs",
+    "Balloon",
+    "ButtonBox",
+    "EntryField",
+    "Group",
+    "LabeledWidget",
+    "MainMenuBar",
+    "MenuBar",
+    "MessageBar",
+    "MessageDialog",
+    "NoteBook",
+    "OptionMenu",
+    "PanedWidget",
+    "PromptDialog",
+    "RadioSelect",
+    "ScrolledCanvas",
+    "ScrolledField",
+    "ScrolledFrame",
+    "ScrolledListBox",
+    "ScrolledText",
+    "HistoryText",
+    "SelectionDialog",
+    "TextDialog",
+    "TimeCounter",
+    "AboutDialog",
+    "ComboBox",
+    "ComboBoxDialog",
+    "Counter",
+    "CounterDialog",
 ]
 
 # Work out which version is being bundled.
 
+
 class NuitkaPluginPmw(NuitkaPluginBase):
     plugin_name = "pmw-freezer"
+    plugin_desc = "Required by the Pmw package"
+
+    def __init__(self, need_blt, need_color):
+        self.need_blt = need_blt
+        self.need_color = need_color
+
+    @classmethod
+    def addPluginCommandLineOptions(cls, group):
+        group.add_option(
+            "--include-pmw-blt",
+            action="store_true",
+            dest="need_blt",
+            default=False,
+            help="""\
+Should 'Pmw.Blt' not be included, Default is to include it.""",
+        )
+
+        group.add_option(
+            "--include-pmw-color",
+            action="store_true",
+            dest="need_color",
+            default=False,
+            help="""\
+Should 'Pmw.Color' not be included, Default is to include it.""",
+        )
 
     def onModuleSourceCode(self, module_name, source_code):
         if module_name == "Pmw":
-            pmw_path = self.locateModule(
-                importing      = None,
-                module_name    = "Pmw",
-                module_package = None,
-                warn           = True
-            )
+            pmw_path = self.locateModule(module_name=module_name)
 
             return self._packagePmw(pmw_path)
 
         return source_code
 
     def _packagePmw(self, pmw_path):
-        # From the "__init__.py" of Pwm:
-        def _hasLoader(dirname):  # @ReservedAssignment
+        self.info("Packaging Pmw into single module fpor freezing.")
+
+        # Algorithm is from the "__init__.py" of Pwm:
+
+        def _hasLoader(dirname):
             # Only accept Pmw_V_R_P with single digits, since ordering will
             # not work correctly with multiple digits (for example, Pmw_10_0
             # will be before Pmw_9_9).
@@ -86,10 +132,16 @@ class NuitkaPluginPmw(NuitkaPluginBase):
         candidates.reverse()
 
         if not candidates:
-            sys.exit("Error, cannot find any Pmw versions.")
+            self.sysexit("Error, cannot find any Pmw versions.")
+
+        self.info(
+            "Found the following Pmw version candidates %s." % ",".join(candidates)
+        )
 
         candidate = os.path.join(pmw_path, candidates[0], "lib")
-        version = candidates[0][4:].replace('_', '.')
+        version = candidates[0][4:].replace("_", ".")
+
+        self.info("Picked version %s." % version)
 
         return self._packagePmw2(candidate, version)
 
@@ -98,32 +150,32 @@ class NuitkaPluginPmw(NuitkaPluginBase):
             # Read the filename and modify it so that it can be bundled with the
             # other Pmw files.
             filename = "Pmw" + filename + ".py"
-            text = open(os.path.join(srcdir, filename)).read()
+            text = getFileContents(os.path.join(srcdir, filename))
             text = re.sub(r"import Pmw\>", "", text)
             text = re.sub("INITOPT = Pmw.INITOPT", "", text)
             text = re.sub(r"\<Pmw\.", "", text)
-            text = '\n' + ('#' * 70) + '\n' + "### File: " + filename + '\n' + text
+            text = "\n" + ("#" * 70) + "\n" + "### File: " + filename + "\n" + text
             return text
 
         # Code to import the Color module.
-        colorCode = """
-import PmwColor
+        color_code = """
+from . import PmwColor
 Color = PmwColor
 del PmwColor
 """
         # Code to import the Blt module.
-        bltCode = """
-import PmwBlt
+        blt_code = """
+from . import PmwBlt
 Blt = PmwBlt
 del PmwBlt
 """
         # Code used when not linking with PmwBlt.py.
-        ignoreBltCode = """
+        ignore_blt_code = """
 _bltImported = 1
 _bltbusyOK = 0
 """
         # Code to define the functions normally supplied by the dynamic loader.
-        extraCode = """
+        extra_code = """
 
 ### Loader functions:
 
@@ -131,11 +183,11 @@ _VERSION = '%s'
 
 def setversion(version):
     if version != _VERSION:
-        raise ValueError, 'Dynamic versioning not available'
+        raise ValueError('Dynamic versioning not available')
 
 def setalphaversions(*alpha_versions):
     if alpha_versions != ():
-        raise ValueError, 'Dynamic versioning not available'
+        raise ValueError('Dynamic versioning not available')
 
 def version(alpha = 0):
     if alpha:
@@ -151,28 +203,25 @@ def installedversions(alpha = 0):
 
 """
 
-        # Set this to 0 if you do not use any of the Pmw.Color functions:
-        # Set this to 0 if you do not use any of the Pmw.Blt functions:
-        needBlt = not self.getPluginOptionBool("blt", True)
-        needColor = not self.getPluginOptionBool("color", True)
-
         outfile = StringIO()
 
-        if needColor:
-            outfile.write(colorCode)
+        if self.need_color:
+            outfile.write(color_code)
 
-        if needBlt:
-            outfile.write(bltCode)
+        if self.need_blt:
+            outfile.write(blt_code)
 
-        outfile.write(extraCode % version)
+        outfile.write(extra_code % version)
 
         # Specially handle PmwBase.py filename:
         text = mungeFile("Base")
+        text = re.sub("from . import PmwLogicalFont", "", text)
         text = re.sub("import PmwLogicalFont", "", text)
         text = re.sub("PmwLogicalFont._font_initialise", "_font_initialise", text)
         outfile.write(text)
-        if not needBlt:
-            outfile.write(ignoreBltCode)
+
+        if not self.need_blt:
+            outfile.write(ignore_blt_code)
 
         files.append("LogicalFont")
 
@@ -184,10 +233,10 @@ def installedversions(alpha = 0):
 
 
 class NuitkaPluginDetectorPmw(NuitkaPluginBase):
-    plugin_name = "pmw-freezer"
+    detector_for = NuitkaPluginPmw
 
-    @staticmethod
-    def isRelevant():
+    @classmethod
+    def isRelevant(cls):
         return Options.isStandaloneMode()
 
     def onModuleDiscovered(self, module):
