@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -31,7 +31,11 @@ flexible.
 
 from nuitka import Variables
 from nuitka.PythonVersions import python_version
-from nuitka.utils.InstanceCounters import counted_del, counted_init
+from nuitka.utils.InstanceCounters import (
+    counted_del,
+    counted_init,
+    isCountingInstances,
+)
 
 
 class TooManyArguments(Exception):
@@ -49,26 +53,45 @@ class ParameterSpec(object):
     # variables, pylint: disable=too-many-instance-attributes
 
     __slots__ = (
-        "name", "owner", "normal_args", "normal_variables",
-        "list_star_arg", "dict_star_arg", "list_star_variable",
-        "dict_star_variable", "default_count", "kw_only_args",
-        "kw_only_variables"
+        "name",
+        "owner",
+        "normal_args",
+        "normal_variables",
+        "list_star_arg",
+        "is_list_star_arg_single",
+        "dict_star_arg",
+        "list_star_variable",
+        "dict_star_variable",
+        "default_count",
+        "kw_only_args",
+        "kw_only_variables",
+        "pos_only_args",
+        "pos_only_variables",
     )
 
     @counted_init
-    def __init__(self, ps_name, ps_normal_args, ps_kw_only_args, ps_list_star_arg,
-                 ps_dict_star_arg, ps_default_count):
+    def __init__(
+        self,
+        ps_name,
+        ps_normal_args,
+        ps_pos_only_args,
+        ps_kw_only_args,
+        ps_list_star_arg,
+        ps_dict_star_arg,
+        ps_default_count,
+        ps_is_list_star_arg_single=False,
+    ):
         if type(ps_normal_args) is str:
             if ps_normal_args == "":
                 ps_normal_args = ()
             else:
-                ps_normal_args = ps_normal_args.split(',')
+                ps_normal_args = ps_normal_args.split(",")
 
         if type(ps_kw_only_args) is str:
             if ps_kw_only_args == "":
                 ps_kw_only_args = ()
             else:
-                ps_kw_only_args = ps_kw_only_args.split(',')
+                ps_kw_only_args = ps_kw_only_args.split(",")
 
         assert None not in ps_normal_args
 
@@ -78,12 +101,17 @@ class ParameterSpec(object):
         self.normal_args = tuple(ps_normal_args)
         self.normal_variables = None
 
-        assert ps_list_star_arg is None or type(ps_list_star_arg) is str, \
-          ps_list_star_arg
-        assert ps_dict_star_arg is None or type(ps_dict_star_arg) is str, \
-          ps_dict_star_arg
+        assert (
+            ps_list_star_arg is None or type(ps_list_star_arg) is str
+        ), ps_list_star_arg
+        assert (
+            ps_dict_star_arg is None or type(ps_dict_star_arg) is str
+        ), ps_dict_star_arg
+
+        assert type(ps_is_list_star_arg_single) is bool, ps_is_list_star_arg_single
 
         self.list_star_arg = ps_list_star_arg if ps_list_star_arg else None
+        self.is_list_star_arg_single = ps_is_list_star_arg_single
         self.dict_star_arg = ps_dict_star_arg if ps_dict_star_arg else None
 
         self.list_star_variable = None
@@ -94,26 +122,36 @@ class ParameterSpec(object):
         self.kw_only_args = tuple(ps_kw_only_args)
         self.kw_only_variables = None
 
-    __del__ = counted_del()
+        self.pos_only_args = tuple(ps_pos_only_args)
+        self.pos_only_variables = None
+
+    if isCountingInstances():
+        __del__ = counted_del()
 
     def makeClone(self):
         return ParameterSpec(
-            ps_name          = self.name,
-            ps_normal_args   = self.normal_args,
-            ps_kw_only_args  = self.kw_only_args,
-            ps_list_star_arg = self.list_star_arg,
-            ps_dict_star_arg = self.dict_star_arg,
-            ps_default_count = self.default_count
+            ps_name=self.name,
+            ps_normal_args=self.normal_args,
+            ps_pos_only_args=self.pos_only_args,
+            ps_kw_only_args=self.kw_only_args,
+            ps_list_star_arg=self.list_star_arg,
+            ps_dict_star_arg=self.dict_star_arg,
+            ps_default_count=self.default_count,
         )
 
     def getDetails(self):
         return {
-            "ps_name"          : self.name,
-            "ps_normal_args"   : ','.join(self.normal_args),
-            "ps_kw_only_args"  : ','.join(self.kw_only_args),
-            "ps_list_star_arg" : self.list_star_arg if self.list_star_arg is not None else "",
-            "ps_dict_star_arg" : self.dict_star_arg if self.dict_star_arg is not None else "",
-            "ps_default_count" : self.default_count
+            "ps_name": self.name,
+            "ps_normal_args": ",".join(self.normal_args),
+            "ps_pos_only_args": self.pos_only_args,
+            "ps_kw_only_args": ",".join(self.kw_only_args),
+            "ps_list_star_arg": self.list_star_arg
+            if self.list_star_arg is not None
+            else "",
+            "ps_dict_star_arg": self.dict_star_arg
+            if self.dict_star_arg is not None
+            else "",
+            "ps_default_count": self.default_count,
         }
 
     def checkParametersValid(self):
@@ -127,11 +165,11 @@ class ParameterSpec(object):
         return None
 
     def __repr__(self):
-        parts = [
-            str(normal_arg)
-            for normal_arg
-            in self.normal_args
-        ]
+        parts = [str(normal_arg) for normal_arg in self.pos_only_args]
+        if parts:
+            parts.append("/")
+
+        parts += [str(normal_arg) for normal_arg in self.normal_args]
 
         if self.list_star_arg is not None:
             parts.append("*%s" % self.list_star_arg)
@@ -140,13 +178,10 @@ class ParameterSpec(object):
             parts.append("**%s" % self.dict_star_variable)
 
         if parts:
-            return "<ParameterSpec '%s'>" % ','.join(parts)
+            return "<ParameterSpec '%s'>" % ",".join(parts)
         else:
             return "<NoParameters>"
 
-    def getArgumentCount(self):
-        return len(self.normal_args)
-
     def setOwner(self, owner):
         if self.owner is not None:
             return
@@ -157,8 +192,7 @@ class ParameterSpec(object):
         for normal_arg in self.normal_args:
             if type(normal_arg) is str:
                 normal_variable = Variables.ParameterVariable(
-                    owner          = self.owner,
-                    parameter_name = normal_arg
+                    owner=self.owner, parameter_name=normal_arg
                 )
             else:
                 assert False, normal_arg
@@ -167,45 +201,40 @@ class ParameterSpec(object):
 
         if self.list_star_arg:
             self.list_star_variable = Variables.ParameterVariable(
-                owner          = owner,
-                parameter_name = self.list_star_arg
+                owner=owner, parameter_name=self.list_star_arg
             )
         else:
             self.list_star_variable = None
 
         if self.dict_star_arg:
             self.dict_star_variable = Variables.ParameterVariable(
-                owner          = owner,
-                parameter_name = self.dict_star_arg
+                owner=owner, parameter_name=self.dict_star_arg
             )
         else:
             self.dict_star_variable = None
 
         self.kw_only_variables = [
-            Variables.ParameterVariable(
-                owner          = self.owner,
-                parameter_name = kw_only_arg
-            )
-            for kw_only_arg in
-            self.kw_only_args
+            Variables.ParameterVariable(owner=self.owner, parameter_name=kw_only_arg)
+            for kw_only_arg in self.kw_only_args
+        ]
+
+        self.pos_only_variables = [
+            Variables.ParameterVariable(owner=self.owner, parameter_name=pos_only_arg)
+            for pos_only_arg in self.pos_only_args
         ]
 
     def getDefaultCount(self):
         return self.default_count
 
-    def getPositionalOnlyCount(self):
-        # Virtual method, pylint: disable=no-self-use
-        return 0
-
     def hasDefaultParameters(self):
         return self.getDefaultCount() > 0
 
     def getTopLevelVariables(self):
-        return self.normal_variables + self.kw_only_variables
+        return self.pos_only_variables + self.normal_variables + self.kw_only_variables
 
     def getAllVariables(self):
-        result = list(self.normal_variables)
-
+        result = list(self.pos_only_variables)
+        result += self.normal_variables
         result += self.kw_only_variables
 
         if self.list_star_variable is not None:
@@ -217,7 +246,7 @@ class ParameterSpec(object):
         return result
 
     def getParameterNames(self):
-        result = list(self.normal_args)
+        result = list(self.pos_only_args + self.normal_args)
 
         result += self.kw_only_args
 
@@ -232,6 +261,9 @@ class ParameterSpec(object):
     def getStarListArgumentName(self):
         return self.list_star_arg
 
+    def isStarListSingleArg(self):
+        return self.is_list_star_arg_single
+
     def getListStarArgVariable(self):
         return self.list_star_variable
 
@@ -252,7 +284,10 @@ class ParameterSpec(object):
         return "%s() takes no keyword arguments" % self.name
 
     def getArgumentNames(self):
-        return self.normal_args
+        return self.pos_only_args + self.normal_args
+
+    def getArgumentCount(self):
+        return len(self.normal_args) + len(self.pos_only_args)
 
     def getKwOnlyParameterNames(self):
         return self.kw_only_args
@@ -260,10 +295,42 @@ class ParameterSpec(object):
     def getKwOnlyParameterCount(self):
         return len(self.kw_only_args)
 
+    def getPosOnlyParameterCount(self):
+        return len(self.pos_only_args)
+
+
+def matchCall(
+    func_name,
+    args,
+    kw_only_args,
+    star_list_arg,
+    star_list_single_arg,
+    star_dict_arg,
+    num_defaults,
+    num_posonly,
+    positional,
+    pairs,
+    improved=False,
+):
+    """Match a call arguments to a signature.
+
+    Args:
+        func_name - Name of the function being matched, used to construct exception texts.
+        args - normal argument names
+        kw_only_args -  keyword only argument names (Python3)
+        star_list_arg - name of star list argument if any
+        star_dict_arg - name of star dict argument if any
+        num_defaults - amount of arguments that have default values
+        num_posonly - amount of arguments that must be given by position
+        positional - tuple of argument values given for simulated call
+        pairs - tuple of pairs arg argument name and argument values
+        improved - (bool) should we give better errors than CPython or not.
+    Returns:
+        Dictionary of argument name to value mappings
+    Notes:
+        Based loosely on "inspect.getcallargs" with corrections.
+    """
 
-# Note: Based loosely on "inspect.getcallargs" with corrections.
-def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
-              num_posonly, positional, pairs, improved = False):
     # This is of incredible code complexity, but there really is no other way to
     # express this with less statements, branches, or variables.
     # pylint: disable=too-many-branches,too-many-locals,too-many-statements
@@ -281,7 +348,7 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
     def assign(arg, value):
         if type(arg) is str:
             # Normal case:
-            result[ arg ] = value
+            result[arg] = value
         else:
             # Tuple argument case:
 
@@ -294,10 +361,8 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
                 except StopIteration:
                     raise TooManyArguments(
                         ValueError(
-                            "need more than %d %s to unpack" % (
-                                i,
-                                "values" if i > 1 else "value"
-                            )
+                            "need more than %d %s to unpack"
+                            % (i, "values" if i > 1 else "value")
                         )
                     )
 
@@ -310,9 +375,7 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
             except StopIteration:
                 pass
             else:
-                raise TooManyArguments(
-                    ValueError("too many values to unpack")
-                )
+                raise TooManyArguments(ValueError("too many values to unpack"))
 
     def isAssigned(arg):
         if type(arg) is str:
@@ -328,127 +391,119 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
         assign(arg, value)
 
     # Python3 does this check earlier.
-    if python_version >= 300 and not star_dict_arg:
+    if python_version >= 0x300 and not star_dict_arg:
         for pair in pairs:
             try:
-                arg_index = args.index(pair[0])
+                arg_index = (args + kw_only_args).index(pair[0])
             except ValueError:
-                if improved or python_version >= 370:
+                if improved or python_version >= 0x370:
                     message = "'%s' is an invalid keyword argument for %s()" % (
                         pair[0],
-                        func_name
+                        func_name,
                     )
                 else:
-                    message = "'%s' is an invalid keyword argument for this function" % pair[0]
+                    message = (
+                        "'%s' is an invalid keyword argument for this function"
+                        % pair[0]
+                    )
 
-                raise TooManyArguments(
-                    TypeError(message)
-                )
+                raise TooManyArguments(TypeError(message))
             else:
                 if arg_index < num_posonly:
                     message = "'%s' is an invalid keyword argument for %s()" % (
                         pair[0],
-                        func_name
-                    )
-
-                    raise TooManyArguments(
-                        TypeError(message)
+                        func_name,
                     )
 
+                    raise TooManyArguments(TypeError(message))
 
     if star_list_arg:
         if num_pos > num_args:
-            assign(star_list_arg, positional[ -(num_pos-num_args) : ])
+            value = positional[-(num_pos - num_args) :]
+            assign(star_list_arg, value)
+
+            if star_list_single_arg:
+                if len(value) > 1:
+                    raise TooManyArguments(
+                        TypeError(
+                            "%s expected at most 1 arguments, got %d"
+                            % (func_name, len(value))
+                        )
+                    )
         else:
             assign(star_list_arg, ())
     elif 0 < num_args < num_total:
+        # Special case for no default values.
         if num_defaults == 0:
+            # Special cases text for one argument.
             if num_args == 1:
                 raise TooManyArguments(
                     TypeError(
-                        "%s() takes exactly one argument (%d given)" % (
-                            func_name,
-                            num_total
-                        )
-                    )
-                )
-            else:
-                raise TooManyArguments(
-                    TypeError(
-                        "%s expected %d arguments, got %d" % (
-                            func_name,
-                            num_args,
-                            num_total
-                        )
+                        "%s() takes exactly one argument (%d given)"
+                        % (func_name, num_total)
                     )
                 )
 
-        else:
             raise TooManyArguments(
                 TypeError(
-                    "%s() takes at most %d %s (%d given)" % (
-                        func_name,
-                        num_args,
-                        "argument" if num_args == 1 else "arguments",
-                        num_total
-                    )
+                    "%s expected %d arguments, got %d"
+                    % (func_name, num_args, num_total)
                 )
             )
+
+        raise TooManyArguments(
+            TypeError(
+                "%s() takes at most %d %s (%d given)"
+                % (
+                    func_name,
+                    num_args,
+                    "argument" if num_args == 1 else "arguments",
+                    num_total,
+                )
+            )
+        )
     elif num_args == 0 and num_total:
         if star_dict_arg:
             if num_pos:
                 # Could use num_pos, but Python also uses num_total.
                 raise TooManyArguments(
                     TypeError(
-                        "%s() takes exactly 0 arguments (%d given)" % (
-                            func_name,
-                            num_total
-                        )
+                        "%s() takes exactly 0 arguments (%d given)"
+                        % (func_name, num_total)
                     )
                 )
         else:
             raise TooManyArguments(
-                TypeError(
-                    "%s() takes no arguments (%d given)" % (
-                        func_name,
-                        num_total
-                    )
-                )
+                TypeError("%s() takes no arguments (%d given)" % (func_name, num_total))
             )
 
-    named_argument_names = [
-        pair[0]
-        for pair in
-        pairs
-    ]
+    named_argument_names = [pair[0] for pair in pairs]
 
-    for arg in args:
+    for arg in args + kw_only_args:
         if type(arg) is str and arg in named_argument_names:
             if isAssigned(arg):
                 raise TooManyArguments(
                     TypeError(
-                        "%s() got multiple values for keyword argument '%s'" % (
-                            func_name,
-                            arg
-                        )
+                        "%s() got multiple values for keyword argument '%s'"
+                        % (func_name, arg)
                     )
                 )
-            else:
-                new_pairs = []
 
-                for pair in pairs:
-                    if arg == pair[0]:
-                        assign(arg, pair[1])
-                    else:
-                        new_pairs.append(pair)
+            new_pairs = []
 
-                assert len(new_pairs) == len(pairs) - 1
+            for pair in pairs:
+                if arg == pair[0]:
+                    assign(arg, pair[1])
+                else:
+                    new_pairs.append(pair)
 
-                pairs = new_pairs
+            assert len(new_pairs) == len(pairs) - 1
+
+            pairs = new_pairs
 
     # Fill in any missing values with the None to indicate "default".
     if num_defaults > 0:
-        for arg in args[ -num_defaults : ]:
+        for arg in (kw_only_args + args)[-num_defaults:]:
             if not isAssigned(arg):
                 assign(arg, None)
 
@@ -460,65 +515,78 @@ def matchCall(func_name, args, star_list_arg, star_dict_arg, num_defaults,
         if improved:
             message = "%s() got an unexpected keyword argument '%s'" % (
                 func_name,
-                unexpected
+                unexpected,
             )
         else:
-            message = "'%s' is an invalid keyword argument for this function" % unexpected
+            message = (
+                "'%s' is an invalid keyword argument for this function" % unexpected
+            )
 
-        raise TooManyArguments(
-            TypeError(message)
-        )
+        raise TooManyArguments(TypeError(message))
 
-    unassigned = num_args - len(
-        [
-            arg
-            for arg in args
-            if isAssigned(arg)
-        ]
-    )
+    unassigned = num_args - len([arg for arg in args if isAssigned(arg)])
 
     if unassigned:
         num_required = num_args - num_defaults
 
+        # Special case required arguments.
         if num_required > 0 or improved:
             if num_defaults == 0 and num_args != 1:
                 raise TooManyArguments(
                     TypeError(
-                        "%s expected %d arguments, got %d" % (
-                            func_name,
-                            num_args,
-                            num_total
-                        )
+                        "%s expected %d arguments, got %d"
+                        % (func_name, num_args, num_total)
                     )
                 )
 
             if num_required == 1:
-                arg_desc = "1 argument" if python_version < 350 else "one argument"
+                arg_desc = "1 argument" if python_version < 0x350 else "one argument"
             else:
                 arg_desc = "%d arguments" % num_required
 
             raise TooManyArguments(
                 TypeError(
-                    "%s() takes %s %s (%d given)" % (
+                    "%s() takes %s %s (%d given)"
+                    % (
                         func_name,
                         "at least" if num_defaults > 0 else "exactly",
                         arg_desc,
-                        num_total
-                    )
-                )
-            )
-        else:
-            raise TooManyArguments(
-                TypeError(
-                    "%s expected %s%s, got %d" % (
-                        func_name,
-                        ( "at least " if python_version < 300 else "" )
-                            if num_defaults > 0
-                        else "exactly ",
-                        "%d arguments" % num_required,
-                        num_total
+                        num_total,
                     )
                 )
             )
 
+        raise TooManyArguments(
+            TypeError(
+                "%s expected %s%s, got %d"
+                % (
+                    func_name,
+                    ("at least " if python_version < 0x300 else "")
+                    if num_defaults > 0
+                    else "exactly ",
+                    "%d arguments" % num_required,
+                    num_total,
+                )
+            )
+        )
+
+    unassigned = len(kw_only_args) - len(
+        [arg for arg in kw_only_args if isAssigned(arg)]
+    )
+    if unassigned:
+        raise TooManyArguments(
+            TypeError(
+                "%s missing %d required keyword-only argument%s: %s"
+                % (
+                    func_name,
+                    unassigned,
+                    "s" if unassigned > 1 else "",
+                    " and ".join(
+                        "'%s'"
+                        % [arg.getName() for arg in kw_only_args if not isAssigned(arg)]
+                    ),
+                )
+            )
+        )
+
     return result
