@@ -17,33 +17,42 @@
 #
 """ Common test infrastructure functions. To be used by test runners. """
 
-from __future__ import print_function
-
 import ast
 import atexit
 import gc
 import hashlib
 import os
-import re
 import shutil
-import subprocess
+import signal
 import sys
 import tempfile
+import threading
+import time
 from contextlib import contextmanager
 from optparse import OptionGroup, OptionParser
 
-from nuitka.freezer.DependsExe import getDependsExePath
+from nuitka.__past__ import subprocess
+from nuitka.PythonVersions import (
+    getPartiallySupportedPythonVersions,
+    getSupportedPythonVersions,
+)
 from nuitka.Tracing import OurLogger, my_print
 from nuitka.tree.SourceReading import readSourceCodeFromFilename
 from nuitka.utils.AppDirs import getCacheDir
-from nuitka.utils.Execution import check_output, withEnvironmentVarOverriden
+from nuitka.utils.Execution import check_output, getNullInput, getNullOutput
 from nuitka.utils.FileOperations import (
+    areSamePaths,
+    getExternalUsePath,
     getFileContentByLine,
     getFileContents,
     getFileList,
+    isPathBelowOrSameAs,
     makePath,
+    openTextFile,
     removeDirectory,
 )
+from nuitka.utils.Jinja2 import getTemplate
+from nuitka.utils.Utils import getOS
 
 from .SearchModes import (
     SearchModeAll,
@@ -131,10 +140,10 @@ print("Anaconda" if os.path.exists(os.path.join(sys.prefix, 'conda-meta')) else
     _python_vendor = version_output.split(b"\n")[3].strip()
 
     if str is not bytes:
-        _python_version_str = _python_version_str.decode("utf-8")
-        _python_arch = _python_arch.decode("utf-8")
-        _python_executable = _python_executable.decode("utf-8")
-        _python_vendor = _python_vendor.decode("utf-8")
+        _python_version_str = _python_version_str.decode("utf8")
+        _python_arch = _python_arch.decode("utf8")
+        _python_executable = _python_executable.decode("utf8")
+        _python_vendor = _python_vendor.decode("utf8")
 
     assert type(_python_version_str) is str, repr(_python_version_str)
     assert type(_python_arch) is str, repr(_python_arch)
@@ -194,11 +203,9 @@ def convertUsing2to3(path, force=False):
     command = [os.environ["PYTHON"], "-m", "py_compile", path]
 
     if not force:
-        with open(path) as source_file:
-            if "xrange" not in source_file.read():
-                with open(os.devnull, "w") as stderr:
-                    if check_result(command, stderr=stderr):
-                        return path, False
+        if "xrange" not in getFileContents(path):
+            if check_result(command, stderr=getNullOutput()):
+                return path, False
 
     filename = os.path.basename(path)
 
@@ -227,22 +234,20 @@ def convertUsing2to3(path, force=False):
 
     command += ("-w", "-n", "--no-diffs", new_path)
 
-    with open(os.devnull, "w") as devnull:
-        try:
-            check_output(command, stderr=devnull)
+    try:
+        check_output(command, stderr=getNullOutput())
 
-        except subprocess.CalledProcessError:
-            if os.name == "nt":
-                raise
+    except subprocess.CalledProcessError:
+        if os.name == "nt":
+            raise
 
-            command[0:3] = ["2to3"]
+        command[0:3] = ["2to3"]
 
-            check_output(command, stderr=devnull)
+        check_output(command, stderr=getNullOutput())
 
-    with open(new_path) as result_file:
-        data = result_file.read()
+    data = getFileContents(new_path)
 
-    with open(new_path, "w") as result_file:
+    with openTextFile(new_path, "w") as result_file:
         result_file.write("__file__ = %r\n" % os.path.abspath(path))
         result_file.write(data)
 
@@ -280,11 +285,11 @@ def decideFilenameVersionSkip(filename):
         return False
 
     # Skip tests that require Python 2 at maximum.
-    if filename.endswith("_2.py") and _python_version > (3,):
+    if filename.endswith("_2.py") and _python_version >= (3,):
         return False
 
     # Skip tests that require Python 3.7 at maximum.
-    if filename.endswith("_37.py") and _python_version > (3, 8):
+    if filename.endswith("_37.py") and _python_version >= (3, 8):
         return False
 
     # Skip tests that require Python 3.2 at least.
@@ -319,12 +324,26 @@ def decideFilenameVersionSkip(filename):
     if filename.endswith("39.py") and _python_version < (3, 9):
         return False
 
+    # Skip tests that require Python 3.10 at least.
+    if filename.endswith("310.py") and _python_version < (3, 10):
+        return False
+
     return True
 
 
 def decideNeeds2to3(filename):
     return _python_version >= (3,) and not filename.endswith(
-        ("32.py", "33.py", "34.py", "35.py", "36.py", "37.py", "38.py", "39.py")
+        (
+            "32.py",
+            "33.py",
+            "34.py",
+            "35.py",
+            "36.py",
+            "37.py",
+            "38.py",
+            "39.py",
+            "310.py",
+        )
     )
 
 
@@ -383,7 +402,7 @@ def compareWithCPython(
         if os.path.exists(compare_with_cpython):
             command = [sys.executable, compare_with_cpython, path, "silent"]
         else:
-            sys.exit("Error, cannot find Nuitka comparison runner.")
+            test_logger.sysexit("Error, cannot locate Nuitka comparison runner.")
 
     if extra_flags is not None:
         command += extra_flags
@@ -411,8 +430,7 @@ def compareWithCPython(
         os.unlink(path)
 
     if result == 2:
-        sys.stderr.write("Interrupted, with CTRL-C\n")
-        sys.exit(2)
+        test_logger.sysexit("Interrupted, with CTRL-C\n", exit_code=2)
 
 
 def checkCompilesNotWithCPython(dirname, filename, search_mode):
@@ -435,8 +453,7 @@ def checkCompilesNotWithCPython(dirname, filename, search_mode):
 def checkSucceedsWithCPython(filename):
     command = [_python_executable, filename]
 
-    with open(os.devnull, "w") as devnull:
-        result = subprocess.call(command, stdout=devnull, stderr=subprocess.STDOUT)
+    result = subprocess.call(command, stdout=getNullOutput(), stderr=subprocess.STDOUT)
 
     return result == 0
 
@@ -465,330 +482,32 @@ def hasDebugPython():
     return False
 
 
-def isExecutableCommand(command):
-    path = os.environ["PATH"]
+def displayRuntimeTraces(logger, path):
+    if not os.path.exists(path):
+        # TODO: Have a logger package passed.
+        logger.sysexit("Error, cannot find %r (%r)." % (path, os.path.abspath(path)))
 
-    suffixes = (".exe",) if os.name == "nt" else ("",)
+    path = os.path.abspath(path)
 
-    for part in path.split(os.pathsep):
-        if not part:
-            continue
-
-        for suffix in suffixes:
-            if os.path.isfile(os.path.join(part, command + suffix)):
-                return True
-
-    return False
-
-
-def getRuntimeTraceOfLoadedFiles(path, trace_stderr=True):
-    """ Returns the files loaded when executing a binary. """
-
-    # This will make a crazy amount of work,
-    # pylint: disable=I0021,too-many-branches,too-many-locals,too-many-statements
-
-    result = []
+    # TODO: Merge code for building command with below function, this is otherwise
+    # horribly bad.
 
     if os.name == "posix":
-        if sys.platform == "darwin" or sys.platform.startswith("freebsd"):
-            if not isExecutableCommand("dtruss"):
-                sys.exit(
-                    """\
-Error, needs 'dtruss' on your system to scan used libraries."""
-                )
-
-            if not isExecutableCommand("sudo"):
-                sys.exit(
-                    """\
-Error, needs 'sudo' on your system to scan used libraries."""
-                )
-
-            args = ("sudo", "dtruss", "-t", "open", path)
+        # Run with traces to help debugging, specifically in CI environment.
+        if getOS() in ("Darwin", "FreeBSD"):
+            test_logger.info("dtruss:")
+            os.system("sudo dtruss %s" % path)
         else:
-            if not isExecutableCommand("strace"):
-                sys.exit(
-                    """\
-Error, needs 'strace' on your system to scan used libraries."""
-                )
-
-            args = (
-                "strace",
-                "-e",
-                "file",
-                "-s4096",  # Some paths are truncated otherwise.
-                path,
-            )
-
-        # Ensure executable is not polluted with third party stuff,
-        # tests may fail otherwise due to unexpected libs being loaded
-        with withEnvironmentVarOverriden("LD_PRELOAD", None):
-            process = subprocess.Popen(
-                args=args, stdout=subprocess.PIPE, stderr=subprocess.PIPE
-            )
-
-            _stdout_strace, stderr_strace = process.communicate()
-            exit_strace = process.returncode
-
-            if exit_strace != 0:
-                if str is not bytes:
-                    stderr_strace = stderr_strace.decode("utf8")
-
-                my_print(stderr_strace, file=sys.stderr)
-                sys.exit("Failed to run strace.")
-
-            with open(path + ".strace", "wb") as f:
-                f.write(stderr_strace)
-
-            for line in stderr_strace.split(b"\n"):
-                if process.returncode != 0 and trace_stderr:
-                    my_print(line)
-
-                if not line:
-                    continue
-
-                # Don't consider files not found. The "site" module checks lots
-                # of things.
-                if b"ENOENT" in line:
-                    continue
-
-                if line.startswith(b"stat(") and b"S_IFDIR" in line:
-                    continue
-
-                # Allow stats on the python binary, and stuff pointing to the
-                # standard library, just not uses of it. It will search there
-                # for stuff.
-                if (
-                    line.startswith(b"lstat(")
-                    or line.startswith(b"stat(")
-                    or line.startswith(b"readlink(")
-                ):
-                    filename = line[line.find(b"(") + 2 : line.find(b", ") - 1]
-
-                    # At least Python3.7 considers the default Python3 path.
-                    if filename == b"/usr/bin/python3":
-                        continue
-
-                    if filename in (
-                        b"/usr/bin/python3." + version
-                        for version in (b"5", b"6", b"7", b"8", b"9")
-                    ):
-                        continue
-
-                    binary_path = _python_executable
-                    if str is not bytes:
-                        binary_path = binary_path.encode("utf-8")
-
-                    found = False
-                    while binary_path:
-                        if filename == binary_path:
-                            found = True
-                            break
-
-                        if binary_path == os.path.dirname(binary_path):
-                            break
-
-                        binary_path = os.path.dirname(binary_path)
-
-                        if filename == os.path.join(
-                            binary_path,
-                            b"python"
-                            + (
-                                "%d%d" % (_python_version[0], _python_version[1])
-                            ).encode("utf8"),
-                        ):
-                            found = True
-                            continue
-
-                    if found:
-                        continue
-
-                result.extend(
-                    os.path.abspath(match)
-                    for match in re.findall(b'"(.*?)(?:\\\\0)?"', line)
-                )
-
-            if sys.version.startswith("3"):
-                result = [s.decode("utf-8") for s in result]
-    elif os.name == "nt":
-        subprocess.call(
-            (
-                getDependsExePath(),
-                "-c",
-                "-ot%s" % path + ".depends",
-                "-f1",
-                "-pa1",
-                "-ps1",
-                "-pp0",
-                "-pl1",
-                path,
-            )
-        )
-
-        inside = False
-        for line in getFileContentByLine(path + ".depends"):
-            if "| Module Dependency Tree |" in line:
-                inside = True
-                continue
-
-            if not inside:
-                continue
-
-            if "| Module List |" in line:
-                break
-
-            if "]" not in line:
-                continue
-
-            # Skip missing DLLs, apparently not needed anyway.
-            if "?" in line[: line.find("]")]:
-                continue
-
-            dll_filename = line[line.find("]") + 2 :].rstrip()
-            dll_filename = os.path.normcase(dll_filename)
-
-            assert os.path.isfile(dll_filename), repr(dll_filename)
-
-            # The executable itself is of course exempted.
-            if dll_filename == os.path.normcase(os.path.abspath(path)):
-                continue
-
-            result.append(dll_filename)
-
-        os.unlink(path + ".depends")
-
-    result = list(sorted(set(result)))
-
-    return result
-
-
-def checkRuntimeLoadedFilesForOutsideAccesses(loaded_filenames, white_list):
-    # A lot of special white listing is required.
-    # pylint: disable=too-many-branches,too-many-statements
-
-    result = []
-
-    for loaded_filename in loaded_filenames:
-        loaded_filename = os.path.normpath(loaded_filename)
-        loaded_filename = os.path.normcase(loaded_filename)
-        loaded_basename = os.path.basename(loaded_filename)
-
-        ok = False
-        for entry in white_list:
-            if loaded_filename.startswith(entry):
-                ok = True
-
-            while entry:
-                old_entry = entry
-                entry = os.path.dirname(entry)
-
-                if old_entry == entry:
-                    break
-
-                if loaded_filename == entry:
-                    ok = True
-                    break
-        if ok:
-            continue
-
-        if loaded_filename.startswith(("/etc/", "/usr/etc")):
-            continue
-
-        if loaded_filename.startswith("/proc/") or loaded_filename == "/proc":
-            continue
-
-        if loaded_filename.startswith("/dev/"):
-            continue
-
-        if loaded_filename.startswith("/tmp/"):
-            continue
-
-        if loaded_filename.startswith("/run/"):
-            continue
-
-        if loaded_filename.startswith("/sys/"):
-            continue
-
-        if loaded_filename.startswith("/usr/lib/locale/"):
-            continue
-
-        if loaded_filename.startswith("/usr/share/locale/"):
-            continue
-
-        if loaded_filename.startswith("/usr/share/X11/locale/"):
-            continue
-
-        # Themes may of course be loaded.
-        if loaded_filename.startswith("/usr/share/themes"):
-            continue
-        if "gtk" in loaded_filename and "/engines/" in loaded_filename:
-            continue
-
-        # Terminal info files are OK too.
-        if loaded_filename.startswith("/lib/terminfo/"):
-            continue
-
-        # System C libraries are to be expected.
-        if loaded_basename.startswith(
-            (
-                "ld-linux-x86-64.so",
-                "libc.so.",
-                "libpthread.so.",
-                "libm.so.",
-                "libdl.so.",
-                "libBrokenLocale.so.",
-                "libSegFault.so",
-                "libanl.so.",
-                "libcidn.so.",
-                "libcrypt.so.",
-                "libmemusage.so",
-                "libmvec.so.",
-                "libnsl.so.",
-                "libnss_compat.so.",
-                "libnss_db.so.",
-                "libnss_dns.so.",
-                "libnss_files.so.",
-                "libnss_hesiod.so.",
-                "libnss_nis.so.",
-                "libnss_nisplus.so.",
-                "libpcprofile.so",
-                "libresolv.so.",
-                "librt.so.",
-                "libthread_db-1.0.so",
-                "libthread_db.so.",
-                "libutil.so.",
-            )
-        ):
-            continue
-
-        # Taking these from system is harmless and desirable
-        if loaded_basename.startswith(("libz.so", "libgcc_s.so")):
-            continue
-
-        # TODO: Unclear, loading gconv from filesystem of installed system
-        # may be OK or not. I think it should be.
-        if loaded_basename == "gconv-modules.cache":
-            continue
-        if "/gconv/" in loaded_filename:
-            continue
-        if loaded_basename.startswith("libicu"):
-            continue
-
-        # GTK may access X files.
-        if loaded_basename == ".Xauthority":
-            continue
-
-        result.append(loaded_filename)
-
-    return result
+            test_logger.info("strace:")
+            os.system("strace -s4096 -e file %s" % path)
 
 
 def hasModule(module_name):
-    with open(os.devnull, "w") as devnull:
-        result = subprocess.call(
-            (os.environ["PYTHON"], "-c", "import %s" % module_name),
-            stdout=devnull,
-            stderr=subprocess.STDOUT,
-        )
+    result = subprocess.call(
+        (os.environ["PYTHON"], "-c", "import %s" % module_name),
+        stdout=getNullOutput(),
+        stderr=subprocess.STDOUT,
+    )
 
     return result == 0
 
@@ -908,7 +627,7 @@ def checkReferenceCount(checked_function, max_rounds=20, explain=False):
     # Clean start conditions.
     assert sys.exc_info() == (None, None, None), sys.exc_info()
 
-    print(checked_function.__name__ + ": ", end="")
+    my_print(checked_function.__name__ + ": ", end="")
     sys.stdout.flush()
 
     disablePrinting()
@@ -947,29 +666,32 @@ def checkReferenceCount(checked_function, max_rounds=20, explain=False):
     reenablePrinting()
 
     if result:
-        print("PASSED")
+        my_print("PASSED")
     else:
-        print("FAILED", ref_count1, ref_count2, "leaked", ref_count2 - ref_count1)
+        my_print(
+            "FAILED %d %d leaked %d" % (ref_count1, ref_count2, ref_count2 - ref_count1)
+        )
 
         if explain:
             print("REPORT of differences:")
             assert m1
             assert m2
 
+            # Using items will unwanted usages, pylint: disable=consider-using-dict-items
             for key in m1:
                 if key not in m2:
-                    print("*" * 80)
-                    print("extra:", m1[key], key)
+                    my_print("*" * 80)
+                    my_print("extra:", m1[key], key)
                 elif m1[key] != m2[key]:
-                    print("*" * 80)
-                    print(m1[key], "->", m2[key], key)
+                    my_print("*" * 80)
+                    my_print(m1[key], "->", m2[key], key)
                 else:
                     pass
 
             for key in m2:
                 if key not in m1:
-                    print("*" * 80)
-                    print("missing:", m2[key], key)
+                    my_print("*" * 80)
+                    my_print("missing:", m2[key], key)
 
                     # print m1[key]
 
@@ -1065,7 +787,7 @@ Defaults to off.""",
     elif mode == "coverage":
         return SearchModeCoverage()
     else:
-        assert False
+        test_logger.sysexit("Error, using unknown search mode %r" % mode)
 
 
 def reportSkip(reason, dirname, filename):
@@ -1101,7 +823,7 @@ def executeReferenceChecked(prefix, names, tests_skipped, tests_stderr, explain=
         # Avoid unraisable output.
         try:
             if number in tests_stderr:
-                sys.stderr = open(os.devnull, "wb")
+                sys.stderr = getNullOutput()
         except OSError:  # Windows
             if not checkReferenceCount(names[name], explain=explain):
                 result = False
@@ -1153,13 +875,6 @@ def withPythonPathChange(python_path):
             old_path = None
             os.environ["PYTHONPATH"] = python_path
 
-    #     print(
-    #         "Effective PYTHONPATH in %s is %r" % (
-    #             sys.modules["__main__"],
-    #             os.environ.get("PYTHONPATH", "")
-    #         )
-    #     )
-
     yield
 
     if python_path:
@@ -1383,7 +1098,7 @@ def compileLibraryTest(search_mode, stage_dir, decide, action):
 
 
 def run_async(coro):
-    """ Execute a coroutine until it's done. """
+    """Execute a coroutine until it's done."""
 
     values = []
     result = None
@@ -1397,7 +1112,7 @@ def run_async(coro):
 
 
 def async_iterate(g):
-    """ Execute async generator until it's done. """
+    """Execute async generator until it's done."""
 
     # Test code for Python3, catches all kinds of exceptions.
     # pylint: disable=broad-except
@@ -1442,18 +1157,6 @@ def getTestingCPythonOutputsCacheDir():
     return result
 
 
-@contextmanager
-def withDirectoryChange(path, allow_none=False):
-    if path is not None or not allow_none:
-        old_cwd = os.getcwd()
-        os.chdir(path)
-
-    yield
-
-    if path is not None or not allow_none:
-        os.chdir(old_cwd)
-
-
 def scanDirectoryForTestCases(dirname, template_context=None):
     filenames = os.listdir(dirname)
 
@@ -1464,9 +1167,6 @@ def scanDirectoryForTestCases(dirname, template_context=None):
         or filename.endswith(".j2")
     ]
 
-    # Jinja2 environment is optional.
-    env = None
-
     for filename in sorted(filenames):
         if not decideFilenameVersionSkip(filename):
             continue
@@ -1475,22 +1175,14 @@ def scanDirectoryForTestCases(dirname, template_context=None):
             # Needs to be a dictionary with template arguments.
             assert template_context is not None
 
-            if env is None:
-                import jinja2
-
-                env = jinja2.Environment(
-                    loader=jinja2.FileSystemLoader("."),
-                    trim_blocks=True,
-                    lstrip_blocks=True,
-                )
-                env.undefined = jinja2.StrictUndefined
-
-            template = env.get_template(filename)
+            template = getTemplate(
+                package_name=None, template_name=filename, template_subdir=dirname
+            )
 
             code = template.render(name=template.name, **template_context)
 
             filename = filename[:-3]
-            with open(filename, "w") as output:
+            with openTextFile(filename, "w") as output:
                 output.write(
                     "'''Automatically generated test, not part of releases or git.\n\n'''\n"
                 )
@@ -1500,6 +1192,25 @@ def scanDirectoryForTestCases(dirname, template_context=None):
         yield filename
 
 
+def scanDirectoryForTestCaseFolders(dirname):
+    filenames = os.listdir(dirname)
+
+    for filename in sorted(filenames):
+        filename = os.path.join(dirname, filename)
+        filename = os.path.relpath(filename)
+
+        if (
+            not os.path.isdir(filename)
+            or filename.endswith(".build")
+            or filename.endswith(".dist")
+        ):
+            continue
+
+        filename_main = getMainProgramFilename(filename)
+
+        yield filename, filename_main
+
+
 def setupCacheHashSalt(test_code_path):
     assert os.path.exists(test_code_path)
 
@@ -1507,7 +1218,10 @@ def setupCacheHashSalt(test_code_path):
         git_cmd = ["git", "ls-tree", "-r", "HEAD", test_code_path]
 
         process = subprocess.Popen(
-            args=git_cmd, stdout=subprocess.PIPE, stderr=subprocess.PIPE
+            args=git_cmd,
+            stdin=getNullInput(),
+            stdout=subprocess.PIPE,
+            stderr=subprocess.PIPE,
         )
 
         stdout_git, stderr_git = process.communicate()
@@ -1542,7 +1256,8 @@ def displayFileContents(name, path):
     test_logger.info("Contents of %s %r:" % (name, path))
 
     if os.path.exists(path):
-        os.system("cat %r" % path)
+        for line in getFileContentByLine(path):
+            my_print(line)
     else:
         test_logger.info("Does not exist.")
 
@@ -1568,16 +1283,15 @@ def checkRequirements(filename):
         if line.startswith("# nuitka-skip-unless-"):
             if line[21:33] == "expression: ":
                 expression = line[33:]
-                with open(os.devnull, "w") as devnull:
-                    result = subprocess.call(
-                        (
-                            os.environ["PYTHON"],
-                            "-c",
-                            "import sys, os; sys.exit(not bool(%s))" % expression,
-                        ),
-                        stdout=devnull,
-                        stderr=subprocess.STDOUT,
-                    )
+                result = subprocess.call(
+                    (
+                        os.environ["PYTHON"],
+                        "-c",
+                        "import sys, os; sys.exit(not bool(%s))" % expression,
+                    ),
+                    stdout=getNullOutput(),
+                    stderr=subprocess.STDOUT,
+                )
                 if result != 0:
                     return (False, "Expression '%s' evaluated to false" % expression)
 
@@ -1592,3 +1306,433 @@ def checkRequirements(filename):
                         )
     # default return value
     return (True, "")
+
+
+class DelayedExecutionThread(threading.Thread):
+    def __init__(self, timeout, func):
+        threading.Thread.__init__(self)
+        self.timeout = timeout
+
+        self.func = func
+
+    def run(self):
+        time.sleep(self.timeout)
+        self.func()
+
+
+def executeAfterTimePassed(timeout, func):
+    alarm = DelayedExecutionThread(timeout=timeout, func=func)
+    alarm.start()
+
+
+def killProcess(name, pid):
+    """Kill a process in a portable way.
+
+    Right now SIGINT is used, unclear what to do on Windows
+    with Python2 or non-related processes.
+    """
+
+    if str is bytes and os.name == "nt":
+        test_logger.info("Using taskkill on test process %r." % name)
+        os.system("taskkill.exe /PID %d" % pid)
+    else:
+        test_logger.info("Killing test process %r." % name)
+        os.kill(pid, signal.SIGINT)
+
+
+def checkLoadedFileAccesses(loaded_filenames, current_dir):
+    # Many details to consider, pylint: disable=too-many-branches,too-many-statements
+
+    current_dir = os.path.normpath(current_dir)
+    current_dir = os.path.normcase(current_dir)
+    current_dir_ext = os.path.normcase(getExternalUsePath(current_dir))
+
+    illegal_accesses = []
+
+    for loaded_filename in loaded_filenames:
+        orig_loaded_filename = loaded_filename
+
+        loaded_filename = os.path.normpath(loaded_filename)
+        loaded_filename = os.path.normcase(loaded_filename)
+        loaded_basename = os.path.basename(loaded_filename)
+
+        if os.name == "nt":
+            if areSamePaths(
+                os.path.dirname(loaded_filename),
+                os.path.normpath(os.path.join(os.environ["SYSTEMROOT"], "System32")),
+            ):
+                continue
+            if areSamePaths(
+                os.path.dirname(loaded_filename),
+                os.path.normpath(os.path.join(os.environ["SYSTEMROOT"], "SysWOW64")),
+            ):
+                continue
+
+            if r"windows\winsxs" in loaded_filename:
+                continue
+
+            # Github actions have these in PATH overriding SYSTEMROOT
+            if r"windows performance toolkit" in loaded_filename:
+                continue
+            if r"powershell" in loaded_filename:
+                continue
+            if r"azure dev spaces cli" in loaded_filename:
+                continue
+            if r"tortoisesvn" in loaded_filename:
+                continue
+
+        if loaded_filename.startswith(current_dir):
+            continue
+
+        if loaded_filename.startswith(os.path.abspath(current_dir)):
+            continue
+
+        if loaded_filename.startswith(current_dir_ext):
+            continue
+
+        ignore = True
+        for ignored_dir in (
+            # System configuration is OK
+            "/etc",
+            "/usr/etc",
+            "/usr/local/etc",
+            # Runtime user state and kernel information is OK.
+            "/proc",
+            "/dev",
+            "/run",
+            "/sys",
+            "/tmp",
+            "/var",
+            # Locals may of course be loaded.
+            "/usr/lib/locale",
+            "/usr/share/locale",
+            "/usr/share/X11/locale",
+            # Themes may of course be loaded.
+            "/usr/share/themes",
+            # Terminal info files are OK too.
+            "/lib/terminfo",
+        ):
+            if isPathBelowOrSameAs(ignored_dir, loaded_filename):
+                ignore = False
+                break
+        if not ignore:
+            continue
+
+        # Themes may of course be loaded.
+        if loaded_filename.startswith("/usr/share/themes"):
+            continue
+        if "gtk" in loaded_filename and "/engines/" in loaded_filename:
+            continue
+
+        if loaded_filename in (
+            "/usr",
+            "/usr/local",
+            "/usr/local/lib",
+            "/usr/share",
+            "/usr/local/share",
+            "/usr/lib64",
+        ):
+            continue
+
+        # TCL/tk for tkinter for non-Windows is OK.
+        if loaded_filename.startswith(
+            (
+                "/usr/lib/tcltk/",
+                "/usr/share/tcltk/",
+                "/usr/lib/tcl/",
+                "/usr/lib64/tcl/",
+            )
+        ):
+            continue
+        if loaded_filename in (
+            "/usr/lib/tcltk",
+            "/usr/share/tcltk",
+            "/usr/lib/tcl",
+            "/usr/lib64/tcl",
+        ):
+            continue
+
+        if loaded_filename in (
+            "/lib",
+            "/lib64",
+            "/lib/sse2",
+            "/lib/tls",
+            "/lib64/tls",
+            "/usr/lib/sse2",
+            "/usr/lib/tls",
+            "/usr/lib64/tls",
+        ):
+            continue
+
+        if loaded_filename in ("/usr/share/tcl8.6", "/usr/share/tcl8.5"):
+            continue
+        if loaded_filename in (
+            "/usr/share/tcl8.6/init.tcl",
+            "/usr/share/tcl8.5/init.tcl",
+        ):
+            continue
+        if loaded_filename in (
+            "/usr/share/tcl8.6/encoding",
+            "/usr/share/tcl8.5/encoding",
+        ):
+            continue
+
+        # System SSL config on Linux. TODO: Should this not be included and
+        # read from dist folder.
+        if loaded_basename == "openssl.cnf":
+            continue
+
+        # Taking these from system is harmless and desirable
+        if loaded_basename.startswith(("libz.so", "libgcc_s.so")):
+            continue
+
+        # System C libraries are to be expected.
+        if loaded_basename.startswith(
+            (
+                "ld-linux-x86-64.so",
+                "libc.so.",
+                "libpthread.so.",
+                "libm.so.",
+                "libdl.so.",
+                "libBrokenLocale.so.",
+                "libSegFault.so",
+                "libanl.so.",
+                "libcidn.so.",
+                "libcrypt.so.",
+                "libmemusage.so",
+                "libmvec.so.",
+                "libnsl.so.",
+                "libnss_compat.so.",
+                "libnss_db.so.",
+                "libnss_dns.so.",
+                "libnss_files.so.",
+                "libnss_hesiod.so.",
+                "libnss_nis.so.",
+                "libnss_nisplus.so.",
+                "libpcprofile.so",
+                "libresolv.so.",
+                "librt.so.",
+                "libthread_db-1.0.so",
+                "libthread_db.so.",
+                "libutil.so.",
+            )
+        ):
+            continue
+
+        # System C++ standard library is also OK.
+        if loaded_basename.startswith("libstdc++."):
+            continue
+
+        # Curses library is OK from system too.
+        if loaded_basename.startswith("libtinfo.so."):
+            continue
+
+        # Loaded by C library potentially for DNS lookups.
+        if loaded_basename.startswith(
+            (
+                "libnss_",
+                "libnsl",
+                # Some systems load a lot more, this is CentOS 7 on OBS
+                "libattr.so.",
+                "libbz2.so.",
+                "libcap.so.",
+                "libdw.so.",
+                "libelf.so.",
+                "liblzma.so.",
+                # Some systems load a lot more, this is Fedora 26 on OBS
+                "libselinux.so.",
+                "libpcre.so.",
+                # And this is Fedora 29 on OBS
+                "libblkid.so.",
+                "libmount.so.",
+                "libpcre2-8.so.",
+                # CentOS 8 on OBS
+                "libuuid.so.",
+            )
+        ):
+            continue
+
+        # Loaded by dtruss on macOS X.
+        if loaded_filename.startswith("/usr/lib/dtrace/"):
+            continue
+
+        # Loaded by cowbuilder and pbuilder on Debian
+        if loaded_basename == ".ilist":
+            continue
+        if "cowdancer" in loaded_filename:
+            continue
+        if "eatmydata" in loaded_filename:
+            continue
+
+        # Loading from home directories is OK too.
+        if (
+            loaded_filename.startswith("/home/")
+            or loaded_filename.startswith("/data/")
+            or loaded_filename.startswith("/root/")
+            or loaded_filename in ("/home", "/data", "/root")
+        ):
+            continue
+
+        # For Debian builders, /build is OK too.
+        if loaded_filename.startswith("/build/") or loaded_filename == "/build":
+            continue
+
+        # TODO: Unclear, loading gconv from filesystem of installed system
+        # may be OK or not. I think it should be.
+        if loaded_basename == "gconv-modules.cache":
+            continue
+        if "/gconv/" in loaded_filename:
+            continue
+        if loaded_basename.startswith("libicu"):
+            continue
+        if loaded_filename.startswith("/usr/share/icu/"):
+            continue
+
+        # Loading from caches is OK.
+        if loaded_filename.startswith("/var/cache/"):
+            continue
+
+        # At least Python3.7 considers the default Python3 path and checks it.
+        if loaded_filename == "/usr/bin/python3":
+            continue
+
+        # Accessing the versioned Python3.x binary is also happening.
+        if loaded_filename in (
+            "/usr/bin/python3." + version for version in ("5", "6", "7", "8", "9", "10")
+        ):
+            continue
+
+        binary_path = _python_executable
+
+        found = False
+        while binary_path:
+            if loaded_filename == binary_path:
+                found = True
+                break
+
+            if binary_path == os.path.dirname(binary_path):
+                break
+
+            binary_path = os.path.dirname(binary_path)
+
+            if loaded_filename == os.path.join(
+                binary_path,
+                "python" + ("%d%d" % (_python_version[0], _python_version[1])),
+            ):
+                found = True
+                break
+
+        if found:
+            continue
+
+        lib_prefix_dir = "/usr/lib/python%d.%s" % (
+            _python_version[0],
+            _python_version[1],
+        )
+
+        # PySide accesses its directory.
+        if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/PySide"):
+            continue
+
+        # GTK accesses package directories only.
+        if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/gtk-2.0/gtk"):
+            continue
+        if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/glib"):
+            continue
+        if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/gtk-2.0/gio"):
+            continue
+        if loaded_filename == os.path.join(lib_prefix_dir, "dist-packages/gobject"):
+            continue
+
+        # PyQt5 and PySide6 seems to do this, but won't use contents then.
+        if loaded_filename in (
+            "/usr/lib/qt6/plugins",
+            "/usr/lib/qt6",
+            "/usr/lib64/qt6/plugins",
+            "/usr/lib64/qt6",
+            "/usr/lib/qt5/plugins",
+            "/usr/lib/qt5",
+            "/usr/lib64/qt5/plugins",
+            "/usr/lib64/qt5",
+            "/usr/lib/x86_64-linux-gnu/qt5/plugins",
+            "/usr/lib/x86_64-linux-gnu/qt5",
+            "/usr/lib/x86_64-linux-gnu",
+            "/usr/lib",
+        ):
+            continue
+
+        # Can look at the interpreters of the system.
+        if loaded_basename in "python3":
+            continue
+        if loaded_basename in (
+            "python%s" + supported_version
+            for supported_version in (
+                getSupportedPythonVersions() + getPartiallySupportedPythonVersions()
+            )
+        ):
+            continue
+
+        # Current Python executable can actually be a symlink and
+        # the real executable which it points to will be on the
+        # loaded_filenames list. This is all fine, let's ignore it.
+        # Also, because the loaded_filename can be yet another symlink
+        # (this is weird, but it's true), let's better resolve its real
+        # path too.
+        if os.path.realpath(loaded_filename) == os.path.realpath(sys.executable):
+            continue
+
+        # Accessing SE-Linux is OK.
+        if loaded_filename in ("/sys/fs/selinux", "/selinux"):
+            continue
+
+        # Looking at device is OK.
+        if loaded_filename.startswith("/sys/devices/"):
+            continue
+
+        # Allow reading time zone info of local system.
+        if loaded_filename.startswith("/usr/share/zoneinfo/"):
+            continue
+
+        # The access to .pth files has no effect.
+        if loaded_filename.endswith(".pth"):
+            continue
+
+        # Looking at site-package dir alone is alone.
+        if loaded_filename.endswith(("site-packages", "dist-packages")):
+            continue
+
+        # QtNetwork insist on doing this it seems.
+        if loaded_basename.startswith(("libcrypto.so", "libssl.so")):
+            continue
+
+        # macOS uses these:
+        if loaded_basename in (
+            "libcrypto.1.0.0.dylib",
+            "libssl.1.0.0.dylib",
+            "libcrypto.1.1.dylib",
+        ):
+            continue
+
+        # Linux onefile uses this
+        if loaded_basename.startswith("libfuse.so."):
+            continue
+
+        # MSVC run time DLLs, due to SxS come from system.
+        if loaded_basename.upper() in ("MSVCRT.DLL", "MSVCR90.DLL"):
+            continue
+
+        illegal_accesses.append(orig_loaded_filename)
+
+    return illegal_accesses
+
+
+def getMainProgramFilename(filename):
+    for filename_main in os.listdir(filename):
+        if filename_main.endswith(("Main.py", "Main")):
+            return filename_main
+
+    test_logger.sysexit(
+        """\
+Error, no file ends with 'Main.py' or 'Main' in '%s', incomplete test case."""
+        % (filename)
+    )
