@@ -20,25 +20,22 @@
 """
 
 
-from __future__ import print_function
-
 import os
-import subprocess
 import sys
 from contextlib import contextmanager
 
-from nuitka.__past__ import unicode  # pylint: disable=I0021,redefined-builtin
-from nuitka.utils.Execution import check_call
-from nuitka.utils.FileOperations import removeDirectory
+from nuitka.__past__ import unicode
+from nuitka.utils.Execution import check_call, executeProcess
+from nuitka.utils.FileOperations import removeDirectory, withDirectoryChange
 
-from .Common import my_print, withDirectoryChange
+from .Common import my_print
 
 
 class Virtualenv(object):
     def __init__(self, env_dir):
         self.env_dir = os.path.abspath(env_dir)
 
-    def runCommand(self, commands):
+    def runCommand(self, commands, style=None):
         if type(commands) in (str, unicode):
             commands = [commands]
 
@@ -49,11 +46,15 @@ class Virtualenv(object):
                 commands = [". bin/activate"] + commands
 
             command = " && ".join(commands)
+
+            if style is not None:
+                my_print("Executing: %s" % command, style=style)
+
             assert os.system(command) == 0, command
 
     def runCommandWithOutput(self, commands, style=None):
         """
-        Returns the stdout,stderr from process.communicate()
+        Returns the stdout,stderr,exit_code from running command
         """
         if type(commands) in (str, unicode):
             commands = [commands]
@@ -64,19 +65,17 @@ class Virtualenv(object):
             else:
                 commands = [". bin/activate"] + commands
 
-            popen_arg = " && ".join(commands)
+            # Build shell command.
+            command = " && ".join(commands)
 
             if style is not None:
-                my_print("Executing: %s" % popen_arg, style=style)
+                my_print("Executing: %s" % command, style=style)
 
             # Use subprocess and also return outputs, stdout, stderr, result
-            process = subprocess.Popen(
-                args=popen_arg,
-                stdout=subprocess.PIPE,
-                stderr=subprocess.PIPE,
+            return executeProcess(
+                command=command,
                 shell=True,
             )
-            return process.communicate()
 
     def getVirtualenvDir(self):
         return self.env_dir
@@ -89,7 +88,8 @@ def withVirtualenv(env_name, base_dir=None, python=None, delete=True, style=None
     Activating for actual use will be your task.
     """
 
-    print("Creating virtualenv for quick test:")
+    if style is not None:
+        my_print("Creating a virtualenv:")
 
     if python is None:
         python = sys.executable
@@ -104,7 +104,7 @@ def withVirtualenv(env_name, base_dir=None, python=None, delete=True, style=None
     with withDirectoryChange(base_dir, allow_none=True):
         command = [python, "-m", "virtualenv", env_name]
         if style is not None:
-            my_print("Executing: %s" % " ".join(command))
+            my_print("Executing: %s" % " ".join(command), style=style)
         check_call(command)
 
         yield Virtualenv(env_dir)
