@@ -48,9 +48,8 @@ special attribute lookups for "__enter__" and "__exit__", calls of them,
 catching and passing in exceptions raised.
 
 """
-
+import marshal
 import os
-import sys
 
 from nuitka import (
     ModuleRegistry,
@@ -58,18 +57,21 @@ from nuitka import (
     OutputDirectories,
     SourceCodeReferences,
 )
-from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
-    long,
-    unicode,
+from nuitka.__past__ import long, unicode
+from nuitka.Caching import (
+    getCachedImportedModulesNames,
+    hasCachedImportedModulesNames,
 )
+from nuitka.containers.oset import OrderedSet
+from nuitka.Errors import CodeTooComplexCode
 from nuitka.freezer.Standalone import detectEarlyImports
 from nuitka.importing import Importing
 from nuitka.importing.ImportCache import addImportedModule
 from nuitka.importing.PreloadedPackages import getPthImportedPackages
 from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
 from nuitka.nodes.AttributeNodes import (
-    ExpressionAttributeLookup,
     StatementAssignmentAttribute,
+    makeExpressionAttributeLookup,
 )
 from nuitka.nodes.BuiltinFormatNodes import (
     ExpressionBuiltinAscii,
@@ -91,8 +93,12 @@ from nuitka.nodes.ExceptionNodes import (
     StatementRaiseException,
     StatementReraiseException,
 )
+from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.GeneratorNodes import StatementGeneratorReturn
-from nuitka.nodes.ImportNodes import makeExpressionAbsoluteImportNode
+from nuitka.nodes.ImportNodes import (
+    isHardModuleWithoutSideEffect,
+    makeExpressionImportModuleFixed,
+)
 from nuitka.nodes.LoopNodes import StatementLoopBreak, StatementLoopContinue
 from nuitka.nodes.ModuleAttributeNodes import (
     ExpressionModuleAttributeFileRef,
@@ -103,23 +109,33 @@ from nuitka.nodes.ModuleNodes import (
     CompiledPythonPackage,
     PythonMainModule,
     PythonShlibModule,
+    makeUncompiledPythonModule,
 )
-from nuitka.nodes.OperatorNodes import (
-    ExpressionOperationUnary,
-    makeBinaryOperationNode,
+from nuitka.nodes.NodeMakingHelpers import (
+    makeRaiseExceptionStatementFromInstance,
 )
+from nuitka.nodes.OperatorNodes import makeBinaryOperationNode
+from nuitka.nodes.OperatorNodesUnary import makeExpressionOperationUnary
 from nuitka.nodes.ReturnNodes import makeStatementReturn
 from nuitka.nodes.SliceNodes import makeExpressionBuiltinSlice
 from nuitka.nodes.StatementNodes import StatementExpressionOnly
 from nuitka.nodes.StringConcatenationNodes import ExpressionStringConcatenation
 from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
 from nuitka.nodes.YieldNodes import ExpressionYieldFromWaitable
+from nuitka.optimizations.BytecodeDemotion import demoteSourceCodeToBytecode
 from nuitka.Options import shallWarnUnusualCode
+from nuitka.pgo.PGO import decideCompilationFromPGO
 from nuitka.plugins.Plugins import Plugins
 from nuitka.PythonVersions import python_version
-from nuitka.Tracing import memory_logger, plugins_logger, unusual_logger
+from nuitka.Tracing import (
+    general,
+    memory_logger,
+    optimization_logger,
+    plugins_logger,
+    recursion_logger,
+    unusual_logger,
+)
 from nuitka.utils import MemoryUsage
-from nuitka.utils.FileOperations import splitPath
 from nuitka.utils.ModuleNames import ModuleName
 
 from . import SyntaxErrors
@@ -160,6 +176,7 @@ from .ReformulationImportStatements import (
     pushFutureSpec,
 )
 from .ReformulationLambdaExpressions import buildLambdaNode
+from .ReformulationMatchStatements import buildMatchNode
 from .ReformulationNamespacePackages import (
     createImporterCacheAssignment,
     createNamespacePackage,
@@ -213,7 +230,6 @@ if str is not bytes:
             source_ref=source_ref,
         )
 
-
 else:
 
     def buildVariableReferenceNode(provider, node, source_ref):
@@ -496,7 +512,7 @@ def buildStatementLoopBreak(provider, node, source_ref):
 
 
 def buildAttributeNode(provider, node, source_ref):
-    return ExpressionAttributeLookup(
+    return makeExpressionAttributeLookup(
         expression=buildNode(provider, node.value, source_ref),
         attribute_name=mangleName(node.attr, provider),
         source_ref=source_ref,
@@ -550,14 +566,17 @@ def buildExprOnlyNode(provider, node, source_ref):
 
 
 def buildUnaryOpNode(provider, node, source_ref):
-    if getKind(node.op) == "Not":
+    operator = getKind(node.op)
+
+    # Delegate this one to boolean operation code.
+    if operator == "Not":
         return buildBoolOpNode(provider=provider, node=node, source_ref=source_ref)
-    else:
-        return ExpressionOperationUnary(
-            operator=getKind(node.op),
-            operand=buildNode(provider, node.operand, source_ref),
-            source_ref=source_ref,
-        )
+
+    operand = buildNode(provider, node.operand, source_ref)
+
+    return makeExpressionOperationUnary(
+        operator=operator, operand=operand, source_ref=source_ref
+    )
 
 
 def buildBinaryOpNode(provider, node, source_ref):
@@ -579,7 +598,7 @@ def buildBinaryOpNode(provider, node, source_ref):
 
 
 def buildReprNode(provider, node, source_ref):
-    return ExpressionOperationUnary(
+    return makeExpressionOperationUnary(
         operator="Repr",
         operand=buildNode(provider, node.value, source_ref),
         source_ref=source_ref,
@@ -620,7 +639,7 @@ def buildFormattedValueNode(provider, node, source_ref):
             value=value, encoding=None, errors=None, source_ref=source_ref
         )
     elif conversion == 2:
-        value = ExpressionOperationUnary(
+        value = makeExpressionOperationUnary(
             operator="Repr", operand=value, source_ref=source_ref
         )
     elif conversion == 1:
@@ -710,6 +729,7 @@ setBuildingDispatchers(
         "FormattedValue": buildFormattedValueNode,
         "NamedExpr": buildNamedExprNode,
         "Slice": buildSliceNode,
+        "Match": buildMatchNode,
     },
     path_args2={
         "Constant": buildNamedConstantNode,  # Python3.8
@@ -723,27 +743,25 @@ setBuildingDispatchers(
 )
 
 
-def buildParseTree(provider, source_code, source_ref, is_module, is_main):
+def buildParseTree(provider, ast_tree, source_ref, is_module, is_main):
     # There are a bunch of branches here, mostly to deal with version
     # differences for module default variables. pylint: disable=too-many-branches
 
+    # Maybe one day, we do exec inlining again, that is what this is for,
+    # then is_module won't be True, for now it always is.
     pushFutureSpec()
     if is_module:
-        provider.future_spec = getFutureSpec()
+        provider.setFutureSpec(getFutureSpec())
 
-    body = parseSourceCodeToAst(
-        source_code=source_code,
-        filename=source_ref.getFilename(),
-        line_offset=source_ref.getLineNumber() - 1,
-    )
-
-    body, doc = extractDocFromBody(body)
+    body, doc = extractDocFromBody(ast_tree)
 
     if is_module and is_main and python_version >= 0x360:
         provider.markAsNeedsAnnotationsDictionary()
 
     result = buildStatementsNode(provider=provider, nodes=body, source_ref=source_ref)
 
+    # After building, we can verify that all future statements were where they
+    # belong, namely at the start of the module.
     checkFutureImportsOnlyAtStart(body)
 
     internal_source_ref = source_ref.atInternal()
@@ -753,26 +771,29 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
     if is_module:
         # Add import of "site" module of main programs visibly in the node tree,
         # so recursion and optimization can pick it up, checking its effects.
-        if is_main and "no_site" not in Options.getPythonFlags():
-            for path_imported_name in getPthImportedPackages():
-                statements.append(
-                    StatementExpressionOnly(
-                        expression=makeExpressionAbsoluteImportNode(
-                            module_name=path_imported_name, source_ref=source_ref
-                        ),
-                        source_ref=source_ref,
-                    )
-                )
-
+        if is_main and not Options.hasPythonFlagNoSite():
             statements.append(
                 StatementExpressionOnly(
-                    expression=makeExpressionAbsoluteImportNode(
+                    expression=makeExpressionImportModuleFixed(
                         module_name="site", source_ref=source_ref
                     ),
                     source_ref=source_ref,
                 )
             )
 
+            for path_imported_name in getPthImportedPackages():
+                if isHardModuleWithoutSideEffect(path_imported_name):
+                    continue
+
+                statements.append(
+                    StatementExpressionOnly(
+                        expression=makeExpressionImportModuleFixed(
+                            module_name=path_imported_name, source_ref=source_ref
+                        ),
+                        source_ref=source_ref,
+                    )
+                )
+
         statements.append(
             StatementAssignmentVariableName(
                 provider=provider,
@@ -914,9 +935,17 @@ def buildParseTree(provider, source_code, source_ref, is_module, is_main):
         assert False
 
 
-def decideCompilationMode(is_top, module_name, source_ref):
-    result = Plugins.decideCompilation(module_name, source_ref)
+def decideCompilationMode(is_top, module_name, for_pgo):
+    """Decide the compilation mode for a module.
 
+    module_name - The module to decide compilation mode for.
+    for_pgo - consider PGO information or not
+    """
+
+    result = Plugins.decideCompilation(module_name)
+
+    # Cannot change mode of __main__ to bytecode, that is not going
+    # to work currently.
     if result == "bytecode" and is_top:
         plugins_logger.warning(
             """\
@@ -927,138 +956,123 @@ required to compiled."""
         )
         result = "compiled"
 
+    # Plugins need to win over PGO, as they might know it better
+    if result is None and not for_pgo:
+        result = decideCompilationFromPGO(module_name=module_name)
+
+    # Default if neither plugins nor PGO have expressed an opinion
+    if result is None:
+        result = "compiled"
+
     return result
 
 
-def decideModuleTree(filename, package, is_shlib, is_top, is_main):
-    # Many variables, branches, due to the many cases, pylint: disable=too-many-branches
+def _createModule(
+    module_name,
+    source_code,
+    source_ref,
+    is_shlib,
+    is_namespace,
+    is_package,
+    is_top,
+    is_main,
+    main_added,
+):
+    # Many details due to the caching done here.
+    # pylint: disable=too-many-locals
 
-    assert package is None or type(package) is ModuleName
-    assert filename is not None
+    if is_shlib:
+        result = PythonShlibModule(module_name=module_name, source_ref=source_ref)
+    elif is_main:
+        result = PythonMainModule(
+            main_added=main_added,
+            module_name=module_name,
+            mode=decideCompilationMode(
+                is_top=is_top, module_name=module_name, for_pgo=False
+            ),
+            future_spec=None,
+            source_ref=source_ref,
+        )
 
-    if is_main and os.path.isdir(filename):
-        source_filename = os.path.join(filename, "__main__.py")
-
-        if not os.path.isfile(source_filename):
-            sys.stderr.write(
-                "%s: can't find '__main__' module in '%s'\n"
-                % (os.path.basename(sys.argv[0]), filename)
-            )
-            sys.exit(2)
-
-        filename = source_filename
-
-        main_added = True
+        checkPythonVersionFromCode(source_code)
+    elif is_namespace:
+        result = createNamespacePackage(module_name, is_top, source_ref)
     else:
-        main_added = False
+        mode = decideCompilationMode(
+            is_top=is_top, module_name=module_name, for_pgo=False
+        )
 
-    if os.path.isfile(filename):
-        source_filename = filename
+        if (
+            mode == "bytecode"
+            and not is_top
+            and hasCachedImportedModulesNames(module_name, source_code)
+        ):
 
-        source_ref = SourceCodeReferences.fromFilename(filename=filename)
+            optimization_logger.info(
+                "'%s' is included as bytecode." % (module_name.asString())
+            )
+            result = makeUncompiledPythonModule(
+                module_name=module_name,
+                filename=source_ref.getFilename(),
+                bytecode=demoteSourceCodeToBytecode(
+                    module_name=module_name,
+                    source_code=source_code,
+                    filename=source_ref.getFilename(),
+                ),
+                user_provided=False,
+                technical=False,
+                is_package=is_package,
+            )
 
-        if is_main:
-            module_name = ModuleName("__main__")
-        else:
-            # Derive module name from filename.
-            module_name = os.path.basename(filename)
-            if is_shlib:
-                module_name = module_name.split(".")[0]
-            elif module_name.endswith(".py"):
-                module_name = module_name[:-3]
+            used_modules = OrderedSet()
 
-            if "." in module_name:
-                sys.stderr.write(
-                    "Error, '%s' is not a proper python module name.\n" % (module_name)
+            for used_module_name in getCachedImportedModulesNames(
+                module_name=module_name, source_code=source_code
+            ):
+                (_module_name, module_filename, _finding,) = Importing.locateModule(
+                    module_name=used_module_name,
+                    parent_package=None,
+                    level=0,
                 )
 
-                sys.exit(2)
+                used_modules.add((used_module_name, os.path.relpath(module_filename)))
 
-            module_name = ModuleName.makeModuleNameInPackage(module_name, package)
+            result.setUsedModules(used_modules)
 
-        if is_shlib:
-            result = PythonShlibModule(module_name=module_name, source_ref=source_ref)
-        elif is_main:
-            result = PythonMainModule(
-                main_added=main_added,
-                mode=decideCompilationMode(False, module_name, source_ref),
-                future_spec=None,
-                source_ref=source_ref,
-            )
+            # Not used anymore
+            source_code = None
         else:
-            result = CompiledPythonModule(
-                module_name=module_name,
-                is_top=is_top,
-                mode=decideCompilationMode(is_top, module_name, source_ref),
-                future_spec=None,
-                source_ref=source_ref,
-            )
+            if is_package:
+                result = CompiledPythonPackage(
+                    module_name=module_name,
+                    is_top=is_top,
+                    mode=mode,
+                    future_spec=None,
+                    source_ref=source_ref,
+                )
+            else:
+                result = CompiledPythonModule(
+                    module_name=module_name,
+                    is_top=is_top,
+                    mode=mode,
+                    future_spec=None,
+                    source_ref=source_ref,
+                )
 
-    elif Importing.isPackageDir(filename):
-        if is_top:
-            module_name = splitPath(filename)[-1]
-        else:
-            module_name = os.path.basename(filename)
-
-        module_name = ModuleName.makeModuleNameInPackage(module_name, package)
-
-        source_filename = os.path.join(filename, "__init__.py")
-
-        if not os.path.isfile(source_filename):
-            source_ref, result = createNamespacePackage(
-                module_name=module_name, is_top=is_top, module_relpath=filename
-            )
-            source_filename = None
-        else:
-            source_ref = SourceCodeReferences.fromFilename(
-                filename=os.path.abspath(source_filename)
-            )
-
-            result = CompiledPythonPackage(
-                module_name=module_name,
-                is_top=is_top,
-                mode=decideCompilationMode(is_top, module_name, source_ref),
-                future_spec=None,
-                source_ref=source_ref,
-            )
-    else:
-        sys.stderr.write(
-            "%s: can't open file '%s'.\n" % (os.path.basename(sys.argv[0]), filename)
-        )
-        sys.exit(2)
-
-    return result, source_ref, source_filename
+    return result
 
 
-class CodeTooComplexCode(Exception):
-    """The code of the module is too complex.
-
-    It cannot be compiled, with recursive code, and therefore the bytecode
-    should be used instead.
-
-    Example of this is "idnadata".
-    """
-
-
-def createModuleTree(module, source_ref, source_code, is_main):
+def createModuleTree(module, source_ref, ast_tree, is_main):
     if Options.isShowMemory():
         memory_watch = MemoryUsage.MemoryWatch()
 
-    try:
-        module_body = buildParseTree(
-            provider=module,
-            source_code=source_code,
-            source_ref=source_ref,
-            is_module=True,
-            is_main=is_main,
-        )
-    except RuntimeError as e:
-        if "maximum recursion depth" in e.args[0]:
-            raise CodeTooComplexCode(
-                module.getFullName(), module.getCompileTimeFilename()
-            )
-
-        raise
+    module_body = buildParseTree(
+        provider=module,
+        ast_tree=ast_tree,
+        source_ref=source_ref,
+        is_module=True,
+        is_main=is_main,
+    )
 
     if module_body.isStatementsFrame():
         module_body = makeStatementsSequenceFromStatement(statement=module_body)
@@ -1076,42 +1090,32 @@ def createModuleTree(module, source_ref, source_code, is_main):
         )
 
 
-def buildModuleTree(filename, package, is_top, is_main):
-    module, source_ref, source_filename = decideModuleTree(
-        filename=filename,
-        package=package,
-        is_top=is_top,
+def buildMainModuleTree(filename, is_main):
+    # Detect to be frozen modules if any, so we can consider to not follow
+    # to them.
+
+    if is_main:
+        # TODO: Doesn't work for deeply nested packages at all.
+        if Options.hasPythonFlagPackageMode():
+            module_name = ModuleName(os.path.basename(filename) + ".__main__")
+        else:
+            module_name = ModuleName("__main__")
+    else:
+        module_name = Importing.getModuleNameAndKindFromFilename(filename)[0]
+
+    module, _added = buildModule(
+        module_name=module_name,
+        module_filename=filename,
+        source_code=None,
+        is_top=True,
         is_main=is_main,
         is_shlib=False,
+        is_fake=False,
+        hide_syntax_error=False,
     )
 
-    if is_top:
-        ModuleRegistry.addRootModule(module)
-
-        OutputDirectories.setMainModule(module)
-
-        # Detect to be frozen modules if any, so we can consider to not recurse
-        # to them.
-        if Options.isStandaloneMode():
-            detectEarlyImports()
-
-    # If there is source code associated (not the case for namespace packages of
-    # Python3.3 or higher, then read it.
-    if source_filename is not None:
-        source_code = readSourceCodeFromFilename(
-            module_name=module.getFullName(), source_filename=source_filename
-        )
-
-        if is_main:
-            checkPythonVersionFromCode(source_code)
-
-        # Read source code.
-        createModuleTree(
-            module=module,
-            source_ref=source_ref,
-            source_code=source_code,
-            is_main=is_main,
-        )
+    if Options.isStandaloneMode() and is_main:
+        module.setEarlyModules(detectEarlyImports())
 
     # Main modules do not get added to the import cache, but plugins get to see it.
     if module.isMainModule():
@@ -1120,3 +1124,184 @@ def buildModuleTree(filename, package, is_top, is_main):
         addImportedModule(imported_module=module)
 
     return module
+
+
+def _makeModuleBodyFromSyntaxError(exc, module_name, module_filename):
+    if module_filename not in Importing.warned_about:
+        Importing.warned_about.add(module_filename)
+
+        recursion_logger.warning(
+            """\
+Cannot follow import to module '%s' because of %r."""
+            % (module_name, exc.__class__.__name__)
+        )
+
+    source_ref = SourceCodeReferences.fromFilename(filename=module_filename)
+
+    module = CompiledPythonModule(
+        module_name=module_name,
+        is_top=False,
+        mode="compiled",
+        future_spec=FutureSpec(),
+        source_ref=source_ref,
+    )
+
+    module_body = makeModuleFrame(
+        module=module,
+        statements=(
+            makeRaiseExceptionStatementFromInstance(
+                source_ref=source_ref, exception=exc
+            ),
+        ),
+        source_ref=source_ref,
+    )
+
+    module_body = makeStatementsSequenceFromStatement(statement=module_body)
+    module.setChild("body", module_body)
+
+    return module
+
+
+def _makeModuleBodyTooComplex(module_name, module_filename, source_code, is_package):
+    if module_filename not in Importing.warned_about:
+        Importing.warned_about.add(module_filename)
+
+        recursion_logger.warning(
+            """\
+Cannot follow import to import module '%r' ('%r') because code is too complex."""
+            % (
+                module_name,
+                module_filename,
+            )
+        )
+
+    module = makeUncompiledPythonModule(
+        module_name=module_name,
+        filename=module_filename,
+        bytecode=marshal.dumps(
+            compile(source_code, module_filename, "exec", dont_inherit=True)
+        ),
+        is_package=is_package,
+        user_provided=True,
+        technical=False,
+    )
+
+    ModuleRegistry.addUncompiledModule(module)
+
+
+def buildModule(
+    module_name,
+    module_filename,
+    source_code,
+    is_top,
+    is_main,
+    is_shlib,
+    is_fake,
+    hide_syntax_error,
+):
+    # Many details to deal with, pylint: disable=too-many-branches,too-many-locals
+    (
+        main_added,
+        is_package,
+        is_namespace,
+        source_ref,
+        source_filename,
+    ) = Importing.decideModuleSourceRef(
+        filename=module_filename,
+        module_name=module_name,
+        is_main=is_main,
+        is_fake=is_fake,
+        logger=general,
+    )
+
+    if Options.hasPythonFlagPackageMode():
+        if is_top and Options.shallMakeModule():
+            optimization_logger.warning(
+                "Python flag -m (package_mode) has no effect in module mode, it's only for executables."
+            )
+        elif is_main and not main_added:
+            optimization_logger.warning(
+                "Python flag -m (package_mode) only works on packages with '__main__.py'."
+            )
+
+    # Read source code if necessary. Might give a SyntaxError due to not being proper
+    # encoded source.
+    if source_filename is not None and not is_namespace and not is_shlib:
+        try:
+            # For fake modules, source is provided directly.
+            if source_code is None:
+                source_code = readSourceCodeFromFilename(
+                    module_name=module_name, source_filename=source_filename
+                )
+        except SyntaxError as e:
+            # Avoid hiding our own syntax errors.
+            if not hasattr(e, "generated_by_nuitka"):
+                raise
+
+            # Do not hide SyntaxError in main module.
+            if not hide_syntax_error:
+                raise
+
+            module = _makeModuleBodyFromSyntaxError(
+                exc=e, module_name=module_name, module_filename=module_filename
+            )
+            return module, True
+
+        try:
+            ast_tree = parseSourceCodeToAst(
+                source_code=source_code,
+                module_name=module_name,
+                filename=source_filename,
+                line_offset=0,
+            )
+        except (SyntaxError, IndentationError) as e:
+            # Do not hide SyntaxError if asked not to.
+            if not hide_syntax_error:
+                raise
+
+            module = _makeModuleBodyFromSyntaxError(
+                exc=e, module_name=module_name, module_filename=module_filename
+            )
+            return module, True
+        except CodeTooComplexCode:
+            # Do not hide CodeTooComplexCode in main module.
+            if is_main:
+                raise
+
+            module = _makeModuleBodyTooComplex(
+                module_name=module_name,
+                module_filename=module_filename,
+                source_code=source_code,
+                is_package=is_package,
+            )
+            return module, False
+    else:
+        ast_tree = None
+        source_code = None
+
+    module = _createModule(
+        module_name=module_name,
+        source_code=source_code,
+        source_ref=source_ref,
+        is_top=is_top,
+        is_main=is_main,
+        is_shlib=is_shlib,
+        is_namespace=is_namespace,
+        is_package=is_package,
+        main_added=main_added,
+    )
+
+    if is_top:
+        ModuleRegistry.addRootModule(module)
+
+        OutputDirectories.setMainModule(module)
+
+    if module.isCompiledPythonModule() and source_code is not None:
+        createModuleTree(
+            module=module,
+            source_ref=source_ref,
+            ast_tree=ast_tree,
+            is_main=is_main,
+        )
+
+    return module, True
