@@ -24,19 +24,20 @@ import os
 
 from nuitka import Options
 from nuitka.nodes.AssignNodes import StatementAssignmentVariableName
-from nuitka.nodes.AttributeNodes import ExpressionAttributeLookup
+from nuitka.nodes.AttributeNodes import makeExpressionAttributeLookup
 from nuitka.nodes.CallNodes import ExpressionCallNoKeywords
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.ContainerMakingNodes import (
     makeExpressionMakeList,
     makeExpressionMakeTuple,
+    makeExpressionMakeTupleOrConstant,
 )
 from nuitka.nodes.DictionaryNodes import StatementDictOperationSet
 from nuitka.nodes.FutureSpecs import FutureSpec
 from nuitka.nodes.ImportNodes import (
     ExpressionImportModuleNameHard,
     ExpressionImportName,
-    makeExpressionAbsoluteImportNode,
+    makeExpressionImportModuleFixed,
 )
 from nuitka.nodes.ModuleAttributeNodes import (
     ExpressionModuleAttributeFileRef,
@@ -46,7 +47,6 @@ from nuitka.nodes.ModuleNodes import CompiledPythonPackage
 from nuitka.nodes.SubscriptNodes import ExpressionSubscriptLookup
 from nuitka.nodes.VariableRefNodes import ExpressionVariableNameRef
 from nuitka.PythonVersions import python_version
-from nuitka.SourceCodeReferences import SourceCodeReference
 
 from .TreeHelpers import makeStatementsSequenceFromStatement
 from .VariableClosure import completeVariableClosures
@@ -54,29 +54,39 @@ from .VariableClosure import completeVariableClosures
 
 def _makeCall(module_name, import_name, attribute_name, source_ref, *args):
     return ExpressionCallNoKeywords(
-        called=ExpressionAttributeLookup(
+        called=makeExpressionAttributeLookup(
             expression=ExpressionImportModuleNameHard(
                 module_name=module_name, import_name=import_name, source_ref=source_ref
             ),
             attribute_name=attribute_name,
             source_ref=source_ref,
         ),
-        args=makeExpressionMakeTuple(elements=args, source_ref=source_ref),
+        args=makeExpressionMakeTupleOrConstant(
+            elements=args, user_provided=True, source_ref=source_ref
+        ),
         source_ref=source_ref,
     )
 
 
-def createPathAssignment(package, source_ref):
-    if Options.getFileReferenceMode() == "original":
-        path_value = makeConstantRefNode(
-            constant=[os.path.dirname(source_ref.getFilename())],
+def getNameSpacePathExpression(package, source_ref):
+    """Create the __path__ expression for a package."""
+
+    reference_mode = Options.getFileReferenceMode()
+
+    if reference_mode == "original":
+        return makeConstantRefNode(
+            constant=[package.getCompileTimeDirectory()],
+            source_ref=source_ref,
+        )
+    elif reference_mode == "frozen":
+        return makeConstantRefNode(
+            constant=[],
             source_ref=source_ref,
-            user_provided=True,
         )
     else:
         elements = [
             ExpressionCallNoKeywords(
-                called=ExpressionAttributeLookup(
+                called=makeExpressionAttributeLookup(
                     expression=ExpressionImportModuleNameHard(
                         module_name="os", import_name="path", source_ref=source_ref
                     ),
@@ -127,34 +137,45 @@ def createPathAssignment(package, source_ref):
 
                 elements.append(path_part)
 
-        path_value = makeExpressionMakeList(elements=elements, source_ref=source_ref)
+        return makeExpressionMakeList(elements=elements, source_ref=source_ref)
 
+
+def createPathAssignment(package, source_ref):
     return StatementAssignmentVariableName(
         provider=package,
         variable_name="__path__",
-        source=path_value,
+        source=getNameSpacePathExpression(package=package, source_ref=source_ref),
         source_ref=source_ref,
     )
 
 
-def createPython3NamespacePath(package, module_relpath, source_ref):
+def createPython3NamespacePath(package, source_ref):
     return StatementAssignmentVariableName(
         provider=package,
         variable_name="__path__",
         source=ExpressionCallNoKeywords(
             called=ExpressionImportName(
-                module=makeExpressionAbsoluteImportNode(
+                module=makeExpressionImportModuleFixed(
                     module_name="_frozen_importlib"
                     if python_version < 0x350
                     else "_frozen_importlib_external",
                     source_ref=source_ref,
                 ),
                 import_name="_NamespacePath",
-                level=None,
+                level=0,
                 source_ref=source_ref,
             ),
-            args=makeConstantRefNode(
-                constant=(package.getFullName().asString(), [module_relpath], None),
+            args=makeExpressionMakeTupleOrConstant(
+                elements=(
+                    makeConstantRefNode(
+                        constant=package.getFullName().asString(),
+                        user_provided=True,
+                        source_ref=source_ref,
+                    ),
+                    getNameSpacePathExpression(package=package, source_ref=source_ref),
+                    makeConstantRefNode(constant=None, source_ref=source_ref),
+                ),
+                user_provided=True,
                 source_ref=source_ref,
             ),
             source_ref=source_ref,
@@ -163,12 +184,7 @@ def createPython3NamespacePath(package, module_relpath, source_ref):
     )
 
 
-def createNamespacePackage(module_name, is_top, module_relpath):
-    source_ref = SourceCodeReference.fromFilenameAndLine(
-        filename=module_relpath, line=1
-    )
-    source_ref = source_ref.atInternal()
-
+def createNamespacePackage(module_name, is_top, source_ref):
     package = CompiledPythonPackage(
         module_name=module_name,
         is_top=is_top,
@@ -178,9 +194,7 @@ def createNamespacePackage(module_name, is_top, module_relpath):
     )
 
     if python_version >= 0x300:
-        statement = createPython3NamespacePath(
-            package=package, module_relpath=module_relpath, source_ref=source_ref
-        )
+        statement = createPython3NamespacePath(package=package, source_ref=source_ref)
     else:
         statement = createPathAssignment(package, source_ref)
 
@@ -188,7 +202,7 @@ def createNamespacePackage(module_name, is_top, module_relpath):
 
     completeVariableClosures(package)
 
-    return source_ref, package
+    return package
 
 
 def createImporterCacheAssignment(package, source_ref):
