@@ -59,7 +59,7 @@ from nuitka.nodes.ComparisonNodes import ExpressionComparisonIs
 from nuitka.nodes.ConditionalNodes import makeStatementConditional
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
 from nuitka.nodes.LoopNodes import StatementLoop, StatementLoopBreak
-from nuitka.nodes.OperatorNodes import ExpressionOperationNot
+from nuitka.nodes.OperatorNodesUnary import ExpressionOperationNot
 from nuitka.nodes.StatementNodes import StatementsSequence
 from nuitka.nodes.VariableRefNodes import ExpressionTempVariableRef
 
@@ -74,7 +74,7 @@ from .TreeHelpers import (
 
 
 def buildWhileLoopNode(provider, node, source_ref):
-    # The while loop is re-formulated according to developer manual. The
+    # The while loop is re-formulated according to Developer Manual. The
     # condition becomes an early condition to break the loop. The else block is
     # taken if a while loop exits normally, i.e. because of condition not being
     # true. We do this by introducing an indicator variable.
