@@ -24,6 +24,7 @@ import __future__
 import ast
 
 from nuitka import Constants, Options
+from nuitka.Errors import CodeTooComplexCode
 from nuitka.nodes.CallNodes import makeExpressionCall
 from nuitka.nodes.CodeObjectSpecs import CodeObjectSpec
 from nuitka.nodes.ConstantRefNodes import makeConstantRefNode
@@ -59,8 +60,8 @@ def extractDocFromBody(node):
     body = node.body
     doc = None
 
-    # Work around ast.get_docstring breakage.
-    if node.body and getKind(body[0]) == "Expr":
+    # Work around "ast.get_docstring" breakage.
+    if body and getKind(body[0]) == "Expr":
         if getKind(body[0].value) == "Str":  # python3.7 or earlier
             doc = body[0].value.s
             body = body[1:]
@@ -71,19 +72,26 @@ def extractDocFromBody(node):
                 doc = body[0].value.value
             body = body[1:]
 
-        if "no_docstrings" in Options.getPythonFlags():
+        if Options.hasPythonFlagNoDocstrings():
             doc = None
 
     return body, doc
 
 
-def parseSourceCodeToAst(source_code, filename, line_offset):
+def parseSourceCodeToAst(source_code, module_name, filename, line_offset):
     # Workaround: ast.parse cannot cope with some situations where a file is not
     # terminated by a new line.
     if not source_code.endswith("\n"):
         source_code = source_code + "\n"
 
-    body = ast.parse(source_code, filename)
+    try:
+        body = ast.parse(source_code, filename)
+    except RuntimeError as e:
+        if "maximum recursion depth" in e.args[0]:
+            raise CodeTooComplexCode(module_name, filename)
+
+        raise
+
     assert getKind(body) == "Module"
 
     if line_offset > 0:
@@ -327,9 +335,14 @@ def buildNode(provider, node, source_ref, allow_none=False):
         # Very likely the stack overflow, which we will turn into too complex
         # code exception, don't warn about it with a code dump then.
         raise
+    except KeyboardInterrupt:
+        # User interrupting is not a problem with the source, but tell where
+        # we got interrupted.
+        optimization_logger.info("Interrupted at '%s'." % source_ref)
+        raise
     except:
         optimization_logger.warning(
-            "Problem at '%s' with %s." % (source_ref, ast.dump(node))
+            "Problem at '%s' with %s." % (source_ref.getAsString(), ast.dump(node))
         )
         raise
 
@@ -386,14 +399,13 @@ def buildAnnotationNode(provider, node, source_ref):
         m = {}
         exec(r, m)
 
-        # TODO: In Python3.9, we should get https://bugs.python.org/issue35143
-        # to be used.
-        if Options.is_debug:
-            assert python_version <= 390
+        value = m["__annotations__"]["x"]
 
-        return makeConstantRefNode(
-            constant=m["__annotations__"]["x"], source_ref=source_ref
-        )
+        if Options.is_debug and python_version >= 0x390:
+            # TODO: In Python3.9+, we should only use ast.unparse
+            assert value == ast.unparse(node)
+
+        return makeConstantRefNode(constant=value, source_ref=source_ref)
 
     return buildNode(provider, node, source_ref)
 
@@ -511,7 +523,8 @@ def makeStatementsSequence(statements, allow_none, source_ref):
 
     if statements:
         return StatementsSequence(
-            statements=mergeStatements(statements), source_ref=source_ref
+            statements=mergeStatements(statements, allow_none=allow_none),
+            source_ref=source_ref,
         )
     else:
         return None
