@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Part of "Nuitka", an optimizing Python compiler that is compatible and
 #     integrates with CPython, but also works on its own.
@@ -26,24 +26,34 @@ complete.
 
 from nuitka.nodes.AssignNodes import (
     StatementAssignmentVariable,
-    StatementDelVariable
+    StatementDelVariable,
+    StatementReleaseVariable,
 )
 from nuitka.nodes.FunctionNodes import MaybeLocalVariableUsage
 from nuitka.nodes.LocalsDictNodes import (
     ExpressionLocalsVariableRef,
-    ExpressionLocalsVariableRefORFallback,
+    ExpressionLocalsVariableRefOrFallback,
     StatementLocalsDictOperationDel,
-    StatementLocalsDictOperationSet
+    StatementLocalsDictOperationSet,
+)
+from nuitka.nodes.NodeMakingHelpers import (
+    makeConstantReplacementNode,
+    mergeStatements,
+)
+from nuitka.nodes.OperatorNodes import makeExpressionOperationBinaryInplace
+from nuitka.nodes.VariableRefNodes import (
+    ExpressionTempVariableRef,
+    makeExpressionVariableRef,
 )
-from nuitka.nodes.NodeMakingHelpers import makeConstantReplacementNode
-from nuitka.nodes.VariableRefNodes import ExpressionVariableRef
 from nuitka.PythonVersions import (
     getErrorMessageExecWithNestedFunction,
-    python_version
+    python_version,
 )
+from nuitka.Variables import isSharedAmongScopes, releaseSharedScopeInformation
 
 from .Operations import VisitorNoopMixin, visitTree
 from .ReformulationFunctionStatements import addFunctionVariableReleases
+from .ReformulationTryFinallyStatements import makeTryFinallyStatement
 from .SyntaxErrors import raiseSyntaxError
 
 # Note: We do the variable scope assignment, as an extra step from tree
@@ -55,39 +65,39 @@ from .SyntaxErrors import raiseSyntaxError
 # lookups are to be done immediately, and one where it is delayed. This is
 # basically class vs. function scope handling.
 
-class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
-    """ Variable closure phase 1: Find assignments and early closure references.
 
-        In class context, a reference to a variable must be obeyed immediately,
-        so that "variable = variable" takes first "variable" as a closure and
-        then adds a new local "variable" to override it from there on. For the
-        not early closure case of a function, this will not be done and only
-        assignments shall add local variables, and references will be ignored
-        until phase 2.
+class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
+    """Variable closure phase 1: Find assignments and early closure references.
+
+    In class context, a reference to a variable must be obeyed immediately,
+    so that "variable = variable" takes first "variable" as a closure and
+    then adds a new local "variable" to override it from there on. For the
+    not early closure case of a function, this will not be done and only
+    assignments shall add local variables, and references will be ignored
+    until phase 2.
     """
 
     @staticmethod
     def _handleNonLocal(node):
         # Take closure variables for non-local declarations.
 
-        for non_local_names, source_ref in node.consumeNonlocalDeclarations():
+        for (
+            non_local_names,
+            user_provided,
+            source_ref,
+        ) in node.consumeNonlocalDeclarations():
             for non_local_name in non_local_names:
 
-                variable = node.takeVariableForClosure(
-                    variable_name = non_local_name
-                )
+                variable = node.takeVariableForClosure(variable_name=non_local_name)
 
-                node.registerProvidedVariable(variable)
+                node.getLocalsScope().registerClosureVariable(variable)
 
-                if variable.isModuleVariable():
+                if variable.isModuleVariable() and user_provided:
                     raiseSyntaxError(
-                        "no binding for nonlocal '%s' found" % (
-                            non_local_name
-                        ),
-                        source_ref
+                        "no binding for nonlocal '%s' found" % (non_local_name),
+                        source_ref,
                     )
 
-
                 variable.addVariableUser(node)
 
     @staticmethod
@@ -104,11 +114,12 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     )
 
                     if class_variable.isModuleVariable():
-                        qualname_node = qualname_assign.subnode_value
+                        qualname_node = qualname_assign.subnode_source
 
                         new_node = makeConstantReplacementNode(
-                            constant = class_variable.getName(),
-                            node     = qualname_node
+                            constant=class_variable.getName(),
+                            node=qualname_node,
+                            user_provided=True,
                         )
 
                         parent = qualname_node.parent
@@ -134,9 +145,10 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
     @staticmethod
     def _shouldUseLocalsDict(provider, variable_name):
-        return provider.isExpressionClassBody() and \
-               (not provider.hasProvidedVariable(variable_name) or \
-               provider.getProvidedVariable(variable_name).getOwner() is provider)
+        return provider.isExpressionClassBody() and (
+            not provider.hasProvidedVariable(variable_name)
+            or provider.getProvidedVariable(variable_name).getOwner() is provider
+        )
 
     def onLeaveNode(self, node):
         if node.isStatementAssignmentVariableName():
@@ -146,26 +158,79 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             # Classes always assign to locals dictionary except for closure
             # variables taken.
             if self._shouldUseLocalsDict(provider, variable_name):
-                new_node = StatementLocalsDictOperationSet(
-                    locals_scope  = provider.getFunctionLocalsScope(),
-                    variable_name = variable_name,
-                    value         = node.subnode_source,
-                    source_ref    = node.source_ref
-                )
+                if node.subnode_source.isExpressionOperationInplace():
+                    temp_scope = provider.allocateTempScope("class_inplace")
+
+                    tmp_variable = provider.allocateTempVariable(
+                        temp_scope=temp_scope, name="value"
+                    )
+
+                    statements = mergeStatements(
+                        statements=(
+                            StatementAssignmentVariable(
+                                variable=tmp_variable,
+                                source=node.subnode_source.subnode_left,
+                                source_ref=node.source_ref,
+                            ),
+                            makeTryFinallyStatement(
+                                provider=provider,
+                                tried=(
+                                    StatementAssignmentVariable(
+                                        variable=tmp_variable,
+                                        source=makeExpressionOperationBinaryInplace(
+                                            left=ExpressionTempVariableRef(
+                                                variable=tmp_variable,
+                                                source_ref=node.source_ref,
+                                            ),
+                                            right=node.subnode_source.subnode_right,
+                                            operator=node.subnode_source.getOperator(),
+                                            source_ref=node.source_ref,
+                                        ),
+                                        source_ref=node.source_ref,
+                                    ),
+                                    StatementLocalsDictOperationSet(
+                                        locals_scope=provider.getLocalsScope(),
+                                        variable_name=variable_name,
+                                        value=ExpressionTempVariableRef(
+                                            variable=tmp_variable,
+                                            source_ref=node.source_ref,
+                                        ),
+                                        source_ref=node.source_ref,
+                                    ),
+                                ),
+                                final=StatementReleaseVariable(
+                                    variable=tmp_variable, source_ref=node.source_ref
+                                ),
+                                source_ref=node.source_ref,
+                            ),
+                        )
+                    )
+
+                    node.parent.replaceStatement(node, statements)
+
+                else:
+                    new_node = StatementLocalsDictOperationSet(
+                        locals_scope=provider.getLocalsScope(),
+                        variable_name=variable_name,
+                        value=node.subnode_source,
+                        source_ref=node.source_ref,
+                    )
+
+                    node.parent.replaceChild(node, new_node)
             else:
                 variable = provider.getVariableForAssignment(
-                    variable_name = variable_name
+                    variable_name=variable_name
                 )
 
                 new_node = StatementAssignmentVariable(
-                    variable   = variable,
-                    source     = node.subnode_source,
-                    source_ref = node.source_ref
+                    variable=variable,
+                    source=node.subnode_source,
+                    source_ref=node.source_ref,
                 )
 
                 variable.addVariableUser(provider)
 
-            node.parent.replaceChild(node, new_node)
+                node.parent.replaceChild(node, new_node)
 
             del node.parent
             del node.provider
@@ -177,22 +242,21 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             if self._shouldUseLocalsDict(provider, variable_name):
                 # Classes always assign to locals dictionary except for closure
                 # variables taken.
-
-                # TODO: This is losing the "tolerant" annotation.
                 new_node = StatementLocalsDictOperationDel(
-                    locals_scope  = provider.getFunctionLocalsScope(),
-                    variable_name = variable_name,
-                    source_ref    = node.source_ref
+                    locals_scope=provider.getLocalsScope(),
+                    variable_name=variable_name,
+                    tolerant=node.tolerant,
+                    source_ref=node.source_ref,
                 )
             else:
                 variable = provider.getVariableForAssignment(
-                    variable_name = variable_name
+                    variable_name=variable_name
                 )
 
                 new_node = StatementDelVariable(
-                    variable   = variable,
-                    tolerant   = node.tolerant,
-                    source_ref = node.source_ref
+                    variable=variable,
+                    tolerant=node.tolerant,
+                    source_ref=node.source_ref,
                 )
 
                 variable.addVariableUser(provider)
@@ -203,9 +267,8 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             parent.replaceChild(node, new_node)
 
     def onEnterNode(self, node):
-        # Mighty complex code with lots of branches and statements, but it
-        # couldn't be less without making it more difficult.
-        # pylint: disable=too-many-branches,too-many-statements
+        # Mighty complex code with lots of branches, but we aim to get rid of it.
+        # pylint: disable=too-many-branches
 
         if node.isExpressionVariableNameRef():
             provider = node.provider
@@ -213,25 +276,26 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
             if provider.isExpressionClassBody():
                 if node.needsFallback():
                     variable = provider.getVariableForReference(
-                        variable_name = node.getVariableName()
+                        variable_name=node.getVariableName()
                     )
 
-                    new_node = ExpressionLocalsVariableRefORFallback(
-                        locals_scope  = provider.getFunctionLocalsScope(),
-                        variable_name = node.getVariableName(),
-                        fallback      = ExpressionVariableRef(
-                            variable   = variable,
-                            source_ref = node.source_ref
+                    new_node = ExpressionLocalsVariableRefOrFallback(
+                        locals_scope=provider.getLocalsScope(),
+                        variable_name=node.getVariableName(),
+                        fallback=makeExpressionVariableRef(
+                            variable=variable,
+                            locals_scope=provider.getLocalsScope(),
+                            source_ref=node.source_ref,
                         ),
-                        source_ref    = node.source_ref
+                        source_ref=node.source_ref,
                     )
 
                     variable.addVariableUser(provider)
                 else:
                     new_node = ExpressionLocalsVariableRef(
-                        locals_scope  = provider.getFunctionLocalsScope(),
-                        variable_name = node.getVariableName(),
-                        source_ref    = node.source_ref
+                        locals_scope=provider.getLocalsScope(),
+                        variable_name=node.getVariableName(),
+                        source_ref=node.source_ref,
                     )
 
                 parent = node.parent
@@ -240,14 +304,13 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                 parent.replaceChild(node, new_node)
         elif node.isExpressionTempVariableRef():
             if node.getVariable().getOwner() != node.getParentVariableProvider():
-                node.getParentVariableProvider().addClosureVariable(
-                    node.getVariable()
-                )
+                node.getParentVariableProvider().addClosureVariable(node.getVariable())
         elif node.isExpressionGeneratorObjectBody():
-            self._handleNonLocal(node)
+            if python_version >= 0x300:
+                self._handleNonLocal(node)
 
             # Only Python3.4 or later allows for generators to have qualname.
-            if python_version >= 340:
+            if python_version >= 0x340:
                 self._handleQualnameSetup(node)
         elif node.isExpressionCoroutineObjectBody():
             self._handleNonLocal(node)
@@ -258,50 +321,21 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
 
             self._handleQualnameSetup(node)
         elif node.isExpressionClassBody():
-            self._handleNonLocal(node)
+            if python_version >= 0x300:
+                self._handleNonLocal(node)
 
             # Python3.4 allows for class declarations to be made global, even
             # after they were declared, so we need to fix this up.
-            if python_version >= 340:
+            if python_version >= 0x340:
                 self._handleQualnameSetup(node)
         elif node.isExpressionFunctionBody():
-            self._handleNonLocal(node)
+            if python_version >= 0x300:
+                self._handleNonLocal(node)
 
             # Python 3.4 allows for class declarations to be made global, even
             # after they were declared, so we need to fix this up.
-            if python_version >= 340:
+            if python_version >= 0x340:
                 self._handleQualnameSetup(node)
-        # Attribute access of names of class functions should be mangled, if
-        # they start with "__", but do not end in "__" as well.
-        elif node.isExpressionAttributeLookup() or \
-             node.isStatementAssignmentAttribute() or \
-             node.isStatementDelAttribute():
-            attribute_name = node.getAttributeName()
-
-            if attribute_name.startswith("__") and \
-               not attribute_name.endswith("__"):
-                seen_function = False
-
-                current = node
-
-                while True:
-                    current = current.getParentVariableProvider()
-
-                    if current.isCompiledPythonModule():
-                        break
-
-                    if current.isExpressionClassBody():
-                        if seen_function:
-                            node.setAttributeName(
-                                "_%s%s" % (
-                                    current.getName().lstrip('_'),
-                                    attribute_name
-                                )
-                            )
-
-                        break
-                    else:
-                        seen_function = True
         # Check if continue and break are properly in loops. If not, raise a
         # syntax error.
         elif node.isStatementLoopBreak() or node.isStatementLoopContinue():
@@ -319,21 +353,18 @@ class VariableClosureLookupVisitorPhase1(VisitorNoopMixin):
                     else:
                         message = "'break' outside loop"
 
-                    raiseSyntaxError(
-                        message,
-                        node.getSourceReference(),
-                    )
+                    raiseSyntaxError(message, node.getSourceReference())
 
 
 class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
-    """ Variable closure phase 2: Find assignments and references.
+    """Variable closure phase 2: Find assignments and references.
 
-        In class context, a reference to a variable must be obeyed immediately,
-        so that "variable = variable" takes first "variable" as a closure and
-        then adds a new local "variable" to override it from there on.
+    In class context, a reference to a variable must be obeyed immediately,
+    so that "variable = variable" takes first "variable" as a closure and
+    then adds a new local "variable" to override it from there on.
 
-        So, assignments for early closure, accesses will already have a
-        variable set now, the others, only in this phase.
+    So, assignments for early closure, accesses will already have a
+    variable set now, the others, only in this phase.
     """
 
     @staticmethod
@@ -342,27 +373,27 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
 
         variable_name = node.getVariableName()
 
-        variable = provider.getVariableForReference(
-            variable_name = variable_name
-        )
+        variable = provider.getVariableForReference(variable_name=variable_name)
 
         # Need to catch functions with "exec" and closure variables not allowed.
-        if python_version < 300 and provider.isExpressionFunctionBodyBase():
+        if python_version < 0x300 and provider.isExpressionFunctionBodyBase():
             was_taken = provider.hasTakenVariable(variable_name)
 
-            if not was_taken and \
-               variable.getOwner() is not provider:
+            if not was_taken and variable.getOwner() is not provider:
                 parent_provider = provider.getParentVariableProvider()
 
                 while parent_provider.isExpressionClassBody():
                     parent_provider = parent_provider.getParentVariableProvider()
 
-                if parent_provider.isExpressionFunctionBody() and \
-                   parent_provider.isUnqualifiedExec():
+                if (
+                    parent_provider.isExpressionFunctionBody()
+                    and parent_provider.isUnqualifiedExec()
+                ):
                     raiseSyntaxError(
-                        getErrorMessageExecWithNestedFunction() % parent_provider.getName(),
+                        getErrorMessageExecWithNestedFunction()
+                        % parent_provider.getName(),
                         node.getSourceReference(),
-                        display_line = False # Wrong line anyway
+                        display_line=False,  # Wrong line anyway
                     )
 
         return variable
@@ -376,19 +407,23 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
             except MaybeLocalVariableUsage:
                 variable_name = node.getVariableName()
 
-                new_node = ExpressionLocalsVariableRefORFallback(
-                    locals_scope  = provider.getFunctionLocalsScope(),
-                    variable_name = variable_name,
-                    fallback      = ExpressionVariableRef(
-                        variable   = node.getParentModule().getVariableForReference(variable_name),
-                        source_ref = node.source_ref
+                new_node = ExpressionLocalsVariableRefOrFallback(
+                    locals_scope=provider.getLocalsScope(),
+                    variable_name=variable_name,
+                    fallback=makeExpressionVariableRef(
+                        variable=node.getParentModule().getVariableForReference(
+                            variable_name
+                        ),
+                        locals_scope=provider.getLocalsScope(),
+                        source_ref=node.source_ref,
                     ),
-                    source_ref    = node.source_ref
+                    source_ref=node.source_ref,
                 )
             else:
-                new_node = ExpressionVariableRef(
-                    variable   = variable,
-                    source_ref = node.source_ref
+                new_node = makeExpressionVariableRef(
+                    variable=variable,
+                    locals_scope=provider.getLocalsScope(),
+                    source_ref=node.source_ref,
                 )
 
                 variable.addVariableUser(provider)
@@ -400,28 +435,26 @@ class VariableClosureLookupVisitorPhase2(VisitorNoopMixin):
 
 
 class VariableClosureLookupVisitorPhase3(VisitorNoopMixin):
-    """ Variable closure phase 3: Find errors and complete frame variables.
+    """Variable closure phase 3: Find errors and complete frame variables.
 
-        In this phase, we can do some fix-ups and find errors. We might e.g.
-        detect that a "del" was executed on a shared variable, which is not
-        allowed for Python 2.x, so it must be caught. The parsing wouldn't do
-        that.
+    In this phase, we can do some fix-ups and find errors. We might e.g.
+    detect that a "del" was executed on a shared variable, which is not
+    allowed for Python 2.x, so it must be caught. The parsing wouldn't do
+    that.
 
-        Also, frame objects for functions should learn their variable names.
+    Also, frame objects for functions should learn their variable names.
     """
 
     def onEnterNode(self, node):
-        if python_version < 300 and node.isStatementDelVariable():
+        if python_version < 0x300 and node.isStatementDelVariable():
             variable = node.getVariable()
 
-            if not variable.isModuleVariable() and \
-               variable.isSharedAmongScopes():
+            if not variable.isModuleVariable() and isSharedAmongScopes(variable):
                 raiseSyntaxError(
                     """\
-can not delete variable '%s' referenced in nested scope""" % (
-                       variable.getName()
-                    ),
-                    node.getSourceReference()
+can not delete variable '%s' referenced in nested scope"""
+                    % (variable.getName()),
+                    node.getSourceReference(),
                 )
         elif node.isStatementsFrame():
             node.updateLocalNames()
@@ -435,18 +468,21 @@ can not delete variable '%s' referenced in nested scope""" % (
                     class_var = node.takeVariableForClosure("__class__")
                     class_var.addVariableUser(node)
 
-                    node.registerProvidedVariable(class_var)
+                    node.getLocalsScope().registerClosureVariable(class_var)
                     while node != class_var.getOwner():
                         node = node.getParentVariableProvider()
-                        node.registerProvidedVariable(class_var)
+                        node.getLocalsScope().registerClosureVariable(class_var)
 
 
 def completeVariableClosures(tree):
     visitors = (
         VariableClosureLookupVisitorPhase1(),
         VariableClosureLookupVisitorPhase2(),
-        VariableClosureLookupVisitorPhase3()
+        VariableClosureLookupVisitorPhase3(),
     )
 
     for visitor in visitors:
         visitTree(tree, visitor)
+
+    # Only used to detect syntax errors.
+    releaseSharedScopeInformation(tree)
