@@ -23,70 +23,55 @@ binaries (needed for exec) and run them capturing outputs.
 
 
 import os
-import shutil
-import subprocess
 from contextlib import contextmanager
 
+from nuitka.__past__ import (  # pylint: disable=I0021,redefined-builtin
+    WindowsError,
+    subprocess,
+)
 from nuitka.PythonVersions import python_version
 from nuitka.Tracing import general
 
+from .Download import getCachedDownloadedMinGW64
+from .FileOperations import getExternalUsePath
 from .Utils import getArchitecture, getOS, isWin32Windows
 
+# Cache, so we avoid repeated command lookups.
+_executable_command_cache = {}
 
-def callExec(args):
-    """Do exec in a portable way preserving exit code.
 
-    On Windows, unfortunately there is no real exec, so we have to spawn
-    a new process instead.
-    """
+def _getExecutablePath(filename, search_path):
+    # Append ".exe" suffix  on Windows if not already present.
+    if getOS() == "Windows" and not filename.lower().endswith(".exe"):
+        filename += ".exe"
 
-    # On Windows os.execl does not work properly
-    if isWin32Windows():
-        args = list(args)
-        del args[1]
+    # Now check in each path element, much like the shell will.
+    path_elements = search_path.split(os.pathsep)
 
-        try:
-            process = subprocess.Popen(args=args)
-            process.communicate()
-            # No point in cleaning up, pylint: disable=protected-access
-            try:
-                os._exit(process.returncode)
-            except OverflowError:
-                # Seems negative values go wrong otherwise,
-                # see https://bugs.python.org/issue28474
-                os._exit(process.returncode - 2 ** 32)
-        except KeyboardInterrupt:
-            # There was a more relevant stack trace already, so abort this
-            # right here, pylint: disable=protected-access
-            os._exit(2)
-    else:
-        # The star arguments is the API of execl
-        os.execl(*args)
+    for path_element in path_elements:
+        path_element = path_element.strip('"')
+
+        full = os.path.join(path_element, filename)
+
+        if os.path.exists(full):
+            return full
 
 
 def getExecutablePath(filename):
-    """ Find an execute in PATH environment. """
+    """Find an execute in PATH environment."""
 
-    if python_version >= 0x300:
-        return shutil.which(filename)
-    else:
-        # Append ".exe" suffix  on Windows if not already present.
-        if getOS() == "Windows" and not filename.lower().endswith(".exe"):
-            filename += ".exe"
+    # Search in PATH environment.
+    search_path = os.environ.get("PATH", "")
 
-        # Search in PATH environment.
-        search_path = os.environ.get("PATH", "")
+    key = (filename, search_path)
+    if key not in _executable_command_cache:
+        _executable_command_cache[key] = _getExecutablePath(filename, search_path)
 
-        # Now check in each path element, much like the shell will.
-        path_elements = search_path.split(os.pathsep)
+    return _executable_command_cache[key]
 
-        for path_element in path_elements:
-            path_element = path_element.strip('"')
 
-            full = os.path.join(path_element, filename)
-
-            if os.path.exists(full):
-                return full
+def isExecutableCommand(command):
+    return getExecutablePath(command) is not None
 
 
 def getPythonExePathWindows(search, arch):
@@ -132,6 +117,30 @@ def getPythonExePathWindows(search, arch):
                 return candidate
 
 
+class NuitkaCalledProcessError(subprocess.CalledProcessError):
+    def __init__(self, retcode, cmd, output, stderr):
+        # False alarm, pylint: disable=super-init-not-called
+
+        subprocess.CalledProcessError(self, retcode, cmd)
+
+        # Python2 doesn't have this otherwise, but needs it.
+        self.stderr = stderr
+        self.output = output
+        self.cmd = cmd
+        self.returncode = retcode
+
+    def __str__(self):
+        result = subprocess.CalledProcessError.__str__(self)
+
+        if self.output:
+            result += " Output was %r." % self.output.strip()
+
+        if self.stderr:
+            result += " Error was %r." % self.stderr.strip()
+
+        return result
+
+
 def check_output(*popenargs, **kwargs):
     """Call a process and check result code.
 
@@ -145,8 +154,12 @@ def check_output(*popenargs, **kwargs):
     if "stdout" in kwargs:
         raise ValueError("stdout argument not allowed, it will be overridden.")
 
+    if "stderr" not in kwargs:
+        kwargs["stderr"] = subprocess.PIPE
+
     process = subprocess.Popen(stdout=subprocess.PIPE, *popenargs, **kwargs)
-    output, _unused_err = process.communicate()
+
+    output, stderr = process.communicate()
     retcode = process.poll()
 
     if retcode:
@@ -154,7 +167,7 @@ def check_output(*popenargs, **kwargs):
         if cmd is None:
             cmd = popenargs[0]
 
-        raise subprocess.CalledProcessError(retcode, cmd, output=output)
+        raise NuitkaCalledProcessError(retcode, cmd, output=output, stderr=stderr)
 
     return output
 
@@ -176,14 +189,21 @@ def check_call(*popenargs, **kwargs):
         )
 
 
+def callProcess(*popenargs, **kwargs):
+    """Call a process and return result code."""
+    subprocess.call(*popenargs, **kwargs)
+
+
 @contextmanager
 def withEnvironmentPathAdded(env_var_name, *paths):
+    assert os.path.sep not in env_var_name
+
     paths = [path for path in paths if path]
     path = os.pathsep.join(paths)
 
     if path:
         if str is not bytes and type(path) is bytes:
-            path = path.decode("utf-8")
+            path = path.decode("utf8")
 
         if env_var_name in os.environ:
             old_path = os.environ[env_var_name]
@@ -203,7 +223,7 @@ def withEnvironmentPathAdded(env_var_name, *paths):
 
 @contextmanager
 def withEnvironmentVarOverriden(env_var_name, value):
-    """ Change an environment and restore it after context. """
+    """Change an environment and restore it after context."""
 
     if env_var_name in os.environ:
         old_value = os.environ[env_var_name]
@@ -226,7 +246,7 @@ def withEnvironmentVarOverriden(env_var_name, value):
 
 @contextmanager
 def withEnvironmentVarsOverriden(mapping):
-    """ Change multiple environment variables and restore them after context. """
+    """Change multiple environment variables and restore them after context."""
 
     old_values = {}
 
@@ -266,10 +286,24 @@ def wrapCommandForDebuggerForExec(*args):
     """
 
     gdb_path = getExecutablePath("gdb")
-    lldb_path = getExecutablePath("lldb")
 
-    if gdb_path is None and lldb_path is None:
-        general.sysexit("Error, no 'gdb' or 'lldb' binary found in path.")
+    # Windows extra ball, attempt the downloaded one.
+    if isWin32Windows():
+        from nuitka.Options import assumeYesForDownloads
+
+        mingw64_gcc_path = getCachedDownloadedMinGW64(
+            target_arch=getArchitecture(),
+            assume_yes_for_downloads=assumeYesForDownloads(),
+        )
+
+        with withEnvironmentPathAdded("PATH", os.path.dirname(mingw64_gcc_path)):
+            gdb_path = getExecutablePath("gdb")
+
+    if gdb_path is None:
+        lldb_path = getExecutablePath("lldb")
+
+        if lldb_path is None:
+            general.sysexit("Error, no 'gdb' or 'lldb' binary found in path.")
 
     if gdb_path is not None:
         args = (gdb_path, "gdb", "-ex=run", "-ex=where", "-ex=quit", "--args") + args
@@ -364,3 +398,72 @@ def getNullOutput():
         return subprocess.NULLDEV
     except AttributeError:
         return open(os.devnull, "wb")
+
+
+def getNullInput():
+    try:
+        return subprocess.NULLDEV
+    except AttributeError:
+        # File is supposed to stay open, pylint: disable=consider-using-with
+        subprocess.NULLDEV = open(os.devnull, "rb")
+        return subprocess.NULLDEV
+
+
+def executeToolChecked(logger, command, absence_message, stderr_filter=None):
+    """Execute external tool, checking for success and no error outputs, returning result."""
+
+    tool = command[0]
+
+    if not isExecutableCommand(tool):
+        logger.sysexit(absence_message)
+
+    # Allow to avoid repeated scans in PATH for the tool.
+    command[0] = getExecutablePath(tool)
+
+    process = subprocess.Popen(
+        command,
+        stdin=getNullInput(),
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        shell=False,
+    )
+
+    stdout, stderr = process.communicate()
+    result = process.poll()
+
+    if stderr_filter is not None:
+        stderr = stderr_filter(stderr)
+
+    if result != 0:
+        logger.sysexit("Error, call to %r failed: %s -> %s." % (tool, command, stderr))
+    elif stderr:
+        logger.sysexit(
+            "Error, call to %r gave warnings: %s -> %s." % (tool, command, stderr)
+        )
+
+    return stdout
+
+
+def executeProcess(
+    command, env=None, needs_stdin=False, shell=False, external_cwd=False
+):
+    if not env:
+        env = os.environ
+
+    process = subprocess.Popen(
+        command,
+        stdin=subprocess.PIPE if needs_stdin else getNullInput(),
+        stdout=subprocess.PIPE,
+        stderr=subprocess.PIPE,
+        shell=shell,
+        # On Windows, closing file descriptions is now working with capturing outputs.
+        close_fds=not isWin32Windows(),
+        env=env,
+        # For tools that want short paths to work.
+        cwd=getExternalUsePath(os.getcwd()) if external_cwd else None,
+    )
+
+    stdout, stderr = process.communicate()
+    exit_code = process.wait()
+
+    return stdout, stderr, exit_code
