@@ -26,9 +26,18 @@ import fnmatch
 import os
 
 
+def checkModuleName(value):
+    return ".." not in str(value) and not (
+        str(value).endswith(".") or str(value) == "."
+    )
+
+
 class ModuleName(str):
     def __init__(self, value):
-        assert ".." not in str(value), value
+        assert checkModuleName(value), value
+
+        # TODO: Disallow some conversion, e.g. from module, function, etc.
+        # objects, and white list what types we accept.
 
         str.__init__(value)
 
@@ -82,6 +91,14 @@ class ModuleName(str):
 
         return self.splitModuleBasename()[0]
 
+    def getRelativePackageName(self, level):
+        result = ".".join(self.asString().split(".")[: -level + 1])
+
+        if result == "":
+            return None
+        else:
+            return ModuleName(result)
+
     def getTopLevelPackageName(self):
         """Get the top level package name.
 
@@ -104,7 +121,7 @@ class ModuleName(str):
         return self.splitModuleBasename()[1]
 
     def splitModuleBasename(self):
-        """ Split a module into package name and module name."""
+        """Split a module into package name and module name."""
 
         if "." in self:
             package_part = ModuleName(self[: self.rfind(".")])
@@ -116,7 +133,7 @@ class ModuleName(str):
         return package_part, module_name
 
     def splitPackageName(self):
-        """ Split a module into the top level package name and remaining module name."""
+        """Split a module into the top level package name and remaining module name."""
 
         if "." in self:
             package_part = ModuleName(self[: self.find(".")])
