@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -122,7 +122,7 @@ def branchingFunction(a, b, c):
         print("outer else")
 
     print("Complex conditional expression:")
-    v = (3 if a - 1 else 0) or (b or (c * 2 if c else 6) if b - 1 else a and b and c)
+    v = (3 if a + 1 else 0) or (b or (c * 2 if c else 6) if b - 1 else a and b and c)
     print(v)
 
     if True:
