@@ -20,6 +20,9 @@
 """
 
 # pylint: disable=broad-except,global-variable-not-assigned,redefined-outer-name
+# pylint: disable=comparison-with-itself,use-dict-literal,use-list-literal
+# pylint: disable=consider-using-with,invalid-bytes-returned,unspecified-encoding
+# pylint: disable=invalid-str-returned,no-self-use,unnecessary-comprehension
 
 from __future__ import print_function
 
@@ -613,18 +616,20 @@ elif isinstance(v, complex):
 else:
     print("Oops, must not happen.")
 
-# Test for any builtins
-print(any([None, None, None]))
-print(any([None, 4, None]))
-print(any([]))
-print(any([0] * 20000))
-print(any([0] * 255))
-print(any(range(260)))
-print(any(range(1, 270)))
-print(any(range(2, 1024, 5)))
+print("Tests for abs():")
+print(abs(-(1000000 ** 10)))
+print(abs(len([1, 2, 3])))
+print(abs(-100))
+print(abs(float("nan")))
+print("abs() with list:")
+try:
+    print(abs([1, 2]))
+except Exception as e:
+    print("caught ", repr(e))
 
 
-def S():
+# Making a condition >42 incrementally True
+def S1():
     print("Yielding 40")
     yield 40
     print("Yielding 60")
@@ -633,7 +638,57 @@ def S():
     yield 30
 
 
-print(any(x > 42 for x in S()))
+# Making a condition >42 incrementally False
+def S2():
+    print("Yielding 60")
+    yield 60
+    print("Yielding 40")
+    yield 40
+    print("Yielding 30")
+    yield 30
+
+
+# Test for "all" built-in
+print(all(x > 42 for x in S1()))
+print(all(x > 42 for x in S2()))
+
+print("Disallowed all() without args:")
+try:
+    print(all())
+except Exception as e:
+    print("caught ", repr(e))
+
+print("all() with float not iterable:")
+try:
+    print(all(1.0))
+except Exception as e:
+    print("caught ", repr(e))
+
+print("all() with float type not iterable:")
+try:
+    print(any(float))
+except Exception as e:
+    print("caught ", repr(e))
+
+print("all with compile time lists:")
+print(all([None, None, None]))
+print(all([None, 4, None]))
+print(all([]))
+print(all([0] * 20000))
+print(all([0] * 255))
+print("all with compile time ranges:")
+print(all(range(1, 10000)))
+print(all(range(10000)))
+print(all(range(2, 999, 4)))
+print("all with compile time strings and bytes:")
+print(all("Nuitka rocks!"))
+print(all("string"))
+print(all(u"unicode"))
+print(all(b"bytes"))
+print(all(b"bytes\0"))
+
+print(any(x > 42 for x in S1()))
+print(any(x > 42 for x in S2()))
 
 print("Disallowed any() without args:")
 try:
@@ -656,29 +711,34 @@ except Exception as e:
 print("any() with sets:")
 print(any(set([0, 1, 2, 3, 3])))
 print(any({1: "One", 2: "Two"}))
-print("Tests for abs():")
-print(abs(-(1000000 ** 10)))
-print(abs(len([1, 2, 3])))
-print(abs(-100))
-print(abs(float("nan")))
-print("abs() with list:")
-try:
-    print(abs([1, 2]))
-except Exception as e:
-    print("caught ", repr(e))
 
-# Test for all builtins
-print(all([None, None, None]))
-print(all([None, 4, None]))
-print(all([]))
-print(all([0] * 20000))
-print(all([0] * 255))
-print("All with ranges:")
-print(all(range(1, 10000)))
-print(all(range(10000)))
-print(all(range(2, 999, 4)))
-print("All with strings:")
-print(all("Nuitka rocks!"))
-print(all("string"))
-print(all(u"unicode"))
-print(all(b"bytes"))
+print("any with compile time lists:")
+print(any([None, None, None]))
+print(any([None, 4, None]))
+print(any([]))
+print(any([0] * 20000))
+print(any([0] * 255))
+print("any with compile time ranges:")
+print(any(range(1, 10000)))
+print(any(range(10000)))
+print(any(range(2, 999, 4)))
+print("any with compile time strings and bytes:")
+print(any("Nuitka rocks!"))
+print(any("string"))
+print(any(u"unicode"))
+print(any(b"bytes"))
+print(any(b"bytes\0"))
+
+print("Tests for zip():")
+print(zip("abc", "cdd"))
+print(zip([1, 2, 3], [2, 3, 4]))
+print(zip([1, 2, 3], "String"))
+try:
+    zip(1, "String")
+except TypeError as e:
+    print("Occured", repr(e))
+print(zip())
+x = [(u, v) for (u, v) in zip(range(8), reversed(range(8)))]
+print(x)
+for v in zip([1, 2, 3], "String"):
+    print(v)
