@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -21,7 +21,8 @@ With newline and stuff."""
 
 from __future__ import print_function
 
-import os, sys
+import os
+import sys
 
 print("doc:", __doc__)
 print("filename:", os.path.basename(__file__))
@@ -29,14 +30,15 @@ print("builtins:", __builtins__)
 print("debug", __debug__)
 print("debug in builtins", __builtins__.__debug__)
 
-print("__initializing__", end = ' ')
+print("__initializing__", end=" ")
 try:
-    print(__initializing__)  # @UndefinedVariable
+    print(__initializing__)
 except NameError:
     print("not found")
 
+
 def checkFromFunction():
-    frame = sys._getframe(1)
+    frame = sys._getframe(1)  # pylint: disable=protected-access
 
     def displayDict(d):
         if "__loader__" in d:
@@ -47,11 +49,17 @@ def checkFromFunction():
             d = dict(d)
             d["__file__"] = "<__file__ removed>"
 
+        if "__compiled__" in d:
+            d = dict(d)
+            del d["__compiled__"]
+
         import pprint
+
         return pprint.pformat(d)
 
     print("Globals", displayDict(frame.f_globals))
     print("Locals", displayDict(frame.f_locals))
     print("Is identical", frame.f_locals is frame.f_globals)
 
+
 checkFromFunction()
