@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -22,12 +22,12 @@ b = 7
 c = [7, 8]
 d = 15
 
-print('+', a + b)
-print('-', a - b)
-print('*', a * b)
-print('/', a / b)
+print("+", a + b)
+print("-", a - b)
+print("*", a * b)
+print("/", a / b)
 print("//", a // b)
-print('%', b % a)
+print("%", b % a)
 print("& (2)", a & b)
 print("| (2)", a | b)
 print("& (3)", a & b & d)
@@ -39,8 +39,8 @@ print("<<", a << b)
 print(">>", b >> a)
 print("in", b in c)
 print("not in", b not in c)
-print('<', a < b)
-print('>', a > b)
+print("<", a < b)
+print(">", a > b)
 print("==", a == b)
 print("<=", a <= b)
 print(">=", a >= b)
@@ -48,24 +48,24 @@ print("!=", a != b)
 print("is", a is b)
 print("is not", a is not b)
 
-print('~', ~ b)
-print('-', - b)
-print('+', + b)
+print("~", ~b)
+print("-", -b)
+print("+", +b)
 
-l =  {('a', 'c') : "a,c", 'b' : 2, 'c' : 3, 'd' : 4  }
-l[ 'l', ] = '6'
+l = {("a", "c"): "a,c", "b": 2, "c": 3, "d": 4}
+l["l",] = "6"
 
 
 print("Extended slicing:")
-print("Should be a,c:", l[ 'a', 'c'])
+print("Should be a,c:", l["a", "c"])
 
 print("Short form of extended slicing:")
 
 d = {}
 # d[1] = 1
 d[1,] = 2
-d[1,2] = 3
-d[1,2,3] = 4
+d[1, 2] = 3
+d[1, 2, 3] = 4
 L = list(d)
 L.sort()
 print(L)
