@@ -1,5 +1,5 @@
 # -*- coding: utf-8 -*-
-#     Copyright 2017, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -37,7 +37,7 @@ a = "partial print"
 # b doesn't exist
 
 try:
-    print a, undefined_global  # @UndefinedVariable
+    print a, undefined_global
 except Exception, e:
     print "then occurred", repr(e)
 
