@@ -15,7 +15,7 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-""" Reference counting tests.
+""" Reference counting tests for features of Python3.5 or higher.
 
 These contain functions that do specific things, where we have a suspect
 that references may be lost or corrupted. Executing them repeatedly and
@@ -25,6 +25,9 @@ These are Python3.5 specific constructs, that will give a SyntaxError or
 not be relevant on older versions.
 """
 
+# While we use that for comparison code, no need to compile that.
+# nuitka-project: --nofollow-import-to=nuitka
+
 import os
 import sys
 
