@@ -25,6 +25,9 @@ These are for Python2 only constructs, which will give SyntaxError on
 other versions.
 """
 
+# While we use that for comparison code, no need to compile that.
+# nuitka-project: --nofollow-import-to=nuitka
+
 import os
 import sys
 
@@ -98,6 +101,19 @@ def simpleFunction7():
         pass
 
 
+def simpleFunction8():
+    from decimal import *
+
+    d1 = Decimal("5")
+    d2 = Decimal("2")
+
+    (p, q) = divmod(d1, d2)
+    (p, q) = divmod(d1, 4)
+    (p, q) = divmod(7, d1)
+
+    return q, p
+
+
 # These need stderr to be wrapped.
 tests_stderr = ()
 
