@@ -60,11 +60,11 @@ def main():
             "expect_success",
             # Keep no temporary files.
             "remove_output",
-            # Include imported files, mostly nothing though.
-            "recurse_all",
+            # Do not follow imports.
+            "--nofollow-imports",
             # Use the original __file__ value, at least one case warns about things
             # with filename included.
-            "original_file",
+            "--file-reference-choice=original",
             # Cache the CPython results for re-use, they will normally not change.
             "cpython_cache",
             # To understand what is slow.
@@ -76,8 +76,6 @@ def main():
         # This test should be run with the debug Python, and makes outputs to
         # standard error that might be ignored.
         if filename.startswith("Referencing"):
-            extra_flags.append("recurse_not:nuitka")
-
             if hasDebugPython():
                 extra_flags.append("python_debug")
 
@@ -112,9 +110,6 @@ def main():
                 needs_2to3=decideNeeds2to3(filename),
             )
 
-            if search_mode.abortIfExecuted():
-                break
-
     search_mode.finish()
 
 
