@@ -48,9 +48,7 @@ sys.path.insert(
 )
 
 # isort:start
-from nuitka.TreeXML import toString
-
-from nuitka.tools.testing.Common import (  # isort:skip
+from nuitka.tools.testing.Common import (
     check_output,
     convertUsing2to3,
     createSearchMode,
@@ -58,8 +56,9 @@ from nuitka.tools.testing.Common import (  # isort:skip
     my_print,
     setup,
 )
+from nuitka.TreeXML import toString
 
-python_version = setup()
+python_version = setup(suite="optimiations")
 
 search_mode = createSearchMode()
 
@@ -90,7 +89,7 @@ def isConstantExpression(expression):
 
     return kind.startswith("Constant") or kind in (
         "ImportModuleHard",
-        "ImportName",
+        "ImportModuleNameHard",
         "ModuleAttributeFileRef",
         "ModuleLoaderRef",
     )
@@ -128,14 +127,6 @@ def checkSequence(filename, statements):
                     if called_expression.attrib["builtin_name"] == "print":
                         continue
 
-        # Trying of printing is fine, but not needed
-        if kind == "Try" and False:
-            tried = getRole(statement, "tried")
-
-            checkSequence(filename, getRole(tried[0], "statements"))
-
-            continue
-
         if kind == "FrameModule":
             checkSequence(filename, getRole(statement, "statements"))
 
@@ -278,9 +269,6 @@ def main():
 
             my_print("OK.")
 
-            if search_mode.abortIfExecuted():
-                break
-
     search_mode.finish()
 
 
