@@ -48,12 +48,11 @@ from nuitka.tools.testing.Common import (
     getTempDir,
     my_print,
     setup,
-    withExtendedExtraOptions,
 )
 
 
 def main():
-    setup()
+    setup(suite="packages")
 
     search_mode = createSearchMode()
 
@@ -62,51 +61,54 @@ def main():
             continue
 
         extra_flags = [
+            "--module",
             "expect_success",
             "remove_output",
-            "module_mode",
             "two_step_execution",
         ]
 
-        # The use of "__main__" in the test package gives a warning.
-        if filename == "sub_package":
-            extra_flags.append("ignore_warnings")
-
         active = search_mode.consider(dirname=None, filename=filename)
 
         if active:
-            my_print("Consider output of recursively compiled package:", filename)
+            my_print("Consider output of compiled package:", filename)
 
             filename_main = None
-            for filename_main in os.listdir(filename):
-                if not os.path.isdir(os.path.join(filename, filename_main)):
-                    continue
 
-                if filename_main not in ("..", "."):
-                    break
+            filename_main = os.path.join(
+                filename, "".join(part.title() for part in filename.split("_")) + ".py"
+            )
+            if os.path.exists(filename_main):
+                filename_main = os.path.basename(filename_main)
             else:
-                search_mode.onErrorDetected(
-                    """\
-Error, no package in dir '%s' found, incomplete test case."""
-                    % filename
+                filename_main = None
+
+            if filename_main is None:
+                for filename_main in os.listdir(filename):
+                    if not os.path.isdir(os.path.join(filename, filename_main)):
+                        continue
+
+                    if filename_main not in ("..", "."):
+                        break
+                else:
+                    search_mode.onErrorDetected(
+                        """\
+Error, no package in test directory '%s' found, incomplete test case."""
+                        % filename
+                    )
+
+                extra_flags.append(
+                    "--include-package=%s" % os.path.basename(filename_main)
                 )
 
-            extensions = ["--include-package=%s" % os.path.basename(filename_main)]
+            extra_flags.append("--output-dir=%s" % getTempDir())
 
-            if "--output-dir" not in os.environ.get("NUITKA_EXTRA_OPTIONS", ""):
-                extensions.append("--output-dir=%s" % getTempDir())
-
-            with withExtendedExtraOptions(*extensions):
-                compareWithCPython(
-                    dirname=filename,
-                    filename=filename_main,
-                    extra_flags=extra_flags,
-                    search_mode=search_mode,
-                    needs_2to3=False,
-                )
-
-            if search_mode.abortIfExecuted():
-                break
+            compareWithCPython(
+                dirname=filename,
+                filename=filename_main,
+                extra_flags=extra_flags,
+                search_mode=search_mode,
+                needs_2to3=False,
+            )
 
     search_mode.finish()
 
