@@ -1,4 +1,4 @@
-#     Copyright 2019, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -17,6 +17,5 @@
 #
 from foo import entry
 
-
-if __name__ == '__main__':
-	entry.main()
+if __name__ == "__main__":
+    entry.main()
