@@ -44,42 +44,23 @@ from nuitka.tools.testing.Common import (
     createSearchMode,
     my_print,
     reportSkip,
+    scanDirectoryForTestCaseFolders,
     setup,
     withPythonPathChange,
 )
 
 
-def getMainProgramFilename(filename):
-    for filename_main in os.listdir(filename):
-        if filename_main.endswith(("Main.py", "Main")):
-            return filename_main
-
-    sys.exit(
-        """\
-Error, no file ends with 'Main.py' or 'Main' in %s, incomplete test case."""
-        % (filename)
-    )
-
-
 def main():
     # Complex stuff, even more should become common code though.
     # pylint: disable=too-many-branches,too-many-statements
 
-    python_version = setup(needs_io_encoding=True)
+    python_version = setup(suite="programs", needs_io_encoding=True)
 
     search_mode = createSearchMode()
 
     extra_options = os.environ.get("NUITKA_EXTRA_OPTIONS", "")
 
-    for filename in sorted(os.listdir(".")):
-        if (
-            not os.path.isdir(filename)
-            or filename.endswith(".build")
-            or filename.endswith(".dist")
-        ):
-            continue
-
-        filename = os.path.relpath(filename)
+    for filename, filename_main in scanDirectoryForTestCaseFolders("."):
 
         # For these, we expect that they will fail.
         expected_errors = [
@@ -105,6 +86,8 @@ def main():
         if python_version < (3,):
             expected_errors.append("named_imports")
 
+        extra_variant = []
+
         if filename not in expected_errors:
             extra_flags = ["expect_success"]
         else:
@@ -137,6 +120,12 @@ def main():
         else:
             extra_flags.append("--file-reference-choice=runtime")
 
+        # Run it as a package and also as directory.
+        if filename == "package_program":
+            # Not really supported for 2.6
+            if python_version >= (2, 7):
+                extra_variant.append("--python-flag=-m")
+
         # Cannot include the files with syntax errors, these would then become
         # ImportError, but that's not the test. In all other cases, use two
         # step execution, which will not add the program original source to
@@ -178,8 +167,6 @@ def main():
         if active:
             my_print("Consider output of recursively compiled program:", filename)
 
-            filename_main = getMainProgramFilename(filename)
-
             extra_python_path = [
                 os.path.abspath(os.path.join(filename, entry))
                 for entry in os.listdir(filename)
@@ -198,6 +185,16 @@ def main():
                     needs_2to3=False,
                 )
 
+                if extra_variant:
+                    my_print("Extra variation %r." % extra_variant)
+                    compareWithCPython(
+                        dirname=filename,
+                        filename=filename_main,
+                        extra_flags=extra_flags + extra_variant,
+                        search_mode=search_mode,
+                        needs_2to3=False,
+                    )
+
     search_mode.finish()
 
 
