@@ -1,4 +1,4 @@
-#     Copyright 2018, Kay Hayen, mailto:kay.hayen@gmail.com
+#     Copyright 2021, Kay Hayen, mailto:kay.hayen@gmail.com
 #
 #     Python tests originally created or extracted from other peoples work. The
 #     parts were too small to be protected.
@@ -15,13 +15,20 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-print( "some_package.star_importing, doing the star import" )
+""" Make a star import, and demonstrate that it has impact on the modules dict.
 
-print("Before", sorted(dir()))
+"""
+
+from __future__ import print_function
+
+from .pyexpat import *  # pylint: disable=unused-wildcard-import,wildcard-import
+
+print("some_package.star_importing, doing the star import")
+
+print("Before", sorted(x for x in dir() if x != "__compiled__"))
 
-from .pyexpat import *
 
 lala = 1
-print("After", sorted(dir()))
+print("After", sorted(x for x in dir() if x != "__compiled__"))
 
-print( "Finished" )
+print("Finished")
