@@ -17,8 +17,24 @@
 #     See the License for the specific language governing permissions and
 #     limitations under the License.
 #
-import numpy
+from __future__ import print_function
 
 # nuitka-skip-unless-imports: numpy
 
-numpy.test()
+# nuitka-project: --standalone
+# nuitka-project: --enable-plugin=numpy
+
+# Make sure, the usual bad ones are not included with anti-bloat.
+
+# nuitka-project: --enable-plugin=anti-bloat
+# nuitka-project: --noinclude-setuptools-mode=error
+# nuitka-project: --noinclude-pytest-mode=error
+# nuitka-project: --noinclude-custom-mode=numpy.distutils:error
+
+import numpy as np
+
+a = np.arange(15).reshape(3, 5)
+
+print("An array", a)
+
+print("OK.")
\ No newline at end of file
